# This is a first pass at defining metadata for old NDOS reconstructed files.

import os
import sys
import subprocess

# Globval Variables
VERBOSE = False
DEV     = False
TEST    = False 
SAMFAIL = False
FFILE   = ""
MFILE   = ""
MDDIR   = "/nova/ana/users/janzirn/MDTmp/"

def showHelp():
    print("Usage:")
    print("DeclareReco.py [options] file\n") 
    print("Declares listed file to SAM and adds their location to SAM.\n")
    print("Options:")
    print("-v,--verbose       Verbose")
    print("-t,--test          Test run (do not issue SAM commands)")
    print("-d,--dev           Use the development SAM db")

def getMetadata(file):
  global MFILE

  size    = os.path.getsize(file)
  chkSum  = subprocess.check_output("samweb file-checksum "+file,stderr=subprocess.STDOUT,shell=True) 
  fname=os.path.basename(file)

  # Figure out release
  # This needs work and will be hardcoded for the first couple of backups
  #relstartid = fname.find("S1")
  #relendid   = fname.find("_",relstartid)
  #release    = fname[relstartid:relendid]
  release = "S12.02.14"

  # Figure out run number
  # The run number is never the first thing in a name
  runstartid = fname.find("r",fname.find("_"))
  runendid   = fname.find("_",runstartid)
  fullRunNum = fname[runstartid+1:runendid]
  runNum     = fullRunNum.lstrip('0')
  #TODO Check that it's actually a valid run number, cast to int make sure it works


  # Figuge out subrun number
  srunstartid = fname.find("s",runendid)
  srunendid   = fname.find("_",srunstartid)
  fullSubRun  = fname[srunstartid+1:srunendid]
  subRun      = fullSubRun.lstrip('0')

  # Not going to work for subrun 0, also 
  # check that we have a valid subrun number
  try:
    int(fullSubRun)
    if subRun == '':
      subRun = '0'
  except ValueError:
    print fullSubRun, " is not a valid subrun number"
    sys.exit(1)

  # Figure out version number
  verstartid = fname.find("v")
  if int(verstartid) < 0:
    version = "1"
  else:
    verendid   = fname.find("_",verstartid)
    version    = fname[verstartid:verendid].lstrip('0')

  # Make sure it's a valid version, if not set it to 1
  try:
    int(version)
  except ValueError:
    version = "1"

  parent = "ndos_r"+fullRunNum+"_s"+fullSubRun+"_t01.raw" 
  parentMD = subprocess.check_output("samweb -e nova get-metadata "+parent,
				     stderr=subprocess.STDOUT,shell=True)

  # Parse the output of the parent meta data file and get the run/subrun times
  sMD = parentMD.split()

  try:
    runStartTime = sMD[sMD.index("Online.RunStartTime:")+1]
  except ValueError:
    runStartTime = ''

  try:
    runEndTime = sMD[sMD.index("Online.RunEndTime:")+1]
  except ValueError:
    runEndTime = ''

  try:
    srunStartTime = sMD[sMD.index("Online.SubRunStartTime:")+1]
  except ValueError:
    srunStartTime = ''

  try:
    srunEndTime = sMD[sMD.index("Online.SubRunEndTime:")+1]
  except ValueError:
    srunEndTime = ''

  # JSON format of Metadata
  jD = "{"
  jD+= '\n\t"file_name": "'+fname+'",'
  jD+= '\n\t"file_type": "importedDetector",'
  jD+= '\n\t"file_size": '+str(size)+','
  jD+= '\n\t"crc": '+str(chkSum)+','
  jD+= '\n\t"file_format": "root",'
  jD+= '\n\t"data_stream": "1",'
  jD+= '\n\t"data_tier": "reconstructed",'
  jD+= '\n\t"Nova.DetectorID": "ndos",'
  jD+= '\n\t"Nova.HornConfig": "none",'
  jD+= '\n\t"Nova.HornPolarity": "none",'
  jD+= '\n\t"Nova.SubVersion": "'+version+'",'
  jD+= '\n\t"Nova.Label": "preMeta",'
  jD+= '\n\t"Reconstructed.base_release": "'+release+'",'

  # The development database has different value types 
  # and different values for run_type from the normal database
  if DEV:
    jD+= '\n\t"Online.RunStartTime": "'+runStartTime+'",'
    jD+= '\n\t"Online.RunEndTime": "'+runEndTime+'",'
    jD+= '\n\t"Online.SubRunStartTime": "'+srunStartTime+'",'
    jD+= '\n\t"Online.SubRunEndTime": "'+srunEndTime+'",'
    jD+= '\n\t"Online.Subrun": "'+subRun+'",'
    jD+= '\n\t"Online.RunNumber": "'+runNum+'",'
    jD+= '\n\t"runs": [ ['+str(runNum)+',"physics"] ]'
  else:
    jD+= '\n\t"Online.RunStartTime": '+runStartTime+','
    jD+= '\n\t"Online.RunEndTime": '+runEndTime+','
    jD+= '\n\t"Online.SubRunStartTime": '+srunStartTime+','
    jD+= '\n\t"Online.SubRunEndTime": '+srunEndTime+','
    jD+= '\n\t"Online.Subrun": '+subRun+','
    jD+= '\n\t"Online.RunNumber": '+runNum+','
    jD+= '\n\t"parents": [{"file_name": "'+parent+'"}],'
    jD+= '\n\t"runs": [ ['+str(runNum)+',"commissioning"] ]'
  jD+= '\n}'

  if VERBOSE:
    print jD
  
  # Define the output file and write the data to it
  mdfile = open(MDDIR+os.path.basename(file)+'.md','w')
  mdfile.write(jD)
  mdfile.close()

  # Set the global variable so we can use it in main
  MFILE = MDDIR+os.path.basename(file)+'.md'

# Parse arguments and set global variables accordingly
def getArgs(argv):
  global VERBOSE
  global TEST
  global DEV
  global FFILE
  for opt in argv:
    if opt == '-h':
      showHelp()
      sys.exit()
    else:
      if opt in ("-v","--verbose"):
	VERBOSE = True
      if opt in ("-t","--test"):
	TEST = True
      if opt in ("-d","--dev"):
	DEV = True
      if os.path.exists(opt):
	if FFILE == '':
	  FFILE = opt
	else:
	  showHelp()
	  sys.exit(2)
  if FFILE == '':
    showHelp()
    sys.exit(3)

# For debug reasons
def printGlobvar():
  print "VERBOSE=", VERBOSE
  print "DEV=", DEV
  print "TEST=", TEST
  print "FFILE=", FFILE
  print "MFILE=", MFILE
  print "MDDIR=", MDDIR
  print "SAMFAIL=", SAMFAIL
  sys.exit()

# Delete the meta data file
def cleanUp(file):
  if os.path.isfile(file):
    os.remove(file)
  else:
    print file, "Is not a valid Meta Data file"
    sys.exit(3)

# Actually declare the metadata to SAM
def declareMetadata(file):
  global SAMFAIL
  if DEV:
    devOpt='--dev'
  else:
    devOpt=''

  if VERBOSE:
    print "samweb declare-file "+devOpt+" -e nova "+file

  try:
    subprocess.check_call("samweb declare-file "+devOpt+" -e nova "+file,stderr=subprocess.STDOUT,shell=True) 
  except subprocess.CalledProcessError:
    SAMFAIL = True
    return

  if VERBOSE:
    print "samweb add-file-location "+devOpt+" -e nova "+os.path.basename(FFILE)+" novadata:"+os.path.dirname(FFILE)

  try:
    subprocess.check_call("samweb add-file-location "+devOpt+" -e nova "+os.path.basename(FFILE)+" novadata:"+os.path.dirname(FFILE)
			,stderr=subprocess.STDOUT,shell=True) 
  except subprocess.CalledProcessError:
    SAMFAIL = True
    return

# The meat and potatoes
def main(argv):
  getArgs(argv)
  #printGlobvar()
  getMetadata(FFILE)
  
  if not TEST:
    declareMetadata(MFILE)

  cleanUp(MFILE)

  if SAMFAIL:
    sys.exit(1)
  else:
    sys.exit()

if __name__ == "__main__":
  main(sys.argv[1:])

