#!/bin/bash

##############
#Not sure how to run with a user defined test release?
#Should ask Marc about this soon. This is a feature 
#that we will need.

##############
#Initialize some variables before parceing arguments
conf=/dev/null
mcgen=false
#jobscript=`which art_sam_wrap.sh`
JOBSCRIPTt=${SRT_PUBLIC_CONTEXT}/Metadata/samUtils/art_sam_wrap.sh
label=false
helpfile=${SRT_PUBLIC_CONTEXT}/Metadata/samUtils/jobsubmissionwrap_help.txt
numfiles=1
NJOBS=
time=`date +%s`
project="${USER}-${time}"

#
# parse options we know, collect rest in $args
while [ $# -gt 0 ] 
do
    case "x$1" in 
	x-s|x--dataset)     dataset="$2";             shift; shift; continue;;
	x-c|x--config)      conf="$2";                shift; shift; continue;;
	x-D|x--destination) dest="$2";                shift; shift; continue;;
	x-R|x--rename)      initial="$2"; swap="$3";  shift; shift; shift; continue;;
	x--jobname)         jobname="$2"; label=true; shift; shift; continue;;
	x-v|x--release)     release="$2";             shift; shift; continue;;
	x-n|x--events)      numevents="-n $2";        shift; shift; continue;;
	x-N|x--jobs)        njobs="$2";               shift; shift; continue;;
	x-G|x--mcgen)       mcgen=true;               shift; continue;;
	x-o|x--out)         outname="-o $2";          shift; shift; continue;;
	x-f|x--files        numfiles="$2";            shift; shift; continue;;
	x-p|x--project      project="$2";             shift; shift; continue;;
	x-h|x--help)        cat $helpfile;            shift; continue;;
	*)                  args="$args \"$1\"";      shift; continue;;
    esac
    break
done
$rename="s/$initial/$swap/"

##############
#set either mc generation from stored sam fcl or process using passed fcl
if [ "$mcgen" = "true" ]; then
    config="--getconfig"
else
    config="--config $conf"
fi

##############
#set the name of the job different from art_sam_wrap.sh 
#for differentiating between job sets within condor_tmp area
if [ "$label" = "true" ]; then
    ln -s $JOBSCRIPT /nova/app/condor_exec/${USER}/$jobname
    $JOBSCRIPT=/nova/app/condor_exec/${USER}/$jobname
fi

##############
# Setup the Job specific parameters
DATASET=$dataset
NUMEVENTS="$numevents"
NUMFILES=${numfiles}
NUMJOBS=$njobs
OUTPUT_DIR="$dest"
EXECUTABLE="nova"

#############
# Set the Experiment Name
export EXPERIMENT=nova
#############
# Set the NOvA Base Release to use
nova_release="$release"

#############
# Setup the Common Tools Area (needed for IFDH and Jobsub)
source /grid/fermiapp/products/common/etc/setups.sh
PRODUCTS=/nusoft/app/externals:$PRODUCTS

#############
# Setup Jobsub and IFDH (art)
setup jobsub_tools v1_2n
setup ifdh_art v1_2_10 -z /nusoft/app/externals -q e4:debug:nu

#############
# Configure IFDH (and optional debug level)
export IFDH_BASE_URI="http://samweb.fnal.gov:8480/sam/nova/api"
export IFDH_DEBUG=1
unset  IFDH_STAGE_VIA
#############
# Set the SAM station to use. 
# Options are nova or nova-int (for the integration station)
export SAM_STATION="nova"

##########################################
# The Magic Jobsub command
#--rename2 "s/daq/reco" \
#--addoutput '*.root' \
#--rename2 unique \
jobsub \
    -N ${NUMJOBS} \
    --opportunistic \
    --nowrapfile \
    -e SAM_STATION \
    -e IFDH_BASE_URI \
    -e IFDH_DEBUG \
    -e EXPERIMENT \
    -g \
    --dataset_definition=${DATASET} \
    --project_name=${project} \
    ${JOBSCRIPT} \
            --export EXTERNALS='$PRODUCTS' \
            ${config} \
	    --config 'Utilities/batch/recoproductionjob.fcl' \
            --source '$CVMFS_DISTRO_BASE/novasvn/srt/srt.sh' \
            --source '$CVMFS_DISTRO_BASE/novasvn/setup/setup_novasoft.sh:-r:'"$nova_release" \
	    --limit ${NUMFILES} \
            --rename $rename \
            --rename2 unique \
            --dest srm://fg-bestman1.fnal.gov:10443/srm/v2/server?SFN=${OUTPUT_DIR} \
	    -X ${EXECUTABLE} \
  	        --sam-file-type=importedSimulated \
	        --sam-application-family=nova \
	        --sam-data-tier=reco \
	        --sam-application-version=$nova_release \
                ${NUMEVENTS} ${outname} \
