##########################################
# Function Declarations
#
# For Job-specific parameters, 
#   skip this section
##########################################
function do_submit()
{

# Determine whether to use --config or --getconfig
CONFIG="--config ${FCL_FILE}"
if [ "x${FCL_FILE}" == "x" ]; then
    CONFIG="--getconfig"
fi
HIGHPRIO="-G 'nova_high_prio'"
if [ "x${highprio}" == "x" ]; then
    HIGHPRIO=""
fi
# Submit Onsite AND Offsite
#
# The Magic Jobsub command
jobsub \
    -N ${JOBSUB_NUMJOBS} \
    --site ${SITES} \
    --OS=SL5,SL6 \
    --opportunistic \
    --nowrapfile \
    --X509_USER_PROXY $X509_USER_PROXY \
    -e SAM_STATION \
    -e SAM_PROJECT_NAME \
    -e IFDH_BASE_URI \
    -e IFDH_DEBUG \
    -e EXPERIMENT \
    -e NOVADBTIMEOUT \
    -e NOVADBUSER  \
    -e NOVADBPWDFILE \
    -l "+JobType = \"MC\"" \
    -g \
    ${HIGHPRIO} \
    ${JOBSCRIPT} \
            --multifile \
            --export EXTERNALS='$PRODUCTS' \
            --export CVMFS_DISTRO_BASE='$PRODUCTS/..' \
            ${CONFIG} \
            --source '$CVMFS_DISTRO_BASE/novasvn/srt/srt.sh' \
            --source '$CVMFS_DISTRO_BASE/novasvn/setup/setup_novasoft_cvmfs.sh:-b:maxopt:-r:'"$nova_release" \
            --source '$CVMFS_DISTRO_BASE/novasvn/setup/setup_novagrid.sh:-r:'"${NOVA_RELEASE}:-b:maxopt" \
	    --source "/cvmfs/oasis.opensciencegrid.org/fermilab/products/common/etc/setups.sh;setup:sam_web_client:v1_5:-z:/cvmfs/oasis.opensciencegrid.org/fermilab/products/common/db" \
            --export PID_LIB_PATH='$CVMFS_DISTRO_BASE/nova/data/pidlibs/S14-04-29' \
            --limit ${NUMFILES} \
            --self-destruct-timer $TIMEOUT \
            --postscript '${SRT_PUBLIC_CONTEXT}/Metadata/samUtils/copyOutFiles.sh' \
            --export DEST=${OUTPUT_DIR} \
	    -X ${EXECUTABLE} \
              --outTier=out1:pidpart \
              --outTier=out2:lemsum \
              --copyOut \
              --hashDirs

}

function make_project() {
# Set up SAM-specific parameters and create a project if necessary

# Set the SAM station to use. 
# Options are nova or nova-int (for the integration station)
export SAM_STATION="nova"

# If there's no project name defined, create a default name
if [[ "x$PROJECTNAME" == "x" ]]; then
  DATE="`date +%Y%m%d_%H%M%S_%N`"
  PROJECTNAME="${USER}-${JOBNAME}-${DATE}"
  ifdh startProject "${PROJECTNAME}" $SAM_STATION "$DATASET" $USER $SAM_STATION
  if [ $? -eq 0 ]; then
      export SAM_PROJECT_NAME="${PROJECTNAME}"
  else
     echo "Error starting project!" >&2
     exit 1
  fi
else
  # See if this is an already-running project...
  ifdh findProject "${PROJECTNAME}"
  if [ $? -ne 0 ]; then
    # It's not; start it
    ifdh startProject "${PROJECTNAME}" $SAM_STATION "$DATASET" $USER $SAM_STATION
  fi
  if [ $? -eq 0 ]; then
    export SAM_PROJECT_NAME="${PROJECTNAME}"
  else
    echo "Error starting project!" >&2
    exit 1
  fi
fi

# For keeping track...
echo "$SAM_PROJECT_NAME" >>projectnames.log

}

function submit_prioritized {
  # Submit half the jobs to the NOvA Dedicated sites. The rest can go anywhere
  local ok_to_submit=0
  local definition_files=`samweb -e nova count-definition-files $DATASET`
  if [ $definition_files -ge ${NUMINPUTS} ] && [ ${RECOVERY_DATASET} -eq 0 ]; then
    ok_to_submit=1
  fi

  if [ $definition_files -gt 0 ] && [ ${RECOVERY_DATASET} -eq 1 ]; then
    ok_to_submit=1
  elif [ ${RECOVERY_DATASET} -eq 1 ]; then
    return 0
  fi

  if [ ${ok_to_submit} -eq 1 ]; then
    if [ $NUMJOBS -lt 0 ]; then
      NUMJOBS=$definition_files
    fi
    make_project
    JOBSUB_NUMJOBS=$(( $NUMJOBS / 2 + $NUMJOBS % 2 ))
    highprio=
#    SITES="FNAL_nova,FNAL_nova_opportunistic"
    SITES="OSC,Harvard,SMU,SMU_nova"
    do_submit
    JOBSUB_NUMJOBS=$(( 2 * $NUMJOBS / 4 ))
#    highprio=1
#    SITES="FNAL_nova,FNAL_nova_opportunistic,FZU,Harvard,MWT2,Nebraska,Omaha,OSC,SMU,SMU_nova,UCSD,Wisconsin"
    do_submit
    return 0
  fi

  return 1
}
##########################################
# END of Function Declarations
##########################################

##########################################
# Basic Configuration
#
# If you're using this script to start
# multiple projects, these are the parameters
# that should be shared between all of them
##########################################

##############
# These will have to change based on the type of job you're running
OUTPUT_DIR="/pnfs/nova/scratch/fts/ParticleID_dropbox" # Where are the output files going to go?
#OUTPUT_DIR="/nova/ana/users/eflumerf" # Where are the output files going to go?
TIMEOUT=12960000 # If the job runs longer than this (in seconds), it will try to kill itself to get log files back
ROLE=Production # Which role to use, options are blank, Production
NUMINPUTS=1 # What is the target number of files in the input dataset?
NUMJOBS=-1 # How many jobs are we going to start? -1 for as many are in the dataset
RECOVERY_DATASET=0 # Is this a recovery run? 1: Yes, 0: No
DATA_TIER="pidpart" # Data tier: one of out1:artdaq, reco, ...
FILETYPE="importedSimulated" # File Type: importedSimulated for MC, importedDetector for Data
FCL_FILE='${SRT_PUBLIC_CONTEXT}/Utilities/batch/pidproductionjob.fcl' # Job FCL file to use, blank for getconfig case
#DATASET="prodreco_S14-03-25_FDGENIE" # Dataset definition to use
#JOBNAME="SAMPIDJobFDGenie" # Name of this job

##########################################
# Other Environment Variables needed to start jobs
# 
# Most of these can be left alone unless you know
# EXACTLY what you're doing
##########################################

#############
# Job wrapper parameters
#EXECUTABLE="nova"
EXECUTABLE='${SRT_PUBLIC_CONTEXT}/Metadata/samUtils/runNovaSAM.py'
NUMFILES=1
JOBSCRIPT="${SRT_PUBLIC_CONTEXT}/Metadata/samUtils/art_sam_wrap.sh"

#############
# Set the Experiment Name
export EXPERIMENT=nova

#############
# Set the NOvA Base Release to use
nova_release="S14-05-05"
NOVA_RELEASE="S14-05-05"

#############
# Setup the Common Tools Area (needed for IFDH and Jobsub)
source /grid/fermiapp/products/common/etc/setups.sh
PRODUCTS=/nusoft/app/externals:/grid/fermiapp/products/common/db:$PRODUCTS

#############
# Configure IFDH (and optional debug level)
export IFDH_BASE_URI="http://samweb.fnal.gov:8480/sam/nova/api"
export IFDH_DEBUG=1
unset  IFDH_STAGE_VIA

#############
# Configure Grid Proxy
if [[ "x$ROLE" == "x" ]]; then
  X509_USER_PROXY=/scratch/$USER/grid/$USER.$EXPERIMENT.proxy
else
  X509_USER_PROXY=/scratch/$USER/grid/$USER.$EXPERIMENT.$ROLE.proxy
fi

#############
# Setup Jobsub and IFDH (art)
setup sam_web_client
setup jobsub_tools v1_2q
setup ifdh_art v1_4_1 -z /nusoft/app/externals -q e4:debug:nu
#if [ "`which $JOBSCRIPT`" != $JOBSCRIPT ]; then
#    JOBSCRIPT="`which $JOBSCRIPT`"
#fi

#############
# Loop Control Parameters
FHCStarted=0
RHCStarted=0

##########################################
# Submission Wait Loop
# 
# This script will wait for all of anticipated files to be declared
# then it will submit jobs
##########################################

while [ $FHCStarted -eq 0 ] && [ $RHCStarted -eq 0 ]; do
  # Example of single-project submission (parameters defined above)
  # if [ $JobStarted -eq 0 ]; then
  #   submit_prioritized
  #   JobStarted=$(( 1 - $? ))
  # fi
 
  # I haven't started FHC NonSwap Jobs yet...

  if [ $FHCStarted -eq 0 ]; then
    NUMJOBS=-1
    DATASET=S14-03-25-GenieFD-Reco-Draining
    JOBNAME=SAMPIDProductionJob
    PROJECTNAME=
    submit_prioritized
    FHCStarted=$(( 1 - $? ))
  fi
  # Be nice about samweb calls, space things out a little bit
  if [ $FHCStarted -eq 0 ] && [ $RHCStarted -eq 0 ]; then
    sleep 300
  fi
done

