##########################################
# Function Declarations
#
# For Job-specific parameters, 
#   skip this section
##########################################
function do_submit()
{

# Determine whether to use --config or --getconfig
CONFIG="--config ${FCL_FILE}"
if [ "x${FCL_FILE}" == "x" ]; then
    CONFIG="--getconfig"
fi

# Don't submit in debug mode
if [ $DEBUG -eq 1 ]; then
  DEBUGSTRING="echo"
fi

# Submit Onsite AND Offsite
#
# The Magic Jobsub command
${DEBUGSTRING} jobsub \
    -N ${NUMJOBS} \
    --site ${SITES} \
    --OS=SL5,SL6 \
    --opportunistic \
    --nowrapfile \
    --X509_USER_PROXY $X509_USER_PROXY \
    -e SAM_STATION \
    -e SAM_PROJECT_NAME \
    -e IFDH_BASE_URI \
    -e IFDH_DEBUG \
    -e EXPERIMENT \
    -l "+JobType = \"MC\"" \
    -g \
    ${JOBSCRIPT} \
            --export EXTERNALS='$PRODUCTS' \
            --export CVMFS_DISTRO_BASE='$PRODUCTS/..' \
            ${CONFIG} \
            --source '$CVMFS_DISTRO_BASE/novasvn/srt/srt.sh' \
            --source '$CVMFS_DISTRO_BASE/novasvn/setup/setup_novasoft_cvmfs.sh:-b:maxopt:-r:'"$nova_release" \
	    --limit ${NUMFILES} \
            --addoutput '*.daq.root' \
	    --rename unique \
            --self-destruct-timer $TIMEOUT \
	    --dest srm://fg-bestman1.fnal.gov:10443/srm/v2/server?SFN=${OUTPUT_DIR} \
	    -X ${EXECUTABLE} \
  	        --sam-file-type=${FILETYPE} \
	        --sam-application-family=nova \
	        --sam-data-tier=${DATATIER} \
	        --sam-application-version=$nova_release

}

function make_project() {
# Set up SAM-specific parameters and create a project if necessary

# Set the SAM station to use. 
# Options are nova or nova-int (for the integration station)
export SAM_STATION="nova"

# If there's no project name defined, create a default name
if [[ "x$PROJECTNAME" == "x" ]]; then
  DATE="`date +%Y%m%d_%H%M%S_%N`"
  PROJECTNAME="${USER}-${JOBNAME}-${DATE}"
  ifdh startProject "${PROJECTNAME}" $SAM_STATION "$DATASET" $USER $SAM_STATION
  if [ $? -eq 0 ]; then
      export SAM_PROJECT_NAME="${PROJECTNAME}"
  else
     echo "Error starting project!" >&2
     exit 1
  fi
else
  # See if this is an already-running project...
  ifdh findProject "${PROJECTNAME}"
  if [ $? -ne 0 ]; then
    # It's not; start it
    ifdh startProject "${PROJECTNAME}" $SAM_STATION "$DATASET" $USER $SAM_STATION
  fi
  if [ $? -eq 0 ]; then
    export SAM_PROJECT_NAME="${PROJECTNAME}"
  else
    echo "Error starting project!" >&2
    exit 1
  fi
fi

# For keeping track...
echo "$SAM_PROJECT_NAME" >>projectnames.log

}

function submit_prioritized {
  # Submit half the jobs to the NOvA Dedicated sites. The rest can go anywhere
  if [ `samweb -e nova count-definition-files $DATASET` -ge ${NUMINPUTS} ]; then
    if [ $DEBUG -ne 1 ]; then
    make_project
    fi
    NUMJOBS=$(( $NUMINPUTS / 2 + $NUMINPUTS % 2 ))
    SITES="OSC,Harvard,HARVARD_nova,SMU,SMU_nova"
    do_submit
    NUMJOBS=$(( $NUMINPUTS / 2 ))
    SITES="FNAL_nova,FNAL_nova_opportunistic,FZU,Harvard,HARVARD_nova,KISTI,MIT,MWT2,Nebraska,Omaha,OSC,SMU,SMU_nova,UChicago,UCSD,Wisconsin"
    do_submit
    return 0
  fi

  return 1
}
##########################################
# END of Function Declarations
##########################################

##########################################
# Basic Configuration
#
# If you're using this script to start
# multiple projects, these are the parameters
# that should be shared between all of them
##########################################

##############
# These will have to change based on the type of job you're running
OUTPUT_DIR="/nova/prod/FTS_DropBoxes/MCdaq_dropbox" # Where are the output files going to go?
TIMEOUT=1296000 # If the job runs longer than this (in seconds), it will try to kill itself to get log files back
ROLE=Production # Which role to use, options are blank, Production
NUMINPUTS=704 # What is the target number of files in the input dataset?
DATATIER="out1:artdaq" # Data tier: one of out1:artdaq, reco, ...
FILETYPE="importedSimulated" # File Type: importedSimulated for MC, importedDetector for Data
FCL_FILE= # Job FCL file to use, blank for getconfig case
DATASET= # Dataset definition to use
JOBNAME= # Name of this job
DEBUG=1 # Should I actually submit or not?

##########################################
# Other Environment Variables needed to start jobs
# 
# Most of these can be left alone unless you know
# EXACTLY what you're doing
##########################################

#############
# Job wrapper parameters
EXECUTABLE="nova"
NUMFILES=1
JOBSCRIPT="art_sam_wrap.sh"

#############
# Set the Experiment Name
export EXPERIMENT=nova

#############
# Set the NOvA Base Release to use
nova_release="S14-03-20"

#############
# Setup the Common Tools Area (needed for IFDH and Jobsub)
source /grid/fermiapp/products/common/etc/setups.sh
PRODUCTS=/nusoft/app/externals:/grid/fermiapp/products/common/db:$PRODUCTS

#############
# Configure IFDH (and optional debug level)
export IFDH_BASE_URI="http://samweb.fnal.gov:8480/sam/nova/api"
export IFDH_DEBUG=1
unset  IFDH_STAGE_VIA

#############
# Configure Grid Proxy
if [[ "x$ROLE" == "x" ]]; then
  X509_USER_PROXY=/scratch/$USER/grid/$USER.$EXPERIMENT.proxy
else
  X509_USER_PROXY=/scratch/$USER/grid/$USER.$EXPERIMENT.$ROLE.proxy
fi

#############
# Setup Jobsub and IFDH (art)
setup jobsub_tools v1_2u
setup ifdh_art v1_3_2 -z /nusoft/app/externals -q e4:debug:nu
if [ "`which $JOBSCRIPT`" != $JOBSCRIPT ]; then
    JOBSCRIPT="`which $JOBSCRIPT`"
fi

#############
# Loop Control Parameters
RHCSwapStarted=0
FHCSwapStarted=0
RHCNonSwapStarted=0
FHCNonSwapStarted=0

##########################################
# Submission Wait Loop
# 
# This script will wait for all of anticipated files to be declared
# then it will submit jobs
##########################################

while [ $(($RHCSwapStarted + $FHCSwapStarted + $RHCNonSwapStarted + $FHCNonSwapStarted)) -lt 4 ]; do
  # Example of single-project submission (parameters defined above)
  # if [ $JobStarted -eq 0 ]; then
  #   submit_prioritized
  #   JobStarted=$(( 1 - $? ))
  # fi
 
  # I haven't started RHC Swap Jobs yet...
  if [ $RHCSwapStarted -eq 0 ]; then
    #Dataset, Jobname and Project Name are unique to this configuration
    DATASET=S14-03-20-GenieFD-RHCSwap-Draining
    JOBNAME="SAMProductionJobFDGenieRHCSwap"
    PROJECTNAME="$USER-$JOBNAME-`date +%s`"
    submit_prioritized
    # submit_prioritized return 0 if successful (jobs were submitted), 1 otherwise
    RHCSwapStarted=$(( 1 - $? ))
  fi

  if [ $RHCNonSwapStarted -eq 0 ]; then
    DATASET=S14-03-20-GenieFD-RHCNonSwap-Draining
    JOBNAME="SAMProductionJobFDGenieRHCNonSwap"
    PROJECTNAME="$USER-$JOBNAME-`date +%s`"
    submit_prioritized
    RHCNonSwapStarted=$(( 1 - $? ))
  fi

  if [ $FHCSwapStarted -eq 0 ]; then
    DATASET=S14-03-20-GenieFD-FHCSwap-Draining
    JOBNAME="SAMProductionJobFDGenieFHCSwap"
    PROJECTNAME="$USER-$JOBNAME-`date +%s`"
    submit_prioritized
    FHCSwapStarted=$(( 1 - $? ))
  fi

  if [ $FHCNonSwapStarted -eq 0 ]; then
    DATASET=S14-03-20-GenieFD-FHCNonSwap-Draining
    JOBNAME="SAMProductionJobFDGenieFHCNonSwap"
    PROJECTNAME="$USER-$JOBNAME-`date +%s`"
    submit_prioritized
    FHCNonSwapStarted=$(( 1 - $? ))
  fi

  # Be nice about samweb calls, space things out a little bit
  sleep 300
done

