##############
# Setup the Job specific parameters
DATASET=S14-01-20GenieND-FCLOnly
SITES="Harvard UCSD SMU OSC Nebraska Wisconsin MWT2"
NUMFILES=1
NUMJOBS=40
JOBSCRIPT="${SRT_PUBLIC_CONTEXT}/Metadata/samUtils/art_sam_wrap.sh"
JOBNAME="SAMTestJobNDGenie"
OUTPUT_DIR="/nova/ana/users/eflumerf"
EXECUTABLE="nova"
TIMEOUT=36000

#############
# Set the Experiment Name
export EXPERIMENT=nova
#############
# Set the NOvA Base Release to use
nova_release="S14-01-20"

#############
# Setup the Common Tools Area (needed for IFDH and Jobsub)
source /grid/fermiapp/products/common/etc/setups.sh
PRODUCTS=/nusoft/app/externals:$PRODUCTS

#############
# Setup Jobsub and IFDH (art)
setup jobsub_tools v1_2p
setup ifdh_art v1_3_1 -z /nusoft/app/externals -q e4:debug:nu

#############
# Configure IFDH (and optional debug level)
export IFDH_BASE_URI="http://samweb.fnal.gov:8480/sam/nova/api"
export IFDH_DEBUG=1
unset  IFDH_STAGE_VIA

#############
# Set the SAM station to use. 
# Options are nova or nova-int (for the integration station)
export SAM_STATION="nova"
DATE="`date +%Y%m%d_%H%M%S_%N`"
PROJECTNAME="${USER}-${JOBNAME}-${DATE}"
ifdh startProject "${PROJECTNAME}" $SAM_STATION "$DATASET" $USER $SAM_STATION
if [ $? -eq 0 ]; then
    export SAM_PROJECT_NAME="${PROJECTNAME}"
fi
echo "Project name is: $SAM_PROJECT_NAME"

#############
# Config Option
CONFIG="--config ${FCL_FILE}"
if [ "x${FCL_FILE}" == "x" ]; then
    CONFIG="--getconfig"
fi

##########################################
# Submit Onsite
#

##########################################
# The Magic Jobsub command
# Inactive options:
# --rename2 "s/daq/reco"  
# --dataset_definition=${DATASET}
jobsub \
    -N ${NUMJOBS} \
    --OS=SL5,SL6 \
    --nowrapfile \
    -e SAM_PROJECT_NAME \
    -e SAM_STATION \
    -e IFDH_BASE_URI \
    -e IFDH_DEBUG \
    -e EXPERIMENT \
    -l "+JobType=\"MC\"" \
    -g \
    ${JOBSCRIPT} \
            --export EXTERNALS='$PRODUCTS' \
            --export CVMFS_DISTRO_BASE='$PRODUCTS/..' \
	    ${CONFIG} \
            --source '$CVMFS_DISTRO_BASE/novasvn/srt/srt.sh' \
            --source '$CVMFS_DISTRO_BASE/novasvn/setup/setup_novasoft_cvmfs.sh:-r:'"$nova_release" \
	    --limit ${NUMFILES} \
            --addoutput '*.root' \
	    --rename unique \
            --self-destruct-timer $TIMEOUT \
	    --dest srm://fg-bestman1.fnal.gov:10443/srm/v2/server?SFN=${OUTPUT_DIR} \
	    -X ${EXECUTABLE} \
  	        --sam-file-type=importedSimulated \
	        --sam-application-family=nova \
	        --sam-data-tier=out1:artdaq \
	        --sam-application-version=$nova_release

##########################################
# Submit Offsite
#

for site in ${SITES}; do
#########################################
# The Magic Jobsub command
jobsub \
    -N ${NUMJOBS} \
    --site $site \
    --OS=SL5,SL6 \
    --opportunistic \
    --nowrapfile \
    -e SAM_PROJECT_NAME \
    -e SAM_STATION \
    -e IFDH_BASE_URI \
    -e IFDH_DEBUG \
    -e EXPERIMENT \
    -l "+JobType = \"MC\"" \
    -g \
    ${JOBSCRIPT} \
           --export EXTERNALS='$PRODUCTS' \
           --export CVMFS_DISTRO_BASE='$PRODUCTS/..' \
	    ${CONFIG} \
            --source '$CVMFS_DISTRO_BASE/novasvn/srt/srt.sh' \
           --source '$CVMFS_DISTRO_BASE/novasvn/setup/setup_novasoft_cvmfs.sh:-r:'"$nova_release" \
	    --limit ${NUMFILES} \
           --addoutput '*.root' \
	    --rename unique \
           --self-destruct-timer $TIMEOUT \
    --dest srm://fg-bestman1.fnal.gov:10443/srm/v2/server?SFN=${OUTPUT_DIR} \
	    -X ${EXECUTABLE} \
  	        --sam-file-type=importedSimulated \
	        --sam-application-family=nova \
	        --sam-data-tier=out1:artdaq \
	        --sam-application-version=$nova_release

done
