#!/bin/bash

# ./SubmitAllRecoMCDatasets.sh <simtag_name> <recotag_name>
if [ $# -ge 2 ]
    then
    :
else
    echo Usage: $0 '<simtag>' '<recotag>' '<test?>' '[optional]<official>'
    exit 1
fi

# Only these two need to be configured.
# TODO: Make TEST configurable at command line.
# TODO: Add Error checking
SIM_TAG=$1
RECO_TAG=$2
TEST=$3
OFFICIAL=$4
VALIDATE="validate_"

# if running official, non-validation sets then empty the VALIDATE string
if [ ! -z "$OFFICIAL" ]; then
    VALIDATE=""
fi
# Don't forget to change the SIM_TAG if using a dataset of sim artdaq files
# from a different tag

echo "SIMULATION tag is: $SIM_TAG"
echo "RECONSTRUCTION tag is: $RECO_TAG"

source /grid/fermiapp/nova/novaart/novasvn/setup/setup_nova.sh -r $RECO_TAG -b maxopt

if [ "$TEST" -eq "test" ]; then
    echo "This is a test job"
    export OUTPUT_DIR="/nova/ana/users/$USER/tmp"
    export NUMJOBS=1
else 
    echo "This is NOT a test job. You are submitting everything!"
    export OUTPUT_DIR="/pnfs/nova/scratch/fts/Reconstruction_dropbox/"
    export NUMJOBS=-1
fi

echo $OUTPUT_DIR
echo $NUMJOBS

FCLDIR="Production/fcl"
DIR="Production/submission"

if [ "$NOVA_RELEASE" == "FA14-10-03" ]; then
    FCLDIR="ProductionScripts/fhicl"
    DIR="ProductionScripts/submission"
fi
echo $FCLDIR
echo $DIR
COSMICFCL="prod_reco_cosmic_job.fcl"
GENIEFCL="prod_reco_numi_job.fcl"

# ${SRT_PUBLIC_CONTEXT}/${DIR}/SubmitRecoMCDataset.sh <DATASET> <NUMFILES> <JOBFCL>

if [ "$TEST" -eq "test" ]; then
    echo ${SRT_PUBLIC_CONTEXT}/${DIR}/SubmitRecoMCDataset.sh prod_daq_${SIM_TAG}_fd_genie_fhc_fluxswap_${VALIDATE}draining 1 $FCLDIR/$GENIEFCL
    ${SRT_PUBLIC_CONTEXT}/${DIR}/SubmitRecoMCDataset.sh prod_daq_${SIM_TAG}_fd_genie_fhc_fluxswap_${VALIDATE}draining 1 $FCLDIR/$GENIEFCL
    
else
    #FD GENIE FHC
    echo "Submitting FD GENIE FHC FLUXSWAP..."
    ${SRT_PUBLIC_CONTEXT}/${DIR}/SubmitRecoMCDataset.sh prod_daq_${SIM_TAG}_fd_genie_fhc_fluxswap_${VALIDATE}draining 1 $FCLDIR/$GENIEFCL

    echo "Submitting FD GENIE FHC TAU..."
    ${SRT_PUBLIC_CONTEXT}/${DIR}/SubmitRecoMCDataset.sh prod_daq_${SIM_TAG}_fd_genie_fhc_tau_${VALIDATE}draining 1 $FCLDIR/$GENIEFCL
    
    echo "Submitting FD GENIE FHC NONSWAP..."
    ${SRT_PUBLIC_CONTEXT}/${DIR}/SubmitRecoMCDataset.sh prod_daq_${SIM_TAG}_fd_genie_fhc_nonswap_${VALIDATE}draining 1 $FCLDIR/$GENIEFCL
    
    echo "Submitting FD GENIE RHC NONSWAP..."
    #FD GENIE RHC
    ${SRT_PUBLIC_CONTEXT}/${DIR}/SubmitRecoMCDataset.sh prod_daq_${SIM_TAG}_fd_genie_rhc_nonswap_${VALIDATE}draining 1 $FCLDIR/$GENIEFCL
    
    echo "Submitting FD GENIE FHC FLUXSWAP..."
    ${SRT_PUBLIC_CONTEXT}/${DIR}/SubmitRecoMCDataset.sh prod_daq_${SIM_TAG}_fd_genie_rhc_fluxswap_${VALIDATE}draining 1 $FCLDIR/$GENIEFCL
    
    echo "Submitting FD GENIE RHC TAU..."
    ${SRT_PUBLIC_CONTEXT}/${DIR}/SubmitRecoMCDataset.sh prod_daq_${SIM_TAG}_fd_genie_rhc_tau_${VALIDATE}draining 1 $FCLDIR/$GENIEFCL
    
    # ND CRY
    # NOTE: non-standard dataset here! For future, this needs to be standardised
    echo "Submitting ND CRY..."
    ${SRT_PUBLIC_CONTEXT}/${DIR}/SubmitRecoMCDataset.sh prod_daq_${SIM_TAG}_nd_cry__all_draining 5 $FCLDIR/$COSMICFCL
    
    # FD CRY
    echo "Submitting FD CRY..."
    ${SRT_PUBLIC_CONTEXT}/${DIR}/SubmitRecoMCDataset.sh prod_daq_${SIM_TAG}_fd_cry_all_${VALIDATE}draining 10 $FCLDIR/$COSMICFCL

fi
