#!/bin/bash

kx509

TIER=raw
RELEASE=$1

DET=$(echo $2 | tr [:upper:] [:lower:])

STREAM=$3

case `echo $STREAM` in
    0) MASK=numi ;;
    1) MASK=bnb ;;
    2) MASK=cosmic ;;
    4) MASK=cosmic ;;
esac

WORKDIR=$PWD

TODAY=$(date +%D --date="today")
TODAY_SECONDS=$(date +%s -d "${TODAY}")
YESTERDAY=$(date +%D --date="yesterday")
YESTERDAY_SECONDS=$(date +%s -d "${YESTERDAY}")

read -r -d '' DIMS <<EOF
file_type       = 'importedDetector' and         
data_tier       = '${TIER}'  and not                                                                               
isparentof:     (data_tier = 'artdaq' and DAQ2RawDigit.base_release = '${RELEASE}')  and
Online.Detector = '${DET}' and                                                                                   
Online.Stream   = '${STREAM}' and                                                                                
Online.SubRunStartTime >= '${YESTERDAY_SECONDS}' and
Online.SubRunStartTime < '${TODAY_SECONDS}'
EOF

DSET=${DET}_data_${TIER}_t0${STREAM}_${RELEASE}_keepup.$(date +%m-%d-%Y --date="yesterday")

echo "Creating dataset ${DSET} with the dimensions:"

echo ""

echo "${DIMS} \n"

echo ""

samweb -e nova create-definition "${DSET}" "${DIMS}"


