#!/bin/bash

# These environment variables need to be passed during job submission.
# Using either -e option in jobsub or --export option in art_sam_wrap.sh
: ${SECONDARY_DATASET:?"Need to set SECONDARY_DATASET non-empty"}
: ${NUMBER_OF_SECONDARY:?"Need to set NUMBER_OF_SECONDARY non-empty"}
: ${OVERLAY_FCL:?"Need to set OVERLAY_FCL non-empty"}
: ${conf:?"Need the name of the fcl configuration file (conf) from generation stage"}

# Make sure the generation config file is there.  This catches failures due to 
# ifdh fetchInput timeout.
if [[ !("$conf" = *.fcl && ("$conf" = fardet* || "$conf" = neardet*)) ]]; then
    echo "No original config file found exiting (error code 5)"
    exit 5
fi

echo "NUMBER_OF_SECONDARY: $NUMBER_OF_SECONDARY"


FCL_PATH=`dirname $OVERLAY_FCL`
FCL_FILE=`basename $OVERLAY_FCL`
if [ "$FCL_PATH" = "." ] ; then
    FCL_PATH=$SRT_PUBLIC_CONTEXT/job
fi

if [ "$SPECIAL_NAME" != "" ];then
    if [ "$SPECIAL_NAME" = "geantonly" ]; then
	SPECIAL_NAME="none"
    else
	SPECIAL_NAME=`echo $SPECIAL_NAME | sed s/-geantonly//`
    fi
else
    # If SPECIAL_NAME is not set something when wrong.
    # Set it to to something innocuous.
    SPECIAL_NAME="none"
fi

# Need to setup up samweb client in order to do samweb stuff later

echo 'Verify samweb client'
which_samweb=`which samweb`
echo "samweb is: $which_samweb"

echo "Find the previously generated detector only file" 
to_process_list=`find ./ -name '*.g4.root'` 

echo "Make a directory to hold the secondary files" 
mkdir overlay 
my_pwd=`pwd` 
echo "$my_pwd" 
cd overlay 
echo "pwd: $PWD" 

# Get all files from the secandary file list
#file_list=`samweb -e nova list-definition-files ${SECONDARY_DATASET}` 
samweb -e nova list-definition-files ${SECONDARY_DATASET} > file_list.out
got_file_list=$?

cat file_list.out

# Get the number of files in the file list
let range=`samweb -e nova count-definition-files ${SECONDARY_DATASET}` 

echo $range
got_range=$?

# Block to make a list of NUMBER_OF_SECONDARY files out of file_list of size range
if [[ $got_file_list -ne 0 || $got_range -ne 0 ]]; then
    echo "samweb failure of either $got_file_list or $got_range"
    exit 1
fi

echo "Creating secondary file list"
cat file_list.out | perl -pe "print rand(),' ',$_" | sort -g | head -n ${NUMBER_OF_SECONDARY} | awk '{print $2}' > overlays.txt
echo ""
echo ""
echo ""
echo "**********************"
echo "Secondary file list is:"
cat overlays.txt
list=`cat overlays.txt`

if [ $? -ne 0 ]; then 
    echo "Failed to get secondary file list"
    exit 2
fi

#while read line
for line in $list
do
    echo "Getting secondary file $line"
    ifdh_fetch -e nova $line
    #echo "$line" >> overlays.txt
done

# Concatenate the secondary files into one file.  
# Makes it easier to manage the later overlay job.
# Assumes that nova has already been setup.
# There are two reasons to concatenate the files together.
# 1) Makes it so that the overlay job config fcl can be set to only one name.  
#    Making the scripting easier.
# 2) The art mixer goes through files in the list one at a time.  
#    This will bias the mixed events toward files that
#    were made earlier.  Potentially undersampling some files and 
#    over sampling others.
#    By globbing them all together this effect can be mitigated.
echo "Glob all the secondary files together into one file" 
command="nova -c ${SRT_PUBLIC_CONTEXT}/Utilities/concat_files.fcl -S ./overlays.txt -o ./overlayfiles.root" 
echo "$command" 
eval $command 
if [ $? -ne 0 ]; then
    echo "Failed to concatenate secondary files together"
    exit 3
fi
echo " "
echo "pwd is: $PWD"
ls 
echo " " 
cd $my_pwd 

# Thought.  The mixer uses a constant seed.  
# Should the seed be randomized within this script?...
# Now doing this.  Assuming no one changes the default seed.  
# Add a comment to the standard job, saying don't change it.
ls_cmd="ls $FCL_PATH/$FCL_FILE"
echo "doing $ls_cmd"
echo `eval $ls_cmd`
base_conf=`basename $conf`
hist_file=`echo $base_conf | sed s/root/hist.root/`
fcl_cmd="cat $FCL_PATH/$FCL_FILE | sed s/24580/$RANDOM/ | sed s/config.fcl/$base_conf/ | sed s/_SPECIAL_NAME_/$SPECIAL_NAME/ | sed s/hist.root/$hist_file/ > ./$FCL_FILE"
echo "$fcl_cmd"
eval "$fcl_cmd"


#A file called overlayfiles.root should now exist on the worker node.

# Now make the actual overlay file(s).
# No longer safe to do this in a multi file context because of the 
# above "sed s/config.fcl/$conf/" line above.  
# Because $conf is an environment variable in a multi file context this 
# will hold only the last config file in the getconfig context
echo "to_process_list: $to_process_list" 
echo "now overlay all the found files" 
for f in $to_process_list
do
    out_name=`echo $f | sed s/g4/overlay.daq/ | sed s/_geantonly//` 
    command="nova -c ./$FCL_FILE -s $f -o $out_name --sam-file-type=importedSimulated --sam-application-family=nova --sam-data-tier=artdaq --sam-application-version=$SRT_BASE_RELEASE" 
    echo 'Running Overlay process'
    echo "$command" 
    eval $command
    ls 
    ls $out_name
done
