#!/bin/env python

import os, sys
import samweb_client, ifdh
import pprint
from subprocess import check_call

if len(sys.argv) != 2 :
    print 'Usage: runDAQ2RawDigit.py <input file>'
    exit(1)

inFile = sys.argv[1]

samweb = samweb_client.SAMWebClient(experiment='nova')

metadata = samweb.getMetadata(inFile)

#make all lower case keys
metadata = {k.lower():v for k,v in metadata.items()}

print "Input File = ", inFile, "with metadata:"
pprint.pprint(metadata)
print ''

if "runs" in metadata:
    runNum = metadata["runs"][0][0]
    subNum = metadata["runs"][0][1]
else:
    print "No run number found!"
    exit(1)

if "data_stream" in metadata:
    stream = int(metadata["data_stream"])
else:
    print "No data stream found!"
    exit(1)

if "online.detector" in metadata:
    detector = metadata["online.detector"].lower()
else:
    print "No detector name found!"
    exit(1)
    
if "SRT_BASE_RELEASE" in os.environ:
    release = os.environ["SRT_BASE_RELEASE"]
else:
    print "No release set!"
    exit(1)

if "file_type" in metadata:
    file_type = metadata["file_type"]
else:
    print "No file_type set!"
    exit(1)

if file_type == "importedDetector":
    dataFlag = "data"
elif file_type == "importedSimulated":
    dataFlag = "sim"
else:
    print "Unrecognized file_type:", file_type
    exit(1)

outName = '{0}_r{1:08d}_s{2:02d}_t{3:02d}_{4}_v1_{5}.daq.root'.format(detector, runNum, subNum, stream, release, dataFlag)

print "Output file =", outName

cmdList = ['nova']
cmdList.append('-o')
cmdList.append(outName)
cmdList.append('-c')
cmdList.append('daq2rawdigitjob.fcl')
cmdList.append('--sam-application-family=nova')
cmdList.append('--sam-application-version=' + release)
cmdList.append('--sam-file-type=' + file_type)
cmdList.append('--sam-data-tier=out1:artdaq')
cmdList.append('--sam-stream-name=out1:' + str(stream))
cmdList.append(inFile)

cmd =  ' '.join(cmdList)

print 'Running:', cmd

retCode = check_call(cmdList)

exit(retCode)

    
