#!/bin/bash

doJobsub()
{
    dataset=$1
    
    echo "Submitting DAQ2RawDigit job over $dataset with $NFILES files"

    OUTPUT_DIR=/nova/ana/users/aurisano/daq2rawdigit/
    
	#--site Fermicloud,Wisconsin,SMU,Nebraska,UCSD,MWT2 
        #--OS SL5,SL6 

    #NFILES=10
    
    jobsub \
	-N $NFILES \
	--nowrapfile \
	-g \
	-e IFDH_BASE_URI \
	-e EXPERIMENT \
	-e SAM_STATION \
	--opportunistic \
	--dataset_definition=${dataset} \
	${SRT_PUBLIC_CONTEXT}/Metadata/samUtils/art_sam_wrap.sh \
	--export EXTERNALS='$PRODUCTS' \
	--export CVMFS_DISTRO_BASE='$PRODUCTS/..' \
	--source '$CVMFS_DISTRO_BASE/novasvn/srt/srt.sh' \
	--source '$CVMFS_DISTRO_BASE/novasvn/setup/setup_novasoft.sh:-r:'"$RELEASE" \
	--dest srm://fg-bestman1.fnal.gov:10443/srm/v2/server?SFN=${OUTPUT_DIR} \
	--limit 1 \
	--addoutput '*.daq.root' \
	-X nova \
	-o ${DET}_r%r_s%s_t0${STREAM}_${MASK}_${RELEASE}_v1_data.daq.root \
	-c 'daq2rawdigitjob.fcl' \
	--sam-application-family=nova \
	--sam-application-version=${RELEASE} \
	--sam-file-type=importedDetector \
	--sam-data-tier=out1:artdaq \
	--sam-stream-name=out1:${STREAM}
}

TIER=raw

RELEASE=$1

DET=$(echo $2 | tr [:upper:] [:lower:])

STREAM=$3

case `echo $STREAM` in
    0) MASK=numi ;;
    1) MASK=bnb ;;
    2) MASK=cosmic ;;
    4) MASK=cosmic ;;
esac

WORKDIR=$PWD

export IFDH_BASE_URI="http://samweb.fnal.gov:8480/sam/nova/api"
export SAM_STATION="nova"
export EXPERIMENT=nova
unset  IFDH_STAGE_VIA

#############
#Setup the Common Tools Area (needed for IFDH and Jobsub)
source /grid/fermiapp/products/common/etc/setups.sh
PRODUCTS=/nusoft/app/externals:$PRODUCTS

#############
# Setup Jobsub and IFDH (art)
setup jobsub_tools v1_2h
setup ifdh_art v1_2_10 -z /nusoft/app/externals -q e4:debug:nu

echo ""
DSET=${DET}_data_${TIER}_t0${STREAM}_${RELEASE}_keepup.$(date +%m-%d-%Y --date="yesterday")
NFILES=$(samweb -e nova count-files "defname: $DSET") && doJobsub $DSET

echo ""
DSET=${DET}_data_${TIER}_t0${STREAM}_${RELEASE}_keepup.$(date +%m-%d-%Y --date="3 days ago")
NFILES=$(samweb -e nova count-files "defname: $DSET") && doJobsub $DSET

echo ""
DSET=${DET}_data_${TIER}_t0${STREAM}_${RELEASE}_keepup.$(date +%m-%d-%Y --date="5 days ago")
NFILES=$(samweb -e nova count-files "defname: $DSET") && doJobsub $DSET

echo ""
DSET=${DET}_data_${TIER}_t0${STREAM}_${RELEASE}_keepup.$(date +%m-%d-%Y --date="7 days ago")
echo "Listing Raw file from 7 days ago still unprocessed:"
samweb -e nova list-files "defname: $DSET"




