#!/usr/bin/perl
#
# Build a description of the far detector from gdml fragments
#

#
# List all the files that need to be glomed together
#
if (@ARGV[0] eq "--fardet") {
    @defFile = qw(unit-defs.gdml 
		  fardet/fd-module-defs.gdml 
		  global-defs.gdml
		  fardet/fd-defs.gdml);
    @gdmlFile = qw(materials.gdml
                   global-rotations.gdml
                   module_v.gdml
                   module_h.gdml
                   fardet/fd-planes.gdml
                   fardet/fd-blocks.gdml
                   fardet/fd-enclosure.gdml
                   fardet/fd-section-nabc.gdml
                   fardet/fd-section-d.gdml
                   fardet/fd-section-e.gdml
                   fardet/fd-section-f.gdml
                   fardet/fd-section-g.gdml
                   fardet/fd-section-h.gdml
                   fardet/fd-section-s.gdml
                   fardet/fd-barite-top.gdml
                   fardet/fd-assembly-area.gdml
                   fardet/fd-world.gdml
		   );
}

if (@ARGV[0] eq "--neardet") {
    @defFile = qw(unit-defs.gdml 
		  neardet/nd-module-defs.gdml 
		  global-defs.gdml 
		  ipnd/ipnd-module-defs.gdml 
		  ipnd/ipnd-defs.gdml 
		  neardet/nd-defs.gdml);
    @gdmlFile = qw(materials.gdml
                   global-rotations.gdml
                   ipnd/ipnd-module_v.gdml
                   ipnd/ipnd-module_h.gdml
                   ipnd/ipnd-planes.gdml
		   ipnd/ipnd-blocks.gdml
		   ipnd/ipnd-superblocks.gdml
                   module_v.gdml
                   module_h.gdml
		   neardet/nd-planes.gdml
		   neardet/nd-blocks.gdml
		   neardet/nd-superblocks.gdml
		   neardet/nd-enclosure.gdml
		   neardet/nd-rock.gdml
		   neardet/nd-world.gdml
		   );
}

if (@ARGV[0] eq "--ipnd") {
    @defFile = qw(unit-defs.gdml 
		  neardet/nd-module-defs.gdml 
		  global-defs.gdml 
                  ipnd/ipnd-defs.gdml 
		  ipnd/ipnd-module-defs.gdml
		  ipnd/ipnd-shed-defs.gdml);
    @gdmlFile = qw(materials.gdml
		   global-rotations.gdml
		   ipnd/ipnd-module_v.gdml
                   ipnd/ipnd-module_h.gdml
		   ipnd/ipnd-planes.gdml
		   ipnd/ipnd-blocks.gdml
		   ipnd/ipnd-superblocks.gdml
                   module_v.gdml
                   module_h.gdml
		   neardet/nd-planes.gdml
		   neardet/nd-blocks.gdml
		   neardet/nd-superblocks.gdml
		   ipnd/ipnd-shed-enclosure.gdml
		   ipnd/ipnd-rock.gdml
		   ipnd/ipnd-world.gdml
		   );
}

if (@ARGV[0] eq "--ndos") {
    @defFile = qw(unit-defs.gdml 
		  neardet/nd-module-defs.gdml 
		  global-defs.gdml 
		  ipnd/ipnd-module-defs.gdml
		  ipnd/ipnd-shed-defs.gdml);
    @gdmlFile = qw(materials.gdml
		   global-rotations.gdml
		   ipnd/ipnd-module_v.gdml
                   ipnd/ipnd-module_h.gdml
		   ipnd/ipnd-planes.gdml
		   ipnd/ipnd-blocks.gdml
		   ipnd/ipnd-superblocks.gdml
                   module_v.gdml
                   module_h.gdml
		   neardet/nd-planes.gdml
		   neardet/nd-blocks.gdml
		   neardet/nd-superblocks.gdml
		   ipnd/ipnd-shed-enclosure.gdml
		   ipnd/ipnd-rock.gdml
		   ipnd/ipnd-shed-world.gdml
		   );
}
#
# Build the table of variable replacements
#
$ikey=0;
for ($ifile=0; $ifile<@defFile; ++$ifile) {
    $DEFINES = @defFile[$ifile];
    open(DEFINES) or die("Could not open file $DEFINES");
    foreach $line (<DEFINES>) {
	chomp($line);
	($key,$name,$value) = split(' ',$line);
	if ($key eq "<constant") {
	    $name  =~ tr/ //d;
	    $value =~ tr/ //d;
	    ($tmp,$name)  = split('=',$name);
	    ($tmp,$value) = split('=',$value);
	    ($tmp,$name, $tmp) = split('"',$name);
	    ($tmp,$value,$tmp) = split('"',$value);
	    $value =~ tr/\"//d;
	    @defNAME[$ikey]  = $name;
	    @defVALUE[$ikey] = "($value)";
	    ++$ikey;
	}
    }
    close(DEFINES);
}
#
# Open each file in turn, read line-by-line making variable
# substitutions, and write to standard output
#
print "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n";
print "<gdml xmlns:gdml=\"http://cern.ch/2001/Schemas/GDML\"\n";
print "      xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n";
print "      xsi:noNamespaceSchemaLocation=\"GDMLSchema/gdml.xsd\">\n";
for ($i=0; $i<@gdmlFile; ++$i) 
{ 
    $FILE = @gdmlFile[$i];
    open(FILE) or die("Could not open file @gdmlFile[$i] for read.");
    foreach $line (<FILE>) {
	# 
	# Work backwards to keep dependency order correct
        #
	for ($j=@defNAME-1; $j>=0; --$j) {
	    $line =~ s/@defNAME[$j]/@defVALUE[$j]/g;
	}
	print $line;
    }
    close(FILE);	      
}
print "\n<setup name=\"Default\" version=\"1.0\">\n";
print "  <world ref=\"vWorld\" />\n";
print "</setup>\n\n";
print "</gdml>\n";
