////////////////////////////////////////////////////////////////////////
//
// BPException.h
// Exception handler for track reconstruction failures specific to
// BreaPointFitter.
// author:  mbaird42@fnal.gov
//
////////////////////////////////////////////////////////////////////////
#ifndef BPF_BPEXCEPTION_H
#define BPF_BPEXCEPTION_H

#include <iostream>
#include <exception>

namespace bpfit {

  // Exception types:
  const unsigned int kBAD_KE   = 1;  ///< negative KE value < tol
  const unsigned int kBAD_RADL = 2;  ///< non-positive radiation length
  const unsigned int kNn       = 3;  ///< number of scat surf >= number of measurements
  const unsigned int kINDEX    = 4;  ///< index problem
  const unsigned int kNAN      = 5;  ///< nan encountered
  const unsigned int kSIGX     = 6;  ///< non-positive value of sigx
  const unsigned int kSIGSJ    = 7;  ///< non-positive value of sigSJ
  const unsigned int kBAD_P    = 8;  ///< negative value for p
  const unsigned int kBAD_BETA = 9;  ///< negative value for beta
  const unsigned int kBAD_NDOF = 10; ///< too few hits used to make the track
  const unsigned int kNPLANES  = 11; ///< too few planes in one (or both) views
  const unsigned int kBAD_TRACK= 12; ///< a generic catch-all category for tracking failures

  class BPException : public std::exception {
  public:
    BPException(const char* file, unsigned int line,
		unsigned int flag, double value = 0.0);
    ~BPException() {};

    void Print();

    const char*  fFile;  ///< Name of file that threw the exception.
    unsigned int fLine;  ///< Line number where exception was thrown.
    unsigned int fFlag;  ///< Exception code thrown.
    double       fValue; ///< (optional) Value of bad variable that caused the exception.
    
  };
}

#endif
////////////////////////////////////////////////////////////////////////
