////////////////////////////////////////////////////////////////////////
/// \file  CAFReweight.h
/// \brief Wrapper for reweight neutrino interactions with GENIE
///
/// \author  nathan.mayer@tufts.edu
////////////////////////////////////////////////////////////////////////
#ifndef CAFREWEIGHT_H
#define CAFREWEIGHT_H

#include <vector>
#include <map>
#include <set>

//Nutools
#include "NuReweight/GENIEReweight.h"

//GENIE include
#include "EVGCore/EventRecord.h"

namespace caf { class SRGenerator;      }

namespace cafrwgt{

  class CAFReweight {

  public:
    CAFReweight();
    ~CAFReweight();

#ifndef __GCCXML__

    double CalcWeight(const std::vector<caf::SRGenerator>& gen, unsigned int i=0);
    std::vector<double> CalcWeightVector(const std::vector<caf::SRGenerator>& gen);
    
    void WgtMultiNeutrino() {fWgtMultiNus=true;}
    void WgtBestNeutrino() {fWgtMultiNus=false;}
    void AddReweightValue(rwgt::ReweightLabel_t rLabel, double value, unsigned int i=0);
    rwgt::GENIEReweight *GenieReweight(int i); //Need this it access the rest of the reweight configurations
    void Configure(int i = -1);

    int NReweights() const {return fGRwgt.size();}
#endif


  private:
    bool fWgtMultiNus; ///< For accounting for multiple neutrino interactions in the slice.
    std::vector<rwgt::GENIEReweight*> fGRwgt; ///< GENIE reweighting
    
#ifndef __GCCXML__
    genie::EventRecord RetrieveGHEP(const caf::SRGenerator& gen);
#endif
    
  };

}

#endif // CAFREWEIGHT_H

