////////////////////////////////////////////////////////////////////////
/// \brief  Look up absolute attenuation constants
/// \author Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////
#ifndef CALIB_ABSCACHE_H
#define CALIB_ABSCACHE_H

#include "NovaDAQConventions/DAQConventions.h"

#include <map>
#include <string>

namespace calib
{
  /// \brief Look up absolute attenuation constants
  class AbsCache
  {
  public:
    AbsCache();
    virtual ~AbsCache();

    double GetPECorrToGeVScale(novadaq::cnv::DetId det,
			       int diblock,
			       bool isData);

    void SetCSVSource(const std::string& src){fCSVSource = src;}

    void UseCSVsFromUPS(bool use = true){fUseCSVsFromUPS = use;}

    void SetTag(const std::string& tag){fTag = tag;}
  protected:
    struct Record
    {
      Record() : meu(0), meuErr(0), meuGeV(0) {}
      // The values from the DB
      double meu, meuErr, meuGeV;      
    };

    void LoadFromVldDatabase(novadaq::cnv::DetId det, bool isData);

    std::string fCSVSource;
    bool fUseCSVsFromUPS;
    std::string fTag;

    /// Whether we have (attempted to) read from db or csv yet
    bool fInitialized;

    std::map<int, Record> fRecords; ///< Map is from diblock number
  };

} // end namespace calib

#endif
////////////////////////////////////////////////////////////////////////
