////////////////////////////////////////////////////////////////////////
// \file    AttenCurve.h
// \brief   Store the results of the attenuation fits
// \version $Id: AttenCurve.h,v 1.3 2012-11-11 23:44:14 gsdavies Exp $
////////////////////////////////////////////////////////////////////////
#ifndef CALIB_ATTENCURVE_H
#define CALIB_ATTENCURVE_H

#include <string>
#include <vector>

#include "Calibrator/func/AttenCache.h"
#include "GeometryObjects/OfflineChan.h"
#include "NovaDAQConventions/DAQConventions.h"

namespace calib{

  class AttenCurve
  {
  public:
    // Named constructors

    /// Return a new AttenCurve objects with fields uninitialized
    static AttenCurve* Uninitialized(int det, geo::OfflineChan chan);

    /// Mean response of this channel at this distance from detector centre
    float MeanPEPerCmAt(double w) const;
    float MeanPEPerCmAt(double w, std::string version) const;

    // Are the constants in this channel good? If not, calibrated quantities
    // will be -1
    bool IsCalibrated() const;

    void AddInterpPoint(float w, float factor);

    void WriteToCSVs(FILE* fConsts, FILE* fPoints, bool mc) const;

    struct Pt
    {
      float w, factor;
      bool operator<(float rhs_w) const {return w < rhs_w;}
    };
    std::vector<Pt> interp_pts;

    float atten_length, coeff_exp, background;
    float edge_low, edge_high;
    float coeff_low, coeff_high;
    float chisq;
    float cell_length;
    /// Nonzero in short muon-catcher cells. Positive is closer to readout
    float center_offset;
    int det;
    geo::OfflineChan chan;
    bool initialized;

  protected:
    AttenCurve() {}
  };

  std::ostream& operator<<(std::ostream& os, const AttenCurve& res);

} // end namespace calib

#endif
////////////////////////////////////////////////////////////////////
