/// \file    RawDataDrawer.h
/// \brief   Class to aid in the rendering of RawData objects
/// \author  messier@indiana.edu
/// \version $Id: RawDataDrawer.h,v 1.11 2012-07-19 04:21:45 ksachdev Exp $
#ifndef EVD_RAWDATADRAWER_H
#define EVD_RAWDATADRAWER_H
#ifndef __CINT__
#include <vector>
#include "art/Persistency/Common/PtrVector.h"
#include <cstdint>

#include "RecoBase/CellHit.h"

class TH1F;
namespace art     { class Event;     }
namespace evdb    { class View2D;    }
namespace evdb    { class View3D;    }
namespace geo     { class Geometry;  }
namespace rawdata { 
  class RawDigit;  
  class RawTrigger;
}


namespace evd {
  /// Aid in the rendering of RawData objects
  class RawDataDrawer {
  public:
    RawDataDrawer();
    ~RawDataDrawer();

    void FillTQHisto(const art::Event& evt,
		     TH1F*             thisto,
		     TH1F*             qhisto);

    void RawDigitHardware(const art::Event& evt,
			  evdb::View2D*     view);

    void RawDigit2D(const art::Event& evt,
		    evdb::View2D*     xzview,
		    evdb::View2D*     yzview);

    void RawDigit3D(const art::Event& evt,
		    evdb::View3D*     view);
    
    void GetLimits(double* xmin, double* xmax,
		   double* ymin, double* ymax,
		   double* zmin, double* zmax);
  private:

    int      GetRawDigits(const art::Event&                  evt,
			  std::string const&                   which,
			  std::string const&                   instance,
			  std::vector< const rawdata::RawDigit*>& rawhits);
    int      GetRawTrigger(const art::Event&                    evt,
			   art::PtrVector< rawdata::RawTrigger>& rawtrigger);

    int      HitColor(const rawdata::RawDigit* hit,
		      const std::vector<rawdata::RawDigit>& unfiltered,
		      int* ghosted);

    bool     DimFiltered(const rawdata::RawDigit* hit,
			 const std::vector<rawdata::RawDigit>&  unfiltered);

    void Draw3DHitBoxes(std::vector<const rawdata::RawDigit*>& hits,
			std::vector<rawdata::RawDigit>&        unfiltHits,
			evdb::View3D*                          view);
    
    double   ScaleFactor(double t, double q);

    void     StickFigure(evdb::View3D* view,
			 double x,
			 double y,
			 double z);

  private:
    double fXmin, fXmax; ///< X range spanned by the hits
    double fYmin, fYmax; ///< Y range spanned by the hits
    double fZmin, fZmax; ///< Z range spanned by the hits
  };
}

#endif /* __CINT__ */
#endif
////////////////////////////////////////////////////////////////////////
