/* 
 * File:   DCMSimulator.h
 * Author: denis
 *
 * Created on January 31, 2012, 5:15 PM
 */

#ifndef _DCMSIMULATOR_H
#define	_DCMSIMULATOR_H
#include "DAQDataFormats/RawMilliSlice.h"
#include "DAQChannelMap/DAQChannelMapConstants.h"

#include <vector>
#include <boost/shared_ptr.hpp>

// forward declarations
namespace daqdataformats{
    class FEBSimulator;
    class RawMicroSlice;

}

namespace mcdatadaq {

    /// Shared pointer
    typedef boost::shared_ptr<daqdataformats::RawMicroSlice> RawMicroSlice_shrptr;

    class DCMSimulator {
        public:
            /// Constructor
            DCMSimulator(const daqchannelmap::diblock_t, const daqchannelmap::dcm_id_t);
            /// Destructor
            ~DCMSimulator();

            /// Generate NanoSlice with a given pixel, TimeStamp and PulseHeight
            bool generateNanoSlice(const daqchannelmap::dchan, const uint32_t time_stamp, const uint32_t               pulse_height);
            bool generateNanoSlice(const daqchannelmap::dchan, const uint32_t time_stamp, const std::vector<uint32_t>& data);

            /// Write all MicroSlices into the output file
            bool writeMicroSlices();
            /// Write MilliSlice into the output file
            bool writeMilliSlices();
            
            /// Fill MilliSlices out
            daqdataformats::RawMilliSlice* fillMilliSlice();

            /// Advance to next event. Advancing MilliSlice. Clearing MicroSlice vector.
            bool nextEvent();


        private:
            /// Get the instance of the FEB Simulator for a given link ID on this DCM
            daqdataformats::FEBSimulator*  getFEBSimulator(const daqchannelmap::feb_t);
            /// Fill MicroSlice with a given NanoSlice
            bool                           fillMicroSlice(const uint32_t microslice_number, daqdataformats::RawNanoSlice*);
            /// Create a MicroSlice and put it in the vector
            daqdataformats::RawMicroSlice* createMicroSlice(const uint32_t microslice_number);

        private:
            uint32_t fDiblockNumber; ///< Diblock number
            uint32_t fDCMNumber;     ///< DCM number in the diblock

            std::vector<daqdataformats::FEBSimulator*> fFrontEndBoards; ///< All 64 FEBs on the DCM

            std::vector<RawMicroSlice_shrptr >         fMicroSlice;     ///< Vector of all Microslices in the event
            daqdataformats::RawMilliSlice              fMilliSlice;     ///< There is going to be 1 Millislice per event

            /// Output file with hte microslices from this DCM
            FILE* fOutputMicroslicesFile;
            FILE* fOutputMillislicesFile;

    };// end of class DCMSimulator
}// end of namespace mcdatadaq
#endif	/* _DCMSIMULATOR_H */

