/* 
 * File:   DataLoggerSimulator.h
 * Author: denis
 *
 * This class simulates a Data Logger in NOvA DAQ
 *
 * Created on February 15, 2012, 10:27 AM
 */

#ifndef _DATALOGGERSIMULATOR_H
#define	_DATALOGGERSIMULATOR_H

#include "DAQDataFormats/RawEvent.h"
#include "DAQDataFormats/RawRunHeader.h"

// forward declarations
namespace daqdataformats{
	class RawDataBlock;
	class RawTrigger;
}

namespace mcdatadaq {

    class DataLoggerSimulator {
        public:
            DataLoggerSimulator();  ///< Constructor
            ~DataLoggerSimulator(); ///< Destructor

            /// Given all DataBlocks from the BufferNodes, produce Event Object
            /// and also add it to the Run Object
            bool produceEvent(const std::vector<daqdataformats::RawDataBlock*>&,
                              daqdataformats::RawTrigger* );

            /// Advance to the next event - clear Event and advance it's number
            bool nextEvent();

        private:
            /// Write Event Object into output file
            bool writeEvent();
            /// Write Run Object into output file
            bool writeRun();

        private:
            daqdataformats::RawEvent       fCurrentEvent;     ///< Event Object
            daqdataformats::RawRunHeader   fCurrentRunHeader;       ///< Run Object

            FILE*                          fOutputEventFile; ///< File for events
            FILE*                          fOutputRunFile;   ///< File for the run
    };// end of class DataLoggerSimulator
}// end of namespace mcdatadaq
#endif	/* _DATALOGGERSIMULATOR_H */

