#ifndef OSC_OSCCALCULATORPMNS_NSI_H
#define OSC_OSCCALCULATORPMNS_NSI_H

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// \file OscCalculatorPMNS_NSI.h                                         //
//                                                                      //
// Adapt the PMNS_NSI calculator to standard interface                   //
// <bckhouse@caltech.edu>						//
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#include "IOscCalculator.h"
#include "PMNS_NSI.h"

namespace osc
{
  /// \brief Optimized version of \ref OscCalculatorPMNS
  ///
  /// Adapt the \ref PMNS_NSI calculator to standard interface
  class OscCalculatorPMNS_NSI: public IOscCalculatorAdjustable
  {
  public:
    OscCalculatorPMNS_NSI();
    virtual ~OscCalculatorPMNS_NSI();

    virtual double P(int flavBefore, int flavAfter, double E);

    virtual void SetL     (double L     ){fPropDirty = true; fL      = L;}
    virtual void SetRho   (double rho   ){fPropDirty = true; fRho    = rho;}
    virtual void SetDmsq21(double dmsq21){fDmDirty   = true; fDmsq21 = dmsq21;}
    virtual void SetDmsq32(double dmsq32){fDmDirty   = true; fDmsq32 = dmsq32;}
    virtual void SetTh12  (double th12  ){fMixDirty  = true; fTh12   = th12;}
    virtual void SetTh13  (double th13  ){fMixDirty  = true; fTh13   = th13;}
    virtual void SetTh23  (double th23  ){fMixDirty  = true; fTh23   = th23;}
    virtual void SetdCP   (double dCP   ){fMixDirty  = true; fdCP    = dCP;}

    virtual void SetEps_ee      (double eps_ee     ){fEpsDirty = true; fEps_ee      = eps_ee;}
    virtual void SetEps_emu     (double eps_emu    ){fEpsDirty = true; fEps_emu     = eps_emu;}
    virtual void SetEps_etau    (double eps_etau   ){fEpsDirty = true; fEps_etau    = eps_etau;}
    virtual void SetEps_mumu    (double eps_mumu   ){fEpsDirty = true; fEps_mumu    = eps_mumu;}
    virtual void SetEps_mutau   (double eps_mutau  ){fEpsDirty = true; fEps_mutau   = eps_mutau;}
    virtual void SetEps_tautau  (double eps_tautau ){fEpsDirty = true; fEps_tautau  = eps_tautau;}
    virtual void SetDelta_emu   (double Delta_emu  ){fEpsDirty = true; fDelta_emu   = Delta_emu;}
    virtual void SetDelta_etau  (double Delta_etau ){fEpsDirty = true; fDelta_etau  = Delta_etau;}
    virtual void SetDelta_mutau (double Delta_mutau){fEpsDirty = true; fDelta_mutau = Delta_mutau;}

  protected:
    PMNS_NSI fPMNS_NSI;

    bool fMixDirty;
    bool fDmDirty;
    bool fPropDirty;
    bool fEpsDirty;
    double fPrevE;
    int fPrevAnti;

    double fEps_ee;
    double fEps_mumu;
    double fEps_tautau;
    double fEps_emu;
    double fEps_etau;
    double fEps_mutau;
    double fDelta_emu;
    double fDelta_etau;
    double fDelta_mutau;

  };

} // namespace

#endif
