////////////////////////////////////////////////////////////////////////
// \file    Veto.h
// \brief   Veto object detailing cuts made
// \author Max Fite - fitex005@umn.edu
////////////////////////////////////////////////////////////////////////
#ifndef Veto_h
#define Veto_h

namespace presel 
{
  ///A class detailing the cuts made on a particular slice
  class Veto
  {
   public:
    //constructors
    Veto();

    #ifndef __GCC_XML__

    Veto(unsigned int NCell,
	 float fFwdDist,
	 float fBakDist,
	 double Ratio,
	 double DirY,
	 double Angle,
	 double AngleVarOld,
	 double AngleVar,
	 bool PassThruOld,
         bool PassThru,
         bool PassAngleFirstAnal,
         bool PassAngle,
         bool Keep);
    
    //Accessor Methods
   unsigned int NCell()              const;
   float        FwdDist()            const;
   float        BakDist()            const;
   double       Ratio()              const;
   double       DirY()               const;
   double       Angle()              const;
   double       AngleVar()           const;
   double       AngleVarOld()        const;
   bool         PassThru()           const;
   bool         PassThruOld()        const;
   bool         PassAngle()          const;
   bool         PassAngleFirstAnal() const;
   bool         Keep()               const;

    //Mutator Methods
    void SetNCell(unsigned int);
    void SetFwdDist(float);
    void SetBakDist(float);
    void SetRatio(double);
    void SetDirY(double);
    void SetAngle(double);
    void SetAngleVar(double);
    void SetAngleVarOld(double);
    void SetPassThru(bool);
    void SetPassThruOld(bool);
    void SetPassAngle(bool);
    void SetPassAngleFirstAnal(bool);
    void SetKeep(bool);

#endif

   private:
    unsigned int fNCell;      ///<Number of hits in the slice
    float        fFwdDist;    ///<Projected live distance to the edge forwards from cosmic track end
    float        fBakDist;    ///<Projected live distance to the edge backwards from cosmic track start
    double       fRatio;      ///< Number of hits in the track relative to the slice
    double       fDirY;       ///<Cosine of CosmicTrack with respect to the y-direction
    double       fAngle;      ///<Cosine of CosmicTrack with respect to the beam direction
    double       fAngleVarOld;///<Difference of squares between beam angle and Y-direction (Also labeled *CosmicVar* *AnglePID* and *AngleVar*); used for determining if pass old angle cut used for first analysis
    double       fAngleVar;   ///<(angle)*(diry+1); used for determining if pass angle cut
    bool         fPassThru;   ///<Boolean variable that determines if the slice passes the through-going check (including ratio > 0.8 check)
    bool         fPassThruOld;///<Boolean variable that determines if the slice passes the through-going check prior to the fix (didn't require ratio > 0.8)
    bool         fPassAngle;  ///<Boolean variable that determines if the slice passes the latest angular cut
    bool         fPassAngleFirstAnal;  ///<Boolean variable that determines if the slice passes the old angular cut used for first analysis
    bool         fKeep;       ///<Boolean variable that determines if the slice was actually kept or thrown away (Also labeled *sel* for *selected*); should be the AND of the through-going check used, the angular check used, and nhits > 20 if turned on (all have to pass for the slice to pass)
  };

} //end namespace presel

#endif //End Veto_h
