////////////////////////////////////////////////////////////////////////
// \file    ReMId.h
// \brief   A pid object for muon identification
// \version $Id: ReMId.h,v 1.1.1.1 2012-09-26 18:38:44 raddatz Exp $
// \author  Nicholas Raddatz - raddatz@physics.umn.edu
////////////////////////////////////////////////////////////////////////
#ifndef REMID_REMID_H
#define REMID_REMID_H

#include <vector>

#include "TVector3.h"
#include "RecoBase/Track.h"
#include "RecoBase/PID.h"

#ifndef __GCCXML__
#include "art/Framework/Principal/Event.h"
#endif // __GCCXML__


namespace remid
{
  class ReMId : public rb::PID
  {
  public:
    
    ReMId();
    ReMId(int pdg, double val);
    void               AddScatter(TVector3 pos, double cosScat);
    void               AddDedx(double dedx, double distFromStart, bool used = true, bool vert = false);
    void               SetMuonDedxLL(double ll);
    void               SetPionDedxLL(double ll);
    void               SetMuonScatLL(double ll);
    void               SetPionScatLL(double ll);
    void               SetDedxUsed(unsigned int idx, bool used);
    void               SetDedxVertex(unsigned int idx, bool vert);
    void               SetContained(bool cont);
    double             AvgDedx() const;
    double             AvgDedxUsed() const;
    unsigned int       NScatters() const;
    unsigned int       NDedx() const;
    TVector3           ScatLocation(unsigned int i) const;
    double             CosScat(unsigned int i)  const;
    double             DedxLocation(unsigned int i) const;
    double             Dedx(unsigned int i) const;
    bool               DedxUsed(unsigned int i) const;
    bool               DedxVertex(unsigned int i) const;
    double             MuonDedxLL() const;
    double             PionDedxLL() const;
    double             MuonScatLL() const;
    double             PionScatLL() const;
    double             DedxSeparation() const;
    double             ScatSeparation() const;
    double             MeasurementFraction() const;
    int                NMeasurementPlanes() const;
    int                NVertexPlanes() const;
    bool               Contained() const;

  protected:
    
    double fMuonScatLL;
    double fPionScatLL;
    double fMuonDedxLL;
    double fPionDedxLL;
    bool   fCont;

    std::vector<TVector3> fscatLocation;
    std::vector<double>   fcosScatAngle;
    std::vector<double>   fdedx;
    std::vector<int>      fdedxvert;
    std::vector<int>      fdedxused;
    std::vector<double>   ffracDistFromStart;
    
  };

#ifndef __GCCXML__
/// Function which finds index of the best ReMId track among a vector of tracks. 
/// Requires the event to fetch associated ReMId objects and the module label
unsigned int HighestPIDTrack(
                const std::vector< art::Ptr<rb::Track> >& sliceTracks,
                const std::string& remidModuleLabel, const art::Event& e);
#endif // __GCCXML__

};
#endif
