////////////////////////////////////////////////////////////////////////
// \file    RVP.h
// \brief   reco var based pid for nue identification
// \version 2012-11-06
// \author  Xuebing Bu - xbbu@fnal.gov
////////////////////////////////////////////////////////////////////////
#ifndef RVP_H
#define RVP_H

#include "RecoBase/PID.h"



namespace rvp
{
  class RVP : public rb::PID
  {
  public:
    RVP();

#ifndef __GCCXML__
    RVP(int pdg,
        unsigned int ncell,
        float recE,
        float lTrack,
        float ltFrac,
        float mFrac,
        float mHits,
        float ePlane20,
        float eFrac2Plane,
        float eFrac6Plane,
        float eFrac2Sig,
        int   png3D,
        float pngEBal,
        float pid,
        int   png2D,
        float pngEBal2D,
        float eIso3,
        float rvp12);

    //setters
    void SetNCells      (unsigned int ncell ) { fNCells      = ncell;       }
    void SetRecoE       (float recE         ) { fRecoE       = recE;        }
    void SetLongTr      (float lTrack       ) { fLongTr      = lTrack;      }
    void SetLongTrFrac  (float ltFrac       ) { fLongTrFrac  = ltFrac;      }
    void SetMipFrac     (float mFrac        ) { fMipFrac     = mFrac;       }
    void SetMipHits     (float mHits        ) { fMipHits     = mHits;       }
    void SetEPl20Frac   (float ePlane20     ) { fEPl20Frac   = ePlane20;    }
    void SetEFrac2PlWin (float eFrac2Plane  ) { fEFrac2PlWin = eFrac2Plane; }
    void SetEFrac6PlWin (float eFrac6Plane  ) { fEFrac6PlWin = eFrac6Plane; }
    void SetEFrac2SigRd (float eFrac2Sig    ) { fEFrac2SigRd = eFrac2Sig;   }
    void SetProngs3D    (int png3D           ) { fProngs3D    = png3D;        }
    void SetProngEBal3D (float pngEBal       ) { fProngEBal3D = pngEBal;      }
    void SetProngs2D    (int png2D           ) { fProngs2D    = png2D;        }
    void SetProngEBal2D (float pngEBal2D     ) { fProngEBal2D = pngEBal2D;    }
    void SetEIso3Sig    (float eIso3Sig     ) { fEIso3Sig    = eIso3Sig;    }
   void SetRVP12       (float rvp12        ) { fRVP12       = rvp12;       }

    //getters
    unsigned int NCells()    const { return fNCells;      }
    float      RecoE()       const { return fRecoE;       }
    float      LongTr()      const { return fLongTr;      }
    float      LongTrFrac()  const { return fLongTrFrac;  }
    float      MipFrac()     const { return fMipFrac;     }
    float      MipHits()     const { return fMipHits;     }
    float      EPl20Frac()   const { return fEPl20Frac;   }
    float      EFrac2PlWin() const { return fEFrac2PlWin; }
    float      EFrac6PlWin() const { return fEFrac6PlWin; }
    float      EFrac2SigRd() const { return fEFrac2SigRd; }
    int        Prongs3D()    const { return fProngs3D;    }
    float      ProngEBal3D() const { return fProngEBal3D; }
    int        Prongs2D()    const { return fProngs2D;    }
    float      ProngEBal2D() const { return fProngEBal2D; }
    float      EIso3Sig()    const { return fEIso3Sig;    }
    float      RVP12()       const { return fRVP12;       }
#endif // __GCCXML__

  private:
    //current RVP input variables to BDT
    unsigned int fNCells;           ///< Number of cells in slice
    float fRecoE;                   ///< Reco energy of slice
    float fLongTr;                  ///< Longest discrete track merge track
    float fLongTrFrac;              ///< Fraction of cells in longest track out of total cells in slice
    float fMipFrac;                 ///< Fraction of mip hits out of total number of hits
    float fMipHits;                 ///< Number of mip hits in slice
    float fEPl20Frac;               ///< Fraction of energy in first 20 planes out of total slice energy
    float fEFrac2PlWin;             ///< Largest fraction of energy in a 2 plane window out of total slice
    float fEFrac6PlWin;             ///< Largest fraction of energy in a 6 plane window out of total slice
    float fEFrac2SigRd;             ///< Fraction of slice energy within 2sigma of mean energy weighted slice position
    int   fProngs3D;                ///< Number of 3D FuzzyK prongs associated with slice
    float fProngEBal3D;             ///< The energy balance between the two hightest energy 3D prongs in slice
    int   fProngs2D;                ///< Number of 2D FuzzyK prongs associated with slice
    float fProngEBal2D;             ///< The energy balance between highest energy 2D prong and 3D prong in the opposite view
    float fEIso3Sig;                ///< Fraction of slice energy more then 3sigma from mean energy weighted slice position
    float fRVP12;                   ///< PID value for 12 variable RVP for comparison purposes

  };

  // Default constructor. May have to be inline here for reflex purposes?
  RVP::RVP()
    : rb::PID(),
      fNCells(0), fRecoE(-1),
      fLongTr(-1), fLongTrFrac(-1),
      fMipFrac(-1), fMipHits(-1),
      fEPl20Frac(-1),
      fEFrac2PlWin(-1), fEFrac6PlWin(-1), fEFrac2SigRd(-1),
      fProngs3D(0), fProngEBal3D(-1), fProngs2D(0), fProngEBal2D(-1),
      fEIso3Sig(-1),
      fRVP12(-1)
  {
  }
}
#endif
