////////////////////////////////////////////////////////////////////////
// \file    HitMap.h
// \brief   Efficient lookup of hits by plane and cell number
// \version $Id: HitMap.h,v 1.6 2012-05-11 00:39:30 bckhouse Exp $ 
// \author  $Author: bckhouse $
////////////////////////////////////////////////////////////////////////
#ifndef RBHITMAP_H
#define RBHITMAP_H

#include <map>

#include "art/Persistency/Common/PtrVector.h"

#include "GeometryObjects/OfflineChan.h"

namespace rb {
  
  class CellHit;
  class Cluster;
  
  /// \brief Provides efficient lookup of CellHits by plane and cell number
  class HitMap
  {
  public:
    HitMap();
    HitMap(const art::PtrVector<rb::CellHit>& chits);
    HitMap(const rb::Cluster* clust);
    ~HitMap();
    
    void Add(const art::Ptr<rb::CellHit>& chit);
    void Add(const art::PtrVector<rb::CellHit>& chits);
    /// Add all the hits in a cluster, or derived class
    void Add(const rb::Cluster* clust);

    void Remove(unsigned int planeIdx, unsigned int cellIdx);

    void Clear(){fMap.clear();}

    /// Does the map contain any cell at this position?
    bool CellExists(unsigned int planeIdx,
		    unsigned int cellIdx) const;

    /// Return the cell at this position. Empty pointer if no match.
    art::Ptr<rb::CellHit> Cell(unsigned int planeIdx,
			       unsigned int cellIdx) const;

    /// Syntactic sugar for \ref Cell
    art::Ptr<rb::CellHit> operator()(unsigned int planeIdx,
				     unsigned int cellIdx) const;
  protected:

    std::map<geo::OfflineChan, art::Ptr<rb::CellHit> > fMap;
  };
  
} // end namespace

#endif
///////////////////////////////////////////////////////////////////////
