///////////////////////////////////////////////////////////////////////
/// \brief   TODO
/// \author  bianjm@umn.physics.edu
///////////////////////////////////////////////////////////////////////
#ifndef JMSHOWER_EID_H
#define JMSHOWER_EID_H

// ROOT includes
#include "TVector3.h"

namespace jmshower
{
  class EID
  {
  public:
    EID() : fANN(-1),fEnergy(0) {}
    ~EID() {}
#ifndef __GCCXML__
    EID(double ann) : fANN(ann) {}

    void SetANN(double ann);
    double ANN() const; //< Event ANN 

    void SetANNE(double anne);
    double ANNE() const; //< Event ANN trained with NuE energy 

    void SetANNNoCos(double ann);
    double ANNNoCos() const; //< Event ANN 

    void SetANNENoCos(double ann);
    double ANNENoCos() const; //< Event ANN trained with NuE energy 

    void SetDetEnergy(double energy);
    double DetEnergy() const;  //< Total detected energy

    void SetNuEnergy(double energy);
    double NuEnergy() const; //< Reconstructed neutrino energy

    void SetTrueNuEnergy(double energy);
    double TrueNuEnergy() const; //< True neutrino energy

    void SetVtx(TVector3 vtx);
    TVector3 Vtx() const; //< Event Vertex

    void SetEvtMinX(double m);// event MinX, loop over all showers
    double EvtMinX() const;
    void SetEvtMinY(double m);// event MinY, loop over all showers
    double EvtMinY() const;
    void SetEvtMinZ(double m);// event MinZ, loop over all showers
    double EvtMinZ() const;
    void SetEvtMaxX(double m);// event MaxX, loop over all showers
    double EvtMaxX() const;
    void SetEvtMaxY(double m);// event MaxY, loop over all showers
    double EvtMaxY() const;
    void SetEvtMaxZ(double m);// event MaxZ, loop over all showers
    double EvtMaxZ() const;




    void SetEvtNCell(int ncell);// number of cells in a slice
    int EvtNCell() const;

    void SetEvtSumCosTheta(double costheta);//Event direction
    double EvtSumCosTheta() const;


    void SetEvtSumP0(double p);// total P4 without pid
    double EvtSumP0() const;

    void SetEvtSumPt0(double pt); // Transverse momentum without pid
    double EvtSumPt0() const;


    void SetEvtSumP(double p);// total P4
    double EvtSumP() const;

    void SetEvtSumPt(double pt); // Transverse momentum
    double EvtSumPt() const;

    void SetEvtEtot(double etot); // total energy on showers 
    double EvtEtot() const;

    void SetEvtGapTNS(double tns); // Min TNS to closet slice 
    double EvtGapTNS() const;

    void SetEvtNCellToEdge(int ncell);// number of cells in a slice
    int EvtNCellToEdge() const;

    void SetSh1Energy(double sh1energy);
    double Sh1Energy() const; //< Leading shower energy

    void SetSh1ExclEnergy(double sh1exclenergy);
    double Sh1ExclEnergy() const; //< Leading shower energy

    void SetSh1TotalLength(double sh1totallength);
    double Sh1TotalLength() const; //< Leading shower length

    void SetSh1Start(TVector3 sh1start);
    TVector3 Sh1Start() const; //< Leading shower start point, = vertex when fuzzkyvertex is used

    void SetSh1Stop(TVector3 sh1stop);
    TVector3 Sh1Stop() const; //< Leading shower stop point

    void SetSh1Dir(TVector3 sh1dir);
    TVector3 Sh1Dir() const; //< Leading shower dir

    void SetSh1VtxDoca(double doca);
    double Sh1VtxDoca() const;

    void SetSh1Gap(double gap);
    double Sh1Gap() const;

    void SetSh1NPlane(unsigned int nplane);
    unsigned int Sh1NPlane() const;

    void SetSh1XNPlane(unsigned int xnplane);
    unsigned int Sh1XNPlane() const;

    void SetSh1YNPlane(unsigned int ynplane);
    unsigned int Sh1YNPlane() const;

    void SetSh1NCell(unsigned int ncell);
    unsigned int Sh1NCell() const;

    void SetSh1XNCell(unsigned int xncell);
    unsigned int Sh1XNCell() const;

    void SetSh1YNCell(unsigned int yncell);
    unsigned int Sh1YNCell() const;

    void SetSh1PID(int pid);
    int Sh1PID() const;

    void SetSh1NCellToEdge(int ncell);
    int Sh1NCellToEdge() const;

    void SetSh1Sh2Dang(double sh1sh2dang);
    double Sh1Sh2Dang() const; //< 2nd shower energy

    void SetSh2Energy(double sh1energy);
    double Sh2Energy() const; //< 2nd shower energy

    void SetSh2ExclEnergy(double sh1exclenergy);
    double Sh2ExclEnergy() const; //< 2nd shower energy

    void SetSh2TotalLength(double sh1totallength);
    double Sh2TotalLength() const; //< 2nd shower length

    void SetSh2Start(TVector3 sh1start);
    TVector3 Sh2Start() const; //< 2nd shower start point, = vertex when fuzzkyvertex is used

    void SetSh2Stop(TVector3 sh1stop);
    TVector3 Sh2Stop() const; //< 2nd shower stop point

    void SetSh2Dir(TVector3 sh1dir);
    TVector3 Sh2Dir() const; //< 2nd shower dir

    void SetSh2VtxDoca(double doca);
    double Sh2VtxDoca() const;

    void SetSh2Gap(double gap);
    double Sh2Gap() const;

    void SetSh2NPlane(unsigned int nplane);
    unsigned int Sh2NPlane() const;

    void SetSh2XNPlane(unsigned int xnplane);
    unsigned int Sh2XNPlane() const;

    void SetSh2YNPlane(unsigned int ynplane);
    unsigned int Sh2YNPlane() const;

    void SetSh2NCell(unsigned int ncell);
    unsigned int Sh2NCell() const;

    void SetSh2XNCell(unsigned int xncell);
    unsigned int Sh2XNCell() const;

    void SetSh2YNCell(unsigned int yncell);
    unsigned int Sh2YNCell() const;

    void SetSh2PID(int pid);
    int Sh2PID() const;

    void SetSh2NCellToEdge(int ncell);
    int Sh2NCellToEdge() const;

    void SetEGLLL(double egLLL);
    double EGLLL() const;
    void SetEGLLT(double egLLT);
    double EGLLT() const;
    void SetEMuLLL(double emuLLL);
    double EMuLLL() const;
    void SetEMuLLT(double emuLLT);
    double EMuLLT() const;
    void SetEPi0LLL(double epi0LLL);
    double EPi0LLL() const;
    void SetEPi0LLT(double epi0LLT);
    double EPi0LLT() const;
    void SetEPLLL(double epLLL);
    double EPLLL() const;
    void SetEPLLT(double epLLT);
    double EPLLT() const;
    void SetENLLL(double enLLL);
    double ENLLL() const;
    void SetENLLT(double enLLT);
    double ENLLT() const;
    void SetEPiLLL(double epiLLL);
    double EPiLLL() const;
    void SetEPiLLT(double epiLLT);
    double EPiLLT() const;
    void SetVtxGeV(double vtxgev);
    double VtxGeV() const;
    void SetPi0Mass(double pi0mass);
    double Pi0Mass() const;
    void SetShE(double shE);
    double ShE() const;
    void SetGap(double gap);
    double Gap() const;

    void SetELLL(double eLLL);
    double ELLL() const;
    void SetELLT(double eLLT);
    double ELLT() const;
    void SetMuLLL(double muLLL);
    double MuLLL() const;
    void SetMuLLT(double muLLT);
    double MuLLT() const;

    void SetGLLL(double gLLL);
    double GLLL() const;
    void SetInvGLLL(double invgLLL);
    double InvGLLL() const;

    void SetSh1DistToEdge(double dist);
    double Sh1DistToEdge()const;

    void SetEEdge2Cell(double eedge2cell);
    double EEdge2Cell()const;  //Energy deposit within 2 cells to detector edges

    void SetEEdge5Cell(double eedge5cell);
    double EEdge5Cell()const;  //Energy deposit within 5 cells to detector edges

    void SetEEdge10Cell(double eedge10cell);
    double EEdge10Cell()const; //Energy deposit within 10 cells to detector edges

    void SetIsMuon(bool ismuon);
    bool IsMuon() const; //< Is muon like?

    void SetIsInvPhoton(bool isinvphoton);
    bool IsInvPhoton() const; //< Is muon like?


    void SetSigOscW(double weight);
    double SigOscW() const; //< Oscillation weight for signal

    void SetBkgOscW(double weight);
    double BkgOscW() const; //< Oscillation for background
#endif // __GCCXML__
  protected:

    double fANN; 
    double fANNE; 
    double fANNNoCos; 
    double fANNENoCos; 

    double fEnergy;
    double fTrueNuEnergy;
    double fNuEnergy;
    double fDetEnergy;

    int fEvtNCell;
    int fEvtNCellToEdge;
    double fEvtSumCosTheta;
    double fEvtSumP; 
    double fEvtSumPt; 
    double fEvtSumP0; 
    double fEvtSumPt0; 
    double fEvtEtot; 
    double fEvtGapTNS; 

    double fEvtMinX; 
    double fEvtMinY; 
    double fEvtMinZ; 
    double fEvtMaxX; 
    double fEvtMaxY; 
    double fEvtMaxZ; 

    double fSh1Energy;
    double fSh1ExclEnergy;
    double fSh1TotalLength;
    TVector3 fSh1Start;
    TVector3 fSh1Stop;
    TVector3 fSh1Dir;
    double fSh1VtxDoca;
    double fSh1Gap;
    unsigned int fSh1NPlane;
    unsigned int fSh1XNPlane;
    unsigned int fSh1YNPlane;
    unsigned int fSh1NCell;
    unsigned int fSh1XNCell;
    unsigned int fSh1YNCell;
    int fSh1PID;
    int fSh1NCellToEdge;

    double fSh1Sh2Dang;
    double fSh2Energy;
    double fSh2ExclEnergy;
    double fSh2TotalLength;
    TVector3 fSh2Start;
    TVector3 fSh2Stop;
    TVector3 fSh2Dir;
    double fSh2VtxDoca;
    double fSh2Gap;
    unsigned int fSh2NPlane;
    unsigned int fSh2XNPlane;
    unsigned int fSh2YNPlane;
    unsigned int fSh2NCell;
    unsigned int fSh2XNCell;
    unsigned int fSh2YNCell;
    int fSh2PID;
    int fSh2NCellToEdge;




    double fSh1DistToEdge;

    bool fIsMuon;
    bool fIsInvPhoton;
    double fSigOscW;
    double fBkgOscW;
    TVector3 fVtx;

    double fEGLLL;
    double fEGLLT;
    double fEMuLLL;
    double fEMuLLT;
    double fEPi0LLL;
    double fEPi0LLT;
    double fEPLLL;
    double fEPLLT;
    double fENLLL;
    double fENLLT;
    double fEPiLLL;
    double fEPiLLT;
    double fVtxGeV;
    double fPi0Mass;
    double fShE;
    double fGap;

    double fELLL;
    double fELLT;
    double fMuLLL;
    double fMuLLT;
    double fGLLL;
    double fInvGLLL;

    double fEEdge2Cell;
    double fEEdge5Cell;
    double fEEdge10Cell;


  };
}

#endif
