///////////////////////////////////////////////////////////////////////////
/// \brief  A very simple service to remember what detector we're working in
/// \author Christopher Backhouse - bckhouse@caltech.edu
//////////////////////////////////////////////////////////////////////////

#ifndef DETECTORSERVICE_H
#define DETECTORSERVICE_H

#include "art/Framework/Services/Registry/ServiceMacros.h"
#include "art/Framework/Principal/Run.h"

#include "NovaDAQConventions/DAQConventions.h"

namespace rh
{
  /// \brief A very simple service to remember what detector we're working in
  ///
  /// Not quite so simple as you might hope, given that a lot of people want to
  /// know the detector in preBeginRun/beginRun/postBeginRun, for both normal
  /// running and MC generation jobs. The trick of having people pass any Run
  /// object they have handy resolves that.
  class DetectorService
  {
  public:
    DetectorService(const fhicl::ParameterSet& pset,
		    art::ActivityRegistry& reg);
    virtual ~DetectorService();

    /// What detector are we in?
    novadaq::cnv::DetId DetId() const {return fDetId;}
    /// \brief If you have a Run object handy, please call this variant
    ///
    /// You may have won the race, and we haven't found out the detector yet,
    /// but we can learn from your Run object.
    novadaq::cnv::DetId DetId(const art::Run& run);

    /// \brief Call this if you have definitive special knowledge (ie you're
    /// the Geometry service)
    void SetDetector(novadaq::cnv::DetId det){fDetId = det;}

    void preBeginRun(const art::Run& run);
    void postBeginRun(const art::Run& run);

  protected:
    void HandleRun(const art::Run& run);

    novadaq::cnv::DetId fDetId;
  };
}

DECLARE_ART_SERVICE(rh::DetectorService, LEGACY)

#endif
////////////////////////////////////////////////////////////////////////
