////////////////////////////////////////////////////////////////////////
/// \brief   This class describes a particle created in the detector Monte
///          Carlo simulation.
/// \author  seligman@nevis.columbia.edu
/// \date
////////////////////////////////////////////////////////////////////////
#ifndef SIM_PARTICLE_H
#define SIM_PARTICLE_H

#include <functional> // so we can redefine less<> below
#include <iostream>
#include <set>
#include <string>

#include "SimulationBase/MCParticle.h"

namespace simb { class MCTruth; }

namespace sim {
  
  class Particle : public simb::MCParticle {
  public:
    
    /// An indicator for an uninitialized variable (see Particle.cxx).
    static const int s_uninitialized; //! Don't write this as ROOT output
    
    /// Standard constructor.  If the mass is not supplied in the
    /// argument, then the PDG mass is used.
    Particle();
    
    Particle(const int trackId, 
	     const int pdg, 
	     const std::string process,
	     const int mother = -1, 
	     const double mass = s_uninitialized,
	     const int status = -1);
    
    /// Destructor.
    ~Particle();
        
  };
  
} // namespace sim

#endif // SIM_PARTICLE_H
/////////////////////////////////////////////////////////////////////////////
