/////////////////////////////////////////////////////////////////////////
/// \brief   Class to help Slicer4D efficiently manage the hits
/// \authors mbaird42@indiana.edu
/// \date    2013/05/01
/////////////////////////////////////////////////////////////////////////
#ifndef SLICER_POINTMANAGER_H
#define SLICER_POINTMANAGER_H

#include <cmath>
#include <vector>

#include "GeometryObjects/PlaneGeo.h"

namespace slicer {

  /// Helper class for Slicer4D
  class PointManager {

  public:
    // member functions
    PointManager();
    ~PointManager();
    void Initalize(double eps, double pepen,
		   double distpen, double opvplpen,
		   bool usepepen);
    void Reset(unsigned int n, bool skip = true);
    void FindNeighbors();
    void CleanUpNeighborhood();

    std::vector<unsigned int> regionQuery(unsigned int point);
    void changeClustID(unsigned int point, unsigned int id);
    void AddHit(double t, double tres,
		unsigned int p, unsigned int c, double pe,
		geo::View_t v);
    double NeighborScore(unsigned int i, unsigned int pt);

    // The following three functions are only used for analysis/validation.
    void   DebugPrint();
    void   PreparekDistInfo(unsigned int kmax);
    double kDist(unsigned int k, unsigned int pt);

    // data
    std::vector<int>                         fClustIDs;         ///< Assigned cluster ID for each hit
    std::vector< std::vector<unsigned int> > fNeighborList;     ///< A list of neighbors for each hit
    std::vector< std::vector<double      > > fkDistList;        ///< A list of the distances to the closest neighbors for each hit (only used for analysis)

  private:
    std::vector<double>       fTime;     ///< Time for each hit
    std::vector<double>       fTres;     ///< T resolution for each hit
    std::vector<unsigned int> fPlane;    ///< Plane number for each hit
    std::vector<unsigned int> fCell;     ///< Cell number for each hit
    std::vector<unsigned int> fPE;       ///< PE for each hit
    std::vector<geo::View_t>  fView;     ///< View for each hit

    unsigned int fNhits;  ///< Total number of hits
    double       fEps;    ///< Neighbor Score cut off value
    bool         fSkip;   ///< Skip calculating the full neighbor score if hits are too far apart in time.

    bool   fUsePEPen;  ///< Use PE penalty term?
    double fPEPen;     ///< PE penalty
    double fDistPenP;  ///< distance penalty converted to number of planes
    double fDistPenC;  ///< distance penalty converted to number of cells
    double fOpVPlPen;  ///< opposite view plane penalty

    unsigned int NScount; ///< Count of the # of times NeighborScore is called (for debugging)
    unsigned int RQcount; ///< Count of the # of times regionQuery is called (for debugging)

  };
}

#endif
/////////////////////////////////////////////////////////////////////////
