////////////////////////////////////////////////////////////////////////
// \file    SRHeader.h
// \brief   An SRHeader contains identity information for a slice.
// \version $Id: SRHeader.h,v 1.5 2012-11-19 17:32:22 rocco Exp $  
// \author  $Author: rocco $
// \date    $Date: 2012-11-19 17:32:22 $
////////////////////////////////////////////////////////////////////////
#ifndef SRHEADER_H
#define SRHEADER_H

namespace caf
{
  
  typedef enum _det {
    kUNKNOWN,     ///< Unknown detector
    kNEARDET,     ///< Near Detector underground
    kFARDET,      ///< Far Detector at Ash River
    kNDOS,        ///< Prototype Near Detector on the Surface
    kNDSBTEST,  
    kTESTBEAM,
    kNDetector,
    kFCCDAQ
  } Det_t;
  
  /// The SRHeader contains identity information for a slice.
  /// This class is meant to be used in Common Analysis Files as 
  /// part of the StandardRecord.
  class SRHeader
    {
    public:
      ///  Default constructor for SRHeader. 
      SRHeader();
      
      /// Default destructor.
      ~SRHeader();
      
      unsigned int   run;       ///< run number
      unsigned int   subrun;    ///< subrun number
      int            cycle;     ///< MC simulation cycle number
      unsigned int   evt;       ///< ART event number, indexes trigger windows.
      unsigned short subevt;    ///< slice number within spill 
      bool           ismc;      ///< data or MC?  True if MC
      Det_t          det;       ///< Detector, ND = 1, FD = 2, NDOS = 3
      bool           blind;     ///< if true, record has been corrupted for blindness
      bool           filt;      ///< if true, record has ben filtered

      unsigned short dibfirst;  ///< first diblock in detector configuration (1-14)
      unsigned short diblast;   ///< last diblock in detector configuration (1-14)
      unsigned short dibmask;   ///< diblock mask (bitfield, lowest bit = diblock 1)
      unsigned short maskstatus;///< 0 no mask found in DB, 1 mask used ok, 2 masking turned off. If 0 or 2 dibmask is instead the configuration based on what RH says is alive.  dibfirst/last may be wrong in this case.

      unsigned short year;      ///< year of spill  
      unsigned short month;     ///< month of spill  
      unsigned short day;       ///< day of spill within month
      unsigned short doy;       ///< day of spill within year  
      unsigned short hour;      ///< hour of spill  
      unsigned short minute;    ///< minute of spill  
      unsigned short second;    ///< second of spill  

      float          unixtime;  ///< unix time of spill
      
      float subevtstarttime;    ///< time of beginning of slice within spill [ns]
      float subevtendtime;      ///< Slice end time  [ns]
      float subevtmeantime;     ///< Slice mean time [ns]

      unsigned int  nbadchan;   ///< Number of bad channels in a subrun. Ignores channels in diblocks that are masked off for analysis.
      unsigned int  ntotchan;   ///< Total number of channels in the analysis masked region of the detector.
      
      void setDefault();
    };
  
} // end namespace

#endif // SRHEADER_H
//////////////////////////////////////////////////////////////////////////////
