////////////////////////////////////////////////////////////////////////
// \file    SRJMEid.h
// \brief   This class contains the output of the jmshower::NueSel module.
// \version $Id: SRJMEid.h,v 1.4 2012-11-19 17:32:22 rocco Exp $  
// \author  $Author: rocco $
// \date    $Date: 2012-11-19 17:32:22 $
////////////////////////////////////////////////////////////////////////
#ifndef SRJMEID_H
#define SRJMEID_H



namespace caf
{
  /// This class contains the output of the jmshower::NueSel module.
  class SRJMEid 
    {
    public:
      ///  Default constructor. 
      SRJMEid();
      
      /// Default destuctor.
      ~SRJMEid();

      float     ann;        ///< ann output  
      float     annE;        ///< ann output, with E 
      float     annNoCos;        ///< ann output, without cosTheta
      float     annENoCos;        ///< ann output, with E without cosTheta  
      int       ismuon;     ///< Boolean to distinguish muons from electrons. 
      
      ///<Below are the training variables for the EID
      ///<These variables all use the leading (most energetic) shower in each slice
      ///<The first 12 variables are calculating by computing the log likelihood
      ///<that the leading shower (in both transverse and longitudinal directions) 
      ///<is an electron, and then subtracting the loglikelihood that the shower is 
      ///<another particle.
      float     eglll;      ///<Electron - gamma ll for longitudinal shower
      float     egllt;      ///<Electron - gamma ll for ltransverse shower
      float     emulll;     ///<Electron - muon ll for longitudinal shower
      float     emullt;     ///<Electron - muon ll for ltransverse shower
      float     epi0lll;    ///<Electron - Pi0 ll for longitudinal shower
      float     epi0llt;    ///<Electron - Pi0 ll for ltransverse shower
      float     eplll;      ///<Electron - proton ll for longitudinal shower
      float     epllt;      ///<Electron - proton ll for ltransverse shower
      float     enlll;      ///<Electron - neutron ll for longitudinal shower
      float     enllt;      ///<Electron - neutron ll for ltransverse shower
      float     epilll;     ///<Electron - pion ll for longitudinal shower
      float     epillt;     ///<Electron - pion ll for ltransverse shower
      float     vtxgev;     ///<Energy of slice in vertex region
      float     pi0mass;    ///<best pi0 mass hypothesis coming from combinations of JMShowers
      float     shwEFrac;   ///<fraction of energy of leading shower out of total energy of slice
      float     gap;        ///<gap from vertex to start of shower
      float     costheta;   ///<cosine of track with respect to beam direction

      ///<The following variables are not used directly in the training but may be useful 
      float     elll;       ///<electron ll for longitudinal shower
      float     ellt;       ///<electron ll for transverse shower
      float     mulll;      ///<muon ll for longitudinal shower
      float     mullt;      ///<muon ll for transverse shower
      
      void setDefault();
    };
  
} // end namespace

#endif // SRJMEID_H
//////////////////////////////////////////////////////////////////////////////
