////////////////////////////////////////////////////////////////////////
// \file    SRNueSandbox.h
// \brief   Class for storing information necessary for nu_e analysis. 
//          This information might not exist here forever, it may 
//          eventually be moved somewhere else. 
// \version $Id: SRNueSandbox.h,v 1.1 2012-12-06 20:18:33 rocco Exp $  
// \author  $Author: rocco $
// \date    $Date: 2012-12-06 20:18:33 $
#ifndef SRNUESANDBOX_H
#define SRNUESANDBOX_H
#include"TVector3.h"
#include "StandardRecord/SRMichelE.h"
#include "StandardRecord/SRJMEid.h"
#include "StandardRecord/SRJMShower.h"


namespace caf
{
    /// Class for storing information necessary for nu_e analysis. 
    /// This information might not exist here forever, it may 
    /// eventually be moved somewhere else.
    class SRNueSandbox{
    public:
      ///  Default constructor
      SRNueSandbox();
      
      /// Default destuctor
      ~SRNueSandbox();

      /// \name Simple containment
      //@{
      bool  infid;      ///< Geometry::isInFiducialVolume() on slice MeanXYZ
      float slicemeanx; ///< slice's Cluster::MeanXYZ().X()
      float slicemeany; ///< slice's Cluster::MeanXYZ().Y()
      float slicemeanz; ///< slice's Cluster::MeanXYZ().Z()
      //@}

      /// \name WARNING: Truth variables
      //@{
      int npi0; ///< Number of primary pi0
      int npip; ///< Number of primary pi+
      int npim; ///< Number of primary pi-

      float pi0E; ///< Energy of largest primary pi0
      float pipE; ///< Energy of largest primary pi+
      float pimE; ///< Energy of largest primary pi-

      float phot0E;  ///< Energy of largest pi0's largest photon
      float phot1E;  ///< Energy of largest pi0's smallest photon
      float opencos; ///< Opening angle of largest pi0's photons
      float conv0;   ///< Conversion length of photon 0
      float conv1;   ///< Conversion length of photon 1
      //@}
      
      /// \name Ruth's additions
      //@{
      float mincellcosmic;     ///< Mincell using all cosmictracker
      float mincellwallcosmic; ///< Mincellwall using all cosmictracker
      float anglecosmic;       ///< Angle of longest cosmictrack
      int   nplanesfilled;     ///< Number of filled planes
      unsigned int nxplanes;   ///< Extent in planes for slice X-view
      unsigned int nyplanes;   ///< Extent in planes for slice Y-view
      int   ncellrow;          ///< Max number contiguous cells, single plane
      int   ncellsum;          ///< Max number cells in total, single plane
      float mincellavg;        ///< Mincell using regression through event
      float mincellkalman;     ///< Mincell using regression through event (MIP frac events)
      int   nmiphits;          ///< Number of MIP Hits (Adjustable boundary)
      TVector3 	mipmin;        ///< Minimum coordinates of box with MIP range hits
      float ecfNu;             ///< Energy Containment Fraction of the mother nuetrino 
      TVector3 	mipmax;        ///< Maximum coordinates of box with MIP range hits
      //@}

      /// \name Jianming's additions:
      //@{
      unsigned int evtncell;///< total # of cell in a slice
      float evtsumcostheta; ///< event direction
      float evtsump;  ///< event total momentum
      float evtsumpt; ///< event total transverse momentum
      float evtsump0;  ///< event total momentum without pid
      float evtsumpt0; ///< event total transverse momentum without pid

      float evtetot;  ///< event total energy on showers
      float evtminx;  ///< min x of a event, looping over all showers
      float evtminy;  ///< min y of a event, looping over all showers
      float evtminz;  ///< min z of a event, looping over all showers
      float evtmaxx;  ///< max x of a event, looping over all showers
      float evtmaxy;  ///< max y of a event, looping over all showers
      float evtmaxz;  ///< max z of a event, looping over all showers

      float evtgaptns;  ///< tns to closet slice
      int  evtncelltoedge;  ///< ncell to detector edge for the event
      float sh1energy;     ///< Leading shower energy
      float sh1exclenergy; ///< Leading shower excluded energy
      float sh1totalL;     ///< Leading shower length
      TVector3 sh1start;   ///< Leading shower start pt
      TVector3 sh1stop;    ///< Leading shower stop pt
      TVector3 sh1dir;     ///< Leading shower dir
      float sh1vtxdoca;     ///< Leading shower DOCA to vertex
      float sh1gap;         ///< Leading shower gap
      unsigned int sh1nplane;  ///< Leading shower NPlane
      unsigned int sh1xnplane;  ///< Leading shower XView NPlane
      unsigned int sh1ynplane;  ///< /Leading shower YView NPlane
      unsigned int sh1ncell;    ///< Leading shower # of cell
      unsigned int sh1xncell;   ///< Leading shower XView # of cell
      unsigned int sh1yncell;   ///< Leading shower YView # of cell
      int sh1pid;               ///< Leading shower pid based on longitudinal+transverse log likelihoods
      int sh1ncelltoedge;       ///<  ncell to detector edge for the leading shower
      float sh1sh2dang;    //angle between 1st and 2nd shower 

      float sh2energy;     ///< 2nd shower energy
      float sh2exclenergy; ///< 2nd shower excluded energy
      float sh2totalL;     ///< 2nd shower length
      TVector3 sh2start;   ///< 2nd shower start pt
      TVector3 sh2stop;    ///< 2nd shower stop pt
      TVector3 sh2dir;     ///< 2nd shower dir
      float sh2vtxdoca;     ///< 2nd shower DOCA to vertex
      float sh2gap;         ///< 2nd shower gap
      unsigned int sh2nplane;  ///< 2nd shower NPlane
      unsigned int sh2xnplane;  ///< 2nd shower XView NPlane
      unsigned int sh2ynplane;  ///< /2nd shower YView NPlane
      unsigned int sh2ncell;    ///< 2nd shower # of cell
      unsigned int sh2xncell;   ///< 2nd shower XView # of cell
      unsigned int sh2yncell;   ///< 2nd shower YView # of cell
      int sh2pid;               ///< 2nd shower pid based on longitudinal+transverse log likelihoods
      int sh2ncelltoedge;       ///< ncell to detector edge for the secondary shower

      float elll;          ///< Electron likelihood
      float sh1d2edge;     ///< Shower distance to edge
      float eedge2cell;    ///< Energy deposit within 2 cells to detector edges
      float eedge5cell;    ///< Energy deposit within 5 cells to detector edges
      float eedge10cell;   ///< Energy deposit within 10 cells to detector edges

      //RecoJMShower PID variables, moved here to make way for LID
      std::vector<SRJMShower> jm;  ///< vector of JMShowers
      size_t njm;  ///< number of jmshowers
      SRJMEid    jmid; ///< jmshower pid object
      TVector3   vtx;    ///< jmshower vertex
      float      time;   ///< jmshower vertex time      [ns]
      float      E;      ///< jmshower neutrino energy estimate
      float      depE;   ///< jmshower energy deposited
      float      shwE;   ///< jmshower energy of primary shower
      //@}


      /// \name Chris' variables
      //@{
      float fracangchanges;     ///< See \ref FillNueSandbox::FracAngChanges
      float fracangchangesloose;///< See \ref FillNueSandbox::FracAngChanges
      float fracmodalhits;      ///< See \ref FillNueSandbox::FracModalHits
      float fracnoncontig;      ///< See \ref FillNueSandbox::FracNonContiguous
      float fracnoncontigloose; ///< See \ref FillNueSandbox::FracNonContiguous
      int   maxgap;             ///< See \ref FillNueSandbox::MaxGap

      //@}

      /// \name Himansu's variables
      //@{
      /// Average dE/dx of the longest 3D prong in the slice. Planes with 
      /// 0 hits are not counted. See \ref FillNueSandbox::GetdEdx
      float dedxpng1;           
      /// Average dE/dx of the 2nd longest 3D prong in the slice. Planes 
      /// with 0 hits are not counted. See \ref FillNueSandbox::GetdEdx
      float dedxpng2;
      //@}


    };
  
} // end namespace

#endif // SRNUESANDBOX_H
//////////////////////////////////////////////////////////////////////////////
