////////////////////////////////////////////////////////////////////////
// \file    SRRvp.h
// \brief   This class contains the RVP PID output.  
// \version $Id: SRRvp.h,v 1.1 2012-12-07 06:37:29 gsdavies Exp $  
// \author  gsdavies@iastate.edu
// \date    $Date: 2012-12-07 06:37:29 $
////////////////////////////////////////////////////////////////////////
#ifndef SRRVP_H
#define SRRVP_H

namespace caf
{
  /// This class contains the RVP PID output.  
  class SRRvp
    {
    public:
      ///  Default constructor. 
      SRRvp();
      
      /// Default destuctor.
      ~SRRvp();

      float   pid;          ///< The output value from the BDT
      int     ncell;        ///<number of cells in slice
      float   recoE;        ///<reco energy of slice
      float   longtr;       ///<longest discrete track merge track
      float   longtrfrac;   ///<fraction of cells in longest track out of total cells in slice
      float   mipfrac;      ///<fraction of mip hits out of total number of hits
      float   miphits;      ///<number of mip hits in slice
      float   epl20frac;    ///<fraction of energy in first 20 planes out of total slice energy
      float   efrac2plwin;  ///<largest fraction of energy in a 2 plane window out of total slice
      float   efrac6plwin;  ///<largest fraction of energy in a 6 plane window out of total slice
      float   efrac2sigrd;  ///<fraction of slice energy within 2sigma of mean energy weighted slice position
      int     prongs3D;     ///<number of 3D FuzzyK prongs associated with slice
      float   prongEbal3D;  ///<the energy balance between the two hightest energy 3D prongs in slice
      int     prongs2D;     ///<number of 2D FuzzyK prongs associated with slice
      float   prongEbal2D;  ///<the energy balance between highest energy 2D prong and 3D prong in the opposite view
      float   eiso3sig;     ///<fraction of slice energy more then 3sigma from mean energy weighted slice position
      float   rvp12;        ///<PID value for 12 variable RVP for comparison purposes

      void setDefault();

};
  
} // end namespace

#endif // SRRVP_H
/////////////////////////////////////////////////////////////////////////
