////////////////////////////////////////////////////////////////////////
// \file    SRSpill.h
// \brief   An SRSpill contains information on the NuMI spill.
// \version $Id: SRSpill.h,v 1.5 2012-11-19 17:32:22 rocco Exp $  
// \author  $Author: edniner $
// \date    $Date: 2012-11-19 17:32:22 $
////////////////////////////////////////////////////////////////////////
#ifndef SRSPILL_H
#define SRSPILL_H

#include "StandardRecord/SRHeader.h"

namespace caf
{
  /// The SRSpill contains information about the NuMI spill and POT
  /// associated with the slice, as well as EventQuality info on
  /// spill by spill basis.
  /// This class is meant to be used in Common Analysis Files as 
  /// part of the StandardRecord.
  class SRSpill
    {
    public:
      ///  Default constructor for SRSpill 
      SRSpill();
      
      /// Default destuctor.
      ~SRSpill();

      unsigned int   run;       ///< run number
      unsigned int   subrun;    ///< subrun number
      unsigned int   evt;       ///< ART event number, indexes trigger windows.

      Det_t          det;       ///< Detector, ND = 1, FD = 2, NDOS = 3
      bool           ismc;      ///< data or MC?  True if MC

      bool                 isgoodspill;         ///< Was the pot for a spill good? (only applicable to data, default true)
      unsigned long int    spilltimesec;        ///< Spill time in seconds [s] 
      unsigned long int    spilltimensec;       ///< Spill time in naonoseconds [ns]
      unsigned long int    gpsspilltimesec;     ///< Spill time from GPS [s]
      unsigned long int    gpsspilltimensec;    ///< Spill time from GPS [ns]
      signed long long int deltaspilltimensec;  ///< Delta time [ns]
      float                spillpot;            ///< POT in spill including factor of 1e12 so that a user does not have to apply this correction 
      float                livetime;            ///< Length of readout [s]

      float                hornI;               ///< Horn current 
      float                posx;                ///< x position on target 
      float                posy;                ///< y position on target 
      float                widthx;              ///< Spill width in x dimension
      float                widthy;              ///< Spill width in y dimension

      unsigned short dibfirst;  ///< first diblock in detector configuration (1-14)
      unsigned short diblast;   ///< last diblock in detector configuration (1-14)
      unsigned short dibmask;   ///< diblock mask (bitfield, lowest bit = diblock 1)
      unsigned short maskstatus;///< 0 no mask found in DB, 1 mask used ok, 2 masking turned off. If 0 or 2 dibmask is instead the configuration based on what RH says is alive.  dibfirst/last may be wrong in this case.

      unsigned int         nmissingdcms;        ///< # of missing DCMs
      float                fracdcm3hits;        ///< fraction of DCM3 hits in horizontal modules
      unsigned int         nouttimehits;        ///< # of out-of-time hits
      unsigned int         nnoisyapds;          ///< # of noisy APDs
      unsigned int         nmissingdcmslg;      ///< # of DCMS with 63 or more bad FEBs (LiveGeometry, subset of baddcmslg)
      unsigned int         nbaddcmslg;           ///< # of DCMS with too many bad channels (LiveGeometry)

      /// \brief How many hits at the DCM edge are matched in the adjacent DCM?
      ///
      /// Low values mean out-of-sync detector
      float                dcmedgematchfrac;
      unsigned int         nmicroslices;         ///< # of micro slices
      int                  ndcms;                ///< # of DCMs in partition; may not = # of LIVE DCMs = (hdr.diblast-hdr.dibfirst+1)*12

      unsigned char trigger; ///< The trigger type from RawTrigger::fTriggerMask_TriggerType
    };
  
} // end namespace caf

#endif // SRSPILL_H
//////////////////////////////////////////////////////////////////////////////
