////////////////////////////////////////////////////////////////////////
/// \brief   An SRTruthBranch contains a vector of SRTruth.  
///          It is intended for use in the Common Analysis File ROOT trees.
///
/// \author  Dominick Rocco
/// \date    Sept. 2012
////////////////////////////////////////////////////////////////////////
#ifndef SRTRUTHBRANCH_H
#define SRTRUTHBRANCH_H

#include "StandardRecord/SRCosmic.h"
#include "StandardRecord/SRNeutrino.h"
#include "StandardRecord/SRTruth.h"
#include "StandardRecord/SRGenerator.h"
#include "StandardRecord/SRGenieWeights.h"
#include "StandardRecord/SRGlobalTruth.h"

#include <vector>

namespace caf
{
  class SRTruthBranch 
    {
    public:
      ///  Default constructor for SRTruthBranch. 
      SRTruthBranch();
      /// Default destuctor.  
      ~SRTruthBranch();
      
      std::vector<SRNeutrino> nu;       ///< implemented as a vector to maintain mc.nu structure, i.e. not a pointer, but with 0 or 1 entries. 
      std::vector<SRCosmic> cosmic;     ///< implemented as a vector to maintain mc.cosmic structure 
      std::vector<SRGenerator> gen;


      std::vector<SRNeutrino> allnus;   ///< vector holding all Neutrinos 
      std::vector<SRCosmic> allcosmics; ///< vector holding all Cosmics
      std::vector<SRGenerator> allgen;
      
      short faveidxeff;           ///<  Index of favorite in allnus when sorted by slicer efficiency      
      short faveidxenergy;        ///<  Index of favorite in allnus when sorted by energy 
      short faveidxpur;           ///<  Index of favorite in allnus when sorted by purity 
      short faveidxeffpur;        ///<  Index of favorite in allnus when sorted by product of efficiency and purity 
      short faveidxeffthenpur;    ///<  Index of favorite in allnus when neutrinos are sorted by efficiency and slices break ties by purity 

      short nnu;                  ///<  Number of neutrinos in nu vector (0 or 1)
      short ncosmic;              ///<  Number of cosmics in cosmic vector (0 or 1)
      short ngen;                 ///<  Number of SRGenerator objects in gen vector (0 or 1)
      short nallnus;              ///<  Number of neutrinos in allnus vector 
      short nallcosmics;          ///<  Number of cosmics in allcosmics vector 
      short nallgen;              ///<  Number of SRGenerator objects in allgen vector

      SRGlobalTruth global;       ///< Information about the event from which the slice came. 
                                  ///< Variables in this branch should be used with caution since it can be 
                                  ///< duplicated across entries (slices) in the CAF tree.         

      std::vector<SRGenieWeights> rwgt;

      void setDefault();

    };
  
} // end namespace

#endif // SRTRUTHBRANCH_H
//////////////////////////////////////////////////////////////////////////////
