////////////////////////////////////////////////////////////////////////
// \file    SRVertexBranch.h
// \brief   An SRVertexBranch contains vectors of SRVertexes.  
//          It is intended for use in the Common Analysis ROOT trees.
//
// \version $Id: SRVertexBranch.h,v 1.6 2012-11-19 17:32:23 rocco Exp $
// \author  $Author: rocco $
// \date    $Date: 2012-11-19 17:32:23 $
////////////////////////////////////////////////////////////////////////
#ifndef SRVERTEXBRANCH_H
#define SRVERTEXBRANCH_H
#include "StandardRecord/SRVertex.h"
#include "StandardRecord/SRElastic.h"
#include "StandardRecord/SRHoughVertex.h"

#include <vector>

namespace caf
{
  
  /// The SRVertexBranch contains vectors of SRVertex.  
  /// This class is meant to be used in Common Analysis Files as 
  /// part of the StandardRecord.
  class SRVertexBranch
    {
    public:
      ///  Default constructor for SRTrackBranch. 
      SRVertexBranch();
      
      /// Default destuctor.  
      ~SRVertexBranch();
      
      std::vector<SRElastic> elastic;     ///< Vector of vertices found by Elastic Arms
      size_t                 nelastic;    ///< Number of vertices in elastic vector (elastic.size())

      std::vector<SRHoughVertex> hough;
      size_t nhough;
      
      void fillSizes();
      
    };
  
} // end namespace

#endif // SRVERTEXBRANCH_H
//////////////////////////////////////////////////////////////////////////////
