////////////////////////////////////////////////////////////////////////
// \file    StandardRecord.h
// \brief   The StandardRecord is the primary top-level object in the 
//          Common Analysis File trees.   
// \version $Id: StandardRecord.h,v 1.7 2012-12-06 20:18:33 rocco Exp $
// \author  $Author: rocco $
// \date    $Date: 2012-12-06 20:18:33 $
////////////////////////////////////////////////////////////////////////
#ifndef STANDARDRECORD_H
#define STANDARDRECORD_H

#include "StandardRecord/SRHeader.h"
#include "StandardRecord/SRSpill.h"
#include "StandardRecord/SRSlice.h"
#include "StandardRecord/SRTrackBranch.h"
#include "StandardRecord/SRShowerBranch.h"
#include "StandardRecord/SRVertexBranch.h"
#include "StandardRecord/SRMichelE.h"
#include "StandardRecord/SREnergyBranch.h"
#include "StandardRecord/SRIDBranch.h"
#include "StandardRecord/SRTruthBranch.h"
#include "StandardRecord/SRSandbox.h"
#include "StandardRecord/SRParentBranch.h"

/// Common Analysis Files
namespace caf
{
  
  /// \brief   The StandardRecord is the primary top-level object in the 
  ///          Common Analysis File trees.   
  
  class StandardRecord  
  {
    
  public:
    StandardRecord();
    ~StandardRecord();
    
    SRHeader       hdr;    ///< Header branch: run, subrun, etc. 
    SRSpill        spill;  ///< Beam spill branch: pot, beam current, etc.
    SRSlice        slc;    ///< Slice branch: nhit, extents, time, etc. 
    SRTrackBranch  trk;    ///< Track branch: nhit, len, etc.
    SRShowerBranch shw;    ///< Shower branch: nhit, len, etc. 
    SRVertexBranch vtx;    ///< Vertex branch: location, time, etc. 
    SRMichelE      me;     ///< Michel electron branch
    SREnergyBranch energy; ///< Energy estimator branch 
    SRIDBranch     sel;    ///< Selector (PID) branch 
    SRSandbox      sand;   ///< Sandoboxes for up and coming analysis variables
    SRTruthBranch  mc;     ///< Truth branch for MC: energy, flavor, etc.
    SRParentBranch parent; ///< True parent branch for matching, e.g. MRCC
  };
  
} // end namespace

#endif // STANDARDRECORD_H
//////////////////////////////////////////////////////////////////////////////
