////////////////////////////////////////////////////////////////////////
// \file    TimingFitAlg.h
// \brief   Algorithm to determine the direction of tracks using timing
// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef TF_TIMINGFITALG_H
#define TF_TIMINGFITALG_H

#include <vector>

#include "TimingFit/TimingFitResult.h"

namespace rb{class Track;}

namespace tf
{
  /// Represent one distance/time pair on a track
  struct Pt
  {
    Pt(double _L, double _t, double _e) : L(_L), t(_t), err(_e) {}
    double L, t, err;
  };

  /// Algorithm to determine the direction of tracks using timing
  class TimingFitAlg
  {
  public:
    TimingFitAlg();

    TimingFitResult HoughFit(const rb::Track* track) const;

    /// Helper for \ref HoughFit. May be useful for more advanced users
    TimingFitResult HoughFitPts(const std::vector<Pt>& pts) const;
  };
}

#endif
