//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
// $Id: NovaG4MonopoleFieldSetup.hh,v 1.2 2012-09-20 21:45:43 greenc Exp $
// GEANT4 tag $Name: not supported by cvs2svn $
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

//
// ------------------------------------------------------------
//        GEANT 4  include file implementation
// ------------------------------------------------------------
//
// NovaG4MonopoleFieldSetup is responsible for setting up a magnetic field
// and the ability to use it with two different equation of motions, 
// one for monopoles and another for the rest of the particles. 
// 
// 

// =======================================================================
// Created:  13 May 2010, B. Bozsogi
//  2012-02-04:  R. Hatcher - incorporate into NOvA framework from base G4 v4.9.4.p02
// =======================================================================

#ifndef NovaG4MonopoleFieldSetup_H
#define NovaG4MonopoleFieldSetup_H

#include "Geant4/G4MagneticField.hh"
#include "Geant4/G4UniformMagField.hh"

class G4FieldManager;
class G4ChordFinder;
class G4Mag_UsualEqRhs;
class G4MagIntegratorStepper;

namespace g4n {

class NovaG4MonopoleEquation;

class NovaG4MonopoleFieldMessenger;

class NovaG4MonopoleFieldSetup
{
public:  
  void InitialiseAll();    //  Set parameters and call method below
  void SetMagField(G4double fieldValue);
  void SetStepperAndChordFinder(G4int val);

  static NovaG4MonopoleFieldSetup* GetMonopoleFieldSetup();
  
protected:
  NovaG4MonopoleFieldSetup(G4ThreeVector) ;  //  The value of the field
  NovaG4MonopoleFieldSetup() ;               //  A zero field

 ~NovaG4MonopoleFieldSetup() ;     
 
protected:
  G4FieldManager*         GetGlobalFieldManager() ;   // static 

  G4FieldManager*         fFieldManager ;
  G4ChordFinder*          fChordFinder ;
  G4Mag_UsualEqRhs*       fEquation ; 
  NovaG4MonopoleEquation*     fMonopoleEquation ;
  G4MagneticField*        fMagneticField ; 

  G4MagIntegratorStepper* fStepper ;
  G4MagIntegratorStepper* fMonopoleStepper ;

  G4double                fMinStep ;
  
private:  
  static NovaG4MonopoleFieldSetup*  fMonopoleFieldSetup;
  
  NovaG4MonopoleFieldMessenger*     fMonopoleFieldMessenger;
 
};

} // end-of-namespace g4n

#endif
