////////////////////////////////////////////////////////////////////////
// Class:       AirFilter
// Module Type: filter
// File:        AirFilter_module.cc
//
// Generated at Mon Apr 28 11:48:33 2014 by Martin Frank using artmod
// from cetpkgsupport v1_05_03.
////////////////////////////////////////////////////////////////////////

#include "RecoBase/Cluster.h"

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"

#include <memory>
#include <iostream>

namespace air {
  class AirFilter;
}

class air::AirFilter : public art::EDFilter {
public:
  explicit AirFilter(fhicl::ParameterSet const & p);
  virtual ~AirFilter();

  bool filter(art::Event & e) override;

private:
  bool parallel_muon_slice(rb::Cluster const& slice) const;

  std::string slice_label_;
};



air::AirFilter::AirFilter(fhicl::ParameterSet const & p) : 
  slice_label_(p.get<std::string>("slice_label"))
{
}



air::AirFilter::~AirFilter()
{
}



bool air::AirFilter::filter(art::Event & e)
{
  art::Handle<std::vector<rb::Cluster> > slices;
  e.getByLabel(slice_label_, slices);

  std::cout << "MF: slicer4D: " << slices->size() << std::endl;

  unsigned n_triggered_slices = 0;
  unsigned n_slice = 0;
  for (auto const& slice : *slices)
  {
    // if (parallel_muon_slice(slice))
    // {
    std::cout << "MF: Event " << e.event()
	      << "\nMF: Slice " << n_slice++
      // << "\nMF: \tmean_adc  = " << slice.TotalADC() / slice.NCell()
	      << "\nMF: \ttotal_adc = " << slice.TotalADC()
      // << "\nMF: \tz_extent  = " << slice.ExtentZ()
	      << "\nMF: \tn_cell    = " << slice.NCell()
	      << std::endl;

    // return true;
    // }

    if (slice.TotalADC() > 375000)
      ++n_triggered_slices;
  }
  
  std::cout << "MF: Event " << e.event() << " has " << n_triggered_slices
	    << " triggered slices (Slicer4D)."
	    << std::endl;

  return false;
}



bool air::AirFilter::parallel_muon_slice(rb::Cluster const& slice) const
{
  if (slice.IsNoise())
    return false;
  
  if (slice.TotalADC() < 4e4 || slice.TotalADC() > 5e4)
    return false;

  if (slice.ExtentZ() < 900)
    return false;

  if (slice.NCell() < 250 || slice.NCell() > 300)
    return false;

  auto mean_adc = slice.TotalADC() / slice.NCell();

  if (mean_adc < 140 || mean_adc > 170)
    return false;

  return true;
}



DEFINE_ART_MODULE(air::AirFilter)
