////////////////////////////////////////////////////////////////////////
// Class:       AirTree
// Module Type: analyzer
// File:        AirTree_module.cc
//
// Generated at Wed Apr  9 09:57:06 2014 by Martin Frank using artmod
// from cetpkgsupport v1_04_02.
////////////////////////////////////////////////////////////////////////

#include "RecoBase/CellHit.h"
#include "RecoBase/Cluster.h"
// #include "MCCheater/BackTracker.h"

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"

#include <TH1.h>
#include <TTree.h>

#include <map>


namespace air {
  class AirTree;
}

class air::AirTree : public art::EDAnalyzer {
public:
  explicit AirTree(fhicl::ParameterSet const & p);
  virtual ~AirTree();

  void analyze(art::Event const & e) override;

  void beginJob() override;

private:
  std::string hit_label_, slice_label_;

  TTree *hit_tree_, *slice_tree_;
  std::map<std::string, double> ht_, st_;

  std::map<std::string, TH1*> h_;
};



air::AirTree::AirTree(fhicl::ParameterSet const & p) :
  EDAnalyzer(p),
  hit_label_(p.get<std::string>("hit_label")),
  slice_label_(p.get<std::string>("slice_label"))
{}



air::AirTree::~AirTree()
{
}



void air::AirTree::beginJob()
{
  // set the branch names
  std::vector<std::string> hit_names { "adc" };
  std::vector<std::string> slice_names { "sum_adc" };

  // handle all of the tree setup
  art::ServiceHandle<art::TFileService> tfs;
  hit_tree_ = tfs->make<TTree>("HitTree", "Air Shower Tree for All Hits");
  slice_tree_ = tfs->make<TTree>("SliceTree", "Air Shower Tree for All Slices");

  for (auto const& name : hit_names)
    hit_tree_->Branch(name.c_str(), &ht_[name], (name + "/D").c_str());
  for (auto const& name : slice_names)
    slice_tree_->Branch(name.c_str(), &st_[name], (name + "/D").c_str());

  // h_["energy"] = tfs->make<TH1F>("energy", "Energy Deposition", 1000, 0, 10);
}



void air::AirTree::analyze(art::Event const & e)
{
  // initialize all of the branches
  const double default_value = -9999;
  for (auto & branch : ht_)
    branch.second = default_value;
  for (auto & branch : st_)
    branch.second = default_value;
  
  art::Handle<std::vector<rb::CellHit> > hits;
  e.getByLabel(hit_label_, hits);
  art::Handle<std::vector<rb::Cluster> > slices;
  e.getByLabel(slice_label_, slices);

  // art::ServiceHandle<cheat::BackTracker> bt;

  for (auto const& hit : *hits)
  {
    ht_["adc"] = hit.ADC(0);

    hit_tree_->Fill();
  }

  unsigned n_slice = 0;
  for (auto const& slice : *slices)
  {
    if (!slice.IsNoise())
    {
      ++n_slice;
      st_["sum_adc"] = slice.TotalADC();

      slice_tree_->Fill();
    }
  }
}



DEFINE_ART_MODULE(air::AirTree)
