##########################################################################
# This script is for submitting a SAM project via jobsub to FNAL resources.
# You need to have sourced the release/build that you intend to submit with
# and made a dataset to run over.

############################################
# USERS EDIT THE BELOW PARAMETERS          #
############################################
# Setup the Job specific parameters
JOBNAME='airtree'
DATASET='mfrank_r14023_DDenergy'
NUMFILES=1
NUMJOBS=1
OUTPUT_DIR="/nova/ana/users/mfrank/air/samout"
JOBFCL="${SRT_PRIVATE_CONTEXT}/job/airtreejob.fcl"

#############################################
# USER INTERVENTION ENDS HERE, IN PRINCIPLE #
#############################################
ROLE=""
DATA_TIER="reco"

# Setup the SAM project here
if [[ "x$PROJECTNAME" == "x" ]]; then
    DATE="`date +%Y%m%d_%H%M`"
    PROJECTNAME="${USER}-${JOBNAME}-${DATE}"
    ifdh startProject "${PROJECTNAME}" $SAM_STATION "$DATASET" $USER $SAM_STATION
    if [ $? -eq 0 ]; then
	export SAM_PROJECT_NAME="${PROJECTNAME}"
    fi
else
  export SAM_PROJECT_NAME="${PROJECTNAME}"
fi

if [ ${NUMJOBS} -le 0 ]; then
  export NUMPROJECTFILES=`samweb -e nova count-definition-files ${DATASET}`
  if [ ${NUMFILES} -eq 1 ]; then
    export NUMJOBS=${NUMPROJECTFILES}
  else
    export NUMJOBS=`echo "(${NUMPROJECTFILES}/ ${NUMFILES}) + 1" | bc`
  fi
fi

##########################################
# Configure Grid Proxy
if [[ "x$ROLE" == "x" ]]; then
  X509_USER_PROXY=/scratch/$USER/grid/$USER.$EXPERIMENT.proxy
else
  X509_USER_PROXY=/scratch/$USER/grid/$USER.$EXPERIMENT.$ROLE.proxy
fi

###########################################
# Give log output names a sensible name based on the PROJECTNAME
#ln -s $SCRIPT /nova/app/condor-exec/$USER/${PROJECTNAME}.sh
ln -s /nova/app/users/mfrank/src/air-S14-04-14/Metadata/samUtils/art_sam_wrap.sh /nova/app/condor-exec/$USER/${PROJECTNAME}.sh
JOBSCRIPT=/nova/app/condor-exec/$USER/${PROJECTNAME}.sh

##########################################
# Print out some useful information
echo "NUMJOBS IS $NUMJOBS with $NUMFILES per job"
echo "DATASET: " $DATASET
echo "Project name is: $SAM_PROJECT_NAME"
echo "Station Monitor: http://samweb.fnal.gov:8480/station_monitor/nova/stations/nova/projects/$SAM_PROJECT_NAME"
echo "$SAM_PROJECT_NAME" >> $0.projectname.log

##########################################
# The Magic Jobsub command
jobsub \
    -N ${NUMJOBS} \
    --site ${SITES} \
    --OS=SL5,SL6 \
    --opportunistic \
    --nowrapfile \
    --X509_USER_PROXY $X509_USER_PROXY \
    -e SAM_PROJECT_NAME \
    -e SAM_STATION \
    -e IFDH_BASE_URI \
    -e IFDH_DEBUG \
    -e EXPERIMENT \
    -e NOVADBTIMEOUT \
    -e NOVADBUSER  \
    -e NOVADBPWDFILE \
    -g \
    ${JOBSCRIPT} \
      --multifile \
      --export EXTERNALS='$PRODUCTS' \
      --config $JOBFCL \
      --source '/grid/fermiapp/nova/novaart/novasvn/srt/srt.sh' \
      --source '${SRT_DIST}/setup/setup_novasoft.sh:-r:'"${NOVA_RELEASE}:-b:${BUILD}" \
      --source '${SRT_PUBLIC_CONTEXT}/setup/setup_novagrid.sh:-r:'"${NOVA_RELEASE}:-b:${BUILD}" \
      --source "${SRT_PRIVATE_CONTEXT}/AirShower/setup_testrel.sh" \
      --limit ${NUMFILES} \
      --postscript "${SRT_PRIVATE_CONTEXT}/AirShower/copyOutFiles.sh" \
      --export DEST=${OUTPUT_DIR} \
      --exe "${SRT_PRIVATE_CONTEXT}/AirShower/runNovaSAM.py"

