///////////////////////////////////////////////////////////
//
// \file    BPFEnergy.h
// \brief   Header for the energy object created by
//          BPFEnergy_module.cc.
// \version
// \author  Michael Baird - mbaird42@fnal.gov
//
///////////////////////////////////////////////////////////
#ifndef BPFIT_BPFENERGY_H
#define BPFIT_BPFENERGY_H

namespace bpfit
{
  class BPFEnergy{
  public:

    // {Con,De}structors:
    BPFEnergy();
    BPFEnergy(float E1, float Eres1, float E2, float Eres2, float E3, float Eres3,
	      float EventID, float PMuon, float DirZMuon,
	      float N3DProngs, float EFuzzyK3D, float ERemain, float SumPE);
    ~BPFEnergy();

    // Set and Get functions:
    void SetE1       (float E)         { fE1 = E; }
    void SetEres1    (float Eres)      { fEres1 = Eres; }
    void SetE2       (float E)         { fE2 = E; }
    void SetEres2    (float Eres)      { fEres2 = Eres; }
    void SetE3       (float E)         { fE3 = E; }
    void SetEres3    (float Eres)      { fEres3 = Eres; }
    void SetEventID  (float EventID)   { fEventID = EventID; }
    void SetPMuon    (float PMuon)     { fPMuon = PMuon; }
    void SetDirZMuon (float DirZMuon)  { fDirZMuon = DirZMuon; }
    void SetN3DProngs(float N3DProngs) { fN3DProngs = N3DProngs; }
    void SetEFuzzyK3D(float EFuzzyK3D) { fEFuzzyK3D = EFuzzyK3D; }
    void SetERemain  (float ERemain)   { fERemain = ERemain; }
    void SetSumPE    (float SumPE)     { fSumPE = SumPE; }

    float E1()        const { return fE1; }
    float Eres1()     const { return fEres1; }
    float E2()        const { return fE2; }
    float Eres2()     const { return fEres2; }
    float E3()        const { return fE3; }
    float Eres3()     const { return fEres3; }
    float EventID()   const { return fEventID; }
    float PMuon()     const { return fPMuon; }
    float DirZMuon()  const { return fDirZMuon; }
    float N3DProngs() const { return fN3DProngs; }
    float EFuzzyK3D() const { return fEFuzzyK3D; }
    float ERemain()   const { return fERemain; }
    float SumPE()     const { return fSumPE; }

  private:
    float fE1;        ///< Total event energy (using an unoscillated training sample)
    float fEres1;     ///< Estimated event energy resolution (using an unoscillated training sample)
    float fE2;        ///< Total event energy (using a training sample with dm^2 = 2.4e-3, sin^2(2th23) = 0.95)
    float fEres2;     ///< Estimated event energy resolution (using a training sample with dm^2 = 2.4e-3, sin^2(2th23) = 0.95)
    float fE3;        ///< Total event energy (using a training sample with dm^2 = 2.4e-3, sin^2(2th23) = 1.0)
    float fEres3;     ///< Estimated event energy resolution (using a training sample with dm^2 = 2.4e-3, sin^2(2th23) = 1.0)
    float fEventID;   ///< Can be used to store event IDs (remid, best BPF muon PID, etc.)
    float fPMuon;     ///< BPF reconstructed momentum for the prong with the best muon PID value
    float fDirZMuon;  ///< BPF reconstructed track direction for the prong with the best muon PID value
    float fN3DProngs; ///< Number of fuzzyK 3D prongs.
    float fEFuzzyK3D; ///< Summed energy from hits on fuzzyK 3D prongs (not including the prong with the best muonPID)
    float fERemain;   ///< Summed remaining energy in the slice (not on fuzzyK 3D prongs)
    float fSumPE;     ///< Summed PE for all hits not on the muon track
  };
}

#endif
///////////////////////////////////////////////////////////
