#include "TCanvas.h"
#include "TH1D.h"

void calhit_validation_macro()
{

  TFile fold("/nova/data/mc/S13-06-05/cosmics/fd/hadd/fd_S13-06-05_v1_cosmics_200.sim.reco.validhists.root");
  fold.cd("calhitana");
  TTree *hitold = gDirectory->Get("fHitTree");
  TTree *evtold = gDirectory->Get("fEvtTree");

  TFile fnew("/nova/prod/mc/S14-03-06/reco/validation/hadd/fardet_cosmics_S14-03-06.sim.daq.reco.hist.root");
  fnew.cd("calhitana");
  TTree *hitnew = gDirectory->Get("fHitTree");
  TTree *evtnew = gDirectory->Get("fEvtTree");
  
  cout<<"Got the files and trees\n";  

  int nadc, oadc, ntdc, otdc, nnoise, onoise;
  float npe, ope, ntns, otns, npee, opee;
  unsigned int nplane, oplane, ncell, ocell;

  hitold->SetBranchAddress("Pe", &ope);
  hitold->SetBranchAddress("Adc", &oadc);
  hitold->SetBranchAddress("Tdc", &otdc);
  hitold->SetBranchAddress("Tns", &otns);
  hitold->SetBranchAddress("Plane", &oplane);
  hitold->SetBranchAddress("Cell", &ocell);

  evtold->SetBranchAddress("Noise", &onoise);
  evtold->SetBranchAddress("PePerTrueE", &opee);

  hitnew->SetBranchAddress("Pe", &npe);
  hitnew->SetBranchAddress("Adc", &nadc);
  hitnew->SetBranchAddress("Tdc", &ntdc);
  hitnew->SetBranchAddress("Tns", &ntns);
  hitnew->SetBranchAddress("Plane", &nplane);
  hitnew->SetBranchAddress("Cell", &ncell);

  evtnew->SetBranchAddress("Noise", &nnoise);
  evtnew->SetBranchAddress("PePerTrueE", &npee);

  cout<<"Set tree branches\n";

  TH1D *hnpe = new TH1D("npe",";PE;Hits",250,0,2500); 
  TH1D *hope = new TH1D("ope",";PE;Hits",250,0,2500); 
  hnpe->Sumw2();
  hope->Sumw2();

  TH1D *hnadc = new TH1D("nadc",";ADC[0];Hits",210,0,4200); 
  TH1D *hoadc = new TH1D("oadc",";ADC[0];Hits",210,0,4200); 
  hnadc->Sumw2();
  hoadc->Sumw2();

  TH1D *hntdc = new TH1D("ntdc",";TDC;Hits",200,0,40000); 
  TH1D *hotdc = new TH1D("otdc",";TDC;Hits",200,0,40000); 
  hntdc->Sumw2();
  hotdc->Sumw2();

  TH1D *hntns = new TH1D("ntns",";TNS;Hits",300,0,600000);
  TH1D *hotns = new TH1D("otns",";TNS;Hits",300,0,600000); 
  hntns->Sumw2();
  hotns->Sumw2();

  TH1D *hnplane = new TH1D("nplane",";Plane;Hits",900,-0.5,899.5); 
  TH1D *hoplane = new TH1D("oplane",";Plane;Hits",900,-0.5,899.5); 
  hnplane->Sumw2();
  hoplane->Sumw2();

  TH1D *hncell = new TH1D("ncell",";Cell;Hits",384,-0.5,383.5); 
  TH1D *hocell = new TH1D("ocell",";Cell;Hits",384,-0.5,383.5); 
  hncell->Sumw2();
  hocell->Sumw2();

  TH2D *hncp = new TH2D("ncp","Hit map;Plane;Cell",900,-0.5,899.5,384,-0.5,383.5); 
  TH2D *hocp = new TH2D("ocp","Hit map;Plane;Cell",900,-0.5,899.5,384,-0.5,383.5); 

  TH1D *hnnoise = new TH1D("nnoise",";Noise Hits;Event", 300,18700,19900);
  TH1D *honoise = new TH1D("onoise",";Noise Hits;Event", 300,18700,19900);
  hnnoise->Sumw2();
  honoise->Sumw2();

  TH1D *hnpee = new TH1D("npee",";PE per FLS Energy [PE/GeV];Event", 300,13000,25000);
  TH1D *hopee = new TH1D("opee",";PE per FLS Energy [PE/GeV];Event", 300,13000,25000);
  hnpee->Sumw2();
  hopee->Sumw2();

  TLegend *leg = new TLegend(0.55,0.7,0.8,0.85);
  leg->SetTextSize(0.04);
  leg->AddEntry(hnpe, "New", "l");
  leg->AddEntry(hope, "Old", "l");
    
  cout<<"Declared histograms\n";

  int nnh = hitnew->GetEntries(); 
  int noh = hitold->GetEntries(); 

  int nne = evtnew->GetEntries(); 
  int noe = evtold->GetEntries(); 

  cout<<nnh<<" "<<noh<<" "<<nne<<" "<<noe<<endl;
  cout<<"scale pes by "<<((float)(nnh*100)/(float)(nne))<<" and "<<((float)(noh*100)/(float)(noe))<<endl;

  cout<<"Entries in new hit tree: "<<nnh<<endl;

  for(int e = 0; e < nnh; ++e){
    hitnew->GetEntry(e);

    hnpe->Fill(npe);
    hnadc->Fill(nadc);
    hntdc->Fill(ntdc);
    hntns->Fill(ntns);
    hnplane->Fill(nplane);
    hncell->Fill(ncell);
    hncp->Fill(nplane, ncell);

  }// end loop over hitnew entries

  cout<<"Entries in old hit tree: "<<noh<<endl;

  for(int e = 0; e < noh; ++e){
    hitold->GetEntry(e);

    hope->Fill(ope);
    hoadc->Fill(oadc);
    hotdc->Fill(otdc);
    hotns->Fill(otns);
    hoplane->Fill(oplane);
    hocell->Fill(ocell);
    hocp->Fill(oplane, ocell);

  }// end loop over hitnew entries

  cout<<"Entries in new evt tree: "<<nne<<endl;

  for(int e = 0; e < nne; ++e){
    evtnew->GetEntry(e);

    hnpee->Fill(npee);
    hnnoise->Fill(nnoise);

  }// end loop over hitnew entries

  cout<<"Entries in old evt tree: "<<noe<<endl;

  for(int e = 0; e < noe; ++e){
    evtold->GetEntry(e);

    hopee->Fill(opee);
    honoise->Fill(onoise);

  }// end loop over hitnew entries


  // makes sense tonormalize everything to the same number of events. 
  // this will show any differences in hitrate. 
  // number of events in old file is noe, and in new is nne
  // normalizing to 100 events

  TCanvas *c1 = new TCanvas("c1");
  c1->SetLogy();
  hnpe->SetMarkerSize(0);
  hope->SetMarkerSize(0);
  hnpe->Scale( nnh*100/(float)nne);
  hope->Scale( noh*100/(float)noe);
  
  hnpe->SetLineColor(2);
  hnpe->Draw("ehist");
  hnpe->GetYaxis()->SetRangeUser(0.001, 1.5* hnpe->GetBinContent(hnpe->GetMaximumBin()) );
  hope->Draw("sameehist");
  leg->Draw("same");
  c1->SaveAs("../results/plots/pe-log.pdf");

  TCanvas *c2 = new TCanvas("c2");
  c2->SetLogy();
  hnadc->SetMarkerSize(0);
  hoadc->SetMarkerSize(0);
  hnadc->Scale( nnh*100/(float)nne);
  hoadc->Scale( noh*100/(float)noe);
  hnadc->SetLineColor(2);
  hnadc->Draw("ehist");
  hnadc->GetYaxis()->SetRangeUser(0.001, 1.5* hnadc->GetBinContent(hnadc->GetMaximumBin()) );
  hoadc->Draw("sameehist");
  leg->Draw("same");
  c2->SaveAs("../results/plots/adc-log.pdf");

  TCanvas *c3 = new TCanvas("c3");
  c3->SetLogy();
  hntdc->SetMarkerSize(0);
  hotdc->SetMarkerSize(0);
  hntdc->Scale( nnh*100/(float)nne);
  hotdc->Scale( noh*100/(float)noe);
  hntdc->SetLineColor(2);
  hntdc->Draw("ehist");
  hntdc->GetYaxis()->SetRangeUser(0.001, 1.5* hntdc->GetBinContent(hntdc->GetMaximumBin()) );
  hotdc->Draw("sameehist");
  leg->Draw("same");
  c3->SaveAs("../results/plots/tdc-log.pdf");

  TCanvas *c4 = new TCanvas("c4");
  c4->SetLogy();
  hntns->SetMarkerSize(0);
  hotns->SetMarkerSize(0);
  hntns->Scale( nnh*100/(float)nne);
  hotns->Scale( noh*100/(float)noe);
  hntns->SetLineColor(2);
  hntns->Draw("ehist");
  hntns->GetYaxis()->SetRangeUser(0.001, 1.5* hntns->GetBinContent(hntns->GetMaximumBin()) );
  hotns->Draw("sameehist");
  leg->Draw("same");
  c4->SaveAs("../results/plots/tns-log.pdf");

  TCanvas *c5 = new TCanvas("c5");
  c5->SetLogy();
  hnplane->SetMarkerSize(0);
  hoplane->SetMarkerSize(0);
  hnplane->Scale( nnh*100/(float)nne);
  hoplane->Scale( noh*100/(float)noe);
  hnplane->SetLineColor(2);
  hnplane->Draw("ehist");
  hnplane->GetYaxis()->SetRangeUser(0.001, 1.5* hnplane->GetBinContent(hnplane->GetMaximumBin()) );
  hoplane->Draw("sameehist");
  leg->Draw("same");
  c5->SaveAs("../results/plots/plane-log.pdf");

  TCanvas *c6 = new TCanvas("c6");
  c6->SetLogy();
  hncell->SetMarkerSize(0);
  hocell->SetMarkerSize(0);
  hncell->Scale( nnh*100/(float)nne);
  hocell->Scale( noh*100/(float)noe);
  hncell->SetLineColor(2);
  hncell->Draw("ehist");
  hncell->GetYaxis()->SetRangeUser(0.001, 1.5* hncell->GetBinContent(hncell->GetMaximumBin()) );
  hocell->Draw("sameehist");
  leg->Draw("same");
  c6->SaveAs("../results/plots/cell-log.pdf");

  TCanvas *c7 = new TCanvas("c7");
  c7->SetLogy();
  hnnoise->SetMarkerSize(0);
  honoise->SetMarkerSize(0);
  hnnoise->Scale( nnh*100/(float)nne);
  honoise->Scale( noh*100/(float)noe);
  hnnoise->SetLineColor(2);
  hnnoise->Draw("ehist");
  hnnoise->GetYaxis()->SetRangeUser(0.001, 1.5* hnnoise->GetBinContent(hnnoise->GetMaximumBin()) );
  honoise->Draw("ehistsame");
  leg->Draw("same");
  c7->SaveAs("../results/plots/noise-log.pdf");

  TCanvas *c8 = new TCanvas("c8");
  c8->SetLogy();
  hnpee->SetMarkerSize(0);
  hopee->SetMarkerSize(0);
  hnpee->Scale( nnh*100/(float)nne);
  hopee->Scale( noh*100/(float)noe);
  hnpee->SetLineColor(2);
  hnpee->Draw("ehist");
  hnpee->GetYaxis()->SetRangeUser(0.001, 1.5* hnpee->GetBinContent(hnpee->GetMaximumBin()) );
  hopee->Draw("ehistsame");
  leg->Draw("same");
  c8->SaveAs("../results/plots/pee-log.pdf");

  TCanvas *c9 = new TCanvas("c9");
  hncp->Draw("colz");
  c9->SaveAs("../results/plots/ncp-log.pdf");

  TCanvas *c10 = new TCanvas("c10");
  hocp->Draw("colz");
  c10->SaveAs("../results/plots/ocp-log.pdf");

  return;
}
