////////////////////////////////////////////////////////////////////////
// Class:       DetRespDrift
// Module Type: filter
// File:        DetRespDrift_module.cc
//
// Generated at Wed Jul 8 2015 by Luke Vinton 
////////////////////////////////////////////////////////////////////////


// framework includes....
#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art/Framework/Core/FindManyP.h" 

// novasoft stuff
#include "CalibrationDataProducts/PCHit.h"
#include "RecoBase/Track.h"
#include "RecoBase/CellHit.h"
#include "Calibrator/Calibrator.h"
#include "GeometryObjects/GeometryBase.h"

// C++
#include <memory>
#include <string>
#include <iostream>
#include <vector>

// ROOT
#include "TH2D.h"
#include "TFile.h"
#include "TVector3.h"
#include "TTree.h"

namespace caldp {
  class PCHit;
}


namespace calib {
  class DetRespDrift;
  
  class DetRespDrift : public art::EDFilter {
  public:
    explicit DetRespDrift(fhicl::ParameterSet const & p);
    virtual ~DetRespDrift();
    
    bool filter(art::Event & e) override;
    void reconfigure(fhicl::ParameterSet const & p) override;    
    void beginJob() override;
    void endJob() override;
    bool beginRun(art::Run & r) override;
    bool endRun(art::Run & r) override;
    
  private:
    double getPECorr(art::Event & e,caldp::PCHit const &pchit);

    TTree* fTree;
    //declare variables for ttree
    int run;
    uint32_t EventTimeHigh;
    double path;
    double trueE;
    double PECorr;
    double pe;
    std::string pathQual;


    std::string fPCHitLabel;
    std::string fQualXYName;
    std::string fQualZName;
    std::string fQualAvgName;
    std::string fLabel;

  };
  
  
  DetRespDrift::DetRespDrift(fhicl::ParameterSet const & p)
  {
    this->reconfigure(p);

  }
  
  DetRespDrift::~DetRespDrift()
  {
    // Clean up dynamic memory and other resources here.
  }
  
  bool DetRespDrift::filter(art::Event & e)
  {
    /// Get the reconstructed tracks                                           
    art::Handle<std::vector<rb::Track> > tracks;
    e.getByLabel(fLabel, tracks);
    if( tracks->size() == 0 ) return true;

    run = (int)(e.run());
    art::Timestamp ts = e.time();
    EventTimeHigh = ts.timeHigh();

    art::FindManyP<caldp::PCHit> PCHitXYGetter(tracks, e, art::InputTag(fPCHitLabel,fQualXYName)); 
    art::FindManyP<caldp::PCHit> PCHitZGetter(tracks, e, art::InputTag(fPCHitLabel,fQualZName)); 
    art::FindManyP<caldp::PCHit> PCHitAvgGetter(tracks, e, art::InputTag(fPCHitLabel,fQualAvgName)); 

    if( !PCHitAvgGetter.isValid() && !PCHitXYGetter.isValid() && !PCHitZGetter.isValid()) return false;    

    // loop over stopping tracks in event
    for( unsigned int i=0; i<tracks->size(); ++i ){

      // loop over PCHits associated to stopping track
      std::vector<art::Ptr<caldp::PCHit> > hitsXY = PCHitXYGetter.at(i);
      std::vector<art::Ptr<caldp::PCHit> > hitsZ = PCHitZGetter.at(i);
      std::vector<art::Ptr<caldp::PCHit> > hitsAvg = PCHitAvgGetter.at(i);

      // XY hits
      for( unsigned int j=0; j<hitsXY.size(); ++j ){
	
	path = hitsXY[j]->Path();
	trueE = hitsXY[j]->TrueMeV();
	PECorr = this->getPECorr(e,*hitsXY[j]);
	pe = hitsXY[j]->PE();
	pathQual = "XY";

	fTree->Fill(); //fill ttree
      }

      // Z hits
      for( unsigned int j=0; j<hitsZ.size(); ++j ){
	
	path = hitsZ[j]->Path();
	trueE = hitsZ[j]->TrueMeV();
	PECorr = this->getPECorr(e,*hitsZ[j]);
	pe = hitsZ[j]->PE();
	pathQual = "Z";

	fTree->Fill(); //fill ttree
      }

      // Avg hits
      for( unsigned int j=0; j<hitsAvg.size(); ++j ){
	
	path = hitsAvg[j]->Path();
	trueE = hitsAvg[j]->TrueMeV();
	PECorr = this->getPECorr(e,*hitsAvg[j]);
	pe = hitsAvg[j]->PE();
	pathQual = "Avg";

	fTree->Fill(); //fill ttree
      }

    }

    return true;

  }

  double DetRespDrift::getPECorr(art::Event & e, caldp::PCHit const &pchit){
   
    //Calibrator necessary to get Atten information                             
    art::ServiceHandle<calib::Calibrator> calibrator;

    //Make mock cellhit
    rb::CellHit cellhit;
    cellhit.SetPlane(pchit.Plane());
    cellhit.SetCell(pchit.Cell());
    cellhit.SetView(pchit.View());
    cellhit.SetTNS(pchit.TNS(),pchit.GoodTime());
    if(!e.isRealData())cellhit.SetMC(); // assume only filled for MC...
    cellhit.SetPE(pchit.PE());

    double pecorr = calibrator->GetPECorr(cellhit, pchit.W());
    return pecorr;
  }

  void DetRespDrift::reconfigure(fhicl::ParameterSet const & pset)
  {
    fPCHitLabel = pset.get< std::string >("PCHitLabel"); //Label of PCHitList module            
    fQualXYName = pset.get< std::string >("QualXYName"); //Instance label, "XY" quality hits    
    fQualZName = pset.get< std::string >("QualZName"); //Instance label, "Z" quality hits    
    fQualAvgName = pset.get< std::string >("QualAvgName"); //Instance label, "Avg" quality hits    
    fLabel = pset.get<std::string>("Label");

  }
  
  void DetRespDrift::beginJob()
  {
    // histograms for output
    art::ServiceHandle<art::TFileService> tfs;

    fTree = tfs->make<TTree>("fTree","tree to hold variables");
    fTree->Branch("run",&run);
    fTree->Branch("EventTimeHigh",&EventTimeHigh);
    fTree->Branch("pe",&pe);
    fTree->Branch("PECorr",&PECorr);
    fTree->Branch("trueE",&trueE);
    fTree->Branch("path",&path);
    fTree->Branch("pathQual",&pathQual);


  }
  
  bool DetRespDrift::beginRun(art::Run &r){
    return true;
  }

  bool DetRespDrift::endRun(art::Run &r){

    art::ServiceHandle<art::TFileService> tfs;

    return true;
  }

  void DetRespDrift::endJob()
  {

  }
}// end namespace
DEFINE_ART_MODULE(calib::DetRespDrift)

