#include "TCanvas.h"
#include "TF1.h"
#include "TFile.h"
#include "TH2.h"
#include "TLatex.h"
#include "TLegend.h"
#include "TLine.h"
#include "TProfile.h"
#include "TString.h"
#include "TStyle.h"
#include "TTree.h"

// Put a "NOvA preliminary" tag in the corner, further than default
void Preliminary()
{
  TLatex* prelim = new TLatex(.99, .95, "NO#nuA Preliminary");
  prelim->SetTextColor(kBlue);
  prelim->SetNDC();
  prelim->SetTextSize(2/30.);
  prelim->SetTextAlign(32);
  prelim->Draw();
}

void plot(TFile* f, TString var, int view)
{
  new TCanvas;

  const TString viewStr = view ? "Y view" : "X view";

  TLegend* leg = new TLegend(.15, .65, .4, .875);
  leg->SetFillStyle(0);

  TProfile* p = (TProfile*)f->Get(TString::Format("prof_%s_%s.eps", var.Data(), view ? "Y" : "X"));

  p->SetLineColor(kRed);
  p->SetMarkerColor(kRed);
  p->Draw();

  p->SetTitle(viewStr+", true deposits > 15MeV");
  p->GetYaxis()->SetTitle("Mean Reconstructed / True Energy");
  p->GetXaxis()->CenterTitle();
  p->GetYaxis()->CenterTitle();


  TLine* one = new TLine(p->GetXaxis()->GetXmin(), 1, 
			 p->GetXaxis()->GetXmax(), 1);
  one->SetLineWidth(2);
  one->SetLineStyle(7);
  one->Draw();
  p->Draw("same");

  TProfile* pu = (TProfile*)f->Get(TString::Format("prof_uncorr_%s_%s.eps", var.Data(), view ? "Y" : "X"));

  pu->SetLineColor(kBlue);
  pu->SetMarkerColor(kBlue);
  pu->Draw("same");

  leg->AddEntry(pu, "Uncalibrated", "el");
  leg->AddEntry(p, "Calibrated", "el");
  leg->Draw();

  if(var == "W")
    p->GetYaxis()->SetRangeUser(0, 3);
  else
    p->GetYaxis()->SetRangeUser(.5, 1.5);
  
  Preliminary();
  gPad->Print(TString::Format("plots/assess_vs_%s_%s.eps", var.Data(), view ? "Y" : "X"));


  p->Draw();
  one->Draw();
  p->Draw("same");

  if(var == "W")
    p->GetYaxis()->SetRangeUser(0.9, 1.1);
  else
    p->GetYaxis()->SetRangeUser(0.95, 1.05);

  Preliminary();
  gPad->Print(TString::Format("plots/assess_vs_%s_%s_zoom.eps", var.Data(), view ? "Y" : "X"));




  // Slice plots are only interesting for W view
  if(var != "W") return;

  int sliceCols[5] = {kBlue, kBlue+2, kBlack, kRed+2, kRed};

  TLegend* leg2 = new TLegend(.45, .6, .875, .85);
  leg2->SetFillStyle(0);

  new TCanvas;
  for(int i = 4; i >= 0; --i){
    TH1* proj = (TH1*)f->Get(TString::Format("slice_%s_%d.eps", view ? "Y" : "X", i));

    proj->SetTitle(p->GetTitle());
    proj->Scale(1./proj->Integral());
    proj->SetLineColor(sliceCols[i]);
    proj->GetXaxis()->CenterTitle();
    proj->GetXaxis()->SetRangeUser(0, 3);
    proj->GetXaxis()->SetTitle("Reconstructed / True Energy");
    proj->DrawClone((i != 4) ? "same" : "");
    if(i == 0) leg2->AddEntry(proj, "Far from readout");
    if(i == 4) leg2->AddEntry(proj, "Near readout");
  }

  leg2->Draw();

  Preliminary();
  gPad->Print(TString::Format("plots/assess_slices_%s.eps", view ? "Y" : "X"));

  new TCanvas;
  for(int i = 0; i < 5; ++i){
    TH1* proj = (TH1*)f->Get(TString::Format("slice_uncorr_%s_%d.eps", view ? "Y" : "X", i));
    proj->SetTitle(p->GetTitle());
    proj->Scale(1./proj->Integral());
    proj->SetLineColor(sliceCols[i]);
    proj->GetXaxis()->CenterTitle();
    proj->GetXaxis()->SetRangeUser(0, 3);
    proj->GetXaxis()->SetTitle("Uncalibrated Response / True Energy");
    proj->DrawClone(i ? "same" : "");
  }

  leg2->Draw();

  Preliminary();
  gPad->Print(TString::Format("plots/assess_slices_uncorr_%s.eps", view ? "Y" : "X"));
}

void plot_assess_pretty()
{
  TFile* f = new TFile("plots/plot_assess.root");

  // Avoid preliminary label
  gStyle->SetTitleAlign(12);
  gStyle->SetTitleX(.1);

  for(int view = 0; view < 2; ++view){
    plot(f, "W", view);
    plot(f, "cell", view);
    plot(f, "plane", view);
  }
}
