///////////////////////////////////////////////////////////////////////////
/// \brief   Histograms used by attenuation calibration
/// \author  Christopher Backhouse - bckhouse@caltech.edu
/// \date    Apr 2013
///////////////////////////////////////////////////////////////////////////
#ifndef CALDP_ATTENHISTS_H
#define CALDP_ATTENHISTS_H

#include "TH2.h"

#include "GeometryObjects/OfflineChan.h"

#include <map>

namespace caldp
{
  /// Histograms used by attenuation calibration
  class AttenHists
  {
  public:
    AttenHists() : WPE(0), WPE_corr(0), WPE_corr_xy(0), WPE_corr_z(0), WPE_corr_xy_truth(0) {}

#ifndef __GCCXML__
    AttenHists& operator+=(const AttenHists& rhs)
    {
      Add(WPE, rhs.WPE);
      Add(WPE_corr, rhs.WPE_corr);
      Add(WPE_corr_xy, rhs.WPE_corr_xy);
      Add(WPE_corr_z, rhs.WPE_corr_z);
      Add(WPE_corr_xy_truth, rhs.WPE_corr_xy_truth);

      return *this;
    }
#endif

    TH2F *WPE, *WPE_corr, *WPE_corr_xy, *WPE_corr_z, *WPE_corr_xy_truth;

  protected:
    void Add(TH2F*& lhs, TH2F* rhs)
    {
      if(lhs) lhs->Add(rhs); else lhs = new TH2F(*rhs);
    }
  };

  /// AttenHists for many channels
  class AttenHistsMap
  {
  public:
#ifndef __GCCXML__
    AttenHists& GetHists(const geo::OfflineChan& c)
    {
      return fMap[c];
    }

    AttenHists& GetHists(int plane, int cell)
    {
      return fMap[geo::OfflineChan(plane, cell)];
    }

    /// Intended for writing out/drawing all the histograms
    std::map<int, std::vector<TH2F*>> GetAllHistsByPlane()
    {
      std::map<int, std::vector<TH2F*>> ret;
      for(auto& it: fMap){
        const int plane = it.first.Plane();
        const AttenHists& hs = it.second;

        ret[plane].push_back(hs.WPE);
        ret[plane].push_back(hs.WPE_corr);
        ret[plane].push_back(hs.WPE_corr_xy);
        ret[plane].push_back(hs.WPE_corr_z);
        ret[plane].push_back(hs.WPE_corr_xy_truth);
      }
      return ret;
    }

    AttenHistsMap& operator+=(const AttenHistsMap& rhs)
    {
      for(auto& it: rhs.fMap) fMap[it.first] += it.second;
      return *this;
    }

    void Clear()
    {
      fMap.clear();
    }
#endif
  protected:
    std::map<geo::OfflineChan, AttenHists> fMap;
  };
}

#endif
