////////////////////////////////////////////////////////////////////////
/// \brief   Hold drift constants from current run
/// \author bckhouse@caltech.edu
/// \date
////////////////////////////////////////////////////////////////////////
#ifndef CALIB_DRIFTCACHE_H
#define CALIB_DRIFTCACHE_H

//#include <map>

#include <string>
#include <ctime>

#include "NovaDAQConventions/DAQConventions.h"
#include "GeometryObjects/OfflineChan.h"

namespace nova{namespace dbi{class Table;}}

namespace calib
{
  class DriftCache
  {
  public:

    DriftCache();
    virtual ~DriftCache();

    double GetFactor(int plane, int cell, time_t tevt);

    void SetVld(int det, time_t tstart, time_t tstop);
    void SetCSVFile(std::string f) { fCSVFile = f;}

  protected:
    void Load();

    int fDet;
    int fFactorIdx;
    time_t fTStart;
    time_t fTStop;

    time_t fEvtTime;

    std::string fCSVFile;
    nova::dbi::Table* fTbl;
  };

} // end namespace calib

#endif
////////////////////////////////////////////////////////////////////////
