///
/// \file    CanaNt.h
/// \brief   Ntuple produced by the Cana module
/// \version $Id: CanaNt.h,v 1.11 2011-07-26 02:08:47 messier Exp $
/// \author  messier@indiana.edu
///
#ifndef COMMISSIONING_CANANT_H
#define COMMISSIONING_CANANT_H
#include "TTimeStamp.h"

namespace comi {

  /// Ntuple produced by the cana module
  class CanaNt {
  public:
    int   run;       ///< run number
    int   subrun;    ///< subrun number
    int   evt;       ///< event number
    int   iwin;      ///< window number
    int   trgsrc;    ///< trigger source
    int   nhit;      ///< Number of hits on track in peak time window
    int   nhitx;     ///< Number of x view hits on track in peak time window
    int   nhity;     ///< Number of y view hits on track in peak time window
    int   plane1x;   ///< 1st plane in x view
    int   plane2x;   ///< last plane in x view
    int   plane1y;   ///< 1st plane in y view
    int   plane2y;   ///< last plane in y view
    int   cell1x;    ///< 1st cell in x view
    int   cell2x;    ///< last cell in x view
    int   cell1y;    ///< 1st cell in y view
    int   cell2y;    ///< last cell in y view
    float t1;        ///< start time window (usec)
    float t2;        ///< end time window (usec)
    float tave;      ///< time where peak occurs (ave over hits) (usec)
    float trms;      ///< rms width of hits inside of window (usec)
    float cosnumi;   ///< Cosine of angle of track wrt to NuMI beam axis
    float cosbnb;    ///< Cosine of angle of track wrt to BNB beam axis
    float cosdet;    ///< Cosine of theta wrt to detector z-axis
    float phidet;    ///< Azimuthal angle in detector x-y plane
    float coscosmic; ///< Cosine of zenith angle (point back wrt. vertical)
    float phicosmic; ///< Azimuthal angle (point back wrt. vertical)
    float pot;       ///< Proton on target
    float dt;        ///< usec time difference to spill
    int   gbeam;     ///<good beam 
    float x1;        ///< Start position of track (cm)
    float y1;        ///< Start position of track (cm)
    float z1;        ///< Start position of track (cm)
    float x2;        ///< End position of track (cm)
    float y2;        ///< End position of track (cm)
    float z2;        ///< End position of track (cm)
    float dedgef;    ///< distance to detector edge going north along track (cm)
    float dedgeb;    ///< distance to detector edge going south along track (cm)
    int   nunused;   ///< Number of hits in time window but not on the track
    float nux;       ///< MC neutrino interaction x location (cm)
    float nuy;       ///< MC neutrino interaction y location (cm)
    float nuz;       ///< MC neutrino interaction z location (cm)
    float nupx;      ///< MC neutrino x momentum (GeV)
    float nupy;      ///< MC neutrino y momentum (GeV)
    float nupz;      ///< MC neutrino z momentum (GeV)
    float leppx;     ///< MC out going lepton momentum (GeV)
    float leppy;     ///< MC out going lepton momentum (GeV)
    float leppz;     ///< MC out going lepton momentum (GeV)
    int   nupdg;     ///< MC neutrino PDG code
    int   iscc;      ///< 1 = CC interaction, 0 = NC interaction
    int   imode;     ///< interaction mode
    unsigned long long tstart; ///< Time start from raw trigger
    TTimeStamp         utc;    ///< UTC calendar date of event
  public:
    CanaNt(){}
  };
}

#endif
////////////////////////////////////////////////////////////////////////
