////////////////////////////////////////////////////////////////////////
// Class:       HardwareStatusCheck
// Module Type: filter
// File:        HardwareStatusCheck_module.cc
//
// Generated at Thu Nov  7 05:46:29 2013 by Matthew Tamsett using artmod
// from cetpkgsupport v1_04_01.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDFilter.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"

// NOvA includes
#include "RunHistory/RunHistory.h"
#include "Geometry/Geometry.h"
#include "GeometryObjects/CellGeo.h"
#include "RawData/RawDigit.h"
#include "CMap/CMap.h"

// ROOT includes
#include "TH1F.h"
#include "TH2F.h"

#include <memory>
//------------------------------------------------------------------------
namespace DataCheck {
    class HardwareStatusCheck;
}
//------------------------------------------------------------------------
class DataCheck::HardwareStatusCheck : public art::EDFilter {
    public:
        explicit HardwareStatusCheck(fhicl::ParameterSet const & p);
        virtual ~HardwareStatusCheck();
        bool filter(art::Event & evt) override;
        void reconfigure(const fhicl::ParameterSet& pset);
        void beginJob() override;
        bool beginSubRun(art::SubRun & sr) override;
        void endJob() override;
        uint32_t globalDCMNumber(uint32_t diblock, uint32_t dcm);
    private:
        // Declare member data here.
        std::string  fRawDigitModule;               ///< Label of the raw digit module; set to default
        std::vector<uint32_t> fDiblocksToUse;       ///< The diblocks we will check. Don't worry about those with bare FEBs
        unsigned int fTotalDCMsExpected       = 0;
        const unsigned int fExpectedNDCMsPerDiblock = 12; ///< The number of DCMs we expect per diblock in each event
        std::vector<uint32_t> fFoundDCMs;
        std::vector<uint32_t> fExpectedDCMs;
        // histograms
        TH1F*        hNRawDigits;
        TH1F*        hNDCMs;
        TH1F*        hPresentDCMs;
        TH1F*        hMissingDCMs;
};
//------------------------------------------------------------------------
DataCheck::HardwareStatusCheck::HardwareStatusCheck(fhicl::ParameterSet const & pset)
    : fDiblocksToUse(0),
      fFoundDCMs(0),
      fExpectedDCMs(0)
// Initialize member data here.
{
    std::cout << "--- HardwareStatusCheck instantiate" << std::endl;
    // Call appropriate Produces<>() functions here.
    this->reconfigure(pset);
}
//------------------------------------------------------------------------
void DataCheck::HardwareStatusCheck::reconfigure(const fhicl::ParameterSet& pset)
{
    std::cout << "--- HardwareStatusCheck reconfigure" << std::endl;
    fRawDigitModule           = pset.get<std::string>("RawDigitModule", "daq");
    fDiblocksToUse            = pset.get<std::vector<unsigned int> >("DiblocksToUse");
    
    fTotalDCMsExpected = fExpectedNDCMsPerDiblock*fDiblocksToUse.size();
    for (const auto db: fDiblocksToUse){
        for(uint32_t i_dcm =0;i_dcm<fExpectedNDCMsPerDiblock;++i_dcm){
            fExpectedDCMs.push_back(globalDCMNumber(db, i_dcm));
        }
    }
    
    std::cout << "    RawDigitModule:           " << fRawDigitModule            << std::endl;
    std::cout << "    Total DCMs expected:      " << fTotalDCMsExpected         << std::endl;
    std::cout << "    Diblocks to use:          ";
    for (const auto db: fDiblocksToUse) std::cout << " " << db;
    std::cout << std::endl;
    //std::cout << "    abs DCM numbers expected:  " << std::endl;
    //for (const auto i: fExpectedDCMs) std::cout << "        " << i << std::endl;
}
//------------------------------------------------------------------------
DataCheck::HardwareStatusCheck::~HardwareStatusCheck()
{
    // Clean up dynamic memory and other resources here.
    fFoundDCMs.clear();
    fDiblocksToUse.clear();
}
//------------------------------------------------------------------------
bool DataCheck::HardwareStatusCheck::beginSubRun(art::SubRun & sr)
{
    std::cout << "--- HardwareStatusCheck beginSubRun" << std::endl;
    art::ServiceHandle<geo::Geometry> geom;
    novadaq::cnv::DetId detectorid = geom->DetId();
    int fRun = sr.run();
    std::cout << "    detector ID: " << detectorid
              << ", run: "           << fRun
              << std::endl;
    nova::dbi::RunHistory rh(sr.run(), detectorid); 
    rh.LoadDAQRunHistory();
    std::cout << "    loaded run history" << std::endl;
    unsigned int nDB = rh.NDiBlocks();
    std::cout << "    N Di-blocks:   " << nDB << std::endl;
    int nDCMs = rh.NLiveDCMs();
    std::cout << "    N DCMs:        " << nDCMs << std::endl;
    int my_nDCMs = 0;
    int my_active_nDCMs = 0;
    for (unsigned int i=0; i<nDB; ++i) {
        const nova::dbi::RunHistory::DiBlock db = rh.GetDiBlock(i);
        std::vector<nova::dbi::RunHistory::DCM> DCMs =  db.dcm;
        std::cout << "    Diblock[" << i
                  << "] has: "      << DCMs.size()
                  << " DCMs"        << std::endl;
        my_nDCMs+=DCMs.size();
        for (unsigned int j=0; j<DCMs.size(); ++j) {
            if(DCMs[j].isActive){
                my_active_nDCMs++;
                std::vector<nova::dbi::RunHistory::FEB> FEBs = DCMs[j].feb;
                std::cout << "        DCM[" << j
                          << "] has: "      << FEBs.size()
                          << " FEBs"        << std::endl;
                //for (unsigned int k=0; k<FEBs.size(); ++k) {
                    //std::cout << "            FEB[ " << k
                              //<< "], hwid: "         << FEBs[k].hwid
                              //<< ", hasAPD: "        << FEBs[k].hasAPD
                              //<< ", isEnabled: "     << FEBs[k].isEnabled
                              //<< ", isCooled: "      << FEBs[k].isCooled
                              //<< ", tempEnabled: "   << FEBs[k].tempEnabled
                              //<< ", temperature: "   << FEBs[k].temperature
                              //<< ", gain: "          << FEBs[k].gain
                              //<< std::endl;
                //} // end of loop on FEBs
            } // end of loop on active DCMs
        } // end of loop on DCMs
    } // end of loop on diblocks
    std::cout << "    My N DCMs:        " << my_nDCMs << std::endl;
    std::cout << "    My N active DCMs: " << my_active_nDCMs << std::endl;

    //assert(false);
    return true;
}
//------------------------------------------------------------------------
bool DataCheck::HardwareStatusCheck::filter(art::Event & evt)
{
    // Implementation of required member function here.
    //std::cout << "--- HardwareStatusCheck filter. Event: " 
    //          << evt.id().event() 
    //          << std::endl;
    
    // Load services
    art::ServiceHandle<cmap::CMap> cmap;
    
    // Load hit list (RawDigits)
    art::Handle< std::vector<rawdata::RawDigit> > rawdigitcol;
    evt.getByLabel(fRawDigitModule, rawdigitcol);
    //std::cout << "    got " << rawdigitcol->size() << " RawDigits" << std::endl;
    
    fFoundDCMs.clear();
    uint32_t daq_channel = 0;
    uint32_t dcm         = 0;
    uint32_t diblock     = 0;
    uint32_t abs_dcm     = 0; // there are 12 DCMs per diblock
    // Loop over raw digits
    for (size_t i_digit = 0; i_digit < rawdigitcol->size(); ++i_digit){
        daq_channel = rawdigitcol->at(i_digit).DaqChannel();
        
        diblock = cmap->Map()->getDiBlock(daq_channel);
        // are we using this diblock?
        if(std::find(fDiblocksToUse.begin(), fDiblocksToUse.end(), diblock)==fDiblocksToUse.end()){
            continue;
        }
        
        dcm     = cmap->Map()->getDCM(daq_channel);
        abs_dcm = globalDCMNumber(diblock, dcm);
        // has this dcm been found before?
        if(std::find(fFoundDCMs.begin(), fFoundDCMs.end(), abs_dcm)==fFoundDCMs.end()){
            fFoundDCMs.push_back(abs_dcm);
        }
    } // end of loop over raw digits
    //std::cout << "    found " << fFoundDCMs.size() << " DCMs" << std::endl;

    // fill monitoring histograms
    hNRawDigits->Fill(rawdigitcol->size());
    hNDCMs->Fill(fFoundDCMs.size());    
    // all present DCMs
    for (auto const n_dcm: fFoundDCMs){
         hPresentDCMs->Fill(n_dcm);
    }
    // only fill missing DCMs if there are any missing
    if (fFoundDCMs.size()!=fTotalDCMsExpected){
        for (auto const n_dcm: fExpectedDCMs){
            if(std::find(fFoundDCMs.begin(), fFoundDCMs.end(), n_dcm)==fFoundDCMs.end()){
                hMissingDCMs->Fill(n_dcm);
            }
        }
    }
    
    return (fFoundDCMs.size()==fTotalDCMsExpected);
}
//------------------------------------------------------------------------
uint32_t DataCheck::HardwareStatusCheck::globalDCMNumber(uint32_t diblock, uint32_t dcm)
{
    return dcm+fExpectedNDCMsPerDiblock*diblock;
}
//------------------------------------------------------------------------
void DataCheck::HardwareStatusCheck::beginJob()
{
    // Implementation of optional member function here.
    std::cout << "--- HardwareStatusCheck beginJob" << std::endl;
    
    art::ServiceHandle<art::TFileService> f;
    hNRawDigits = f->make<TH1F>("hNRawDigits",  ";NRawDigits;Events",   100,-0.5,10000.5);
    hNDCMs      = f->make<TH1F>("hNDCMs",       ";NDCMs;Events",        169,-0.5,168.5);
    hPresentDCMs= f->make<TH1F>("hPresentDCMs", ";DCM number + 12#timesdiblock number;Events", 168,-0.5,167.5);
    hMissingDCMs= f->make<TH1F>("hMissingDCMs", ";DCM number + 12#timesdiblock number;Events", 168,-0.5,167.5);
}
//------------------------------------------------------------------------
void DataCheck::HardwareStatusCheck::endJob()
{
    // Implementation of optional member function here.
    std::cout << "--- HardwareStatusCheck endJob" << std::endl;
}
//------------------------------------------------------------------------
DEFINE_ART_MODULE(DataCheck::HardwareStatusCheck)
