#ifndef DAQ2RAWDIGIT_NOVARAWINPUTDRIVER_H
#define DAQ2RAWDIGIT_NOVARAWINPUTDRIVER_H

////////////////////////////////////////////////////////////////////////
/// \file  NOvA.h
/// \brief Source to convert raw binary files to root files
///
/// \version $Id: NOvARawInputDriver.h,v 1.7 2012-11-18 18:52:46 gsdavies Exp $
/// \author  brebel@fnal.gov, soderber@fnal.gov, greenc@fnal.gov, paterno@fnal.gov
////////////////////////////////////////////////////////////////////////

#include "DAQ2RawDigit/NOvAInputDriverBase.h"
#include "RawFileParser/RawFileParser.h"

#include "art/Persistency/Provenance/SubRunID.h"
#include "art/Framework/IO/Sources/SourceHelper.h"

#include <string>

// forward declarations
namespace daq2raw {
  class NOvARawInputDriver;
}

namespace fhicl{
    class ParameterSet;
}

namespace art{
    class ProductRegistryHelper;
    class RunPrincipal;
    class SubRunPrincipal;
    class EventPrincipal;
    class FileBlock;
}


class daq2raw::NOvARawInputDriver: public daq2raw::NOvAInputDriverBase{
  /// Class to fill the constraints on a template argument to the class,
  /// art::Source
public:
  // Required constructor
  NOvARawInputDriver(fhicl::ParameterSet const &pset,
                     art::ProductRegistryHelper &help,
                     art::SourceHelper const &pm);

  // Required by art::Source:
  void closeCurrentFile();
  void readFile(std::string const &name,
                art::FileBlock* &fb);
  bool readNext(art::RunPrincipal* const &inR,
                art::SubRunPrincipal* const &inSR,
                art::RunPrincipal* &outR,
                art::SubRunPrincipal* &outSR,
                art::EventPrincipal* &outE);

private:
  art::SourceHelper          fSourceHelper;           ///< Class to help with {Run, SubRun, Event}Principal construction.
  std::string                  fCurrentFilename;          ///< Hang on to the current filename.
  art::SubRunID                fCurrentSubRunID;          ///< Keep track of the current subRun details.
  rawfileparser::RawFileParser fParser;                   ///< Online raw file parsing utility.
  size_t                       fMaxEventSize;             ///< Will not try to read an event bigger than this.
  bool                         fFilterCorruptedNanoslices;///< Filter the corrupted nanaslices in the unpacking?
  bool                         fDoFillFlatDAQData;        ///< Do we need to fill FlatDAQData object?
};  // NOvARawInputDriver

#endif /* DAQ2RAWDIGIT_NOVARAWINPUTDRIVER_H */
// Local Variables:
// mode: c++
// End:

