///////////////////////////////////////////////////////////////////////
// Class:       RawDigit2DAQHit
// Module Type: producer
// File:        RawDigit2DAQHit_module.cc
//
// Generated at Mon Apr 29 07:46:14 2013 by Matthew Tamsett using artmod
// from art v1_02_06.
////////////////////////////////////////////////////////////////////////
#include "art/Framework/Core/EDProducer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"

#include "CMap/CMap.h"
#include "DAQChannelMap/DAQChannelMap.h"
#include "DDTBaseDataProducts/DAQHit.h"
#include "DDTBaseDataProducts/BaseProducts.h"
#include "NovaDAQConventions/DAQConventions.h"
#include "RawData/RawDigit.h"
#include "RawData/RawTrigger.h"
#include "RunHistory/service/DetectorService.h"
#include "SummaryData/RunData.h"
#include "DDTCore/WaveformProcessor.h"

#include <string>
#include <vector>

namespace raw2daq {
  class RawDigit2DAQHit;
}

// using unsigned long long rather than uint64_t to ensure consistent behavior on OSX and LINUX

//--------------------------------------------------------------------
class raw2daq::RawDigit2DAQHit : public art::EDProducer {
public:
  explicit RawDigit2DAQHit(fhicl::ParameterSet const & p);
  virtual ~RawDigit2DAQHit();
  void produce(art::Event & event) override;

private:
  std::string fRawDigitLabel;  ///< label of where to find Raw Digits
  art::ServiceHandle<novaddt::WaveformProcessor> wpr;

protected:
  void FillDAQHits(art::Handle< std::vector<rawdata::RawDigit> > & RawDigits,
                   std::unique_ptr<std::vector<novaddt::DAQHit> > & DAQHits,
                   unsigned long long event_time
                   );
};
//---------------------------------------------------------------------
raw2daq::RawDigit2DAQHit::RawDigit2DAQHit(fhicl::ParameterSet const & p)
    : fRawDigitLabel  (p.get< std::string >("RawDigitLabel"))
{
  // Call appropriate Produces<>() functions here.
  mf::LogInfo("RawDigit2DAQHit") << "=== raw2daq:::RawDigit2DAQHit instantiate";
  mf::LogInfo("RawDigit2DAQHit") << "\t RawDigitLabel: " << fRawDigitLabel;
  produces< std::vector<novaddt::DAQHit>,   art::InEvent>();

}
//---------------------------------------------------------------------
raw2daq::RawDigit2DAQHit::~RawDigit2DAQHit()
{
  // Clean up dynamic memory and other resources here.
}
//---------------------------------------------------------------------
void raw2daq::RawDigit2DAQHit::produce(art::Event & event)
{
  // Implementation of required member function here.
  // get the RawDigit list
  art::Handle< std::vector<rawdata::RawDigit> > RawDigits;
  event.getByLabel(fRawDigitLabel, RawDigits);
  // make a vector to store the DAQHits into
  std::unique_ptr<std::vector<novaddt::DAQHit> > DAQHits(new std::vector<novaddt::DAQHit>);
  
  // Get the raw trigger to get the event time
  art::Handle< std::vector<rawdata::RawTrigger> > RawTriggers; 
  event.getByLabel(fRawDigitLabel, RawTriggers);    
  // Currently no triggers should be configured to have more than one raw trigger per event
  // The vector nature is reserved for exotic supernovae triggers
  assert(RawTriggers->size()==1);
  unsigned long long event_time = RawTriggers->at(0).fTriggerTimingMarker_TimeStart;
  
  // Make a DAQHit for every RawDigit and write it out
  FillDAQHits(RawDigits, DAQHits, event_time);
  // add DAQHits into the event
  event.put(std::move(DAQHits));
}
//---------------------------------------------------------------------
void raw2daq::RawDigit2DAQHit::FillDAQHits(art::Handle< std::vector<rawdata::RawDigit> > & RawDigits,
                                           std::unique_ptr<std::vector<novaddt::DAQHit> > & DAQHits,
                                           unsigned long long event_time
                                           )
{
  const unsigned int nDigits = RawDigits->size();
  art::ServiceHandle<cmap::CMap> cmap;

  art::ServiceHandle<rh::DetectorService> rh;
  const novadaq::cnv::DetId det = rh->DetId();

  // loop over digits
  for(unsigned int iDigit=0; iDigit < nDigits; ++iDigit){
    const rawdata::RawDigit* digit = &(*RawDigits)[iDigit];
    const daqchannelmap::lchan channel       = digit->Channel();
    const daqchannelmap::dchan daqchannel    = cmap->Map()->encodeDChan(channel);

    if (digit->NADC() < 4) // use DCS
      DAQHits->emplace_back(det,
			    daqchannel,
			    novaddt::ADC(digit->ADC()),
			    novaddt::TDC(digit->TDC() + event_time));

    else { // using WaveformProcessor
      novaddt::TDC theTDC;
      novaddt::ADC theADC;
      int16_t adc1 = digit->ADC(1) - digit->ADC(0);
      int16_t adc2 = digit->ADC(2) - digit->ADC(0);
      int16_t adc3 = digit->ADC(3) - digit->ADC(0);

      
      //
      // Time Conversion: Offline TDC -> Raw TDC
      //
      if (event_time > std::numeric_limits<int64_t>::max())
      {
	std::cerr << "\n\tThe event time is too large to fit into int64_t.\n"
		  << std::endl;
	assert(false);
      }

      int32_t offline_tdc = digit->TDC();
      // This casting of uint64 to int64 is okay because our event time only
      // goes up to 56 bits and we checked above.
      int64_t integer_raw_tdc =
	static_cast<int64_t>(event_time) + static_cast<int64_t>(offline_tdc);
      unsigned long long raw_tdc      = static_cast<unsigned long long>(integer_raw_tdc);
      uint32_t raw_tdc_low  = raw_tdc & 0x00000000FFFFFFFF;
      unsigned long long raw_tdc_high = raw_tdc >> 32;


      wpr->setup(raw_tdc_low, raw_tdc_high, adc1, adc2, adc3, digit->ADC());
      
      DAQHits->emplace_back(det,
                            daqchannel,
			    wpr->getADC(),
                            wpr->getTDC());
    }
  } // end of loop on raw digits
}

//---------------------------------------------------------------------
DEFINE_ART_MODULE(raw2daq::RawDigit2DAQHit)
