#include "DDTTools/DDTHelpers.h"

#include "Geometry/Geometry.h"
#include "GeometryObjects/CellGeo.h"

#include <art/Framework/Services/Registry/ServiceHandle.h>

#include <boost/format.hpp>



rb::Cluster ddthelpers::cluster_hits
(std::vector<art::Ptr<rb::CellHit> > const& hits,
 int32_t t_start, int32_t t_end, int16_t adc_cut)
{
  rb::Cluster result;

  for (auto const& hit : hits)
    if (hit->TDC() >= t_start &&
	hit->TDC() <= t_end   &&
	hit->ADC() >= adc_cut)
      result.Add(hit);

  return result;
}



bool ddthelpers::is_equal
(novaddt::DAQHit const& ddthit, art::Ptr<rb::CellHit> const& cellhit,
 novaddt::TDC const& event_time)
{
  int64_t ddt_time = ddthit.TDC().val - event_time;

  if (ddt_time != cellhit->TDC())
    return false;
  
  if (ddthit.ADC().val != cellhit->ADC())
    return false;

  if (ddthit.Plane().val != cellhit->Plane())
    return false;

  if (ddthit.Cell().val != cellhit->Cell())
    return false;

  return true;
}



bool ddthelpers::is_equal
(novaddt::DAQHit const& ddthit, rawdata::RawDigit const& digit,
 novaddt::TDC const& event_time)
{
  int64_t ddt_time = ddthit.TDC().val - event_time;

  if (ddt_time != digit.TDC())
    return false;
  
  if (ddthit.ADC().val != digit.ADC(0))
    return false;

  return true;
}



bool ddthelpers::is_equal
(novaddt::DAQHit const& lhs, novaddt::DAQHit const& rhs)
{
  if (lhs.TDC().val != rhs.TDC().val)
    return false;

  if (lhs.ADC().val != rhs.ADC().val)
    return false;

  return true;
}



void ddthelpers::print
(novaddt::DAQHit const& h, unsigned long long const& event_time)
{
  int64_t relative_time =
    static_cast<int64_t>(h.TDC().val) - static_cast<int64_t>(event_time);

  double time_in_us = static_cast<double>(relative_time) / 64.0e6 * 1.0e6;
  
  std::string view;
  if (h.View().val == daqchannelmap::DetView_TYPE::X_VIEW)
    view = "xz";
  else if (h.View().val == daqchannelmap::DetView_TYPE::Y_VIEW)
    view = "yz";
  else
    view = "bad";

  art::ServiceHandle<geo::Geometry> geom;
  TVector3 position;
  geom->Plane(h.Plane().val)->Cell(h.Cell().val)->GetCenter(position);
  
  boost::format form("(%2s, %3u, %3u) (%6.2f, %6.2f, %6.2f) "
		     "%5u ADC %20u TDC = %6.2f us");
  std::cout << 
    form % view % h.Plane().val % h.Cell().val %
    position.X() % position.Y() % position.Z() %
    h.ADC().val % relative_time % time_in_us
	    << std::endl;
}



void ddthelpers::print(art::Ptr<rb::CellHit> const& h)
{
  boost::format form("%7s %4u %4u %5u %20u");

  std::string view;
  if (h->View() == geo::View_t::kX)
    view = "xz";
  else if (h->View() == geo::View_t::kY)
    view = "yz";
  else
    view = "bad";

  std::cout << 
    form % view % h->Plane() % h->Cell() % 
    h->ADC() % h->TDC()
	    << std::endl;
}



void ddthelpers::print(rawdata::RawDigit const& h)
{
  boost::format form("%7s %4u %4u %5u %20u");

  std::cout << 
    form % "?" % 0 % 0 % 
    h.ADC(0) % h.TDC()
	    << std::endl;
}



void ddthelpers::print(std::vector<novaddt::TriggerDecision> const& decisions)
{
  boost::format form("%20u %6u %6u");

  for (auto const& d : decisions)
    std::cout << 
      form % d.start() % (d.start() + d.duration()) % d.duration()
	      << std::endl;
}
