#include "DQSpillFlags/DAQEventUnpack.h"
#include "DAQDataFormats/RawEvent.h"
#include "DAQDataFormats/RawTrigger.h"
#include "DAQDataFormats/RawTriggerMask.h"
#include "DAQDataFormats/RawDataBlock.h"
#include "DAQDataFormats/RawMicroBlock.h"
#include "DAQDataFormats/RawMicroSlice.h"
#include "DAQDataFormats/RawNanoSlice.h"
#include "DAQChannelMap/DAQChannelMap.h"
#include "DQSpillFlags/DAQEventSummary.h"

using namespace dqsf;

static const double kUSEC_PER_TDC    = (1.0E6/64.0E6);
static const unsigned int kADC_ERROR = 0xFFFFFFFF;

DAQEventUnpack::DAQEventUnpack() { }

//......................................................................

DAQEventUnpack::~DAQEventUnpack() { }

//......................................................................

void DAQEventUnpack::UnpackRawEvent(daqdataformats::RawEvent& rawevt,
				    dqsf::DAQEventSummary&  daqsum)
{
  // ================================================
  // Do whatever analysis you want on the raw event
  // ================================================
  if (rawevt.getHeader()->isEventIncomplete()) daqsum.fEventIncomplete = true;
  unsigned int nDataBlock = rawevt.getDataBlockNumber();

  //
  // Now unpack the associated data blocks
  //
  unsigned register int idata;
  for (idata = 0; idata < nDataBlock; ++idata) {
    this->UnpackDataBlock(rawevt, idata, daqsum);
  }
}

//......................................................................

void DAQEventUnpack::UnpackDataBlock(daqdataformats::RawEvent& rawevt,
				     unsigned int            idata,
				     dqsf::DAQEventSummary&  daqsum)
{
  rawevt.setFloatingDataBlock(idata);
  daqdataformats::RawDataBlock& datablock = (*rawevt.getFloatingDataBlock());

  // =====================================================
  // Do whatever analysis you want on the data block here
  // =====================================================
  if (datablock.isMissingData()) ++daqsum.fNdataBlockMissingData;

  unsigned int nMicroBlock = datablock.getNumMicroBlocks();
  if (nMicroBlock == 0) ++daqsum.fNemptyDataBlock;

  //
  // Now unpack the associated micro slices
  //
  unsigned register int imicro;
  for (imicro = 0; imicro < nMicroBlock; ++imicro) {
    this->UnpackMicroSlice(datablock, imicro, daqsum);
  }
}

//......................................................................

void DAQEventUnpack::UnpackMicroSlice(daqdataformats::RawDataBlock& datablock,
				      unsigned int            imicro,
				      dqsf::DAQEventSummary&  daqsum)
{
  datablock.setFloatingMicroBlock(imicro);

  daqdataformats::RawMicroBlock&
    microblock = (*datablock.getFloatingMicroBlock());

  daqdataformats::RawMicroSlice&
    microslice = (*microblock.getMicroSlice());

  // =====================================================
  // Do whatever analysis you want on the micro slice here
  // =====================================================
  if (!microslice.getDataPresent()) ++daqsum.fNmicroSliceDataNotPresent;
  unsigned int nNanoSlice = microslice.getNumNanoSlices();
  if (nNanoSlice == 0) ++daqsum.fNemptyMicroSlice;
  daqsum.fNtotalNanoSlices += nNanoSlice;

  //
  // Now unpack all the associated nano slices
  //
  unsigned register int inano;
  for (inano = 0; inano < nNanoSlice; ++inano) {
    this->UnpackNanoSlice(microslice, inano, daqsum);
  }
}

//......................................................................

void DAQEventUnpack::UnpackNanoSlice(daqdataformats::RawMicroSlice& microslice,
				     unsigned int            inano,
				     dqsf::DAQEventSummary&  daqsum)
{
  microslice.setFloatingNanoSlice(inano);

  daqdataformats::RawNanoSlice&       nanoslice    = (*microslice.getFloatingNanoSlice());
  daqdataformats::RawNanoSliceHeader& nanoslicehdr = (*nanoslice.getHeader());

  // =====================================================
  // Do whatever analysis you want on the nano slices here
  // =====================================================
  if (!nanoslicehdr.getDataPresent())   ++daqsum.fNanoSliceDataNotPresent;
  if (!nanoslicehdr.getLinkStatus())    ++daqsum.fNanoSliceNoLinkStatus;
  if ( nanoslicehdr.getBufferEmpty())   ++daqsum.fNanoSliceBufferEmpty;
  if ( nanoslicehdr.getBufferFull())    ++daqsum.fNanoSliceBufferFull;
  if ( nanoslicehdr.getCommError())     ++daqsum.fNanoSliceCommError;
  if ( nanoslicehdr.getPacketError())   ++daqsum.fNanoSlicePackerError;
  if ( nanoslicehdr.getOverflowError()) ++daqsum.fNanoSliceOverflowError;
  if ( nanoslicehdr.getADCError())      ++daqsum.fNanoSliceADCError;
}

////////////////////////////////////////////////////////////////////////
