////////////////////////////////////////////////////////////////////////
// \file    Segment.h
// \version $Id: Segment.h,v 1.6 2012-10-30 23:33:09 bckhouse Exp $
// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef DTSEGMENT_H
#define DTSEGMENT_H

#include <ostream>

#include "DiscreteTracker/Types.h"

namespace dt
{
  /// Just the upstream or downstream part, just coordinates
  struct SubSeg
  {
    SubSeg(double a, double b, double c) : z(a), y0(b), y1(c) {}
    double z, y0, y1;
  };

  /// Window the line must pass through from (z,y0)-(z,y1)
  struct Segment
  {
    Segment(){}
    Segment(double a, double b, double c,
            double d, double e, double f,
            bool l, int p, int cl, int ch)
      : zL(a), yL0(b), yL1(c),
        zR(d), yR0(e), yR1(f),
        left(l), plane(p), cellLo(cl), cellHi(ch)
    {
    }
    bool operator==(const Segment& rhs) const;
    bool operator!=(const Segment& rhs) const {return !(*this==rhs);}
    bool operator<(const Segment& rhs) const {return zL < rhs.zL;}
    SubSeg GetSubSeg(Direction dir) const;
    double zL, yL0, yL1;
    double zR, yR0, yR1;
    bool left;
    int plane, cellLo, cellHi;
  };

  std::ostream& operator<<(std::ostream& os, const Segment& seg);
} // end namespace

#endif // DTSEGMENT_H
