////////////////////////////////////////////////////////////////////////
/// \file    HitEva.cxx
/// \brief   TODO
/// \version $Id: HitEva.cxx,v 1.7 2012-09-25 06:30:12 ricken Exp $
/// \author  ricken@fnal.gov
////////////////////////////////////////////////////////////////////////
#include <cmath>
#include <vector>
#include <string>

// ROOT includes
#include "TH1I.h"
#include "TNtuple.h"
#include "TBenchmark.h"

// NOvA includes
#include "Calibrator/Calibrator.h"
#include "GeometryObjects/CellGeo.h"
#include "Geometry/Geometry.h"
#include "GeometryObjects/PlaneGeo.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/Cluster.h"
#include "RecoBase/RecoHit.h"
#include "MCCheater/BackTracker.h"
#include "Simulation/ParticleNavigator.h"

// Framework includes
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"

class TTree;
class TVector3;

namespace geo { class Geometry; }
namespace rb  { class CellHit; class Cluster; }

///tracking algorithms
namespace pa {
  class HitEva : public art::EDAnalyzer {
  public:
    explicit HitEva(fhicl::ParameterSet const& pset);
    virtual ~HitEva();

    void beginJob();
    void analyze(const art::Event& evt);
    void reconfigure(const fhicl::ParameterSet& p);
    
  private:
    std::string fCellHitInput;
    TTree* HitTruth;
    bool _IsNoise, _TagAsNoise;
    int  ph_DCS,   ph_MCS;
    int  tdc_DCS,  tdc_MCS;
    
  };
}

namespace pa{  
  //......................................................................
  HitEva::HitEva(fhicl::ParameterSet const& pset)
    : EDAnalyzer(pset)
  {
    reconfigure(pset);
  }
  
  //......................................................................
  HitEva::~HitEva()
  {
  }
  
  //......................................................................
  void HitEva::reconfigure(const fhicl::ParameterSet& pset)
  {
    fCellHitInput    = pset.get< std::string >("CellHitInput");
  }

  //......................................................................
  void HitEva::beginJob()
  {
    art::ServiceHandle<art::TFileService> tfs;
    HitTruth = tfs->make<TTree>("HitTruth","");

    HitTruth -> Branch("isNoise",    &_IsNoise,      "isNoise/O");
    HitTruth -> Branch("tagAsNoise", &_TagAsNoise,   "tagAsNoise/O");
    HitTruth -> Branch("ph_DCS",     &ph_DCS,        "ph_DCS/I");
    HitTruth -> Branch("ph_MCS",     &ph_MCS,        "ph_MCS/I");
    HitTruth -> Branch("tdc_DCS",    &tdc_DCS,       "tdc_DCS/I");
    HitTruth -> Branch("tdc_MCS",    &tdc_MCS,       "tdc_MCS/I");

  }

  void HitEva::analyze(const art::Event& evt) 
  {
    //make a backtracker
    art::ServiceHandle<cheat::BackTracker> bt;

    art::Handle< std::vector<rb::CellHit> > hitcol;
    evt.getByLabel(fCellHitInput, hitcol);

    for (unsigned i = 0; i!= hitcol->size(); ++i) {
      art::Ptr<rb::CellHit> hit_ptr(hitcol, i);
      _IsNoise = bt->IsNoise(hit_ptr);
      ph_DCS   = hit_ptr->ADC();
      ph_MCS   = ph_DCS;
      tdc_DCS = hit_ptr->TDC();
      tdc_MCS = tdc_DCS;
      _TagAsNoise = false;

      //Start MCS algorithm:
      bool timeshifted = false;
      int ADC_sigma = 0;
      for (int j = 3; j!=0; --j) {
	if (hit_ptr->ADC(j) < hit_ptr->ADC(j-1)) {
	  if (!timeshifted) {
	    tdc_MCS = tdc_MCS+32*j;
	    timeshifted = true;
	  }
	  if (hit_ptr->ADC(j-1) - hit_ptr->ADC(j) >ADC_sigma)
	    ADC_sigma = hit_ptr->ADC(j-1) - hit_ptr->ADC(j);
	}

      }

      if (ph_MCS > 5*ADC_sigma ) {
	if (ph_MCS<hit_ptr->ADC(3) - hit_ptr->ADC(1)) 
	  ph_MCS =  hit_ptr->ADC(3) - hit_ptr->ADC(1);
	if (ph_MCS<hit_ptr->ADC(2) - hit_ptr->ADC(0)) 
	  ph_MCS =  hit_ptr->ADC(2) - hit_ptr->ADC(0);
      }

      else if (timeshifted) 
	_TagAsNoise = true;

      HitTruth->Fill();
    }


    return;

  }   //end analyze

  DEFINE_ART_MODULE(HitEva)

}
