#include <TH1.h>
#include <TH2.h>
#include <TTree.h>
#include "TAxis.h"
#include <TLegend.h>
#include <TFile.h>
#include <TProfile.h>

#include <fstream>
#include <iostream>
#include <cmath>

//This is following the SupperDDTEva convention 
void readTree()
{
  /*
  std::ofstream ofs;
  ofs.open ("trigger_eff.dat", std::ofstream::out | std::ofstream::app);
  */
  TFile* infile = new TFile("test.root","READ");
  TFile* outfile = new TFile("newplots.root","recreate");

  TTree* MC_Tree = (TTree*)infile->Get("superddteva/MCTruth");
  int n_MC_hits,evt_mc, nslice, ci,co, pi,po;
  float beta_MC;
  double edep, pathL;

  MC_Tree->SetBranchAddress("nhits",&n_MC_hits);
  MC_Tree->SetBranchAddress("Beta",&beta_MC);
  MC_Tree->SetBranchAddress("evtID",&evt_mc);
  MC_Tree->SetBranchAddress("nslice",&nslice);
  MC_Tree->SetBranchAddress("PathLength",&pathL);
  MC_Tree->SetBranchAddress("EnergyDump",&edep);
  MC_Tree->SetBranchAddress("Cell_in",&ci);
  MC_Tree->SetBranchAddress("Cell_out",&co);
  MC_Tree->SetBranchAddress("Plane_in",&pi);
  MC_Tree->SetBranchAddress("Plane_out",&po);

  TTree* RC_Tree = (TTree*)infile->Get("superddteva/RCInfo");
  int n_true_hits, n_hits, n_sh, evt;
  float sumADC,beta,cosz; 
  RC_Tree->SetBranchAddress("Beta",&beta);
  RC_Tree->SetBranchAddress("CosZ",&cosz);
  RC_Tree->SetBranchAddress("SumADC",&sumADC);
  RC_Tree->SetBranchAddress("nFLShits",&n_true_hits);
  RC_Tree->SetBranchAddress("nhits",&n_hits);
  RC_Tree->SetBranchAddress("NSH",&n_sh);
  RC_Tree->SetBranchAddress("eventID",&evt);

  TProfile* p_beta_ADC  = new TProfile("beta_ADC","#beta Vs. Mean Slice ADC", 1000, 0, 0.4, 200,4100);
  TProfile* p_beta_st   = new TProfile("beta_st","#beta Vs. Number of Slices Contain Hits From Same MM", 1000, 0, 0.4, 0,5);

  TH1F*     beta_ceff   = new TH1F("beta_ceff", "", 100, 0, 0.04);
  TH1F*     beta_teff   = new TH1F("beta_teff", "", 100, 0, 0.04);
  TH1F*     h_beta      = new TH1F("beta", "", 100, 0, 0.04);

  TH1F*     h_missing   = new TH1F("beta_missing", "", 100, 0, 0.04); 

  TH2F*     M_edepVsnh  = new TH2F("eVn","",100,0,10,30,0.5,30.5);

  TH2F*     CP_bound    = new TH2F("cp","",1000,-50.5,949.5,500,-50.5,449.5);
   

  int nMCevts = 0;
  int evt_mc_tmp = -1;
  for (int i =0; i!=MC_Tree->GetEntries();++i) {
    MC_Tree->GetEntry(i);

    CP_bound->Fill(pi,ci);
    CP_bound->Fill(po,co);

    if (n_MC_hits<1) continue;
    if (evt_mc_tmp==evt_mc) std::cout<<"Oh?! MC duplicated event entry!"<<std::endl;

    h_beta->Fill(beta_MC);
    if (!nslice) {
      h_missing->Fill(beta_MC);
      M_edepVsnh->Fill(edep,n_MC_hits);
    }
    ++nMCevts;
    evt_mc_tmp = evt_mc;
  }

  std::cout<<"number of real MM evts: "<<nMCevts<<std::endl;

  int evt_tmp   = -1;
  int sliceTime = 1;
  int dup_time=0;
  int rc_evts=0;

  //  bool found     = false;

  for (int k = 0; k!=RC_Tree->GetEntries();++k) {
    RC_Tree->GetEntry(k);
    if (n_true_hits<1) continue;
    p_beta_ADC->Fill(beta,sumADC/(n_hits+0.),1);
    if (evt_tmp==evt) {
      ++sliceTime;
      ++dup_time;
    }
    else {
      ++rc_evts;
      beta_ceff ->Fill(beta);
      if (sumADC> 2500*n_hits && n_sh > 6) {
	beta_teff->Fill(beta);
      }
      sliceTime = 1;
      evt_tmp = evt;
    }    
    p_beta_st->Fill(beta,sliceTime,1);
  }
  std::cout<<"RC Events: "<<rc_evts<<std::endl;
  std::cout<<"duplicated Slicing time: "<<dup_time<<std::endl;

  for (int bin =1; bin!=1001; ++bin) {
    float tot = h_beta->GetBinContent(bin);

    float ce = 1;
    float te = 1;
    float mi = 0;
    float ce_err = 0;
    float te_err = 0;
    float mi_err = 0;
 
    if (tot) {
      ce  = beta_ceff->GetBinContent(bin)/tot;
      te  = beta_teff->GetBinContent(bin)/tot;
      mi  = h_missing->GetBinContent(bin)/tot;
      ce_err = sqrt(ce*(1-ce)/tot);
      te_err = sqrt(te*(1-te)/tot);
      mi_err = sqrt(mi*(1-mi)/tot);
    }

    if (ce>1 || te>1) std::cout<<"You have some error!"<<std::endl;

    beta_ceff -> SetBinContent(bin,ce);
    beta_ceff -> SetBinError(bin,ce_err);

    beta_teff -> SetBinContent(bin,te);
    beta_teff -> SetBinError(bin,te_err);

    h_missing -> SetBinContent(bin,mi);
    h_missing -> SetBinError(bin,mi_err);
  }

  h_beta->Write();
  beta_ceff->Write();
  beta_teff->Write();
  p_beta_ADC->Write();
  p_beta_st->Write();
  h_missing->Write();
  M_edepVsnh->Write();
  CP_bound -> Write();
  outfile->Close();
  infile ->Close();
}
   
 
