///
/// \file    HardwareView.h
/// \brief   A view showing hits by hardware address
/// \author  messier@indiana.edu
/// \version $Id: HardwareView.h,v 1.1 2011-03-11 23:13:27 messier Exp $
///
#ifndef EVD_HARDWAREVIEW_H
#define EVD_HARDWAREVIEW_H
#include "EventDisplayBase/Canvas.h"
#include "RQ_OBJECT.h"

namespace evd {
  class TQPad;
  class HeaderPad;
  class HardwarePad;

  /// View of event 
  class HardwareView : public evdb::Canvas {
  public:
    RQ_OBJECT("evd::HardwareView");
  public:
    HardwareView(TGMainFrame* mf);
    ~HardwareView();
    
    const char* Description() const { return "Hardware View"; }
    const char* PrintTag()    const { return "hardware";      }
    void Draw(const char* opt="");

  private:
    HeaderPad*   fHeaderPad;   ///< Show header information
    TQPad*       fRawQ;        ///< Histogram of raw charges
    TQPad*       fRawT;        ///< Histogram of raw times
    TQPad*       fCalQ;        ///< Histogram of calibrated charges
    TQPad*       fCalT;        ///< Histogram of calibrated times
    HardwarePad* fHardwarePad; ///< Hits by hardware address
  };
}
#endif
////////////////////////////////////////////////////////////////////////
