/////////////////////////////////////////////////////////////////////////
/// \file    MCBrief.cxx
/// \brief   Drawing pad for time or charge histograms
/// \author  messier@indiana.edu
/// \version $Id: MCBriefPad.cxx,v 1.6 2011-12-17 04:44:32 messier Exp $
////////////////////////////////////////////////////////////////////////
#include "TBox.h"
#include "TH1F.h"
#include "TPad.h"

#include "EventDisplay/MCBriefPad.h"
#include "EventDisplay/SimulationDrawer.h"
#include "EventDisplayBase/View2D.h"
#include "EventDisplayBase/EventHolder.h"

namespace evd
{
  //......................................................................
  MCBriefPad::MCBriefPad(const char* nm, const char* ti,
			 double x1, double y1,
			 double x2, double y2,
			 const char* /*opt*/) :
    DrawingPad(nm, ti, x1, y1, x2, y2)
  {
    this->Pad()->cd();

    fView = new evdb::View2D();
  }

  //......................................................................

  MCBriefPad::~MCBriefPad() 
  {
    if (fView) { delete fView; fView = 0; }
  }

  //......................................................................

  void MCBriefPad::Draw() 
  {
    fView->Clear();
 
    const art::Event *evt = evdb::EventHolder::Instance()->GetEvent();
    if(evt){
      this->SimulationDraw()->MCTruthShortText(*evt, fView);
      this->SimulationDraw()->MCTruthLongText(*evt, fView);
    }
    
    fPad->cd();
    this->Pad()->Clear();
    fView->Draw();
  }
} // end namespace evd
//////////////////////////////////////////////////////////////////////////
