/// \file    RecoBaseDrawer.h
/// \brief   Class to aid in the rendering of RecoBase objects
/// \author  messier@indiana.edu
/// \version $Id: RecoBaseDrawer.h,v 1.15 2012-08-09 23:38:31 bckhouse Exp $
#ifndef EVD_RECOBASEDRAWER_H
#define EVD_RECOBASEDRAWER_H
#include <vector>

class TH1F;
namespace art      { class Event;    }
namespace evdb     { class View2D;   }
namespace evdb     { class View3D;   }
namespace geo      { class Geometry; class OfflineChan; }
namespace rb { 
  class CellHit;     
  class Cluster;
  class HoughResult;
  class Vertex;
  class Prong;       
  class Track;       
  class Shower;      
}

namespace evd {
  
  /// Aid in the rendering of RecoBase objects
  class RecoBaseDrawer {
  public:
    RecoBaseDrawer();
    ~RecoBaseDrawer();
    
  public:
    void CellHit2D(art::Event const& evt,
		   evdb::View2D*     xzview,
		   evdb::View2D*     yzview);

    void Vertex2D(art::Event const& evt, 
		  evdb::View2D*     xzview, 
		  evdb::View2D*     yzview);

    void Cluster2D(art::Event const& evt, 
		   evdb::View2D*     xzview, 
		   evdb::View2D*     yzview);

    void OfflineChans2D(const art::Event& evt,
                        evdb::View2D*     xzview,
                        evdb::View2D*     yzview);

    void HoughResult2D(art::Event const& evt,
                       evdb::View2D*     xzview,
                       evdb::View2D*     yzview);

    void Prong2D(art::Event const& evt,
		 evdb::View2D*     xzview,
		 evdb::View2D*     yzview);

    void Track2D(const art::Event& evt,
		 evdb::View2D*     xzview,
		 evdb::View2D*     yzview);


    void CellHit3D(art::Event const& evt,
		   evdb::View3D*     view);

    void Cluster3D(art::Event const& evt,
		   evdb::View3D*     view);

    void Prong3D(art::Event const& evt,
		 evdb::View3D*     view);


    void FillTQHisto(art::Event const& evt,
		     TH1F*             thisto,
		     TH1F*             qhisto);

  protected:
    void DrawHoughResults2D(std::vector<const rb::HoughResult*>& htr,
			    evdb::View2D* xzview,
			    evdb::View2D* yzview);

    void DrawVertices2D(std::vector<const rb::Vertex*>& vert,
		        evdb::View2D* xzview,
			evdb::View2D* yzview,
			std::vector< int >&         vertexIndex);

    void DrawCluster2D(std::vector<const rb::Cluster*>& clusts,
                       evdb::View2D*                  xzview,
                       evdb::View2D*                  yzview,
                       std::vector<int>&              clustIndex);

    void DrawProng2D(std::vector<const rb::Prong*>& prong,
		     evdb::View2D*                  xzview,
		     evdb::View2D*                  yzview,
		     std::vector< int >&            prongIndex);

    void DrawTrack2D(std::vector<const rb::Track*>& track,
		     evdb::View2D*                  xzview,
		     evdb::View2D*                  yzview,
		     std::vector< int >&            trackIndex);

    void DrawProng3D(std::vector<const rb::Prong*>& prong,
		     evdb::View3D*                  view,
		     std::vector< int >&            prongIndex);

    void DrawTrack3D(std::vector<const rb::Track*>& track,
		     evdb::View3D*                  view,
		     std::vector< int >&            trackIndex);

  private:
    void GetClusterOutlines(const rb::Cluster* c,
                            int view,
			    std::vector<double>& vzVpts,
			    std::vector<double>& vzZpts);

    void GetClusterOutlinesConvexHull(const rb::Cluster* c,
                                      int viewint,
                                      std::vector<double>& vzVpts,
                                      std::vector<double>& vzZpts) const;

    int GetClusterColor(unsigned int idx) const;
    int DimOutOfTime(double t1, double t2, int c);
    int DimFiltered(const rb::CellHit* chit,
		    std::vector<const rb::CellHit*>  unfiltered,
		    int c);
    double ScaleFactor(double, double);

  };
}

#endif
////////////////////////////////////////////////////////////////////////
