///
/// \file    SimulationDrawer.h
/// \brief   Render the objects from the Simulation package
/// \author  messier@indiana.edu
/// \version $Id: SimulationDrawer.h,v 1.14 2012-09-26 21:40:55 bckhouse Exp $
///
#ifndef EVD_SIMULATIONDRAWER_H
#define EVD_SIMULATIONDRAWER_H
#include <string>
#include <vector>
#include <map>
#include <set>

namespace art  { class Event;       }
namespace evdb { class View2D;      }
namespace evdb { class View3D;      }
namespace geo  { 
  class Geometry; 
  class OfflineChan;    
}
namespace simb { class MCTruth;     } 
namespace sim  { 
  class Particle; 
  class FLSHitList;  
}

namespace evd {
  class SimulationDrawer {
  public:
    SimulationDrawer();
    ~SimulationDrawer();
  public:
    // Drawing functions
    void MCTruthShortText(const art::Event& evt,
			  evdb::View2D*     view);
    void MCTruthLongText(const art::Event& evt,
			 evdb::View2D*     view);

    void MCTruthVectors2D(const art::Event& evt,
			  evdb::View2D*     xzview,
			  evdb::View2D*     yzview);
    void MCTruthVectors3D(const art::Event& evt,
			  evdb::View3D*     view);

    void MCTruthTrajectories2D(const art::Event& evt,
			       evdb::View2D*     xzview,
			       evdb::View2D*     yzview);
    void MCTruthTrajectories3D(const art::Event& evt,
			       evdb::View3D*     view);
    
    void FLSHit2D(const art::Event& evt,
		  evdb::View2D*     xzview,
		  evdb::View2D*     yzview);
    void FLSHit3D(const art::Event& evt,
		  evdb::View3D*     xzview);

    void HiLite(int trkId, bool hlt=true);

#ifndef __CINT__
    void GetLimits(const art::Event* evt,
                   double& xmin, double& xmax,
                   double& ymin, double& ymax,
                   double& zmin, double& zmax,
                   const std::set<geo::OfflineChan>& hmap = {});
#endif /* __CINT __ */

    void GetTimeLimits(const art::Event* evt,
                       double& tmin, double& tmax);

  private:
    int GetMCTruth(const art::Event&              evt,
		   std::vector<simb::MCTruth>& mctruth);

    int GetFLSHits(const art::Event&                evt,
		   const char*                      which,
		   std::vector<sim::FLSHitList>& flshits);

    std::string ShortInteractionSuffix(int iType) const;

    /// Helper function for \ref MCTruthLongText
    void PrintParticleAndOffspring(std::stringstream& ss,
                                   const sim::Particle* part,
				   std::string prefix,
				   int depth) const;

    /// Helper function for MCTruthTrajectories[2|3]D
    void MCTruthTrajectoriesAnyD(const art::Event& evt,
				 evdb::View2D*     xzview,
				 evdb::View2D*     yzview,
				 evdb::View3D*     view3D);

  private:
    std::map<int,bool>       fHighlite;

  };
}

#endif
////////////////////////////////////////////////////////////////////////
