////////////////////////////////////////////////////////////////////////
/// \brief   GENIE interface for flavor modification
///
/// \author  Robert Hatcher <rhatcher \at fnal.gov>
///          Fermi National Accelerator Laboratory
///
/// \date    2010-10-31 initial version
////////////////////////////////////////////////////////////////////////
#include "EventGenerator/GENIE/GFlavorOsc.h"

#include <iostream>
#include <iomanip>
#include <cassert>
#include <cstdlib>

// GENIE includes
#include "Utils/StringUtils.h"
#include "FluxDrivers/GFlavorMixerFactory.h"
// self register with the factory
FLAVORMIXREG3(evgen,GFlavorOsc,evgen::GFlavorOsc)

// NOvA, nusoft or ART headers
#include "messagefacility/MessageLogger/MessageLogger.h"

namespace evgen {
//____________________________________________________________________________
  GFlavorOsc::GFlavorOsc() : fLastEnergy(-999), fLastDist(-999)
{
  UnityProbMatrix();
}

GFlavorOsc::~GFlavorOsc() { ; }

//____________________________________________________________________________
void GFlavorOsc::Config(std::string config)
{
  mf::LogInfo("GFlavorOsc") 
    << "GFlavorOsc::Config \"" << config << "\"" << std::endl;

}

//____________________________________________________________________________
void GFlavorOsc::UnityProbMatrix()
{
  for (int i=0; i<kNFlavors; ++i) {
    for (int j=0; j<kNFlavors; ++j) {
      fProbMatrix[i][j] = ( (i==j) ? 1 : 0 );
    }
  }  
}
//____________________________________________________________________________
double GFlavorOsc::Probability(int pdg_initial, int pdg_final, 
                                double energy, double dist )
{
  // do a bit of caching if the oscillation code supports it efficiently
  if ( fLastEnergy != energy && fLastDist != dist ) {
    // need new values
    fLastEnergy = energy;
    fLastDist   = dist;

    UnityProbMatrix();  // do this only if code below doesn't fill all elements

    // remove this printout after vetting the code
    mf::LogInfo("GFlavorOsc")
      << " new energy " << energy << " or distance " << dist;

    // fill the matrix, or reconfigure the calculator

  }

  // assuming we're using a cached matrix ... otherwise calculate the prob
  double prob = fProbMatrix[PDG2Indx(pdg_initial)][PDG2Indx(pdg_final)];

  // remove this printout after vetting the code
  mf::LogInfo("GFlavorOsc")
    << "saw " << pdg_initial << " => " << pdg_final 
    << " E=" << energy << " dist=" << dist 
    << " prob=" << prob
    << std::endl;

  return prob;
}

//____________________________________________________________________________
void GFlavorOsc::PrintConfig(bool /* verbose */)
{
  mf::LogInfo("GFlavorOsc")
    << "GFlavorOsc::PrintConfig():" << std::endl;
  
}
//____________________________________________________________________________
const char* GFlavorOsc::IndxName(int indx)
{
  static const char* name[] = { "sterile", 
                                "nu_e", "nu_mu", "nu_tau",
                                "nu_e_bar", "nu_mu_bar", "nu_tau_bar" };
  return name[indx];

}

//____________________________________________________________________________
} // namespace evgen

