#include "art/Framework/Core/EmptyEventTimestampPlugin.h"

class NovaMCEmptyEvent: public art::EmptyEventTimestampPlugin
{
public:
  NovaMCEmptyEvent(const fhicl::ParameterSet& pset)
    : EmptyEventTimestampPlugin(pset),
      fFirstTimeSec(pset.get<long>("firstTime")),
      fCurrentTimeSec(fFirstTimeSec),
      fCurrentTimeNSec(0),
      fTimeBetweenEventsNSec(pset.get<long>("timeBetweenEvents"))
  {
  }

  art::Timestamp eventTimestamp(const art::EventID& e) override
  {
    const art::Timestamp ret(fCurrentTimeSec << 32 | fCurrentTimeNSec);

    const long kNanoSecPerSec = 1000000000;

    fCurrentTimeNSec += fTimeBetweenEventsNSec; // add to the nsec
    fCurrentTimeSec += fCurrentTimeNSec/kNanoSecPerSec; // carry to the sec
    fCurrentTimeNSec %= kNanoSecPerSec; // truncate the nsec

    return ret;
  }

  void rewind() override
  {
    fCurrentTimeSec = fFirstTimeSec;
    fCurrentTimeNSec = 0;
  }

protected:
  long fFirstTimeSec;
  long fCurrentTimeSec;
  long fCurrentTimeNSec;
  long fTimeBetweenEventsNSec;
};

DEFINE_ART_EMPTYEVENTTIMESTAMP_PLUGIN(NovaMCEmptyEvent);
