////////////////////////////////////////////////////////////////////////
/// \file  Geometry.h
/// \brief Encapsulate the geometry of one entire detector (near, far,
///        ndos)
///
/// \version $Id: Geometry.h,v 1.39 2012-11-14 19:40:46 bckhouse Exp $
/// \author  messier@indiana.edu
////////////////////////////////////////////////////////////////////////
#ifndef GEO_GEOMETRY_H
#define GEO_GEOMETRY_H

#include "GeometryObjects/GeometryBase.h"

#include "art/Framework/Principal/Run.h"
#include "art/Framework/Services/Registry/ActivityRegistry.h"
#include "art/Framework/Services/Registry/ServiceMacros.h"
#include "TVector3.h"

/// Detector geometry information
namespace geo {
  /// The geometry of one entire detector (near, far, ipnd)
  class Geometry : public GeometryBase
  {
  public:
    Geometry(fhicl::ParameterSet const& pset, art::ActivityRegistry& reg);
    ~Geometry();

    void preBeginRun(art::Run const& run);

    bool     calculateMassesLong(const unsigned int number_of_points) const;

    /// Get direction of the Prong in the beam coordinates
    TVector3 getDirectionInBeamCoordinates(TVector3 const& startPos,
					   TVector3 const& dir) const;

  protected:
    virtual void setDetectorID(novadaq::cnv::DetId) override;

    std::string fLoadedGDMLInfo;
  };
}
DECLARE_ART_SERVICE(geo::Geometry, LEGACY)
#endif // GEO_GEOMETRY_H
////////////////////////////////////////////////////////////////////////
