////////////////////////////////////////////////////////////////////////
/// \file    CoordinateTransformationTest_module.cc
/// \brief   unit tests for the CoordinateTransformation service
/// \author  brebel@fnal.gov
////////////////////////////////////////////////////////////////////////
// ROOT includes
#include "TH1D.h"
#include "TH2D.h"
#include "TVector3.h"
#include "TNtuple.h"
#include "TGeoManager.h"

// NOvA includes
#include "NovaDAQConventions/DAQConventions.h"

// Framework includes
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Services/Optional/TFileDirectory.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art/Framework/Principal/Handle.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"
#include "GeometryObjects/CoordinateTransformation.h"

#include <string>
#include <cmath>
#include <vector>

namespace geo {
  class CoordinateTransformationTest : public art::EDAnalyzer {
  public:
    explicit CoordinateTransformationTest(fhicl::ParameterSet const& pset);
    virtual ~CoordinateTransformationTest();

    void analyze(art::Event const& evt);
    
  private:

    bool printCoordinateTransformations(uint32_t detector_id, uint32_t beam_type);
  };

  //......................................................................
  CoordinateTransformationTest::CoordinateTransformationTest(fhicl::ParameterSet const& pset)
    : EDAnalyzer(pset)
  {
  }

  //......................................................................
  CoordinateTransformationTest::~CoordinateTransformationTest()
  {
  }

  //......................................................................
  void CoordinateTransformationTest::analyze(art::Event const& evt)
  {
  
  
   std::cout<<"Coordinate transformations\n";
   uint32_t detector_id = novadaq::cnv::kNEARDET;
   uint32_t beam_type   = BEAMTYPE_NUMI;
   std::cout<<"Beam type = BEAMTYPE_NUMI,   Detector = NEAR:\n";
   printCoordinateTransformations(detector_id,beam_type);
  
  
   detector_id = novadaq::cnv::kFARDET;
   std::cout<<"Beam type = BEAMTYPE_NUMI,   Detector = FAR:\n";
   printCoordinateTransformations(detector_id,beam_type);

   detector_id = novadaq::cnv::kNDOS;
   std::cout<<"Beam type = BEAMTYPE_NUMI,   Detector = NDOS:\n";
   printCoordinateTransformations(detector_id,beam_type);
  
    return;
  }

  bool CoordinateTransformationTest::printCoordinateTransformations(uint32_t detector_id, uint32_t beam_type){
	
    CoordinateTransformation  coordinate_transformation(detector_id, beam_type);
	
    return coordinate_transformation.print();
  
  }

  DEFINE_ART_MODULE(CoordinateTransformationTest);

}//end namespace
