////////////////////////////////////////////////////////////////////////
/// \file    Geo.h
/// \brief   Collect Geo headers and supply basic geometry functions
/// \author  messier@indiana.edu
/// \version $Id: Geo.h,v 1.2 2012-09-20 21:42:43 greenc Exp $
#ifndef GEOFUNC_GEO_H
#define GEOFUNC_GEO_H
#include <vector>

#include "TVector3.h"

#include "GeometryObjects/GeometryBase.h"

/// Detector geometry definition and interface
namespace geo {
  void ProjectToBoxEdge(const double xyz[],
			const double dxyz[],
			double xlo, double xhi,
			double ylo, double yhi,
			double zlo, double zhi,
			double xyzout[]);
  int WhichWallofBox(const double xyz[],
			double xlo, double xhi,
			double ylo, double yhi,
			double zlo, double zhi);
  bool IntersectsBox(const double xyz[],
		     const double dxyz[],
		     double xlo, double xhi,
		     double ylo, double yhi,
		     double zlo, double zhi);
  void ProjectToBoxEdgeFromOutside(const double xyz[],
				   const double dxyz[],
				   int    axis,
				   double edge,
				   double xyzout[]);
  bool LineIntersection(double x0, double y0, double x1, double y1,
			double X0, double Y0, double X1, double Y1,
			double& x, double& y);
  double ClosestApproach(const double point[],
			 const double intercept[],
			 const double slopes[],
			 double closest[]);
  double ClosestApproach(TVector3 point, TVector3 intercept,
			 TVector3 slopes, TVector3& closest);
  double ClosestApproach(const TVector3& P0,
			 const TVector3& P1,
			 const TVector3& Q0,
			 const TVector3& Q1,
			 double* sc=0,
			 double* tc=0,
			 TVector3* PC=0,
			 TVector3* QC=0);
  double DsqrToLine(double x0, double y0,
		    double x1, double y1,
		    double x2, double y2);
  double LinFit(const std::vector<double>& x,
		const std::vector<double>& y,
		const std::vector<double>& w,
		double* x1, double* y1,
		double* x2, double* y2);
  double LinFitMinDperp(const std::vector<double>& x,
			const std::vector<double>& y,
			const std::vector<double>& w,
			double* x1, double* y1,
			double* x2, double* y2);
  double DistToEdge(double *point,
		    double detHalfWidth,
		    double detHalfHeight,
		    double detLength);
  /// \brief If either endpoint is outside the detector move it to the edge
  ///
  /// If the given ray does not intersect the detector, returns false
  bool ClampRayToDetectorVolume(TVector3* p0, TVector3* p1,
                                const GeometryBase* geom);

  double AverageCellPathLength(geo::View_t view,
			       double dx, double dy, double dz);
}

#endif
////////////////////////////////////////////////////////////////////////

