//////////////////////////////////////////////////////////////////////////
/// \brief   Tool for finding and labeling "islands" around peaks in a Hough map
/// \authors messier@indiana.edu, mibaird@indiana.edu
/// \date    Nov. 2011
/////////////////////////////////////////////////////////////////////////
#ifndef HOUGH_ISLANDSTH2_H
#define HOUGH_ISLANDSTH2_H

#include "TH2S.h"

namespace hough {
  class IslandsTH2 : public TH2S {
  public:
    IslandsTH2(const char* n,
	       const char* t,
	       const TH2* h,
	       double q,
	       bool wrapx,
	       bool wrapy);
    unsigned int Nislands() const { return fNislands; }
    void LabelIslands(const TH2* h);
  private:
    void Mark(const TH2* h, int i, int j, int isl_indx, double th);
    int  Neighbor(int i, int d, int imx, bool wrap);
  private:
    double       fPco;      ///< Number of sigma above average peak height to use for peak threshold
    bool         fWrapX;    ///< Wrap around the x boundaries?
    bool         fWrapY;    ///< Wrap around the y boundaries?
    int          fNx;       ///< Number of bins along x
    int          fNy;       ///< Number of bins along y
    unsigned int fNislands; ///< Number of islands found
  };
}

#endif
////////////////////////////////////////////////////////////////////////
