////////////////////////////////////////////////////////////////////////
/// \brief
/// \author messier@indiana.edu
////////////////////////////////////////////////////////////////////////
#include <cstdio>

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h" 
#include "fhiclcpp/ParameterSet.h" 


namespace mcchk {
  /// A null module for performance checks
  
  class NullAna : public art::EDAnalyzer {
  
  public:
    explicit NullAna(fhicl::ParameterSet const& pset);
    virtual ~NullAna();
    
    void analyze(art::Event const& evt);
    
    int counter;
    int prescale;
    
  private:
    
  };
};


////////////////////////////////////////////////////////////////////////
namespace mcchk {

  //......................................................................  
  NullAna::NullAna(fhicl::ParameterSet const& pset) :
    EDAnalyzer(pset),
    prescale (pset.get< int >("Prescale"))
  { 
    counter  = 0;
  }
    
  //......................................................................
  
  NullAna::~NullAna() { }
  
  //......................................................................

  void NullAna::analyze(art::Event const& evt)
  {
    counter++;
    if(counter%prescale){
      return;
    }
    else{
      printf("Event %d - Passed\n",counter);
      return;
    }

    return;
  }


} // end mcchk

////////////////////////////////////////////////////////////////////////
namespace mcchk
{
  DEFINE_ART_MODULE(NullAna);
}
