////////////////////////////////////////////////////////////////////////
/// \brief  CLass for parsing *.fcl files for their metadata information
/// \author Adam Aurisano - aurisaam@uc.edu
////////////////////////////////////////////////////////////////////////

#include <algorithm>
#include <cstdlib>
#include <fstream>
#include <iostream>
#include <iomanip>
#include <iterator>
#include <map>
#include <string>
#include <vector>

#include "art/Utilities/FirstAbsoluteOrLookupWithDotPolicy.h"
#include "fhiclcpp/intermediate_table.h"
#include "fhiclcpp/make_ParameterSet.h"
#include "fhiclcpp/ParameterSet.h"

#include "MetadataManager.h"

namespace meta {
  
  class MetadataParse {
    
  public:
    explicit MetadataParse(std::string infile);
    
    void PrintMetadata();
    
  };
};

namespace meta {
  
  //.................................................................
  MetadataParse::MetadataParse(std::string infile)
  {
    MetadataManager& fManager = MetadataManager::getInstance();
    
    art::FirstAbsoluteOrLookupWithDotPolicy policy("FHICL_FILE_PATH");
    // parse a configuration file; obtain intermediate form
    fhicl::intermediate_table tbl;
    fhicl::parse_document(infile, policy, tbl);
    
    // convert to ParameterSet
    fhicl::ParameterSet pset;
    fhicl::make_ParameterSet(tbl, pset);
    
    //fetch the appropriate sub-pset and pass to the manager
    fhicl::ParameterSet metaPSet;
    bool foundPSet = pset.get_if_present<fhicl::ParameterSet>("physics.analyzers.metadata.params", metaPSet);
    if(foundPSet) fManager.DoParsing(metaPSet);
  }
  
  //..................................................................
  void MetadataParse::PrintMetadata()
  {
    MetadataManager& fManager = MetadataManager::getInstance();
    
    std::map<std::string, std::string> metamap = fManager.GetMetadata();
  
    std::cout << "{" << std::endl;
    std::cout << " \"file_type\":   \"mc_config\" " << std::endl;
    std::cout << ",\"file_format\":  \"fcl\" " << std::endl;
    
    for(auto& it: metamap){
      std::string key = it.first;
      std::string value = it.second;
      
      // Exclude base_release type entries from fcl metadata
      size_t found = key.find("base_release");
      if(found != std::string::npos) continue;
      
      std::cout << ",\"" << key 
		<< "\":" << "      " 
		<< "\"" << value 
		<< "\"" << std::endl; 
    }
    std::cout << "}" << std::endl;
  }
  
} // end namespace meta

//...................................................................
int main(int argc, char *argv[])
{
  if(argc != 2) return 1;
  
  std::string infile = argv[1]; 
  meta::MetadataParse parser(infile);
  parser.PrintMetadata();
    
  return 0;
}
///////////////////////////////////////////////////////////////////
