////////////////////////////////////////////////////////////////////////
// \file    MichelECluster.h
// \brief   A MichelECluster is a cluster of hits that arise from a Michel 
//          electron cluster at the end of stopping muon track.          
// \version $Id: MichelECluster.h,v 1.6 2012-10-26 22:25:05 bckhouse Exp $
// \author  Kanika Sachdev (ksachdev@physics.umn.edu)
////////////////////////////////////////////////////////////////////////


#ifndef MichelECluster_h
#define MichelECluster_h

#include "RecoBase/Cluster.h"

namespace rb{ class Track;}

namespace me {  
  class MichelECluster: public rb::Cluster
  {
  public:
    MichelECluster();  // Default constructor
    ~MichelECluster(); 

    /// Returns the distance of the michel cluster from the end
    /// of the parent track
    double DistanceToTrackEnd() const;

    /// Returns the time difference between the end of the muon 
    /// track the michel cluster
    double DeltaT() const;

    /// Returns the art::Ptr of the track the michel cluster 
    /// is associated with
    art::Ptr<rb::Track> ParentTrack() const;

    /// A function to set the michel cluster's parent track
    void SetParentTrack( art::Ptr<rb::Track> muTrack);

    /// Determines which end of the track the michel cluster was
    /// found to be. If the returned value is true, the michel
    /// cluster was closer to the track stop point, else it was
    /// closer to the track start point.
    bool AtTrackEnd() const;
    
  protected:

    /// Parent track to the michel electron
    art::Ptr<rb::Track> fParentTrack;

  };
  
} // end namespace me

#endif // MichelECluster_h



