/* 
 * File:   GlobalEventTime.cxx
 * Author: denis
 * 
 * Created on February 10, 2010, 11:31 AM
 */
#ifdef NOVACMAKE
#include "MockDataDAQ/include/GlobalEventTime.h"
#include "MockDataDAQ/include/Configuration.h"
#else
#include "MockDataDAQ/GlobalEventTime.h"
#include "MockDataDAQ/Configuration.h"
#endif
#include "DAQDataFormats/RawNanoSlice.h"
#include <stdio.h>
#include <iostream>


namespace mcdatadaq{

GlobalEventTime* GlobalEventTime::finstanceID = NULL;

//------------------------------------------------------------------------------
GlobalEventTime* GlobalEventTime::getInstance(){

    /// Check if there is an instance active
    /// If there is not, make a new one
    if(finstanceID == NULL){
      const Configuration* config = Configuration::getInstance();
      finstanceID = new GlobalEventTime(config->getEventTimeDifference());
  }
  return finstanceID;
}

//------------------------------------------------------------------------------
GlobalEventTime::GlobalEventTime():
      fEventTimeDifference(DEFAULT_TIME_DIFF)
    , fGlobalEventTime(0)
{
    print();
}

//------------------------------------------------------------------------------
GlobalEventTime::GlobalEventTime(const unsigned long long time_difference,
                                 const unsigned long long global_event_time):
      fEventTimeDifference(time_difference)
    , fGlobalEventTime(global_event_time)
{
    print();
}

//------------------------------------------------------------------------------
//Destructor
GlobalEventTime::~GlobalEventTime() {}


//------------------------------------------------------------------------------
void GlobalEventTime::advanceEventTime(){ fGlobalEventTime = fGlobalEventTime + fEventTimeDifference;}


//------------------------------------------------------------------------------
void GlobalEventTime::print() const{

    std::cout<<"GlobalEventTime::print :"<<std::endl;
    std::cout<<"      First Event Time="<<fGlobalEventTime<<std::endl;
    std::cout<<"      Event Time Difference="<<fEventTimeDifference<<std::endl;
}


//------------------------------------------------------------------------------
void GlobalEventTime::printEventTime()  const {
    printf("------------------------------\n");
    printf("Event time = %llu\n", (unsigned long long)fGlobalEventTime);
    printf("------------------------------\n");
}

//------------------------------------------------------------------------------
void GlobalEventTime::printNanoSliceTime(const daqdataformats::RawNanoSlice* Nanoslice)  const{

    const unsigned long long Nanoslice_Time = fGlobalEventTime + Nanoslice->getTimeStamp();

    printf("------------------------------\n");
    printf("Raw NanoSlice time = %d\n", Nanoslice->getTimeStamp());
    printf("Global Nanoslice time = %llu\n", (unsigned long long)Nanoslice_Time);
    printf("------------------------------\n");
}

//------------------------------------------------------------------------------
unsigned long long GlobalEventTime::getNanoSliceGlobalTime(const daqdataformats::RawNanoSlice* Nanoslice) const {
    const unsigned long long Nanoslice_Time = fGlobalEventTime + Nanoslice->getTimeStamp();
    return Nanoslice_Time;
}

//------------------------------------------------------------------------------
unsigned long long GlobalEventTime::getGlobalEventTime() const{return fGlobalEventTime;}

//------------------------------------------------------------------------------
unsigned long long GlobalEventTime::getNextEventGlobalTime() const{return fGlobalEventTime + fEventTimeDifference;}

}// end of namespace
