

////////////////////////////////////////////////////////////////////////
// \file    MRCCParent.h
// \brief   MRCCParent object is a container for information
//          about the parent slice of a MRCC event. The parent slice
//          is the original slice that a MRCC slice maps to with maximum 
//          purity. Only the information that has been found to be
//          important for the MRCC analysis so far is being added to 
//          this object.
// \author  Kanika Sachdev (ksachdev@physics.umn.edu)
////////////////////////////////////////////////////////////////////////


#ifndef MRCCParent_h
#define MRCCParent_h

namespace murem
{

  class MRCCParent 
  {

  public:
   
    /// Default constructor
    MRCCParent():fParentIdx(-1), fParentEff(0.0), fParentPur(0.0){} 

    /// Default destructor
    ~MRCCParent(){}  
   
    /// Set the index of the parent slice
    void SetParentIndex( int parentIdx) { fParentIdx = parentIdx; }
    /// Returns the index of the parent slice
    int  ParentIndex() const { return fParentIdx; }


    /// Set the efficiency of MRCC slice wrt the parent
    void SetParentEff( float parentEff) { fParentEff = parentEff; } 
    /// Returns the efficiency of MRCC slice wrt the parent
    float ParentEff() const { return fParentEff; }


    /// Set the purity of MRCC slice wrt the parent
    void SetParentPur( float parentPur) { fParentPur = parentPur; }
    /// Returns the purity of MRCC slice wrt the parent
    float ParentPur() const { return fParentPur; }

  private:

    int      fParentIdx;
    float    fParentEff;
    float    fParentPur;

  }; // End of MRCCParent class definition

}// End of namespace murem

#endif
