////////////////////////////////////////////////////////////////////////
// \file    NueSandObj.h
// \brief   Object collecting nue Sandbox variables
// \author  Christopher Backhouse - bckhouse@caltech.edu
////////////////////////////////////////////////////////////////////////

#ifndef NUESAND_NUESANDOBJ_H
#define NUESAND_NUESANDOBJ_H

#include "art/Persistency/Common/PtrVector.h"
#include "TVector3.h"

/// \f$ \nu_e \f$ sandbox
namespace nuesand
{
  /// \f$ \nu_e \f$ sandbox variables
  class NueSandObj
  {
  public:
    NueSandObj()
      : fInFid(false),
      fNPi0(0), fNPiPlus(0), fNPiMinus(0),
      fPi0Energy(0), fPiPlusEnergy(0), fPiMinusEnergy(0),
      fPhot0E(0), fPhot1E(0),
      fOpenCos(0),
      fConvL0(0), fConvL1(0),
      fMincellCosmic(0),
      fMincellWallCosmic(0),
      fMincellAvg(0),
      fMincellKalman(0),
      fAngleCosmic(0),
      fNMIPHits(0),
      fMIPMin(),
      ECFNu(0.0),
      fMIPMax(),
      fFilledPlanes(0),
      fExtentXPlanes(0),
      fExtentYPlanes(0),
      fMaxCellsInPlane(0),
      fMaxRowOfCells(0),
      fFracAngChanges(0),
      fFracAngChangesLoose(0),
      fFracModalHits(0),
      fFracNonContig(0),
      fFracNonContigLoose(0),
      fMaxGap(0),
      fdEdxProng1(0),
      fdEdxProng2(0)
	{}

  NueSandObj(bool infid, TVector3 mean)
    : fInFid(infid), fSliceMean(mean),
      fNPi0(0), fNPiPlus(0), fNPiMinus(0),
      fPi0Energy(0), fPiPlusEnergy(0), fPiMinusEnergy(0),
      fPhot0E(0), fPhot1E(0),
      fOpenCos(0),
      fConvL0(0), fConvL1(0),
      fMincellCosmic(0),
      fMincellWallCosmic(0),
      fMincellAvg(0),
      fMincellKalman(0),
      fAngleCosmic(0),
      fNMIPHits(0),
      fMIPMin(),
      ECFNu(0.0),
      fMIPMax(),
      fFilledPlanes(0),
      fExtentXPlanes(0),
      fExtentYPlanes(0),
      fMaxCellsInPlane(0),
      fMaxRowOfCells(0),
      fFracAngChanges(0),
      fFracAngChangesLoose(0),
      fFracModalHits(0),
      fFracNonContig(0),
      fFracNonContigLoose(0),
      fMaxGap(0),
      fdEdxProng1(0),
      fdEdxProng2(0)
	{}

    bool     InFiducialVolume()  const {return fInFid;}
    TVector3 SliceMeanPosition() const {return fSliceMean;}
    
  protected:
    
    bool fInFid;
    TVector3 fSliceMean;

  public:
    int fNPi0;     ///< Number of primary pi0
    int fNPiPlus;  ///< Number of primary pi+
    int fNPiMinus; ///< Number of primary pi-

    float fPi0Energy; ///< Energy of largest primary pi0
    float fPiPlusEnergy; ///< Energy of largest primary pi+
    float fPiMinusEnergy; ///< Energy of largest primary pi-

    float fPhot0E; ///< Energy of largest pi0's largest photon
    float fPhot1E; ///< Energy of largest pi0's smallest photon
    float fOpenCos; ///< Opening angle of largest pi0's photons
    float fConvL0; ///< Conversion length of photon 0
    float fConvL1; ///< Conversion length of photon 1

    int fMincellCosmic;
    int fMincellWallCosmic;
    int fMincellAvg;
    int fMincellKalman;
    float fAngleCosmic;
    float fNMIPHits;
    TVector3 fMIPMin;
    float ECFNu;
    TVector3 fMIPMax;
    int fFilledPlanes;
    unsigned int fExtentXPlanes;
    unsigned int fExtentYPlanes;
    int fMaxCellsInPlane;
    int fMaxRowOfCells;

    float fFracAngChanges;
    float fFracAngChangesLoose;
    float fFracModalHits;
    float fFracNonContig;
    float fFracNonContigLoose;
    int fMaxGap;

    float fdEdxProng1;
    float fdEdxProng2;
  };

}

#endif
