////////////////////////////////////////////////////////////////////////
// \file    NumuE.cxx
// \brief   An energy object for numu cc events 
// \version $Id: NumuE.cxx,v 1.5 2012-11-14 20:56:10 lein Exp $
// \author  Susan Lein - lein@physics.umn.edu
////////////////////////////////////////////////////////////////////////
#include "NumuEnergy/NumuE.h"

namespace numue
{
  //......................................
  NumuE::NumuE()
    : rb::Energy()
  {
    SetCalCCE(-1);
    SetTrkQEE(-1);
    SetTrkNonQEE(-1);
    SetTrkCCE(-1);
    SetAngleQEE(-1);
    SetAngleQEError(-1);
    SetHadCalE(-1);
    SetHadTrkE(-1);
    SetUCCCE(-1);
    SetNDTrkLenAct(-1);
    SetNDTrkLenCat(-1);
    SetNDTrkCalAct(-1);
    SetNDTrkCalTran(-1);
    SetNDTrkCalCat(-1);
    SetNDHadCalAct(-1);
    SetNDHadCalTran(-1);
    SetNDHadCalCat(-1);
    SetNDHadTrkAct(-1);
    SetNDHadTrkTran(-1);
    SetNDHadTrkCat(-1);
    SetFScatterE(-1);
    SetNDTrkTranX(999);
    SetNDTrkTranY(999);
  }
  //......................................
  NumuE::NumuE(float bestE, float calcc, float trkqe, float trknonqe, float angleqe, float hadcal)
    : rb::Energy(bestE)
  {
    SetCalCCE(calcc);
    SetTrkQEE(trkqe);
    SetTrkNonQEE(trknonqe);
    SetTrkCCE(-1);
    SetAngleQEE(angleqe);
    SetAngleQEError(-1);
    SetHadCalE(hadcal);
    SetHadTrkE(-1);
    SetUCCCE(-1);
    SetNDTrkLenAct(-1);
    SetNDTrkLenCat(-1);
    SetNDTrkCalAct(-1);
    SetNDTrkCalTran(-1);
    SetNDTrkCalCat(-1);
    SetNDHadCalAct(-1);
    SetNDHadCalTran(-1);
    SetNDHadCalCat(-1);
    SetNDHadTrkAct(-1);
    SetNDHadTrkTran(-1);
    SetNDHadTrkCat(-1);
    SetFScatterE(-1);
    SetNDTrkTranX(999);
    SetNDTrkTranY(999);
  }
  //.....................................
  void    NumuE::SetCalCCE(float calcc)
  {
    fCalCCE = calcc;
  }
  //.....................................
  void    NumuE::SetTrkQEE(float trkqe)
  {
    fTrkQEE = trkqe;
  }
  //.....................................
  void    NumuE::SetTrkNonQEE(float trknonqe)
  {
    fTrkNonQEE = trknonqe;
  }
  //.....................................
  void    NumuE::SetTrkCCE(float trkcc)
  {
    fTrkCCE = trkcc;
  }
  //.....................................
  void    NumuE::SetAngleQEE(float angleqe)
  {
    fAngleQEE = angleqe;
  }
  //.....................................
  void    NumuE::SetAngleQEError(float angleqeerror)
  {
    fAngleQEError = angleqeerror;
  }
  //.....................................
  void    NumuE::SetHadCalE(float hadcal)
  {
    fHadCalE = hadcal;
  }
  //.....................................
  void    NumuE::SetHadTrkE(float hadtrk)
  {
    fHadTrkE = hadtrk;
  }
  //.....................................
  void    NumuE::SetUCCCE(float uccc)
  {
    fUCCCE = uccc;
  }
  //.....................................
  void    NumuE::SetNDTrkLenAct(float ndtrklenact)
  {
    fNDTrkLenAct = ndtrklenact;
  }
  //.....................................
  void    NumuE::SetNDTrkLenCat(float ndtrklencat)
  {
    fNDTrkLenCat = ndtrklencat;
  }
  //.....................................
  void    NumuE::SetNDTrkCalAct(float ndtrkcalact)
  {
    fNDTrkCalAct = ndtrkcalact;
  }
  //.....................................
  void    NumuE::SetNDTrkCalTran(float ndtrkcaltran)
  {
    fNDTrkCalTran = ndtrkcaltran;
  }
  //.....................................
  void    NumuE::SetNDTrkCalCat(float ndtrkcalcat)
  {
    fNDTrkCalCat = ndtrkcalcat;
  }
  //.....................................
  void    NumuE::SetNDHadCalAct(float ndhadcalact)
  {
    fNDHadCalAct = ndhadcalact;
  }
  //.....................................
  void    NumuE::SetNDHadCalTran(float ndhadcaltran)
  {
    fNDHadCalTran = ndhadcaltran;
  }
  //.....................................
  void    NumuE::SetNDHadCalCat(float ndhadcalcat)
  {
    fNDHadCalCat = ndhadcalcat;
  }
  //.....................................
  void    NumuE::SetNDHadTrkAct(float ndhadtrkact)
  {
    fNDHadTrkAct = ndhadtrkact;
  }
  //.....................................
  void    NumuE::SetNDHadTrkTran(float ndhadtrktran)
  {
    fNDHadTrkTran = ndhadtrktran;
  }
  //.....................................
  void    NumuE::SetNDHadTrkCat(float ndhadtrkcat)
  {
    fNDHadTrkCat = ndhadtrkcat;
  }
  //.....................................
  void    NumuE::SetFScatterE(float fscattere)
  {
    fFScatterE = fscattere;
  }
  //.....................................
  void    NumuE::SetNDTrkTranX(float ndtrktranx)
  {
    fNDTrkTranX = ndtrktranx;
  }
  //.....................................
  void    NumuE::SetNDTrkTranY(float ndtrktrany)
  {
    fNDTrkTranY = ndtrktrany;
  }
  //.....................................
  void    NumuE::SetHadCluster(rb::Cluster hadcluster)
  {
    fHadCluster = hadcluster;
  }
  //.....................................
  float  NumuE::CalCCE() const
  {
    return fCalCCE;
  }
  //.....................................
  float  NumuE::TrkQEE() const
  {
    return fTrkQEE;
  }
  //.....................................
  float  NumuE::TrkNonQEE() const
  {
    return fTrkNonQEE;
  }
  //.....................................
  float  NumuE::TrkCCE() const
  {
    return fTrkCCE;
  }
  //.....................................
  float  NumuE::AngleQEE() const
  {
    return fAngleQEE;
  }
  //.....................................
  float  NumuE::AngleQEError() const
  {
    return fAngleQEError;
  }
  //.....................................
  float  NumuE::HadCalE() const
  {
    return fHadCalE;
  }
  //.....................................
  float  NumuE::HadTrkE() const
  {
    return fHadTrkE;
  }
  //.....................................
  float  NumuE::UCCCE() const
  {
    return fUCCCE;
  }
  //.....................................
  float  NumuE::NDTrkLenAct() const
  {
    return fNDTrkLenAct;
  }
  //.....................................
  float  NumuE::NDTrkLenCat() const
  {
    return fNDTrkLenCat;
  }
  //.....................................
  float  NumuE::NDTrkCalAct() const
  {
    return fNDTrkCalAct;
  }
  //.....................................
  float  NumuE::NDTrkCalTran() const
  {
    return fNDTrkCalTran;
  }
  //.....................................
  float  NumuE::NDTrkCalCat() const
  {
    return fNDTrkCalCat;
  }
  //.....................................
  float  NumuE::NDHadCalAct() const
  {
    return fNDHadCalAct;
  }
  //.....................................
  float  NumuE::NDHadCalTran() const
  {
    return fNDHadCalTran;
  }
  //.....................................
  float  NumuE::NDHadCalCat() const
  {
    return fNDHadCalCat;
  }
  //.....................................
  float  NumuE::NDHadTrkAct() const
  {
    return fNDHadTrkAct;
  }
  //.....................................
  float  NumuE::NDHadTrkTran() const
  {
    return fNDHadTrkTran;
  }
  //.....................................
  float  NumuE::NDHadTrkCat() const
  {
    return fNDHadTrkCat;
  }
  //.....................................
  float  NumuE::FScatterE() const
  {
    return fFScatterE;
  }
  //.....................................
  float  NumuE::NDTrkTranX() const
  {
    return fNDTrkTranX;
  }
  //.....................................
  float  NumuE::NDTrkTranY() const
  {
    return fNDTrkTranY;
  }
  //.....................................
  rb::Cluster NumuE::HadCluster() const
  {
    return fHadCluster;
  }
}
