////////////////////////////////////////////////////////////////////////
// \file    NumuE.h
// \brief   An energy object for numu cc events
// \version $Id: NumuE.h,v 1.6 2012-11-14 20:56:10 lein Exp $
// \author  Susan Lein - lein@physics.umn.edu
////////////////////////////////////////////////////////////////////////
#ifndef NumuE_h
#define NumuE_h

#include "RecoBase/Energy.h"
#include "RecoBase/Cluster.h"

/// Energy estimators for \f$ \nu_\mu \f$ CC events
namespace numue
{

  enum MuonType {
    kFDMuon, 
    kNDActiveMuon, 
    kNDCatcherMuon, 
    kNDActCatMuon, 
    kNDPiddleMuon};
  
  class NumuE : public rb::Energy
    {
    public:
      
      NumuE();
      NumuE(float bestE, float calcc, float trkqe, float trknonqe, float angleqe, float hadcal);
      
      void               SetCalCCE(float calcc);
      void               SetTrkQEE(float trkqe);
      void               SetTrkNonQEE(float trknonqe);
      void               SetTrkCCE(float trkcce);
      void               SetAngleQEE(float angleqe);
      void               SetAngleQEError(float angleqeerror);
      void               SetHadCalE(float hadcal);
      void               SetHadTrkE(float hadtrk);
      void               SetUCCCE(float uccc);
      void               SetNDTrkLenAct(float ndtrklenact);
      void               SetNDTrkLenCat(float ndtrklencat);
      void               SetNDTrkCalAct(float ndtrkcalact);
      void               SetNDTrkCalTran(float ndtrkcaltran);
      void               SetNDTrkCalCat(float ndtrkcalcat);
      void               SetNDHadCalAct(float ndhadcalact);
      void               SetNDHadCalTran(float ndhadcaltran);
      void               SetNDHadCalCat(float ndhadcalcat);
      void               SetNDHadTrkAct(float ndhadtrkact);
      void               SetNDHadTrkTran(float ndhadtrktran);
      void               SetNDHadTrkCat(float ndhadtrkcat);
      void               SetFScatterE(float fscattere);
      void               SetNDTrkTranX(float ndtrktranx);
      void               SetNDTrkTranY(float ndtrktrany);
      
      void               SetHadCluster(rb::Cluster hadcluster); 
      
      rb::Cluster        HadCluster() const;
      
      float              CalCCE() const;
      float              TrkQEE() const;
      float              TrkNonQEE() const;
      float              TrkCCE() const;
      float              AngleQEE() const;
      float              AngleQEError() const;
      float              HadCalE() const;
      float              HadTrkE() const;
      float              UCCCE() const;
      float              NDTrkLenAct() const;
      float              NDTrkLenCat() const;
      float              NDTrkCalAct() const;
      float              NDTrkCalTran() const;
      float              NDTrkCalCat() const;
      float              NDHadCalAct() const;
      float              NDHadCalTran() const;
      float              NDHadCalCat() const;
      float              NDHadTrkAct() const;
      float              NDHadTrkTran() const;
      float              NDHadTrkCat() const;
      float              FScatterE() const;
      float              NDTrkTranX() const;
      float              NDTrkTranY() const;
      
    protected:
      
      float              fCalCCE;
      float              fTrkQEE;
      float              fTrkNonQEE;
      float              fTrkCCE;
      float              fAngleQEE;
      float              fAngleQEError;
      float              fHadCalE;
      float              fHadTrkE;
      float              fUCCCE;
      float              fNDTrkLenAct;
      float              fNDTrkLenCat;
      float              fNDTrkCalAct;
      float              fNDTrkCalTran;
      float              fNDTrkCalCat;
      float              fNDHadCalAct;
      float              fNDHadCalTran;
      float              fNDHadCalCat;
      float              fNDHadTrkAct;
      float              fNDHadTrkTran;
      float              fNDHadTrkCat;
      float              fFScatterE;
      float              fNDTrkTranX;
      float              fNDTrkTranY;

      rb::Cluster fHadCluster;
    };
  
};
#endif
