////////////////////////////////////////////////////////////////////////
// \file    ScatterEnergy.cxx
// \brief   NN output from training with scattering variables
//          input:  scattering variables from ScatterEstim
//          output: reco energy
// \version $Id: ScatterEnergy.cxx,v 1.1.1 2013/05/15 16:16:41 psihas Exp $
// \author  $Author: psihas $
////////////////////////////////////////////////////////////////////////
#include "ScatterEnergy.h"
#include <cmath>
#include <cstdio>
#include <iostream>

namespace numue
{

  double ScatterEnergy::Value(int index,double in0,double in1,double in2,double in3) {
    input0 = (in0 - 0)/1;
    input1 = (in1 - 0)/1;
    input2 = (in2 - 0)/1;
    input3 = (in3 - 0)/1;
    switch(index) {
    case 0:
      return neuron0x3c5e170();
    default:
      return 0.;
    }
  }

  double ScatterEnergy::Value(int index, double* input) {
    input0 = (input[0] - 0)/1;
    input1 = (input[1] - 0)/1;
    input2 = (input[2] - 0)/1;
    input3 = (input[3] - 0)/1;
    switch(index) {
    case 0:
      return neuron0x3c5e170();
    default:
      return 0.;
    }
  }

  double ScatterEnergy::neuron0x3c57c70() const {
    return input0;
  }

  double ScatterEnergy::neuron0x3c57fb0() const {
    return input1;
  }

  double ScatterEnergy::neuron0x3c582f0() const {
    return input2;
  }

  double ScatterEnergy::neuron0x3c58630() const {
    return input3;
  }

  double ScatterEnergy::input0x3c58aa0() const {
    double input = -0.243759;
    input += synapse0x3c608b0();
    input += synapse0x3c58d50();
    input += synapse0x3c58d90();
    input += synapse0x3c58dd0();
    return input;
  }

  double ScatterEnergy::neuron0x3c58aa0() const {
    double input = input0x3c58aa0();
    return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
  }

  double ScatterEnergy::input0x3c58e10() const {
    double input = 0.695511;
    input += synapse0x3c59150();
    input += synapse0x3c59190();
    input += synapse0x3c591d0();
    input += synapse0x3c59210();
    return input;
  }

  double ScatterEnergy::neuron0x3c58e10() const {
    double input = input0x3c58e10();
    return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
  }

  double ScatterEnergy::input0x3c59250() const {
    double input = -2.19144;
    input += synapse0x3c59590();
    input += synapse0x3c595d0();
    input += synapse0x3c59610();
    input += synapse0x3c59650();
    return input;
  }

  double ScatterEnergy::neuron0x3c59250() const {
    double input = input0x3c59250();
    return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
  }

  double ScatterEnergy::input0x3c59690() const {
    double input = 0.14492;
    input += synapse0x3c599d0();
    input += synapse0x3c59a10();
    input += synapse0x3c59a50();
    input += synapse0x3c59a90();
    return input;
  }

  double ScatterEnergy::neuron0x3c59690() const {
    double input = input0x3c59690();
    return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
  }

  double ScatterEnergy::input0x3c59ad0() const {
    double input = -0.169776;
    input += synapse0x3c59e10();
    input += synapse0x372a310();
    input += synapse0x372a350();
    input += synapse0x3c59f60();
    return input;
  }

double ScatterEnergy::neuron0x3c59ad0() const {
   double input = input0x3c59ad0();
   return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
}

double ScatterEnergy::input0x3c59fa0() const {
   double input = 0.956044;
   input += synapse0x3c5a2e0();
   input += synapse0x3c5a320();
   input += synapse0x3c5a360();
   input += synapse0x3c5a3a0();
   return input;
}

double ScatterEnergy::neuron0x3c59fa0() const {
   double input = input0x3c59fa0();
   return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
}

double ScatterEnergy::input0x3c5a3e0() const {
   double input = 0.335411;
   input += synapse0x3c5a720();
   input += synapse0x3c5a760();
   input += synapse0x3c5a7a0();
   input += synapse0x3c5a7e0();
   return input;
}

double ScatterEnergy::neuron0x3c5a3e0() const {
   double input = input0x3c5a3e0();
   return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
}

double ScatterEnergy::input0x3c5a820() const {
   double input = 0.391416;
   input += synapse0x3c5ab60();
   input += synapse0x3c5aba0();
   input += synapse0x3c5abe0();
   input += synapse0x3c5ac20();
   return input;
}

double ScatterEnergy::neuron0x3c5a820() const {
   double input = input0x3c5a820();
   return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
}

double ScatterEnergy::input0x3c5ac60() const {
   double input = -0.463973;
   input += synapse0x3c5afa0();
   input += synapse0x3c608f0();
   input += synapse0x3c60930();
   input += synapse0x3c59e50();
   return input;
}

double ScatterEnergy::neuron0x3c5ac60() const {
   double input = input0x3c5ac60();
   return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
}

double ScatterEnergy::input0x372a490() const {
   double input = 1.72755;
   input += synapse0x3c59f20();
   input += synapse0x3c5b310();
   input += synapse0x3c5b350();
   input += synapse0x3c5b390();
   return input;
}

double ScatterEnergy::neuron0x372a490() const {
   double input = input0x372a490();
   return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
}

double ScatterEnergy::input0x3c5b3d0() const {
   double input = -0.425939;
   input += synapse0x3c5b710();
   input += synapse0x3c5b750();
   input += synapse0x3c5b790();
   input += synapse0x3c5b7d0();
   input += synapse0x3c5b810();
   input += synapse0x3c5b850();
   input += synapse0x3c5b890();
   input += synapse0x3c5b8d0();
   input += synapse0x3c5b910();
   input += synapse0x3c5b950();
   return input;
}

double ScatterEnergy::neuron0x3c5b3d0() const {
   double input = input0x3c5b3d0();
   return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
}

double ScatterEnergy::input0x3c5b990() const {
   double input = -0.411842;
   input += synapse0x3c5bcd0();
   input += synapse0x3c5bd10();
   input += synapse0x3c5bd50();
   input += synapse0x3c5bd90();
   input += synapse0x3c5bdd0();
   input += synapse0x3c5be10();
   input += synapse0x3c5be50();
   input += synapse0x3c5be90();
   input += synapse0x3c5bed0();
   input += synapse0x3c5bf10();
   return input;
}

double ScatterEnergy::neuron0x3c5b990() const {
   double input = input0x3c5b990();
   return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
}

double ScatterEnergy::input0x3c5bf50() const {
   double input = -0.356588;
   input += synapse0x372a160();
   input += synapse0x372a1a0();
   input += synapse0x3c5c3a0();
   input += synapse0x3c5c3e0();
   input += synapse0x3c5c420();
   input += synapse0x3c5afe0();
   input += synapse0x3c5b020();
   input += synapse0x3c5b060();
   input += synapse0x3c5b0a0();
   input += synapse0x3c5b0e0();
   return input;
}

double ScatterEnergy::neuron0x3c5bf50() const {
   double input = input0x3c5bf50();
   return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
}

double ScatterEnergy::input0x3c5c870() const {
   double input = -0.683731;
   input += synapse0x3c5cbb0();
   input += synapse0x3c5cbf0();
   input += synapse0x3c5cc30();
   input += synapse0x3c5cc70();
   input += synapse0x3c5ccb0();
   input += synapse0x3c5ccf0();
   input += synapse0x3c5cd30();
   input += synapse0x3c5cd70();
   input += synapse0x3c5cdb0();
   input += synapse0x3c5cdf0();
   return input;
}

double ScatterEnergy::neuron0x3c5c870() const {
   double input = input0x3c5c870();
   return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
}

double ScatterEnergy::input0x3c5ce30() const {
   double input = -1.04778;
   input += synapse0x3c5d170();
   input += synapse0x3c5d1b0();
   input += synapse0x3c5d1f0();
   input += synapse0x3c5d230();
   input += synapse0x3c5d270();
   input += synapse0x3c5d2b0();
   input += synapse0x3c5d2f0();
   input += synapse0x3c5d330();
   input += synapse0x3c5d370();
   input += synapse0x3c5d3b0();
   return input;
}

double ScatterEnergy::neuron0x3c5ce30() const {
   double input = input0x3c5ce30();
   return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
}

double ScatterEnergy::input0x3c5d3f0() const {
   double input = 1.86526;
   input += synapse0x3c5d730();
   input += synapse0x3c5d770();
   input += synapse0x3c5d7b0();
   input += synapse0x3c5d7f0();
   input += synapse0x3c5d830();
   return input;
}

double ScatterEnergy::neuron0x3c5d3f0() const {
   double input = input0x3c5d3f0();
   return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
}

double ScatterEnergy::input0x3c5d870() const {
   double input = 0.662894;
   input += synapse0x3c5dbb0();
   input += synapse0x3c5dbf0();
   input += synapse0x3c5dc30();
   input += synapse0x3c5dc70();
   input += synapse0x3c5dcb0();
   return input;
}

double ScatterEnergy::neuron0x3c5d870() const {
   double input = input0x3c5d870();
   return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
}

double ScatterEnergy::input0x3c5dcf0() const {
   double input = 0.320619;
   input += synapse0x3c5e030();
   input += synapse0x3c5e070();
   input += synapse0x3c5e0b0();
   input += synapse0x3c5e0f0();
   input += synapse0x3c5e130();
   return input;
}

double ScatterEnergy::neuron0x3c5dcf0() const {
   double input = input0x3c5dcf0();
   return ((input < -709. ? 0. : (1/(1+exp(-input)))) * 1)+0;
}

double ScatterEnergy::input0x3c5e170() const {
   double input = -0.595635;
   input += synapse0x3c5e4b0();
   input += synapse0x3c5e4f0();
   input += synapse0x3c5e530();
   return input;
}

double ScatterEnergy::neuron0x3c5e170() const {
   double input = input0x3c5e170();
   return (input * 1)+0;
}

double ScatterEnergy::synapse0x3c608b0() const {
   return (neuron0x3c57c70()*0.124177);
}

double ScatterEnergy::synapse0x3c58d50() const {
   return (neuron0x3c57fb0()*-0.044634);
}

double ScatterEnergy::synapse0x3c58d90() const {
   return (neuron0x3c582f0()*-0.264487);
}

double ScatterEnergy::synapse0x3c58dd0() const {
   return (neuron0x3c58630()*2.83015);
}

double ScatterEnergy::synapse0x3c59150() const {
   return (neuron0x3c57c70()*0.0654539);
}

double ScatterEnergy::synapse0x3c59190() const {
   return (neuron0x3c57fb0()*-0.258563);
}

double ScatterEnergy::synapse0x3c591d0() const {
   return (neuron0x3c582f0()*0.0916401);
}

double ScatterEnergy::synapse0x3c59210() const {
   return (neuron0x3c58630()*4.46496);
}

double ScatterEnergy::synapse0x3c59590() const {
   return (neuron0x3c57c70()*0.635446);
}

double ScatterEnergy::synapse0x3c595d0() const {
   return (neuron0x3c57fb0()*0.417829);
}

double ScatterEnergy::synapse0x3c59610() const {
   return (neuron0x3c582f0()*-0.653016);
}

double ScatterEnergy::synapse0x3c59650() const {
   return (neuron0x3c58630()*0.0703743);
}

double ScatterEnergy::synapse0x3c599d0() const {
   return (neuron0x3c57c70()*-0.331704);
}

double ScatterEnergy::synapse0x3c59a10() const {
   return (neuron0x3c57fb0()*-0.308554);
}

double ScatterEnergy::synapse0x3c59a50() const {
   return (neuron0x3c582f0()*0.130074);
}

double ScatterEnergy::synapse0x3c59a90() const {
   return (neuron0x3c58630()*2.61857);
}

double ScatterEnergy::synapse0x3c59e10() const {
   return (neuron0x3c57c70()*0.160263);
}

double ScatterEnergy::synapse0x372a310() const {
   return (neuron0x3c57fb0()*-0.2686);
}

double ScatterEnergy::synapse0x372a350() const {
   return (neuron0x3c582f0()*-0.425967);
}

double ScatterEnergy::synapse0x3c59f60() const {
   return (neuron0x3c58630()*0.81142);
}

double ScatterEnergy::synapse0x3c5a2e0() const {
   return (neuron0x3c57c70()*0.104498);
}

double ScatterEnergy::synapse0x3c5a320() const {
   return (neuron0x3c57fb0()*0.228741);
}

double ScatterEnergy::synapse0x3c5a360() const {
   return (neuron0x3c582f0()*-0.13488);
}

double ScatterEnergy::synapse0x3c5a3a0() const {
   return (neuron0x3c58630()*-0.169195);
}

double ScatterEnergy::synapse0x3c5a720() const {
   return (neuron0x3c57c70()*-0.489546);
}

double ScatterEnergy::synapse0x3c5a760() const {
   return (neuron0x3c57fb0()*-0.13285);
}

double ScatterEnergy::synapse0x3c5a7a0() const {
   return (neuron0x3c582f0()*0.00604201);
}

double ScatterEnergy::synapse0x3c5a7e0() const {
   return (neuron0x3c58630()*-0.528516);
}

double ScatterEnergy::synapse0x3c5ab60() const {
   return (neuron0x3c57c70()*0.179242);
}

double ScatterEnergy::synapse0x3c5aba0() const {
   return (neuron0x3c57fb0()*-0.0687927);
}

double ScatterEnergy::synapse0x3c5abe0() const {
   return (neuron0x3c582f0()*0.369328);
}

double ScatterEnergy::synapse0x3c5ac20() const {
   return (neuron0x3c58630()*-0.601143);
}

double ScatterEnergy::synapse0x3c5afa0() const {
   return (neuron0x3c57c70()*0.142731);
}

double ScatterEnergy::synapse0x3c608f0() const {
   return (neuron0x3c57fb0()*0.362247);
}

double ScatterEnergy::synapse0x3c60930() const {
   return (neuron0x3c582f0()*-0.386656);
}

double ScatterEnergy::synapse0x3c59e50() const {
   return (neuron0x3c58630()*-0.639771);
}

double ScatterEnergy::synapse0x3c59f20() const {
   return (neuron0x3c57c70()*-0.519383);
}

double ScatterEnergy::synapse0x3c5b310() const {
   return (neuron0x3c57fb0()*-0.413791);
}

double ScatterEnergy::synapse0x3c5b350() const {
   return (neuron0x3c582f0()*-1.75648);
}

double ScatterEnergy::synapse0x3c5b390() const {
   return (neuron0x3c58630()*0.0455314);
}

double ScatterEnergy::synapse0x3c5b710() const {
   return (neuron0x3c58aa0()*0.00267344);
}

double ScatterEnergy::synapse0x3c5b750() const {
   return (neuron0x3c58e10()*-0.141334);
}

double ScatterEnergy::synapse0x3c5b790() const {
   return (neuron0x3c59250()*0.887474);
}

double ScatterEnergy::synapse0x3c5b7d0() const {
   return (neuron0x3c59690()*0.229934);
}

double ScatterEnergy::synapse0x3c5b810() const {
   return (neuron0x3c59ad0()*-0.50555);
}

double ScatterEnergy::synapse0x3c5b850() const {
   return (neuron0x3c59fa0()*-0.587792);
}

double ScatterEnergy::synapse0x3c5b890() const {
   return (neuron0x3c5a3e0()*0.422558);
}

double ScatterEnergy::synapse0x3c5b8d0() const {
   return (neuron0x3c5a820()*0.123072);
}

double ScatterEnergy::synapse0x3c5b910() const {
   return (neuron0x3c5ac60()*0.02948);
}

double ScatterEnergy::synapse0x3c5b950() const {
   return (neuron0x372a490()*-1.01348);
}

double ScatterEnergy::synapse0x3c5bcd0() const {
   return (neuron0x3c58aa0()*-0.084771);
}

double ScatterEnergy::synapse0x3c5bd10() const {
   return (neuron0x3c58e10()*-0.0888054);
}

double ScatterEnergy::synapse0x3c5bd50() const {
   return (neuron0x3c59250()*0.713177);
}

double ScatterEnergy::synapse0x3c5bd90() const {
   return (neuron0x3c59690()*-0.164155);
}

double ScatterEnergy::synapse0x3c5bdd0() const {
   return (neuron0x3c59ad0()*-0.146811);
}

double ScatterEnergy::synapse0x3c5be10() const {
   return (neuron0x3c59fa0()*-0.497152);
}

double ScatterEnergy::synapse0x3c5be50() const {
   return (neuron0x3c5a3e0()*0.198825);
}

double ScatterEnergy::synapse0x3c5be90() const {
   return (neuron0x3c5a820()*-0.224238);
}

double ScatterEnergy::synapse0x3c5bed0() const {
   return (neuron0x3c5ac60()*-0.276587);
}

double ScatterEnergy::synapse0x3c5bf10() const {
   return (neuron0x372a490()*-0.633448);
}

double ScatterEnergy::synapse0x372a160() const {
   return (neuron0x3c58aa0()*0.0336298);
}

double ScatterEnergy::synapse0x372a1a0() const {
   return (neuron0x3c58e10()*-0.408048);
}

double ScatterEnergy::synapse0x3c5c3a0() const {
   return (neuron0x3c59250()*0.980242);
}

double ScatterEnergy::synapse0x3c5c3e0() const {
   return (neuron0x3c59690()*-0.144517);
}

double ScatterEnergy::synapse0x3c5c420() const {
   return (neuron0x3c59ad0()*-0.356292);
}

double ScatterEnergy::synapse0x3c5afe0() const {
   return (neuron0x3c59fa0()*-0.895875);
}

double ScatterEnergy::synapse0x3c5b020() const {
   return (neuron0x3c5a3e0()*0.181922);
}

double ScatterEnergy::synapse0x3c5b060() const {
   return (neuron0x3c5a820()*-0.459524);
}

double ScatterEnergy::synapse0x3c5b0a0() const {
   return (neuron0x3c5ac60()*0.364972);
}

double ScatterEnergy::synapse0x3c5b0e0() const {
   return (neuron0x372a490()*-1.64202);
}

double ScatterEnergy::synapse0x3c5cbb0() const {
   return (neuron0x3c58aa0()*0.2525);
}

double ScatterEnergy::synapse0x3c5cbf0() const {
   return (neuron0x3c58e10()*-0.463705);
}

double ScatterEnergy::synapse0x3c5cc30() const {
   return (neuron0x3c59250()*1.70646);
}

double ScatterEnergy::synapse0x3c5cc70() const {
   return (neuron0x3c59690()*0.0884919);
}

double ScatterEnergy::synapse0x3c5ccb0() const {
   return (neuron0x3c59ad0()*-1.08151);
}

double ScatterEnergy::synapse0x3c5ccf0() const {
   return (neuron0x3c59fa0()*-1.23203);
}

double ScatterEnergy::synapse0x3c5cd30() const {
   return (neuron0x3c5a3e0()*-0.19954);
}

double ScatterEnergy::synapse0x3c5cd70() const {
   return (neuron0x3c5a820()*0.0325257);
}

double ScatterEnergy::synapse0x3c5cdb0() const {
   return (neuron0x3c5ac60()*0.314493);
}

double ScatterEnergy::synapse0x3c5cdf0() const {
   return (neuron0x372a490()*-1.74909);
}

double ScatterEnergy::synapse0x3c5d170() const {
   return (neuron0x3c58aa0()*-0.0244779);
}

double ScatterEnergy::synapse0x3c5d1b0() const {
   return (neuron0x3c58e10()*-0.693285);
}

double ScatterEnergy::synapse0x3c5d1f0() const {
   return (neuron0x3c59250()*2.05803);
}

double ScatterEnergy::synapse0x3c5d230() const {
   return (neuron0x3c59690()*0.177365);
}

double ScatterEnergy::synapse0x3c5d270() const {
   return (neuron0x3c59ad0()*-1.1025);
}

double ScatterEnergy::synapse0x3c5d2b0() const {
   return (neuron0x3c59fa0()*-1.46397);
}

double ScatterEnergy::synapse0x3c5d2f0() const {
   return (neuron0x3c5a3e0()*0.114601);
}

double ScatterEnergy::synapse0x3c5d330() const {
   return (neuron0x3c5a820()*0.286396);
}

double ScatterEnergy::synapse0x3c5d370() const {
   return (neuron0x3c5ac60()*0.320963);
}

double ScatterEnergy::synapse0x3c5d3b0() const {
   return (neuron0x372a490()*-1.68032);
}

double ScatterEnergy::synapse0x3c5d730() const {
   return (neuron0x3c5b3d0()*-0.740042);
}

double ScatterEnergy::synapse0x3c5d770() const {
   return (neuron0x3c5b990()*-0.201459);
}

double ScatterEnergy::synapse0x3c5d7b0() const {
   return (neuron0x3c5bf50()*-1.83814);
}

double ScatterEnergy::synapse0x3c5d7f0() const {
   return (neuron0x3c5c870()*-0.775049);
}

double ScatterEnergy::synapse0x3c5d830() const {
   return (neuron0x3c5ce30()*-2.13834);
}

double ScatterEnergy::synapse0x3c5dbb0() const {
   return (neuron0x3c5b3d0()*-0.58143);
}

double ScatterEnergy::synapse0x3c5dbf0() const {
   return (neuron0x3c5b990()*-1.13905);
}

double ScatterEnergy::synapse0x3c5dc30() const {
   return (neuron0x3c5bf50()*-0.729314);
}

double ScatterEnergy::synapse0x3c5dc70() const {
   return (neuron0x3c5c870()*-0.64363);
}

double ScatterEnergy::synapse0x3c5dcb0() const {
   return (neuron0x3c5ce30()*-1.15768);
}

double ScatterEnergy::synapse0x3c5e030() const {
   return (neuron0x3c5b3d0()*-0.324437);
}

double ScatterEnergy::synapse0x3c5e070() const {
   return (neuron0x3c5b990()*-0.943852);
}

double ScatterEnergy::synapse0x3c5e0b0() const {
   return (neuron0x3c5bf50()*-0.599828);
}

  double ScatterEnergy::synapse0x3c5e0f0() const {
    return (neuron0x3c5c870()*-0.980052);
  }

  double ScatterEnergy::synapse0x3c5e130() const {
    return (neuron0x3c5ce30()*-1.03851);
  }

  double ScatterEnergy::synapse0x3c5e4b0() const {
    return (neuron0x3c5d3f0()*3.11935);
  }

  double ScatterEnergy::synapse0x3c5e4f0() const {
    return (neuron0x3c5d870()*1.5416);
  }

  double ScatterEnergy::synapse0x3c5e530() const {
    return (neuron0x3c5dcf0()*1.06868);
  }

}
