////////////////////////////////////////////////////////////////////////
// Class:       LightLevels
// Module Type: analyzer
// File:        LightLevels_module.cc
//
// Generated at Tue Apr  9 23:39:58 2013 by Adam Aurisano using artmod
// from art v1_02_06.
////////////////////////////////////////////////////////////////////////

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Core/FindManyP.h"
#include "art/Framework/Services/Registry/ServiceHandle.h" 
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Persistency/Common/Ptr.h"
#include "art/Persistency/Common/PtrVector.h"
#include "art/Utilities/InputTag.h"

#include "Geometry/Geometry.h"
#include "Geometry/LiveGeometry.h"

#include "GeometryObjects/GeometryBase.h"
#include "GeometryObjects/Geo.h"
#include "GeometryObjects/CellGeo.h"
#include "GeometryObjects/PlaneGeo.h"

#include "ChannelInfo/BadChanList.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/Cluster.h"
#include "RecoBase/Track.h"
#include "Utilities/func/MathUtil.h"
#include "CalibrationUtils/CalibUtil.h"
#include "CalibrationDataProducts/PCHit.h"
#include "CMap/CMap.h"
#include "RunHistory/service/RunHistoryService.h"
#include "NovaDAQConventions/DAQConventions.h"
#include "cetlib/search_path.h"

#include <cmath>
#include <iostream>
#include <iomanip>
#include <map>
#include <unordered_map>
#include <vector>
#include <string>
#include <set>
#include <utility>
#include <algorithm>

#include "TFile.h"
#include "TTree.h"
#include "TH1D.h"
#include "TH2F.h"
#include "TH2D.h"
#include "TH3D.h"
#include "TProfile.h"
#include "TString.h"
#include "TVector3.h"

#include <sys/stat.h>

#include <iostream>
#include <iomanip>

class LightLevels;

class LightLevels : public art::EDAnalyzer {
public:
  explicit LightLevels(fhicl::ParameterSet const & p);
  virtual ~LightLevels();

  void analyze(art::Event const & e) override;

  void beginJob() override;
  void beginRun(const art::Run& run) override;
  void endRun(const art::Run& run) override;
  
  void CreateHistos();

private:
  // Declare member data here.
  TH1D*  fNSpills;
  TH1D*  fNActiveCells;
  TH1D*  fExposure;

  TH2D* fPEperCM;
  TH2D* fPEperCM_X;
  TH2D* fPEperCM_Y;

  TH2D* fPEperCM_all;
  TH2D* fPEperCM_all_X;
  TH2D* fPEperCM_all_Y;

  TH1D* fPEperCM1D;
  TH1D* fPEperCM1D_X;
  TH1D* fPEperCM1D_Y;

  TH1D* fPEperCM1D_all;
  TH1D* fPEperCM1D_all_X;
  TH1D* fPEperCM1D_all_Y;

  TH1D* fPE;
  TH1D* fPE_X;
  TH1D* fPE_Y;

  TH1D* fPE_all;
  TH1D* fPE_all_X;
  TH1D* fPE_all_Y;

  TH1D* fPathLength;
  TH1D* fPathLength_X;
  TH1D* fPathLength_Y;

  TH1D* fPathLength_all;
  TH1D* fPathLength_all_X;
  TH1D* fPathLength_all_Y;

  TH1D* fCosDirX;
  TH1D* fCosDirY;
  TH1D* fCosDirZ;

  TH1D* fPhysicsHits;

  TH1D*  fNoiseHits;
  TH2D*  fNoiseHitsPerCell;
  TH3D*  fNoiseHits3D;

  TTree* fLightLevelTree;

  Float_t _pathLength;
  Float_t _w;
  Float_t _pe;
  Int_t   _plane;
  Int_t   _cell;
  Float_t _distFromEnd;
  Bool_t  _isStopper;
  Float_t _cosX;
  Float_t _cosY;
  Float_t _cosZ;
  Bool_t  _isX;

  double fMaxW;

  bool fFirstTime;
  bool fMakeTree;

  std::vector<std::string> fQualLabels;
};

LightLevels::LightLevels(fhicl::ParameterSet const & p) :
  EDAnalyzer(p), fMakeTree( p.get<bool>("MakeTree") )
{}

LightLevels::~LightLevels()
{}

void LightLevels::beginJob()
{
  fFirstTime = true;
}

void LightLevels::endRun(const art::Run& run)
{
  //Fill fNActiveCells with the number of non-zero cells in fNoiseHitsPerCell
  double nActiveCells(0.0);
  for (int iX = 1; iX <= fNoiseHitsPerCell->GetXaxis()->GetNbins(); ++iX) {
    for (int iY = 1; iY <= fNoiseHitsPerCell->GetYaxis()->GetNbins(); ++iY) {
      if (fNoiseHitsPerCell->GetBinContent(iX, iY) > 0) nActiveCells += 1.0;
    }
  }
  fNActiveCells->SetBinContent(1, nActiveCells);
}

void LightLevels::CreateHistos()
{
  if (!fFirstTime) return;

  art::ServiceHandle<geo::Geometry> geo;
  art::ServiceHandle<art::TFileService> tfs;
  
  fMaxW = 0;
  unsigned int maxCells(0);
  unsigned int nPlanes = geo->NPlanes();
  for (unsigned int plane = 0; plane < nPlanes; plane++) {
    if (geo->Plane(plane)->Ncells() > maxCells) maxCells = geo->Plane(plane)->Ncells();
    for (unsigned int cell = 0; cell < geo->Plane(plane)->Ncells(); cell++) {
      if (geo->Plane(plane)->Cell(cell)->HalfL() > fMaxW) fMaxW = geo->Plane(plane)->Cell(cell)->HalfL(); 
    }
  }

  double cmPerBin = 5.0;
  int    nBinsW   = 2.*fMaxW/cmPerBin;

  fNSpills        = tfs->make<TH1D>("fNSpills",        ";Number of Spills", 1, 0, 2);
  fNActiveCells   = tfs->make<TH1D>("fNActiveCells",   ";Number of Active Cells", 1, 0, 2);
  fExposure       = tfs->make<TH1D>("fExposure",        ";Total Exposure Time (ms)", 1, 0, 2);

  fPEperCM       = tfs->make<TH2D>("pPEperCM",  ";Distance to Readout End of Cell; PE/Pathlength",          nBinsW, 0, 2*fMaxW, 2000, 0, 500);
  fPEperCM_X     = tfs->make<TH2D>("pPEperCM_X",";Distance to Readout End of Cell (X View); PE/Pathlength", nBinsW, 0, 2*fMaxW, 2000, 0, 500);
  fPEperCM_Y     = tfs->make<TH2D>("pPEperCM_Y",";Distance to Readout End of Cell (Y View); PE/Pathlength", nBinsW, 0, 2*fMaxW, 2000, 0, 500);

  fPEperCM_all       = tfs->make<TH2D>("pPEperCM_all",  ";Distance to Readout End of Cell; PE/Pathlength",          nBinsW, 0, 2*fMaxW, 2000, 0, 500);
  fPEperCM_all_X     = tfs->make<TH2D>("pPEperCM_all_X",";Distance to Readout End of Cell (X View); PE/Pathlength", nBinsW, 0, 2*fMaxW, 2000, 0, 500);
  fPEperCM_all_Y     = tfs->make<TH2D>("pPEperCM_all_Y",";Distance to Readout End of Cell (Y View); PE/Pathlength", nBinsW, 0, 2*fMaxW, 2000, 0, 500);
  
  fPEperCM1D     = tfs->make<TH1D>("fPEperCM1D",  ";Mean PE/Pathlength", 2000, 0, 500);
  fPEperCM1D_X   = tfs->make<TH1D>("fPEperCM1D_X",";Mean PE/Pathlength", 2000, 0, 500);
  fPEperCM1D_Y   = tfs->make<TH1D>("fPEperCM1D_Y",";Mean PE/Pathlength", 2000, 0, 500);

  fPEperCM1D_all     = tfs->make<TH1D>("fPEperCM1D_all",  ";Mean PE/Pathlength", 2000, 0, 500);
  fPEperCM1D_all_X   = tfs->make<TH1D>("fPEperCM1D_all_X",";Mean PE/Pathlength", 2000, 0, 500);
  fPEperCM1D_all_Y   = tfs->make<TH1D>("fPEperCM1D_all_Y",";Mean PE/Pathlength", 2000, 0, 500);
  
  fPE           = tfs->make<TH1D>("fPE",   ";Hit PE",         512, 0, 2048);
  fPE_X         = tfs->make<TH1D>("fPE_X", ";Hit PE: X View", 512, 0, 2048);
  fPE_Y         = tfs->make<TH1D>("fPE_Y", ";Hit PE: Y View", 512, 0, 2048);

  fPE_all           = tfs->make<TH1D>("fPE_all",   ";Hit PE",         512, 0, 2048);
  fPE_all_X         = tfs->make<TH1D>("fPE_all_X", ";Hit PE: X View", 512, 0, 2048);
  fPE_all_Y         = tfs->make<TH1D>("fPE_all_Y", ";Hit PE: Y View", 512, 0, 2048);

  fPathLength     = tfs->make<TH1D>("fPathLength",   ";Path Length",         100 ,0, 10);
  fPathLength_X   = tfs->make<TH1D>("fPathLength_X", ";Path Length: X View", 100 ,0, 10);
  fPathLength_Y   = tfs->make<TH1D>("fPathLength_Y", ";Path Length: Y View", 100 ,0, 10);

  fPathLength_all     = tfs->make<TH1D>("fPathLength_all",   ";Path Length",         100 ,0, 10);
  fPathLength_all_X   = tfs->make<TH1D>("fPathLength_all_X", ";Path Length: X View", 100 ,0, 10);
  fPathLength_all_Y   = tfs->make<TH1D>("fPathLength_all_Y", ";Path Length: Y View", 100 ,0, 10);

  fCosDirX       = tfs->make<TH1D>("fCosDirX", ";X Direction Cosine", 100, 0, 1);
  fCosDirY       = tfs->make<TH1D>("fCosDirY", ";Y Direction Cosine", 100, 0, 1);
  fCosDirZ       = tfs->make<TH1D>("fCosDirZ", ";Z Direction Cosine", 100, 0, 1);

  fPhysicsHits   = tfs->make<TH1D>("fPhysicsHits", ";N Physics Hits",1000, 0, 1000);

  fNoiseHits         = tfs->make<TH1D>("fNoiseHits",              ";Noise Slice ADC", 512, 0, 4095);
  fNoiseHitsPerCell  = tfs->make<TH2D>("fNoiseHitsPerCell",       ";Plane;Cell;Noise Hits", nPlanes, 0, nPlanes-1, maxCells, 0, maxCells-1);  
  fNoiseHits3D       = tfs->make<TH3D>("fNoiseHits3D",            ";ADC1-ADC0;ADC2-ADC0;ADC3-ADC0", 150, 0, 300, 150, 0, 300, 150, 0, 300);

  fLightLevelTree = tfs->make<TTree>("lightLevelTree","Information for setting light levels");

  fLightLevelTree->Branch("pathLength",  &_pathLength,  "pathLength/F");
  fLightLevelTree->Branch("w",           &_w,           "w/F");
  fLightLevelTree->Branch("pe",          &_pe,          "pe/F");
  fLightLevelTree->Branch("plane",       &_plane,       "plane/I");
  fLightLevelTree->Branch("cell",        &_cell,        "cell/I");
  fLightLevelTree->Branch("distFromEnd", &_distFromEnd, "distFromENd/F");
  fLightLevelTree->Branch("isStopper",   &_isStopper,   "isStopper/B");
  fLightLevelTree->Branch("isX",         &_isX,         "isX/B");
  fLightLevelTree->Branch("cosX",        &_cosX,        "coxX/F");
  fLightLevelTree->Branch("cosY",        &_cosY,        "coxY/F");
  fLightLevelTree->Branch("cosZ",        &_cosZ,        "cosZ/F");

  fQualLabels.push_back("pathQualXY");
  fQualLabels.push_back("pathQualZ");

  fFirstTime = false;
  return;
}

void LightLevels::beginRun(const art::Run& run)
{
  CreateHistos();
  return;
}

void LightLevels::analyze(art::Event const & evt)
{
  std::string trackLabel = "cosmictrack";
  std::string pclistLabel = "pclist";
  
  art::ServiceHandle<geo::Geometry> geo;

  //if there are no hits, this event was probably filtered

  art::Handle< std::vector<rb::CellHit> > hits;
  evt.getByLabel("calhit", hits);
  if (hits.failedToGet())
    {
      //std::cerr << "Failed to get calhits" << std::endl;
      return;
    }

  fNSpills->Fill(1);

  // get the slices                                                                                                                                                     
  art::Handle< std::vector< rb::Cluster > > slices;
  evt.getByLabel("slicer", slices);
  if (slices.failedToGet()) {
    std::cerr << "Failed to get slices" << std::endl;
    return;
  }

  double minTime(999999.0);
  double maxTime(-999999.0);
  for(unsigned int i = 0; i < hits->size(); ++i)
    {
      double tns = (*hits)[i].TNS();
      if (i == 0) {
        maxTime = tns;
	minTime = tns;
      }
      else {
	if (tns > maxTime) maxTime = tns;
	if (tns < minTime) minTime = tns;
      }
    }
  
  double deltaTMS = fExposure->GetBinContent(1);
  deltaTMS += (maxTime - minTime)/1.0e6;
  fExposure->SetBinContent(1, deltaTMS);
  
  art::ServiceHandle<cmap::CMap> cmap;

  for(unsigned int i = 0; i < slices->size(); ++i) {
    if ((*slices)[i].IsNoise()) {
      for(unsigned int j = 0; j < (*slices)[i].NCell(); ++j) {
	const rb::CellHit* h = (*slices)[i].Cell(j).get();      
	fNoiseHits->Fill( h->ADC() );
	if (h->NADC() > 1) fNoiseHits3D->Fill( h->ADC(1) - h->ADC(0), h->ADC(2) - h->ADC(0), h->ADC(3) - h->ADC(0) );
	const int plane = cmap->GetPlane(h);
	const int cell  = cmap->GetCell(h);
	double nhits = fNoiseHitsPerCell->GetBinContent(plane, cell);
	nhits += 1.0;
	fNoiseHitsPerCell->SetBinContent(plane, cell, nhits);
      }
    }
    else {
      int nhits = (*slices)[i].NCell();
      art::FindManyP<rb::Track> fmptrks(slices, evt, art::InputTag(trackLabel));
      if (!fmptrks.isValid()) {
	std::cerr << "No associated cosmic tracks for slice!" << std::endl;
	return;
      }
      int nTrackHits(0);
      std::vector<art::Ptr<rb::Track> > sliceTracks = fmptrks.at(i);
      for( auto const& slcTrk: sliceTracks ) {
	nTrackHits += slcTrk->NCell();
      }
      fPhysicsHits->Fill(nhits - nTrackHits);
    }
  }

  art::Handle<std::vector<rb::Track> > tracks;
  evt.getByLabel(trackLabel, tracks);
  if (tracks.failedToGet()) {
    std::cerr << "Failed to get cosmic tracks" << std::endl;
    return;
  }      
  if (tracks->empty()) {
    //std::cerr << "Empty cosmic tracks container" << std::endl;
    return;
  }      

  for (size_t trkIdx = 0; trkIdx < tracks->size(); ++trkIdx) {
    const rb::Track* track(&tracks->at(trkIdx));
    
    //3D tracks
    if(track->View() != geo::kXorY) continue;
    
    //sufficient length for track window
    if ( track->TotalLength() < 200. ) continue;
    
    double dx = std::fabs( track->Dir().X() );
    double dy = std::fabs( track->Dir().Y() );
    double dz = std::fabs( track->Dir().Z() );

    fCosDirX->Fill( dx );
    fCosDirY->Fill( dy );
    fCosDirZ->Fill( dz );

    _cosX = dx;
    _cosY = dy;
    _cosZ = dz;

    //Determine if tracks come from outside and stop
    art::ServiceHandle<geo::LiveGeometry> livegeo;
    
    double distStartToFront = livegeo->DistToFront(track->Start());
    double distStartToTop   = livegeo->DistToTop(track->Start());
    double distStartToSide  = livegeo->DistToEdgeX(track->Start());

    //physical starts

    if (distStartToFront < 0) continue;
    if (distStartToTop < 0) continue;
    if (distStartToSide < 10) continue;


    //make sure it start near front for neardet (must be rock muon)
    //if (geo->DetId() == novadaq::cnv::kNEARDET && distStartToFront > 10) continue;
    //make sure it start near top for fardet (must be cosmic muon)
    //if (geo->DetId() == novadaq::cnv::kFARDET && distStartToTop > 10) continue;
    //moving to the activity trigger

    if (distStartToTop > 10) continue;

    // endpoint isn't contained
    double distEndToEdge = std::min(livegeo->DistToEdgeXY(track->Stop()), livegeo->DistToEdgeZ(track->Stop()));
    bool isStopper = true;
    if (distEndToEdge < 50) isStopper = false;

    // fetch pc hits
    for (size_t iQual = 0; iQual < fQualLabels.size(); ++iQual) {
      art::FindManyP<caldp::PCHit> fmpcxy(tracks, evt, art::InputTag(pclistLabel, fQualLabels[iQual].c_str()) );
      if (!fmpcxy.isValid()) {
	std::cerr << "No associated pchits for cosmic tracks!" << std::endl;
	return;
      }  
      
      std::vector<art::Ptr<caldp::PCHit> > pcxy = fmpcxy.at(trkIdx);
      std::cout << pcxy.size() << " pchits" << std::endl;
      for( auto const& iPc: pcxy ) {
	double pathLength  = iPc->Path();
	double w           = iPc->W();
	double pe          = iPc->PE();
	int    plane       = iPc->Plane();
	int    cell        = iPc->Cell();
	
	//remove the first module in each plane (potential light contamination)
	//if (cell < 32) continue;
	
	//don't keep hits in the muon catcher
	if(geo->DetId() == novadaq::cnv::kNEARDET && plane >= int(geo->FirstPlaneInMuonCatcher())) continue;
	
	//maxW - |w| < 0.75*maxW -> away from the edges that might be affected by the roll-offs
	const double maxW = geo->Plane(plane)->Cell(cell)->HalfL();
	bool isCentral = (maxW - std::fabs(w) < 0.75*maxW);
	
	double x = track->TotalLength() - iPc->FlightLen();
	
	_pathLength = pathLength;
	_w = w;
	_pe = pe;
	_plane = plane;
	_cell = cell;
	_distFromEnd = x;
	_isStopper = isStopper;
	if (iPc->View() == geo::kX) _isX = true;
	else                        _isX = false;
	
	if (fMakeTree) fLightLevelTree->Fill();
	
	//remove potentially low efficiency cells and low oil cells
	//int modcell = cell%32;
	//if ( modcell == 7 || modcell == 15 || modcell == 23 || modcell == 31) continue;
	//if ( iPc->View() == geo::kY && modcell == 30) continue;
	
	fPEperCM_all->Fill( maxW - w, pe/pathLength );
	fPathLength_all->Fill( pathLength );
	if (isCentral) {
	  fPE_all->Fill(pe);
	  fPEperCM1D_all->Fill( pe/pathLength );
	}
	if (iPc->View() == geo::kX) {
	  fPEperCM_all_X->Fill( maxW - w, pe/pathLength );
	  fPathLength_all_X->Fill( pathLength );
	  if (isCentral) {
	    fPE_all_X->Fill(pe);
	    fPEperCM1D_all_X->Fill( pe/pathLength );
	  }
	}
	else {
	  fPEperCM_all_Y->Fill( maxW - w, pe/pathLength );
	  fPathLength_all_Y->Fill( pathLength );
	  if (isCentral) { 
	    fPE_all_Y->Fill(pe);
	    fPEperCM1D_all_Y->Fill( pe/pathLength );
	  }
	}
	
	if (!isStopper) continue;
	if (x < 100 || x > 200) continue;
	
	fPEperCM->Fill( maxW - w, pe/pathLength );
	fPathLength->Fill( pathLength );
	if (isCentral) {
	  fPE->Fill(pe);
	  fPEperCM1D->Fill( pe/pathLength );
	}
	if (iPc->View() == geo::kX) {
	  fPEperCM_X->Fill( maxW - w, pe/pathLength );
	  fPathLength_X->Fill( pathLength );
	  if (isCentral) {
	    fPE_X->Fill(pe);
	    fPEperCM1D_X->Fill( pe/pathLength );
	  }
	}
	else {
	  fPEperCM_Y->Fill( maxW - w, pe/pathLength );
	  fPathLength_Y->Fill( pathLength );
	  if (isCentral) {
	    fPE_Y->Fill(pe);
	    fPEperCM1D_Y->Fill( pe/pathLength );
	  }
	}
      }
    }
  }
}

DEFINE_ART_MODULE(LightLevels)
