////////////////////////////////////////////////////////////////////////
/// \brief   Common configuration params for SimpleReadout, 
///          FPGAAlgorithms, NoiseMaker
///
/// \author  Christopher Backhouse
/// \date
////////////////////////////////////////////////////////////////////////
#ifndef RSIM_COMMONPARAMETERS_H
#define RSIM_COMMONPARAMETERS_H

#include "fhiclcpp/ParameterSet.h"
#include <string>

namespace rsim {

  /// Can be used as either a member holding configurations, or a mix-in
  class CommonParameters
  {
  public:
    CommonParameters(const fhicl::ParameterSet& pset);

    double fClocktick;            ///< digitization clock period, ns
    int    fNumClockticksInSpill; ///< number of clockticks in a spill
    int    fNumClockticksPerDigitization; ///< How many ticks between each ADC
    double fADCMaxPE;             ///< maximum signal size that the ADC can handle without saturating (in PE)
    double fAPDExcessNoiseFactor; ///< APD's "excess noise factor" (see .cxx for more)
    double fASICBaselineInADCCounts; ///< ASIC baseline output level, in ADC counts
    bool   fUseDBThresholds;      ///< Use thresholds from DB instead of fcl defaults
    bool   fUseRandomThresholds;  ///< Use thresholds drawn from a histogram instead of fcl defaults
    std::string fThresholdFile;   ///< File containing thresholds for the random threshold option

    double fASICRiseTime;  ///< ASIC shaping curve: rise time, ns
    double fASICFallTime;  ///< ASIC shaping curve: fall time, ns
  };

} // end namespace

#endif
