#ifndef EXCESSNOISEMAKER_H
#define EXCESSNOISEMAKER_H
//////////////////////////////////////////////////////////////////////////
/// \brief   Class for simulating the spread in photo-electrons         //
///          according to the theory of APDs presented in               //
///          P. Balaban, P.E Fleischer, H Zucker,                       //
///          IEEE Trans. Electron Devices 23, 1189-1190, (1979)         //
///          See NOVA-doc-4401 for background                           //
/// \author  aurisaam@ucmail.uc.edu					//
/// \date                                                               //
//////////////////////////////////////////////////////////////////////////

#include <map>
#include <vector>

#include "ReadoutSim/CommonParameters.h"

#include "fhiclcpp/ParameterSet.h"

#include "CLHEP/Random/RandFlat.h"

namespace art { class ParameterSet; }

namespace CLHEP { class RandFlat; }

namespace rsim {

  class ExcessNoiseMaker : protected CommonParameters {
    
  public:
    explicit ExcessNoiseMaker(const fhicl::ParameterSet& pset);
    virtual ~ExcessNoiseMaker() {}

    //Smear input PE by excess noise noise
    double DrawSmearedPE(int nPE, CLHEP::RandFlat* flat);

  protected:
    double ExcessNoisePDF(int nPE_in, double nPE_out);
    //lazy construction of noise templates
    std::map<int, std::vector<double> >::iterator MakeTemplate(int nPE);
    
    std::map<int, std::vector<double> > fPDFs;
    
    double fGain;                ///< APD gain
    double fK;
  };

}

#endif
