////////////////////////////////////////////////////////////////////////
/// \brief   TODO
///
/// \author  Christopher Backhouse - bckhouse@caltech.edu
/// \date
////////////////////////////////////////////////////////////////////////
#ifndef FPGAALGORITHMS_H
#define FPGAALGORITHMS_H

#include "ReadoutSim/CommonParameters.h"

#include "RawData/RawDigit.h"

#include "fhiclcpp/ParameterSet.h"

#include <vector>
class TH1;

namespace rsim
{
  /// Interface implemented by all FPGA algorithms
  class IFPGAAlgorithm: protected CommonParameters
  {
  public:
    friend IFPGAAlgorithm* CreateFPGAAlgorithm(const fhicl::ParameterSet&);

    virtual ~IFPGAAlgorithm();

    virtual std::vector<rawdata::RawDigit>
    ASICToDigits(double* ASIC_Output, int offset) = 0;

    double ThresholdADC() const;
    void FetchThresholds();
    void SetPlaneAndCell(int plane, int cell) {fPlane = plane; fCell = cell;}
    
  protected:
    /// Use \ref CreateFPGAAlgorithm to create
    IFPGAAlgorithm(const fhicl::ParameterSet& pset);

    double fThresholdADC;
    TH1*  fThresholdPDF;
    TH1*  fThresholds;
    std::vector< std::vector< int > > fThresholdMap;
    int fPlane;
    int fCell;
  };

  /// \brief Create and configure the correct algorithm based on \a pset
  IFPGAAlgorithm* CreateFPGAAlgorithm(const fhicl::ParameterSet& pset);

  //---------------- Implementation of specific algorithms ----------------

  class FPGA_DualCorrelatedSampling : public IFPGAAlgorithm
  {
  public:
    friend IFPGAAlgorithm* CreateFPGAAlgorithm(const fhicl::ParameterSet&);

    virtual std::vector<rawdata::RawDigit>
    ASICToDigits(double* ASIC_Output, int offset);

  protected:
    /// Use \ref CreateFPGAAlgorithm to create
    FPGA_DualCorrelatedSampling(const fhicl::ParameterSet& pset);

    int fLookbackSamples; ///< How far back we look to do the background subtraction
    double fRetriggerThreshold; ///< Fraction of threshold we have to fall to after a trigger

    int fVersion; ///< Nanoslice version, describes what points to write out
  };


  class FPGA_MatchedFiltering : public IFPGAAlgorithm
  {
  public:
    friend IFPGAAlgorithm* CreateFPGAAlgorithm(const fhicl::ParameterSet&);

    virtual std::vector<rawdata::RawDigit>
    ASICToDigits(double* ASIC_Output, int offset);

  protected:
    /// Use \ref CreateFPGAAlgorithm to create
    FPGA_MatchedFiltering(const fhicl::ParameterSet& pset);

    double PerfectASICCurve(double t) const;

    double fASICRiseTime;
    double fASICFallTime;
    double fBaselineTime;        ///< start of baseline window, FD config value
    double fEndTime;	         ///< end of matching window, FD config value
  };


  class FPGA_DumpASICOutput : public IFPGAAlgorithm
  {
  public:
    friend IFPGAAlgorithm* CreateFPGAAlgorithm(const fhicl::ParameterSet&);

    virtual std::vector<rawdata::RawDigit>
    ASICToDigits(double* ASIC_Output, int offset);

  protected:
    /// Use \ref CreateFPGAAlgorithm to create
    FPGA_DumpASICOutput(const fhicl::ParameterSet& pset);
  };
}

#endif
