#ifndef NOISEMAKER_H
#define NOISEMAKER_H
//////////////////////////////////////////////////////////////////////////
/// \brief   Class for dealing with correlated noise on the FEB         //
///          in the simulation.                                         //
///          See NOVA-doc-4401 for background                           //
/// \author  rbpatter@caltech.edu					//
/// \date                                                               //
//////////////////////////////////////////////////////////////////////////

#include <vector>

#include "ReadoutSim/CommonParameters.h"

#include "fhiclcpp/ParameterSet.h"

#include "CLHEP/Random/RandGaussQ.h"

namespace art { class ParameterSet; }

namespace CLHEP { class RandGaussQ; }

namespace rsim {

  class NoiseMaker : protected CommonParameters {
    
  public:
    explicit NoiseMaker(const fhicl::ParameterSet& pset);
    virtual ~NoiseMaker() {}

    // Output noise in PE units.  (That is, the gain is divided out.)
    double GetNoise(CLHEP::RandGaussQ* gauss){return GetCurrentNoise(gauss)+
                                                     GetVoltageNoise(gauss);}
    double GetCurrentNoise(CLHEP::RandGaussQ* gauss);
    double GetVoltageNoise(CLHEP::RandGaussQ* gauss);

    double GetNominalThreshold() {return fNominalThreshold;}

  protected:

    void ApplyDCS(std::vector<double> &rawtrace, std::vector<double> &dcstrace);
    double GetRMS(std::vector<double> &dcstrace);

    double fVoltageNoiseScale;
    double fCurrentNoiseScale;

    bool   fNeedInit;

    int    fNumNormSamples;      ///< num samples to use to set norm at init time
    double fTrise;               ///< rise time of shaper (ns)
    double fTfall; 		 ///< fall time of shaper (ns)
    double fTread; 		 ///< time between digitizations (ns)
    double fVoltageNoiseDensity; ///< voltage noise density of APD circuit (nV/Hz^0.5)
    double fDetectorCapacitance; ///< detector capacitance (pF)
    double fLeakageCurrent;      ///< APD leakage current (nA)
    double fGain;                ///< APD gain
    double fThresholdSigma;      ///< Sigma value used for setting thresholds
    
    int fFPGA_DCS_LookbackSamples; ///< For doing DCS to normalize

    void Init(CLHEP::RandGaussQ* gauss);

    // the state:
    double fLastVDraw;
    double fLastCDraw;

    double fNominalThreshold;

  };

}

#endif
