////////////////////////////////////////////////////////////////////////
// \file    FitSum.cxx
// \brief   Kinematic summary object.
// \version $Id: KineSum.cxx,v 1.0 2014-09-25 00:00:00 mbaird42 Exp $
// \author  $Author: mbaird42 $
// \date    $Date: 2014-09-25 00:00:00 $
////////////////////////////////////////////////////////////////////////
#include "RecoBase/FitSum.h"

namespace rb
{
  FitSum::FitSum():
    fPDG(0)
  {
    fChi2[0]=0;
    fChi2[1]=0;
    fChi2[2]=0;
    fChi2[3]=0;
    fNdof[0]=0;
    fNdof[1]=0;
    fNdof[2]=0;
    fNdof[3]=0;
    fFourMom.SetPxPyPzE(0.0,0.0,0.0,0.0);
  }

  FitSum::FitSum(TVector3 dir, double p, double m, int pdg, 
		 double chi2_hit1,        double chi2_hit2,
		 double chi2_angle1,    double chi2_angle2, 
		 int ndof_hit1,              int ndof_hit2,
		 int ndof_angle1,          int ndof_angle2):
    fPDG(pdg)
  {
    fChi2[0]=chi2_hit1;
    fChi2[1]=chi2_hit2;
    fChi2[2]=chi2_angle1;
    fChi2[3]=chi2_angle2;
    fNdof[0]=ndof_hit1;
    fNdof[1]=ndof_hit2;
    fNdof[2]=ndof_angle1;
    fNdof[3]=ndof_angle2;
    this->SetKin(dir,p,m);
  }
  
  //
  // Construct a four vector that contains kinematic 
  // information for each track.
  //
  void FitSum::SetKin(TVector3 dir, double p, double m)
  {
    fFourMom.SetPxPyPzE(dir.X()*p,dir.Y()*p,dir.Z()*p,sqrt(p*p+m*m));
  }
  
} // end namespace
////////////////////////////////////////////////////////////////////////
