/////////////////////////////////////////////////////////////////////////
/// \file    HitList.cxx
/// \brief   A convenient packaging of cell information required for
///          simple geometric track finding and fitting
/// \version $Id: HitList.cxx,v 1.6 2012-09-21 02:32:54 greenc Exp $
/// \author  messier@indiana.edu
/////////////////////////////////////////////////////////////////////////
#include "RecoBase/HitList.h"

#include "Geometry/Geometry.h"
#include "GeometryObjects/CellGeo.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/Cluster.h"

#include "art/Framework/Services/Registry/ServiceHandle.h"

namespace rb
{
  //......................................................................
  HitList::HitList() { }

  //......................................................................

  HitList::HitList(unsigned int n) { this->resize(n); }

  //......................................................................

  HitList::HitList(const rb::Cluster& clust, bool wbyq)
  {
    art::ServiceHandle<geo::Geometry> geom;

    unsigned int n = clust.NCell();
    this->resize(n);

    unsigned register int i;
    for (i=0; i<n; ++i) {
      const rb::CellHit* h = clust.Cell(i).get();

      unsigned int ip = h->Plane();
      unsigned int ic = h->Cell();

      const geo::PlaneGeo* p = geom->Plane(ip);
      const geo::CellGeo*  c = p->   Cell(ic);

      double xyz[3];
      c->GetCenter(xyz);

      double cz = c->HalfD();
      double cxy = c->HalfW();

      double xy = 0.0;
      if (p->View()==geo::kX) xy = xyz[0];
      if (p->View()==geo::kY) xy = xyz[1];

      double w = 1.0;
      if (wbyq) w = h->PE();

      this->SetHit(i, p->View(), xy, xyz[2], h->TNS(), cxy, cz, w);
    }
  }

  //......................................................................

  void HitList::AddHit(geo::View_t v,
           double xy,
           double z,
           double t,
                       double cxy,
                       double cz,
           double w)
  {
    this->push_back(Hit2D(v,xy,z,t,cxy,cz,w));
  }

  //......................................................................

  void HitList::SetHit(unsigned int i,
           geo::View_t v,
           double xy,
           double z,
           double t,
                       double cxy,
                       double cz,
           double w)
  {
    Hit2D& h = (*this)[i];
    h.fView = v;
    h.fXY   = xy;
    h.fZ    = z;
    h.fT    = t;
    h.fCXY  = cxy;
    h.fCZ   = cz;
    h.fW    = w;
  }
} // end namespace rb
////////////////////////////////////////////////////////////////////////
