////////////////////////////////////////////////////////////////////////
// \file    Shower.h
// \brief   A shower is a \ref rb::Prong with a length, other variables which can be commonly used by analysis showers will be added.
// \version $Id: Shower.h,v 1.00 2013-07-12 21:31:55 bianjm Exp $
// \author  Jianming Bian - bianjm@physics.umn.edu
// \date    $Date: 2013-07-12 21:31:55 $
////////////////////////////////////////////////////////////////////////
#ifndef RBSHOWER_H
#define RBSHOWER_H

#include "RecoBase/Prong.h"

#include <vector>

namespace rb
{
  /// \brief A \ref rb::Prong with a length
  class Shower : public Prong
  {
  public:
    /// \name Constructors
    ///
    /// See \ref rb::Prong documentation
    //@{
    Shower():fTotalLength(0.0){}
#ifndef __GCCXML__
    Shower(const rb::Prong& prong
          ,int              id = 0
          ,double           length = 0.0);

    //@}

    virtual ~Shower();

    /// Length (cm) of a shower
    virtual double TotalLength() const;
    /// Set length of a shower
    void SetTotalLength(double length);

    /// Endpoint of the shower
    TVector3 Stop() const;
  protected:
#endif // __GCCXML__
    double fTotalLength;

 };
}

#endif // RBSHOWER_H
