#include "art/Persistency/Common/Wrapper.h"
#include "art/Persistency/Common/Assns.h"
#include "art/Persistency/Common/Ptr.h"

#include "TH1F.h"
#include "TH2F.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TProfile.h"
#include "TGraph.h"
#include "TGraphErrors.h"

#include "RecoBase/CellHit.h"
#include "RecoBase/Cluster.h"
#include "RecoBase/Energy.h"
#include "RecoBase/FilterList.h"
#include "RecoBase/HitMap.h"
#include "RecoBase/HitList.h"
#include "RecoBase/HoughResult.h"
#include "RecoBase/PID.h"
#include "RecoBase/Prong.h"
#include "RecoBase/RecoHit.h"
#include "RecoBase/Track.h"
#include "RecoBase/Shower.h"
#include "RecoBase/Vertex.h"
#include "RecoBase/FitSum.h"

//
// Enable writing of the most common root plotting classes
//
template class std::vector<TH1F>;
template class std::vector<TH1D>;
template class std::vector<TH2F>;
template class std::vector<TH2D>;
template class std::vector<TProfile>;
template class std::vector<TGraph>;
template class std::vector<TGraphErrors>;

template class art::Wrapper<TH1F>;
template class art::Wrapper<TH1D>;
template class art::Wrapper<TH2F>;
template class art::Wrapper<TH2D>;
template class art::Wrapper<TProfile>;
template class art::Wrapper<TGraph>;
template class art::Wrapper<TGraphErrors>;

template class art::Wrapper< std::vector<TH1F> >;
template class art::Wrapper< std::vector<TH1D> >;
template class art::Wrapper< std::vector<TH2F> >;
template class art::Wrapper< std::vector<TH2D> >;
template class art::Wrapper< std::vector<TProfile> >;
template class art::Wrapper< std::vector<TGraph> >;
template class art::Wrapper< std::vector<TGraphErrors> >;

template class std::vector<rb::CellHit>;
template class std::vector<rb::Cluster>;
template class std::vector<rb::HoughPeak>;
template class std::vector<rb::HoughResult>;
template class std::vector<rb::Vertex>;
template class std::vector<rb::Prong>;
template class std::vector<rb::Track>;
template class std::vector<rb::Shower>;
template class std::vector<rb::PID>;
template class std::vector<rb::Energy>;
template class std::vector<rb::FitSum>;

template class art::PtrVector<rb::Track>;
template class art::PtrVector<rb::Shower>;

template class art::Ptr<rb::CellHit>;
template class std::vector<art::Ptr<rb::CellHit> >;
template class art::Wrapper<art::PtrVector<rb::Track> >;
template class art::Wrapper<art::PtrVector<rb::Shower> >;

template class art::Wrapper< std::vector<rb::CellHit>     >;
template class art::Wrapper< std::vector<rb::Cluster>     >;
template class art::Wrapper< std::vector<rb::HoughPeak>   >;
template class art::Wrapper< std::vector<rb::HoughResult> >;
template class art::Wrapper< std::vector<rb::Vertex>      >;
template class art::Wrapper< std::vector<rb::Prong>       >;
template class art::Wrapper< std::vector<rb::Track>       >;
template class art::Wrapper< std::vector<rb::FitSum> >;
template class art::Wrapper< std::vector<rb::Shower>       >;
template class art::Wrapper< std::vector<rb::PID>         >;
template class art::Wrapper< std::vector<rb::Energy>      >;

template class std::pair<art::Ptr<rb::Cluster>,    art::Ptr<rb::CellHit>     >;
template class std::pair<art::Ptr<rb::Cluster>,    art::Ptr<rb::Prong>       >;
template class std::pair<art::Ptr<rb::Cluster>,    art::Ptr<rb::Track>       >;
template class std::pair<art::Ptr<rb::Cluster>,    art::Ptr<rb::Shower>      >;
template class std::pair<art::Ptr<rb::Cluster>,    art::Ptr<rb::Vertex>      >;
template class std::pair<art::Ptr<rb::Cluster>,    art::Ptr<rb::HoughResult> >;
template class std::pair<art::Ptr<rb::Cluster>,    art::Ptr<rb::PID>         >;
template class std::pair<art::Ptr<rb::Cluster>,    art::Ptr<rb::Energy>      >;

template class std::pair<art::Ptr<rb::Prong>,      art::Ptr<rb::CellHit>     >;
template class std::pair<art::Ptr<rb::Prong>,      art::Ptr<rb::Cluster>     >;
template class std::pair<art::Ptr<rb::Prong>,      art::Ptr<rb::Track>       >;
template class std::pair<art::Ptr<rb::Prong>,      art::Ptr<rb::Shower>      >;
template class std::pair<art::Ptr<rb::Prong>,      art::Ptr<rb::Vertex>      >;
template class std::pair<art::Ptr<rb::Prong>,      art::Ptr<rb::PID>         >;
template class std::pair<art::Ptr<rb::Prong>,      art::Ptr<rb::Energy>      >;

template class std::pair<art::Ptr<rb::Track>,      art::Ptr<rb::CellHit>     >;
template class std::pair<art::Ptr<rb::Track>,      art::Ptr<rb::Cluster>     >;
template class std::pair<art::Ptr<rb::Track>,      art::Ptr<rb::Prong>       >;
template class std::pair<art::Ptr<rb::Track>,      art::Ptr<rb::Vertex>      >;
template class std::pair<art::Ptr<rb::Track>,      art::Ptr<rb::PID>         >;
template class std::pair<art::Ptr<rb::Track>,      art::Ptr<rb::Energy>      >;

template class std::pair<art::Ptr<rb::Shower>,      art::Ptr<rb::CellHit>     >;
template class std::pair<art::Ptr<rb::Shower>,      art::Ptr<rb::Cluster>     >;
template class std::pair<art::Ptr<rb::Shower>,      art::Ptr<rb::Prong>       >;
template class std::pair<art::Ptr<rb::Shower>,      art::Ptr<rb::Vertex>      >;
template class std::pair<art::Ptr<rb::Shower>,      art::Ptr<rb::PID>         >;
template class std::pair<art::Ptr<rb::Shower>,      art::Ptr<rb::Energy>      >;

template class std::pair<art::Ptr<rb::Vertex>,     art::Ptr<rb::CellHit>     >;
template class std::pair<art::Ptr<rb::Vertex>,     art::Ptr<rb::Cluster>     >;
template class std::pair<art::Ptr<rb::Vertex>,     art::Ptr<rb::Prong>       >;
template class std::pair<art::Ptr<rb::Vertex>,     art::Ptr<rb::Track>       >;
template class std::pair<art::Ptr<rb::Vertex>,     art::Ptr<rb::Shower>      >;

template class std::pair<art::Ptr<rb::HoughResult>,art::Ptr<rb::Cluster>     >;


template class std::pair<art::Ptr<rb::Energy>,     art::Ptr<rb::Cluster>     >;
template class std::pair<art::Ptr<rb::Energy>,     art::Ptr<rb::Prong>       >;
template class std::pair<art::Ptr<rb::Energy>,     art::Ptr<rb::Track>       >;
template class std::pair<art::Ptr<rb::Energy>,     art::Ptr<rb::Shower>      >;

template class std::pair<art::Ptr<rb::PID>,        art::Ptr<rb::Cluster>     >;
template class std::pair<art::Ptr<rb::PID>,        art::Ptr<rb::Prong>       >;
template class std::pair<art::Ptr<rb::PID>,        art::Ptr<rb::Track>       >;
template class std::pair<art::Ptr<rb::PID>,        art::Ptr<rb::Shower>       >;

template class art::Assns<rb::CellHit,     rb::Cluster,     void>;
template class art::Assns<rb::CellHit,     rb::Prong,       void>;
template class art::Assns<rb::CellHit,     rb::Track,       void>;
template class art::Assns<rb::CellHit,     rb::Shower,      void>;
template class art::Assns<rb::CellHit,     rb::Vertex,      void>;

template class art::Assns<rb::Cluster,     rb::CellHit,     void>;
template class art::Assns<rb::Cluster,     rb::Prong,       void>;
template class art::Assns<rb::Cluster,     rb::Track,       void>;
template class art::Assns<rb::Cluster,     rb::Shower,      void>;
template class art::Assns<rb::Cluster,     rb::Vertex,      void>;
template class art::Assns<rb::Cluster,     rb::HoughResult, void>;
template class art::Assns<rb::Cluster,     rb::PID,         void>;
template class art::Assns<rb::Cluster,     rb::Energy,      void>;

template class art::Assns<rb::Prong,       rb::CellHit,     void>;
template class art::Assns<rb::Prong,       rb::Cluster,     void>;
template class art::Assns<rb::Prong,       rb::Track,       void>;
template class art::Assns<rb::Prong,       rb::Shower,      void>;
template class art::Assns<rb::Prong,       rb::Vertex,      void>;
template class art::Assns<rb::Prong,       rb::PID,         void>;
template class art::Assns<rb::Prong,       rb::Energy,      void>;
template class art::Assns<rb::Prong,       rb::FitSum,     void>;

template class art::Assns<rb::Track,       rb::CellHit,     void>;
template class art::Assns<rb::Track,       rb::Cluster,     void>;
template class art::Assns<rb::Track,       rb::Prong,       void>;
template class art::Assns<rb::Track,       rb::Vertex,      void>;
template class art::Assns<rb::Track,       rb::PID,         void>;
template class art::Assns<rb::Track,       rb::Energy,      void>;
template class art::Assns<rb::Track,       rb::FitSum,     void>;

template class art::Assns<rb::Shower,       rb::CellHit,     void>;
template class art::Assns<rb::Shower,       rb::Cluster,     void>;
template class art::Assns<rb::Shower,       rb::Prong,       void>;
template class art::Assns<rb::Shower,       rb::Vertex,      void>;
template class art::Assns<rb::Shower,       rb::PID,         void>;
template class art::Assns<rb::Shower,       rb::Energy,      void>;

template class art::Assns<rb::Vertex,      rb::CellHit,     void>;
template class art::Assns<rb::Vertex,      rb::Cluster,     void>;
template class art::Assns<rb::Vertex,      rb::Prong,       void>;
template class art::Assns<rb::Vertex,      rb::Track,       void>;
template class art::Assns<rb::Vertex,      rb::Shower,      void>;

template class art::Assns<rb::HoughResult, rb::Cluster,     void>;
template class art::Assns<rb::Energy,      rb::Cluster,     void>;
template class art::Assns<rb::Energy,      rb::Prong,       void>;
template class art::Assns<rb::Energy,      rb::Track,       void>;
template class art::Assns<rb::Energy,      rb::Shower,      void>;

template class art::Assns<rb::PID,         rb::Cluster,     void>;
template class art::Assns<rb::PID,         rb::Prong,       void>;
template class art::Assns<rb::PID,         rb::Track,       void>;
template class art::Assns<rb::PID,         rb::Shower,       void>;

template class art::Assns<rb::FitSum, rb::Prong,  void>;
template class art::Assns<rb::FitSum, rb::Track,  void>;


template class art::Wrapper<art::Assns<rb::CellHit,    rb::Cluster,    void> >;
template class art::Wrapper<art::Assns<rb::CellHit,    rb::Prong,      void> >;
template class art::Wrapper<art::Assns<rb::CellHit,    rb::Track,      void> >;
template class art::Wrapper<art::Assns<rb::CellHit,    rb::Shower,     void> >;
template class art::Wrapper<art::Assns<rb::CellHit,    rb::Vertex,     void> >;

template class art::Wrapper<art::Assns<rb::Cluster,    rb::CellHit,    void> >;
template class art::Wrapper<art::Assns<rb::Cluster,    rb::Prong,      void> >;
template class art::Wrapper<art::Assns<rb::Cluster,    rb::Track,      void> >;
template class art::Wrapper<art::Assns<rb::Cluster,    rb::Shower,     void> >;
template class art::Wrapper<art::Assns<rb::Cluster,    rb::Vertex,     void> >;
template class art::Wrapper<art::Assns<rb::Cluster,    rb::HoughResult,void> >;
template class art::Wrapper<art::Assns<rb::Cluster,    rb::PID,        void> >;
template class art::Wrapper<art::Assns<rb::Cluster,    rb::Energy,     void> >;

template class art::Wrapper<art::Assns<rb::Prong,      rb::CellHit,    void> >;
template class art::Wrapper<art::Assns<rb::Prong,      rb::Cluster,    void> >;
template class art::Wrapper<art::Assns<rb::Prong,      rb::Track,      void> >;
template class art::Wrapper<art::Assns<rb::Prong,      rb::Shower,      void> >;
template class art::Wrapper<art::Assns<rb::Prong,      rb::Vertex,     void> >;
template class art::Wrapper<art::Assns<rb::Prong,      rb::PID,        void> >;
template class art::Wrapper<art::Assns<rb::Prong,      rb::Energy,     void> >;
template class art::Wrapper<art::Assns<rb::Prong,      rb::FitSum,    void> >;

template class art::Wrapper<art::Assns<rb::Track,      rb::CellHit,    void> >;
template class art::Wrapper<art::Assns<rb::Track,      rb::Cluster,    void> >;
template class art::Wrapper<art::Assns<rb::Track,      rb::Prong,      void> >;
template class art::Wrapper<art::Assns<rb::Track,      rb::Vertex,     void> >;
template class art::Wrapper<art::Assns<rb::Track,      rb::PID,        void> >;
template class art::Wrapper<art::Assns<rb::Track,      rb::Energy,     void> >;
template class art::Wrapper<art::Assns<rb::Track,      rb::FitSum,    void> >;

template class art::Wrapper<art::Assns<rb::Shower,     rb::CellHit,    void> >;
template class art::Wrapper<art::Assns<rb::Shower,     rb::Cluster,    void> >;
template class art::Wrapper<art::Assns<rb::Shower,     rb::Prong,      void> >;
template class art::Wrapper<art::Assns<rb::Shower,     rb::Vertex,     void> >;
template class art::Wrapper<art::Assns<rb::Shower,     rb::PID,        void> >;
template class art::Wrapper<art::Assns<rb::Shower,     rb::Energy,     void> >;

template class art::Wrapper<art::Assns<rb::Vertex,     rb::CellHit,    void> >;
template class art::Wrapper<art::Assns<rb::Vertex,     rb::Cluster,    void> >;
template class art::Wrapper<art::Assns<rb::Vertex,     rb::Prong,      void> >;
template class art::Wrapper<art::Assns<rb::Vertex,     rb::Track,      void> >;
template class art::Wrapper<art::Assns<rb::Vertex,     rb::Shower,     void> >;

template class art::Wrapper<art::Assns<rb::HoughResult,rb::Cluster,    void> >;

template class art::Wrapper<art::Assns<rb::FitSum,    rb::Prong,      void> >;
template class art::Wrapper<art::Assns<rb::FitSum,    rb::Track,      void> >;

template class art::Wrapper<art::Assns<rb::Energy,     rb::Cluster,    void> >;
template class art::Wrapper<art::Assns<rb::Energy,     rb::Prong,      void> >;
template class art::Wrapper<art::Assns<rb::Energy,     rb::Track,      void> >;
template class art::Wrapper<art::Assns<rb::Energy,     rb::Shower,     void> >;

template class art::Wrapper<art::Assns<rb::PID,        rb::Cluster,    void> >;
template class art::Wrapper<art::Assns<rb::PID,        rb::Prong,      void> >;
template class art::Wrapper<art::Assns<rb::PID,        rb::Track,      void> >;
template class art::Wrapper<art::Assns<rb::PID,        rb::Shower,     void> >;


template class art::Ptr<rb::Cluster>;
template class art::Ptr<rb::Prong>;
template class art::Ptr<rb::Track>;
template class art::Ptr<rb::Shower>;
template class art::Ptr<rb::Vertex>;
template class art::Ptr<rb::FitSum>;


template class rb::FilterList<rb::Cluster>;
template class art::Wrapper<rb::FilterList<rb::Cluster> >;
template class std::vector<rb::FilterList<rb::Cluster> >;
template class art::Wrapper<std::vector<rb::FilterList<rb::Cluster> > >;


// Currently we don't associate any ROOT objects to any reco objects, and
// having these greatly extends the dictionary compilation time.
/*
template class std::pair<art::Ptr<TH1F>,              art::Ptr<rb::Prong>       >;
template class std::pair<art::Ptr<TH1F>,              art::Ptr<rb::Track>       >;
template class std::pair<art::Ptr<TH1F>,              art::Ptr<rb::Shower>       >;
template class std::pair<art::Ptr<TH1F>,              art::Ptr<rb::HoughResult> >;
template class std::pair<art::Ptr<TH1F>,              art::Ptr<rb::Cluster>     >;

template class std::pair<art::Ptr<TH2F>,              art::Ptr<rb::Prong>       >;
template class std::pair<art::Ptr<TH2F>,              art::Ptr<rb::Track>       >;
template class std::pair<art::Ptr<TH2F>,              art::Ptr<rb::Shower>       >;
template class std::pair<art::Ptr<TH2F>,              art::Ptr<rb::HoughResult> >;
template class std::pair<art::Ptr<TH2F>,              art::Ptr<rb::Cluster>     >;

template class std::pair<art::Ptr<TH1D>,              art::Ptr<rb::Prong>       >;
template class std::pair<art::Ptr<TH1D>,              art::Ptr<rb::Track>       >;
template class std::pair<art::Ptr<TH1D>,              art::Ptr<rb::Shower>       >;
template class std::pair<art::Ptr<TH1D>,              art::Ptr<rb::HoughResult> >;
template class std::pair<art::Ptr<TH1D>,              art::Ptr<rb::Cluster>     >;

template class std::pair<art::Ptr<TH2D>,              art::Ptr<rb::Prong>       >;
template class std::pair<art::Ptr<TH2D>,              art::Ptr<rb::Track>       >;
template class std::pair<art::Ptr<TH2D>,              art::Ptr<rb::Shower>       >;
template class std::pair<art::Ptr<TH2D>,              art::Ptr<rb::HoughResult> >;
template class std::pair<art::Ptr<TH2D>,              art::Ptr<rb::Cluster>     >;

template class std::pair<art::Ptr<TGraph>,            art::Ptr<rb::Prong>       >;
template class std::pair<art::Ptr<TGraph>,            art::Ptr<rb::Track>       >;
template class std::pair<art::Ptr<TGraph>,            art::Ptr<rb::Shower>       >;
template class std::pair<art::Ptr<TGraph>,            art::Ptr<rb::HoughResult> >;
template class std::pair<art::Ptr<TGraph>,            art::Ptr<rb::Cluster>     >;

template class std::pair<art::Ptr<TProfile>,          art::Ptr<rb::Prong>       >;
template class std::pair<art::Ptr<TProfile>,          art::Ptr<rb::Track>       >;
template class std::pair<art::Ptr<TProfile>,          art::Ptr<rb::Shower>       >;
template class std::pair<art::Ptr<TProfile>,          art::Ptr<rb::HoughResult> >;
template class std::pair<art::Ptr<TProfile>,          art::Ptr<rb::Cluster>     >;

template class std::pair<art::Ptr<TGraphErrors>,      art::Ptr<rb::Prong>       >;
template class std::pair<art::Ptr<TGraphErrors>,      art::Ptr<rb::Track>       >;
template class std::pair<art::Ptr<TGraphErrors>,      art::Ptr<rb::Shower>       >;
template class std::pair<art::Ptr<TGraphErrors>,      art::Ptr<rb::HoughResult> >;
template class std::pair<art::Ptr<TGraphErrors>,      art::Ptr<rb::Cluster>     >;

template class std::pair<art::Ptr<rb::Prong>,            art::Ptr<TH1F>         >;
template class std::pair<art::Ptr<rb::Prong>,            art::Ptr<TH2F>         >;
template class std::pair<art::Ptr<rb::Prong>,            art::Ptr<TH1D>         >;
template class std::pair<art::Ptr<rb::Prong>,            art::Ptr<TH2D>         >;
template class std::pair<art::Ptr<rb::Prong>,            art::Ptr<TGraph>       >;
template class std::pair<art::Ptr<rb::Prong>,            art::Ptr<TProfile>     >;
template class std::pair<art::Ptr<rb::Prong>,            art::Ptr<TGraphErrors> >;

template class std::pair<art::Ptr<rb::Track>,            art::Ptr<TH1F>         >;
template class std::pair<art::Ptr<rb::Track>,            art::Ptr<TH2F>         >;
template class std::pair<art::Ptr<rb::Track>,            art::Ptr<TH1D>         >;
template class std::pair<art::Ptr<rb::Track>,            art::Ptr<TH2D>         >;
template class std::pair<art::Ptr<rb::Track>,            art::Ptr<TGraph>       >;
template class std::pair<art::Ptr<rb::Track>,            art::Ptr<TProfile>     >;
template class std::pair<art::Ptr<rb::Track>,            art::Ptr<TGraphErrors> >;

template class std::pair<art::Ptr<rb::Shower>,            art::Ptr<TH1F>         >;
template class std::pair<art::Ptr<rb::Shower>,            art::Ptr<TH2F>         >;
template class std::pair<art::Ptr<rb::Shower>,            art::Ptr<TH1D>         >;
template class std::pair<art::Ptr<rb::Shower>,            art::Ptr<TH2D>         >;
template class std::pair<art::Ptr<rb::Shower>,            art::Ptr<TGraph>       >;
template class std::pair<art::Ptr<rb::Shower>,            art::Ptr<TProfile>     >;
template class std::pair<art::Ptr<rb::Shower>,            art::Ptr<TGraphErrors> >;

template class std::pair<art::Ptr<rb::HoughResult>,      art::Ptr<TH1F>         >;
template class std::pair<art::Ptr<rb::HoughResult>,      art::Ptr<TH2F>         >;
template class std::pair<art::Ptr<rb::HoughResult>,      art::Ptr<TH1D>         >;
template class std::pair<art::Ptr<rb::HoughResult>,      art::Ptr<TH2D>         >;
template class std::pair<art::Ptr<rb::HoughResult>,      art::Ptr<TGraph>       >;
template class std::pair<art::Ptr<rb::HoughResult>,      art::Ptr<TProfile>     >;
template class std::pair<art::Ptr<rb::HoughResult>,      art::Ptr<TGraphErrors> >;

template class std::pair<art::Ptr<rb::Cluster>,          art::Ptr<TH1F>         >;
template class std::pair<art::Ptr<rb::Cluster>,          art::Ptr<TH2F>         >;
template class std::pair<art::Ptr<rb::Cluster>,          art::Ptr<TH1D>         >;
template class std::pair<art::Ptr<rb::Cluster>,          art::Ptr<TH2D>         >;
template class std::pair<art::Ptr<rb::Cluster>,          art::Ptr<TGraph>       >;
template class std::pair<art::Ptr<rb::Cluster>,          art::Ptr<TProfile>     >;
template class std::pair<art::Ptr<rb::Cluster>,          art::Ptr<TGraphErrors> >;

template class art::Assns<TH1F,     rb::Prong,       void>;
template class art::Assns<TH1F,     rb::Track,       void>;
template class art::Assns<TH1F,     rb::Shower,       void>;
template class art::Assns<TH1F,     rb::HoughResult, void>;
template class art::Assns<TH1F,     rb::Cluster,     void>;

template class art::Assns<TH2F,     rb::Prong,       void>;
template class art::Assns<TH2F,     rb::Track,       void>;
template class art::Assns<TH2F,     rb::Shower,       void>;
template class art::Assns<TH2F,     rb::HoughResult, void>;
template class art::Assns<TH2F,     rb::Cluster,     void>;

template class art::Assns<TH1D,     rb::Prong,       void>;
template class art::Assns<TH1D,     rb::Track,       void>;
template class art::Assns<TH1D,     rb::Shower,       void>;
template class art::Assns<TH1D,     rb::HoughResult, void>;
template class art::Assns<TH1D,     rb::Cluster,     void>;

template class art::Assns<TH2D,     rb::Prong,       void>;
template class art::Assns<TH2D,     rb::Track,       void>;
template class art::Assns<TH2D,     rb::Shower,       void>;
template class art::Assns<TH2D,     rb::HoughResult, void>;
template class art::Assns<TH2D,     rb::Cluster,     void>;

template class art::Assns<TGraph,     rb::Prong,       void>;
template class art::Assns<TGraph,     rb::Track,       void>;
template class art::Assns<TGraph,     rb::Shower,      void>;
template class art::Assns<TGraph,     rb::HoughResult, void>;
template class art::Assns<TGraph,     rb::Cluster,     void>;

template class art::Assns<TProfile,     rb::Prong,       void>;
template class art::Assns<TProfile,     rb::Track,       void>;
template class art::Assns<TProfile,     rb::Shower,      void>;
template class art::Assns<TProfile,     rb::HoughResult, void>;
template class art::Assns<TProfile,     rb::Cluster,     void>;

template class art::Assns<TGraphErrors,     rb::Prong,       void>;
template class art::Assns<TGraphErrors,     rb::Track,       void>;
template class art::Assns<TGraphErrors,     rb::Shower,      void>;
template class art::Assns<TGraphErrors,     rb::HoughResult, void>;
template class art::Assns<TGraphErrors,     rb::Cluster,     void>;


template class art::Assns<rb::Prong,           TH1F,               void>;
template class art::Assns<rb::Prong,           TH2F,               void>;
template class art::Assns<rb::Prong,           TH1D,               void>;
template class art::Assns<rb::Prong,           TH2D,               void>;
template class art::Assns<rb::Prong,           TGraph,             void>;
template class art::Assns<rb::Prong,           TProfile,           void>;
template class art::Assns<rb::Prong,           TGraphErrors,       void>;

template class art::Assns<rb::Track,           TH1F,               void>;
template class art::Assns<rb::Track,           TH2F,               void>;
template class art::Assns<rb::Track,           TH1D,               void>;
template class art::Assns<rb::Track,           TH2D,               void>;
template class art::Assns<rb::Track,           TGraph,             void>;
template class art::Assns<rb::Track,           TProfile,           void>;
template class art::Assns<rb::Track,           TGraphErrors,       void>;

template class art::Assns<rb::Shower,          TH1F,               void>;
template class art::Assns<rb::Shower,          TH2F,               void>;
template class art::Assns<rb::Shower,          TH1D,               void>;
template class art::Assns<rb::Shower,          TH2D,               void>;
template class art::Assns<rb::Shower,          TGraph,             void>;
template class art::Assns<rb::Shower,          TProfile,           void>;
template class art::Assns<rb::Shower,          TGraphErrors,       void>;

template class art::Assns<rb::HoughResult,     TH1F,               void>;
template class art::Assns<rb::HoughResult,     TH2F,               void>;
template class art::Assns<rb::HoughResult,     TH1D,               void>;
template class art::Assns<rb::HoughResult,     TH2D,               void>;
template class art::Assns<rb::HoughResult,     TGraph,             void>;
template class art::Assns<rb::HoughResult,     TProfile,           void>;
template class art::Assns<rb::HoughResult,     TGraphErrors,       void>;

template class art::Assns<rb::Cluster,         TH1F,               void>;
template class art::Assns<rb::Cluster,         TH2F,               void>;
template class art::Assns<rb::Cluster,         TH1D,               void>;
template class art::Assns<rb::Cluster,         TH2D,               void>;
template class art::Assns<rb::Cluster,         TGraph,             void>;
template class art::Assns<rb::Cluster,         TProfile,           void>;
template class art::Assns<rb::Cluster,         TGraphErrors,       void>;
*/
