////////////////////////////////////////////////////////////////////////
/// \brief CellHit class
/// \author C. Backhouse - bckhouse@caltech.edu
/// \date
////////////////////////////////////////////////////////////////////////
#include "RecoBase/CellHit.h"

#include <algorithm>
#include <iomanip>
#include <iostream>



namespace rb
{
  //----------------------------------------------------------------------
  CellHit::CellHit() : 
    RawDigit(),
    fPlane(0),
    fCell(0),
    fView(geo::kXorY),
    fPE(0),
    fTNS(0),
    fID(0),
    fCoarseTiming(false)
  {
  }
  
  //----------------------------------------------------------------------
  CellHit::CellHit(const rawdata::RawDigit& rd, int id) : 
    RawDigit(rd),
    fPlane(0),
    fCell(0),
    fView(geo::kXorY),
    fPE(0),
    fTNS(0),
    fID(0),
    fCoarseTiming(false)
  { 
  }

  //----------------------------------------------------------------------
  CellHit::~CellHit()
  {
  }
  
  //----------------------------------------------------------------------
  void CellHit::Print() const
  {
    std::cout << *this;
  }
  
  //------------------------------------------------------------
  bool CellHit::operator< (const CellHit& other) const
  {
    if(fPlane < other.Plane() ) return true;
    if(fPlane == other.Plane() && fCell < other.Cell() ) return true;
    return false;
  }

  //------------------------------------------------------------
  std::ostream& operator<< (std::ostream& o, const CellHit& c)
  {
    o << std::setiosflags(std::ios::fixed) << std::setprecision(2);	
    o << " View = "   << std::setw(5) << std::right << c.View()
      << " ID = "     << std::setw(5) << std::right << c.ID()
      << " Plane = "  << std::setw(5) << std::right << c.Plane()
      << " Cell = "   << std::setw(5) << std::right << c.Cell()
      << " Channel = "<< std::setw(5) << std::right << c.Channel()
      << " TDC = "    << std::setw(5) << std::right << c.TDC()
      << " TNS = "    << std::setw(5) << std::right << c.TNS()
      << " ADC = "    << std::setw(5) << std::right << c.ADC()
      << " PE = "     << std::setw(5) << std::right << c.PE();
    return o;
  }
  
  bool CellHit::operator== (const CellHit& other) const
  {
    if(fPlane           == other.Plane() &&
       fCell            == other.Cell()  &&
       this->ADC()      == other.ADC()    &&
       fTDC[0]          == other.TDC()    )
      return true;
    return false;
  }

  //------------------------------------------------------------
  bool CompareByPlane(const art::Ptr<rb::CellHit>& a,
		      const art::Ptr<rb::CellHit>& b)
  {
    return a->Plane() < b->Plane(); 
  }
  
  //------------------------------------------------------------
  bool CompareByTime(const art::Ptr<rb::CellHit>& a,
		     const art::Ptr<rb::CellHit>& b)
  {
    return a->TNS() < b->TNS(); 
  }
  
  //------------------------------------------------------------
  bool CompareByCell(const art::Ptr<rb::CellHit>& a,
		     const art::Ptr<rb::CellHit>& b)
  {
    return a->Cell() < b->Cell();
  }

  //------------------------------------------------------------
  bool CompareByPlaneAndCell(const art::Ptr<rb::CellHit>& a,
			     const art::Ptr<rb::CellHit>& b)
  {
    if( a->Plane() < b->Plane() )
      return true;
    if( a->Plane() > b->Plane() )
      return false;
    return a->Cell() < b->Cell();
  }

  //------------------------------------------------------------
  void SortByPlane(std::vector<art::Ptr<rb::CellHit> >& c)
  {
    std::sort(c.begin(), c.end(), CompareByPlane);
  }
  void SortByPlane(art::PtrVector<rb::CellHit>& c)
  {
    std::sort(c.begin(), c.end(), CompareByPlane);
  }
  
  //------------------------------------------------------------
  void SortByTime(std::vector<art::Ptr<rb::CellHit> >& c)
  {
    std::sort(c.begin(), c.end(), CompareByTime);
  }
  void SortByTime(art::PtrVector<rb::CellHit>& c)
  {
    std::sort(c.begin(), c.end(), CompareByTime);
  }
  
  //------------------------------------------------------------
  
  void SortByCell(std::vector<art::Ptr<rb::CellHit> >& c)
  {
    std::sort(c.begin(), c.end(), CompareByCell);
  }
  void SortByCell(art::PtrVector<rb::CellHit>& c)
  {
    std::sort(c.begin(), c.end(), CompareByCell);
  }
  
  //------------------------------------------------------------
  void SortByPlaneAndCell(std::vector<art::Ptr<rb::CellHit> >& c)
  {
    std::sort(c.begin(), c.end(), CompareByPlaneAndCell);
  }
  void SortByPlaneAndCell(art::PtrVector<rb::CellHit>& c)
  {
    std::sort(c.begin(), c.end(), CompareByPlaneAndCell);
  }

} // end namespace rb
//////////////////////////////////////////////////////////////////////////////////////////
