////////////////////////////////////////////////////////////////////////
// \file    RecoHit.cxx
// \brief   RecoHit class
// \version $Id: RecoHit.cxx,v 1.7 2012-03-07 22:13:19 bckhouse Exp $
// \author  $Author: bckhouse $
////////////////////////////////////////////////////////////////////////
#include "RecoBase/RecoHit.h"

#include <cassert>
#include <iomanip>
#include <iostream>
#include <iosfwd>
#include <string>

namespace rb
{
  //----------------------------------------------------------------------
  RecoHit::RecoHit() : 
    fPECorr(-1),
    fMIP(-1),
    fGeV(-1),
    fX(0),
    fY(0),
    fZ(0),
    fT(0)
  {
  }

  //----------------------------------------------------------------------
  RecoHit::~RecoHit()
  {
  }

  //----------------------------------------------------------------------
  bool RecoHit::IsCalibrated() const
  {
    return fPECorr >= 0 && fMIP >= 0 && fGeV >= 0;
  }

  //----------------------------------------------------------------------
  void RecoHit::Print() const
  {
    std::cout << *this << std::endl;
  }

  //----------------------------------------------------------------------
  float RecoHit::PECorr() const
  {
    assert(fPECorr >= 0 && "Not calibrated");
    return fPECorr;
  }

  //----------------------------------------------------------------------
  float RecoHit::MIP() const
  {
    assert(fMIP >= 0 && "Not calibrated");
    return fMIP;
  }

  //----------------------------------------------------------------------
  float RecoHit::GeV() const
  {
    assert(fGeV >= 0 && "Not calibrated");
    return fGeV;
  }
  
  //------------------------------------------------------------
  std::ostream& operator<< (std::ostream& o, const RecoHit& r)
  {
    o << std::setiosflags(std::ios::fixed) << std::setprecision(2);	
    o << "PECor = " << std::setw(5) << std::right << r.PECorr()
      << " MIP = "  << std::setw(5) << std::right << r.MIP()
      << " GeV = "  << std::setw(5) << std::right << r.GeV()
      << " X = "    << std::setw(5) << std::right << r.X()
      << " Y = "    << std::setw(5) << std::right << r.Y()
      << " Z = "    << std::setw(5) << std::right << r.Z();
    return o;
  }
} // end namespace rb
///////////////////////////////////////////////////////////////////////////////////
