////////////////////////////////////////////////////////////////////////
/// \brief   TODO
/// \author  bianjm@umn.physics.edu
///////////////////////////////////////////////////////////////////////
#include "RecoJMShower/JMShower.h"
#include "Geometry/Geometry.h"
#include "GeometryObjects/CellGeo.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"

namespace jmshower
{
  //......................................................................
  JMShower::JMShower(const rb::Prong& prong,
                     int id,
                     double length)
    :  rb::Shower(prong,
                 id,
                 length)
  {
  }

  //..................................................................
  void JMShower::SetWindowSize(unsigned int nplane)
  {
    for(unsigned int i =0; i<20;i++){
      fANN[i]=-5;
    }
    for(unsigned int i = 0;i<nplane;i++) {
      fPlaneDedx.push_back(0);
      fPlaneRadius.push_back(0);
      fPlaneProbE.push_back(-99);
      fPlaneProbEqe.push_back(-99);
      fPlaneProbEres.push_back(-99);
      fPlaneProbEdis.push_back(-99);
      fPlaneProbEcoh.push_back(-99);
      fPlaneProbG.push_back(-99);
      fPlaneProbMu.push_back(-99);
      fPlaneProbPi0.push_back(-99);
      fPlaneProbHad.push_back(-99);
      fPlaneProbProton.push_back(-99);
      fPlaneProbNuetron.push_back(-99);
      fPlaneProbPion.push_back(-99);
      TVector3 plhitxyz(0,0,0);
      fPlaneHitXYZ.push_back(plhitxyz);
      fPlaneE1Cell.push_back(-1);
      fPlaneE2Cell.push_back(-1);
      fPlaneCentroidCell.push_back(-1);
      fPlaneHitCell.push_back(0);

    }
  }

  void JMShower::SetHitColSize(unsigned int ncell)
  {
    for(unsigned int i = 0;i<ncell;i++) {
      fHitDistCol.push_back(0);
      fHitDeconvECol.push_back(0);
      fHitSumECol.push_back(0);
    }
  }

  void JMShower::SetStop(TVector3 stop)
  {
    fStop = stop;
  }

  TVector3 JMShower::Stop() const
  {
    return fStop;
  }

  void JMShower::SetHitDist(unsigned int cell, double dist)
  {
    fHitDistCol[cell] = dist;
  }

  double JMShower::HitDist(unsigned int cell) const
  {
    return fHitDistCol[cell];
  }

  void JMShower::SetHitDeconvE(unsigned int cell, double deconve)
  {
    fHitDeconvECol[cell] = deconve;
  }

  double JMShower::HitDeconvE(unsigned int cell) const
  {
    return fHitDeconvECol[cell];
  }


  void JMShower::SetHitSumE(unsigned int cell, double sume)
  {
    fHitSumECol[cell] = sume;
  }

  double JMShower::HitSumE(unsigned int cell) const
  {
    return fHitSumECol[cell];
  }


  void JMShower::SetIsFiducial(bool isfiducial)
  {
    fIsFiducial = isfiducial;
  }
  bool JMShower::IsFiducial() const
  {
    return fIsFiducial;
  }

  void JMShower::SetDistToEdge(double disttoedge)
  {
    fDistToEdge = disttoedge;
  }
  double JMShower::DistToEdge() const
  {
    return fDistToEdge;
  }


  void JMShower::SetDepositEnergy(double depositenergy)
  {
    fDepositEnergy =  depositenergy;
  }
  double JMShower::DepositEnergy() const
  {
    return fDepositEnergy;
  }

  void JMShower::SetExclEnergy(double exclenergy)
  {
    fExclEnergy = exclenergy;
  }
  double JMShower::ExclEnergy() const
  {
    return fExclEnergy;
  }
 


  void JMShower::SetEnergy(double energy)
  {
    fEnergy =  energy;
  }
  double JMShower::Energy() const
  {
    return fEnergy;
  }

  void JMShower::SetVtxGeV(double gev)
  {
    fVtxGeV = gev;
  }
  double JMShower::VtxGeV() const
  {
    return fVtxGeV;
  }

  void JMShower::SetVtxCenterGeV(double gev)
  {
    fVtxCenterGeV = gev;
  }
  double JMShower::VtxCenterGeV() const
  {
    return fVtxCenterGeV;
  }

  void JMShower::SetPi0Mgg(double pi0mass)
  {
    fPi0Mgg = pi0mass;
  }
  double JMShower::Pi0Mgg() const
  {
    return fPi0Mgg;
  }

  void JMShower::SetPi0G2ID(int id)
  {
    fPi0G2ID = id;
  }
  int JMShower::Pi0G2ID() const
  {
    return fPi0G2ID;
  }

  void JMShower::SetTheta(double theta)
  {
    fTheta = theta;
  }
  double JMShower::Theta() const
  {
    return fTheta;
  }


  void JMShower::SetTrkGeV(double gev)
  {
    fTrkGeV = gev;
  }
  double JMShower::TrkGeV() const
  {
    return fTrkGeV;
  }

  void JMShower::SetSliceGeV(double gev)
  {
    fSliceGeV = gev;
  }
  double JMShower::SliceGeV() const
  {
    return fSliceGeV;
  }

  void JMShower::SetSliceEtot(double etot)
  {
    fSliceEtot = etot;
  }
  double JMShower::SliceEtot() const
  {
    return fSliceEtot;
  }

  void JMShower::SetNueEnergy(double nueenergy)
  {
    fNueEnergy = nueenergy;
  }
  double JMShower::NueEnergy() const
  {
    return fNueEnergy;
  }


  void JMShower::SetSliceEdge10GeV(double gev)
  {
    fSliceEdge10GeV = gev;
  }
  double JMShower::SliceEdge10GeV() const
  {
    return fSliceEdge10GeV;
  }

  void JMShower::SetSliceEdge5GeV(double gev)
  {
    fSliceEdge5GeV = gev;
  } 
  double JMShower::SliceEdge5GeV() const
  {
    return fSliceEdge5GeV;
  }

  void JMShower::SetSliceEdge2GeV(double gev)
  {
    fSliceEdge2GeV = gev;
  } 
  double JMShower::SliceEdge2GeV() const
  {
    return fSliceEdge2GeV;
  }


  void JMShower::SetNCellToEdge(int nce)
  {
    fNCellToEdge = nce;
  }
  int JMShower::NCellToEdge() const
  {
    return fNCellToEdge;
  }

  void JMShower::SetClosetEdge(int edge)
  {
    fClosetEdge = edge;
  }

  int JMShower::ClosetEdge() const
  {
    return fClosetEdge;
  }


  void JMShower::SetEdge20GeV(double gev)
  {
    fEdge20GeV = gev;
  }
  double JMShower::Edge20GeV() const
  {
    return fEdge20GeV;
  }
  void JMShower::SetEdge15GeV(double gev)
  {
    fEdge15GeV = gev;
  }

  double JMShower::Edge15GeV() const
  {
    return fEdge15GeV;
  }

  void JMShower::SetEdge10GeV(double gev)
  {
    fEdge10GeV = gev;
  }

  double JMShower::Edge10GeV() const
  {
    return fEdge10GeV;
  }

  void JMShower::SetEdge5GeV(double gev)
  {
    fEdge5GeV = gev;
  } 
  double JMShower::Edge5GeV() const
  {
    return fEdge5GeV;
  }

  void JMShower::SetEdge2GeV(double gev)
  {
    fEdge2GeV = gev;
  } 
  double JMShower::Edge2GeV() const
  {
    return fEdge2GeV;
  }

  void JMShower::SetSliceExclGeV(double gev)
  {
    fSliceExclGeV = gev;
  }
  double JMShower::SliceExclGeV() const
  {
    return fSliceExclGeV;
  }

  void JMShower::SetRadius(double radius)
  {
    fRadius = radius;
  }
  double JMShower::Radius() const
  {
    return fRadius;
  }

  void JMShower::SetSumDRadius(double dradius)
  {
    fSumDRadius = dradius;
  }
  double JMShower::SumDRadius() const
  {
    return fSumDRadius;
  }

  void JMShower::SetSum10DRadius(double dradius)
  {
    fSum10DRadius = dradius;
  }
  double JMShower::Sum10DRadius() const
  {
    return fSum10DRadius;
  }

  void JMShower::SetBackTag(bool backtag)
  {
    fBackTag = backtag;
  }

  bool JMShower::BackTag() const
  {
    return fBackTag;
  }


  void JMShower::SetGap(double gap)
  {
    fGap = gap;
  }

  double JMShower::Gap() const
  {
    return fGap;
  }

  void JMShower::SetContGap(double gap)
  {
    fContGap = gap;
  }

  double JMShower::ContGap() const
  {
    return fContGap;
  }

  void JMShower::SetVtxDoca(double vtxdoca)
  {
    fVtxDoca = vtxdoca;
  }

  double JMShower::VtxDoca() const
  {
    return fVtxDoca;
  }


  void JMShower::SetCentroid(TVector3 centroid)
  {
    fCentroid = centroid;
  }
  TVector3 JMShower::Centroid() const
  {
    return fCentroid;
  }

  void JMShower::SetStartPlane(unsigned int startplane)
  {
    fStartPlane = startplane;
  }
  unsigned int JMShower::StartPlane() const
  {
    return fStartPlane;
  }

  void JMShower::SetStopPlane(unsigned int stopplane)
  {
    fStopPlane = stopplane;
  }
  unsigned int JMShower::StopPlane() const
  {
    return fStopPlane;
  }

  void JMShower::SetNPlane(unsigned int nplane)
  {
    fNPlane = nplane;
  }
  unsigned int JMShower::NPlane() const
  {
    return fNPlane;
  }

  void JMShower::SetXNPlane(unsigned int xnplane)
  {
    fXNPlane = xnplane;
  }
  unsigned int JMShower::XNPlane() const
  {
    return fXNPlane;
  }

  void JMShower::SetYNPlane(unsigned int ynplane)
  {
    fYNPlane = ynplane;
  }
  unsigned int JMShower::YNPlane() const
  {
    return fYNPlane;
  }


  void JMShower::SetXMinPlane(unsigned int xminplane)
  {
    fXMinPlane = xminplane;
  }
  unsigned int JMShower::XMinPlane() const
  {
    return fXMinPlane;
  }

  void JMShower::SetYMinPlane(unsigned int yminplane)
  {
    fYMinPlane = yminplane;
  }
  unsigned int JMShower::YMinPlane() const
  {
    return fYMinPlane;
  }


  void JMShower::SetXMaxPlane(unsigned int xmaxlane)
  {
    fXMaxPlane = xmaxlane;
  }
  unsigned int JMShower::XMaxPlane() const
  {
    return fXMaxPlane;
  }

  void JMShower::SetYMaxPlane(unsigned int ymaxlane)
  {
    fYMaxPlane = ymaxlane;
  }
  unsigned int JMShower::YMaxPlane() const
  {
    return fYMaxPlane;
  }


  void JMShower::SetPlaneHitXYZ(unsigned int plane, TVector3 xyz)
  {
    if(plane>=JMShower::NPlane())return;
//    if(plane>=200)return;
    fPlaneHitXYZ[plane] = xyz; 
  }

  TVector3 JMShower::PlaneHitXYZ(unsigned int plane) const
  {
//    if(plane>=200)return 0;
    if(plane>=JMShower::NPlane()){
      TVector3 p3null(0,0,0);
      return p3null;
    }
    return fPlaneHitXYZ[plane];
  }


  void JMShower::SetPlaneHitCell(unsigned int plane, unsigned int cell)
  {
    if(plane>=JMShower::NPlane())return;
    fPlaneHitCell[plane] = cell;
  }
  unsigned int JMShower::PlaneHitCell(unsigned int plane) const
  {
    if(plane>=JMShower::NPlane())return 0;
    return fPlaneHitCell[plane];
  }

  void JMShower::SetPlaneE1Cell(unsigned int plane, int cell)
  {
    if(plane>=JMShower::NPlane())return;
    fPlaneE1Cell[plane] = cell;
  }

  int JMShower::PlaneE1Cell(unsigned int plane) const
  {
    if(plane>=JMShower::NPlane())return 0;
    return fPlaneE1Cell[plane];
  }

  void JMShower::SetPlaneE2Cell(unsigned int plane, int cell)
  {
    if(plane>=JMShower::NPlane())return;
    fPlaneE2Cell[plane] = cell;
  }

  int JMShower::PlaneE2Cell(unsigned int plane) const
  {
    if(plane>=JMShower::NPlane())return 0;
    return fPlaneE2Cell[plane];
  }

  void JMShower::SetPlaneCentroidCell(unsigned int plane, int cell)
  {
    if(plane>=JMShower::NPlane())return;
    fPlaneCentroidCell[plane] = cell;
  }

  int JMShower::PlaneCentroidCell(unsigned int plane) const
  {
    if(plane>=JMShower::NPlane())return 0;
    return fPlaneCentroidCell[plane];
  }

  void JMShower::SetPlaneDedx(unsigned int plane, double planededx)
  {
    if(plane>=JMShower::NPlane())return;
//    if(plane>=200)return;
    fPlaneDedx[plane] = planededx;
  }

  double JMShower::PlaneDedx(unsigned int plane) const
  {
//    if(plane>=200)return 0;
    if(plane>=JMShower::NPlane())return 0;
    return fPlaneDedx[plane];
  }

  void JMShower::SetPlaneRadius(unsigned int plane, double planerad)
  {
    if(plane>=JMShower::NPlane())return;
//    if(plane>=200)return;
    fPlaneRadius[plane] = planerad;
  }

  double JMShower::PlaneRadius(unsigned int plane) const
  {
//    if(plane>=200)return 0;
    if(plane>=JMShower::NPlane())return 0;
    return fPlaneRadius[plane];
  }

  void JMShower::SetPlaneProb(unsigned int plane, double planeprob, int type)
  {

    if(plane>=JMShower::NPlane())return;

    if(type==0)fPlaneProbE[plane] = planeprob;
    if(type==1)fPlaneProbG[plane] = planeprob;
    if(type==2)fPlaneProbMu[plane] = planeprob;
    if(type==3)fPlaneProbPi0[plane] = planeprob;
    if(type==4)fPlaneProbHad[plane] = planeprob;
    if(type==5)fPlaneProbProton[plane] = planeprob;
    if(type==6)fPlaneProbNuetron[plane] = planeprob;
    if(type==7)fPlaneProbPion[plane] = planeprob;
    if(type==8)fPlaneProbEqe[plane] = planeprob;
    if(type==9)fPlaneProbEres[plane] = planeprob;
    if(type==10)fPlaneProbEdis[plane] = planeprob;
    if(type==11)fPlaneProbEcoh[plane] = planeprob;

    return;
  }

  double JMShower::PlaneProb(unsigned int plane, int type) const
  {
    if(plane>=JMShower::NPlane()) return -99;
    if(type==0)return fPlaneProbE[plane];
    if(type==1)return fPlaneProbG[plane];
    if(type==2)return fPlaneProbMu[plane];
    if(type==3)return fPlaneProbPi0[plane];
    if(type==4)return fPlaneProbHad[plane];
    if(type==5)return fPlaneProbProton[plane];
    if(type==6)return fPlaneProbNuetron[plane];
    if(type==7)return fPlaneProbPion[plane];
    if(type==8)fPlaneProbEqe[plane];
    if(type==9)fPlaneProbEres[plane];
    if(type==10)fPlaneProbEdis[plane];
    if(type==11)fPlaneProbEcoh[plane];
    return -9999;
  }


  void JMShower::SetDedxLongLL(double dedxlongll, int type)
  {
    if(type>=20) return;
    fDedxLongLL[type] = dedxlongll;
  }

  double JMShower::DedxLongLL(int type) const
  {
    if(type>=20)return -9999;
    return fDedxLongLL[type];
  }

  void JMShower::SetDedxInvLongLL(double dedxlongll, int type)
  {
    if(type>=20) return;
    fDedxInvLongLL[type] = dedxlongll;
  }

  double JMShower::DedxInvLongLL(int type) const
  {
    if(type>=20)return -9999;
    return fDedxInvLongLL[type];
  }

  void JMShower::SetCellTransDedx(unsigned int tcell,double celldedx)
  {
    if(tcell>=20)return;
    fCelltransDedx[tcell] = celldedx;
  }

  double JMShower::CellTransDedx(unsigned int tcell) const
  {
    if(tcell>=20)return 0;
    return fCelltransDedx[tcell];
  }


  void JMShower::SetDedxTransLL(double dedxtransll, int type)
  {
    if(type>=20) return;
    fDedxTransLL[type] = dedxtransll;
  }
  double JMShower::DedxTransLL(int type) const
  {
    if(type>=20)return -9999;
    return fDedxTransLL[type];
  }

  void JMShower::SetIsMuon(bool ismuon)
  {
    fIsMuon = ismuon;
  }

  bool JMShower::IsMuon() const
  {
    return fIsMuon;
  }

  void JMShower::SetIsInvPhoton(bool isinvphoton)
  {
    fIsInvPhoton = isinvphoton;
  }

  bool JMShower::IsInvPhoton() const
  {
    return fIsInvPhoton;
  }


  void JMShower::SetNMIPPlane(int nmipplane)
  {
    fNMIPPlane = nmipplane;
  }
  int JMShower::NMIPPlane() const
  {
    return fNMIPPlane;
  }

  void JMShower::SetISlice(unsigned int islice)
  {
    fISlice = islice;
  }
  unsigned int JMShower::ISlice() const
  {
    return fISlice;
  }


  void JMShower::SetANN(unsigned int mode, double ann)
  {
    fANN[mode] = ann;
  }
  double JMShower::ANN(unsigned int mode) const
  {
    return fANN[mode];
  }

  void JMShower::SetEfrac_2slides(double efrac_2slides)
  {
    fEfrac_2slides = efrac_2slides;
  }
  double JMShower::Efrac_2slides() const
  {
    return fEfrac_2slides;
  }

  void JMShower::SetEfrac_4slides(double efrac_4slides)
  {
    fEfrac_4slides = efrac_4slides;
  }
  double JMShower::Efrac_4slides() const
  {
    return fEfrac_4slides;
  }

  void JMShower::SetEfrac_6slides(double efrac_6slides)
  {
    fEfrac_6slides = efrac_6slides;
  }
  double JMShower::Efrac_6slides() const
  {
    return fEfrac_6slides;
  }

  void JMShower::SetEfrac_20slides(double efrac_20slides)
  {
    fEfrac_20slides = efrac_20slides;
  }
  double JMShower::Efrac_20slides() const
  {
    return fEfrac_20slides;
  }

  void JMShower::SetEfrac_2sigmaHOR(double efrac_2sigmaHOR)
  {
    fEfrac_2sigmaHOR = efrac_2sigmaHOR;
  }
  double JMShower::Efrac_2sigmaHOR() const
  {
    return fEfrac_2sigmaHOR;
  }

  void JMShower::SetEfrac_4sigmaHOR(double efrac_4sigmaHOR)
  {
    fEfrac_4sigmaHOR = efrac_4sigmaHOR;
  }
  double JMShower::Efrac_4sigmaHOR() const
  {
    return fEfrac_4sigmaHOR;
  }

  void JMShower::SetEiso_2sigmaHOR(double eiso_2sigmaHOR)
  {
    fEiso_2sigmaHOR = eiso_2sigmaHOR;
  }
  double JMShower::Eiso_2sigmaHOR() const
  {
    return fEiso_2sigmaHOR;
  }
  void JMShower::SetEiso_4sigmaHOR(double eiso_4sigmaHOR)
  {
    fEiso_4sigmaHOR = eiso_4sigmaHOR;
  }
  double JMShower::Eiso_4sigmaHOR() const
  {
    return fEiso_4sigmaHOR;
  }
  void JMShower::SetEiso_6sigmaHOR(double eiso_6sigmaHOR)
  {
    fEiso_6sigmaHOR = eiso_6sigmaHOR;
  }
  double JMShower::Eiso_6sigmaHOR() const
  {
    return fEiso_6sigmaHOR;
  }

  void JMShower::SetNodes(std::vector<TVector3> nodespos, std::vector<double> nodese, std::vector<geo::OfflineChan> xid, std::vector<geo::OfflineChan> yid)
  {
    if(nodespos.size()!=nodese.size())return;
    fNNode = nodespos.size();
    fNodesPos.clear();
    fNodesE.clear();
    fNodesXID.clear();
    fNodesYID.clear();
    for(int i=0;i<fNNode;i++){
      fNodesPos.push_back(nodespos[i]);
      fNodesE.push_back(nodese[i]);
      fNodesXID.push_back(xid[i]);
      fNodesYID.push_back(yid[i]);
    }
  }

  int  JMShower::NNode() const
  {
    return fNNode;
  }

  TVector3 JMShower::NodePos(int node) const
  {
    TVector3 nodepos(0,0,0);
    if(node>=(int)fNodesPos.size())return nodepos;
    nodepos = fNodesPos[node];
    return nodepos;
  }

  double JMShower::NodeE(int node) const
  {
    double  nodee=0;
    if(node>=(int)fNodesE.size())return nodee;
    nodee = fNodesE[node];
    return nodee;
  }

  geo::OfflineChan JMShower::NodeXID(int node) const
  {
    geo::OfflineChan key0(0,0); 
    if(node>=(int)fNodesXID.size())return key0;
    return fNodesXID[node];
  }
  geo::OfflineChan JMShower::NodeYID(int node) const
  {
    geo::OfflineChan key0(0,0); 
    if(node>=(int)fNodesYID.size())return key0;
    return fNodesYID[node];
  }

  void JMShower::SetIxyz(TVector3 ixyz)
  {
    fIxyz = ixyz;
  }
  TVector3 JMShower::Ixyz()const
  {
    return fIxyz;
  }

  void JMShower::SetAsym(double asym)
  {
    fAsym = asym;
  }
  double JMShower::Asym()const
  {
    return fAsym;
  }

  
  std::map< int, std::vector< int > > JMShower::GetPlaneHits()
  {
    std::map< int, std::vector< int > > mapP;
    int ncells = this->NCell();
    for( int i = 0; i < ncells; ++i){
      mapP[ this->Cell(i)->Plane() ].push_back( i ); 
    }// end loop over cells

    return mapP;
  }// end of GetPlaneHits

  //......................................................................

    
  std::map< int, std::vector< int > > JMShower::GetTransHits()
  {
      
    std::map< int, std::vector< int > > mapT;
    art::ServiceHandle<geo::Geometry>   geom;    
    int ncells = this->NCell();
    TVector3 dir   = this->Dir();
    TVector3 start = this->Start();
    TVector3 stop  = this->Stop();

    float xslope = dir.X()/dir.Z();
    float yslope = dir.Y()/dir.Z();
    float planeX = (start.X() - stop.X() ) /2.0;
    float planeY = (start.Y() - stop.Y() ) /2.0;

    
    for( int i = 0; i < ncells; ++i){
      
      int plane    = this->Cell(i)->Plane();
      int cell     = this->Cell(i)->Cell();
      geo::View_t view = this->Cell(i)->View();
	
      // Find the cell in this plane that the shower
      // core passes through
      TVector3 cellCenter;

      geom->Plane(plane)->Cell(0)->GetCenter( cellCenter );
      float cellW  = geom->Plane( plane )->Cell(0)->HalfW();
      
      if( fabs(dir.Z()) > 0.0001 ){
	planeX = start.X() + ( cellCenter.Z() - start.Z() )*xslope;
	planeY = start.Y() + ( cellCenter.Z() - start.Z() )*yslope;
      }
      int trkCell = ( view == geo::kX )? std::ceil( (planeX- cellCenter.X())/cellW )
                                       : std::ceil( (planeY- cellCenter.Y())/cellW );

      
      mapT[ trkCell - cell ].push_back( i );
    }// end loop over cells    

    return mapT;
  }


}
  //......................................................................
