#include <cstdlib>
#include <iostream>
#include <map>
#include <string>
#include <stdlib.h> 
#include <sys/stat.h>
#include <sys/types.h>

#include "TFile.h"
#include "TTree.h"
#include "TChain.h"
#include "TObjString.h"
#include "TSystem.h"
#include "TROOT.h"
#include "TString.h"
#include "TMath.h"
#include "TStopwatch.h"
#include "TLegend.h"
#include "TCanvas.h"
#include "TLine.h"
#include "TSystem.h"
#include "TUnixSystem.h"



//#include "SimulationBase/MCParticle.h"
//#include "SimulationBase/MCNeutrino.h"
//#include "SimulationBase/MCTruth.h"
//#include "Simulation/FLSHit.h"
//#include "Simulation/FLSHitList.h"
//#include "Simulation/Particle.h"
//#include "Simulation/ParticleList.h"
//#include "Simulation/ParticleNavigator.h"
//#include "Simulation/PhotonSignal.h"

#include "TMVA/Factory.h"
#include "TMVA/Tools.h"
#include "TMVA/Reader.h"
#include "TMVA/MethodCuts.h"
#include "TMVA/Tools.h"
#include "TMVA/Config.h"

/**  TMVA-Based Electron ID classification training and diagnostics.
 * 
 *    This class is used to train a specified classifier to recognize charged current nue events and their 
 * backgrounds.  This code is based on JMShower EID and is intended to take advantage of the diagnostics and
 * ability to use different classifiers in TMVA.
 */

#ifndef TMVAEIDTRAINING_H
#define	TMVAEIDTRAINING_H

class TMVAEIDTraining {
public:
    TMVAEIDTraining(TString jobNameIn, TString outPathIn, TString myMethodListIn, TString useENuIn);
    TMVAEIDTraining(const TMVAEIDTraining& orig);
    virtual ~TMVAEIDTraining();
    //    void launchGui();
    void runTraining(TString cSVFileNameIn);
    void setupForReader();
    void setSignalFilesUntransformed(std::vector<TString> inputFileNameIn);
    void setBackgroundFilesUntransformed(std::vector<TString> inputFileNameIn);
    //    void setInputFileTransformed(TString inputFileNameIn);
    void launchGUI();

    std::map<std::string, int> getAlgorithmUseMap() {
        return Use;
    }
    void runDiagnostics();

private:
    //    TUnixSystem* systemPtr;

    enum CSVOperation {
        INIT,
        FILL,
        CLOSE
    };

    
    Int_t kIDNuE = 12;
    Int_t kIDElectron = 11;
    Int_t kIDNuMu = 14;
    Int_t kQE = 0;

    ofstream outfile;
    TStopwatch sw;
    TString fOutPath;
    TString fJobName;
    TString weightDir, plotDir, histDir;
    bool isTransformedInput = true;
    bool methodsBooked = false;
    bool useENu;
    TMVA::Factory *factory;
    TMVA::Reader *reader;
    TString myMethodList;
    // Default MVA methods to be trained + tested
    std::map<std::string, int> Use;
    TString outfileName;
    TFile* outputFile;
    TFile *input;
    TTree *trainingSample;
    //    TChain *trainingSample;
    //    TChain *trainingChainIn;
    TChain *signalIn;
    TChain *backgroundIn;
    TString cSVFileName;

    TH1F* htruenue;
    TH1F* htruenupi0;
    TH1F* hfomBeamNueBG;
    TH1F *hfom;
    TH1F *heff;
    TH1F *hbgeff;
    TH1F *hbgNumuCCeff;
    TH1F *hbgNCeff;
    TH1F *hbgNueCCeff;
    TH1F *bg;
    TH1F *bgNumuCC;
    TH1F *bgNC;
    TH1F *bgNueCC;
    TH1F *sig;
    TH1F *sigbg;

    double scalepotsig;
    double scalepotNumuCC;
    double scalepotNueCC;
    double scalepotNC;
    // Later, will be a loop over different methods
    TString currentAlgorithm = TString("MLP method");
    TString lineBuffer;
    int lineBufferSize;

    //    TString currentAlgorithm = "MLP";




    // <editor-fold defaultstate="collapsed" desc="Signal/Background UNtransformed Tree Variables">
    int evtTrueNuCCNC;
    int evtTrueNuMode;
    int evtTrueNuPdg;
    double evtTrueNuEnergy;
    int evtTrueNuIsFid;
    double evtTrueNuP4[4];
    double evtTrueNuVtx[3];
    int evtOsTag;
    double evtOsProb[20];
    double evtOsRand;
    double evtRwProb[20];
    double evtRwRand[20];
    int evtTruePdg[20];
    int evtTrueMother[20];
    double evtTrueEnergy[20];
    int evtRun;
    int evtSRun;
    int evtEvent;
    int evtNSlice;
    int evtNTrk;
    double evtPi0Mgg0, evtPi0Mgg, evtSh1Pi0Mgg, evtSh1VtxGeV, evtSh1CosTheta, evtSh1Length, evtPi0Eg1;
    double evtPi0Eg2, evtPi0TrueDirMgg0, evtPi0TrueDirMgg;
    double evtPi0StartDist;
    double evtPi0LineDist;
    double evtEtot0;
    double evtEtot;
    double evtSh1SumPt;
    double evtPEtot;
    double evtTrueEtot;
    double evtSh1Enue;
    double evtSh1Energy;
    double evtSh1Energy0;
    double evtTotalRecoGeV;
    double evtSh1SliceGeV;
    double evtSheEnergy;
    double evtSh1Gap;
    double evtSh1IsFid;
    double evtSh1P4[4];
    double evtSh1Start[3];
    double evtSh1DedxLLL[20];
    double evtSh1DedxLLT[20];
    double evtSh1NPlane;
    double evtSh1NMIPPlane;
    double evtSh1ContStartPlane;
    double evtSh2Energy;
    int evtSh1TruePdg;
    double evtSh1TrueDang;
    double evtSh1TrueEDang;
    // JM's counters-- should rename
    int numSignalCorrectProcess = 0, numSignalCorrectShower = 0, numSignalCorrectShowerNotObviousMuon = 0, ns3 = 0;
    int normSig = 0, normBkg = 0, totBkg = 0;
    int numBackgroundNuMuOrNuE0 = 0, numBackgroundPassingFirstShCuts = 0, numBackgroundPassingNotObviousMuonsShCuts2 = 0, nb3 = 0;
    int numBackgroundNueCC0 = 0, numBackgroundPassingFirstShCutsNueCC1 = 0, numBackgroundPassingNotObviousMuonsShCutsNueCC2 = 0, nbNueCC2_a = 0, nbNueCC3 = 0;
    int numBackgroundNumuCC0 = 0, numBackgroundPassingFirstShCutsNumuCC1 = 0, numBackgroundPassingNotObviousMuonsShCutsNumuCC2 = 0, nbNumuCC3 = 0;
    int numBackgroundNC0 = 0, numBackgroundPassingFirstShCutsNC1 = 0, numBackgroundPassingNotObviousMuonsShCutsNC2 = 0, nbNC3 = 0;
    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="TrainingVariables">
    float egLLL, egLLT, emuLLL, emuLLT, epi0LLL, epi0LLT, epLLL, epLLT, enLLL, enLLT, epiLLL, epiLLT, gap, pi0mass;
    float vtxgev, shE, nueRecEnergy, costheta, length, annPi0LineDist, annPi0StartDist;
    Int_t ntrk, type;
    std::string specType;
    float reWeight;
    // </editor-fold>


    void setTMVAAlgorithms();
    void internalTMVASetup();
    void preTrainingSetup();
    void setupTrainingAndTestTrees();
    void bookTMVAMethods();
    void trainTestAndEvaluateTMVAMethods();
    void saveOutput();
    void transformInputTrees(bool fillTrainingSample);
    void dumpTrainingSampleToCSV(CSVOperation operation, TString cSVFileName);
    void setTreeBranchAddresses(TTree* treeIncoming);
    void fillTrainingVariables();
    void preTrainHistograms();
    void saveDiagnosticHists();
    void printTrainingVars();
};

#endif	/* TMVAEIDTRAINING_H */

