////////////////////////////////////////////////////////////////////////
/// \brief   Photon signal class
/// \author  jpaley@indiana.edu
/// \date
////////////////////////////////////////////////////////////////////////
#include "Simulation/PhotonSignal.h"

#include <algorithm>

namespace sim
{
  //----------------------------------------------------------------------
  PhotonSignal::PhotonSignal() : 
    fTimeMean(0.), fNPhoton(0), fCell(0), fPlane(0), fTrackId(0), fPoissonLambda(0)
  {
  }
  
  //----------------------------------------------------------------------
  PhotonSignal::PhotonSignal(double time, int npe) : 
    fTimeMean(time), fNPhoton(npe), fCell(0), fPlane(0), fTrackId(0), fPoissonLambda(0)
  {   
  }
  
  //----------------------------------------------------------------------
  PhotonSignal::~PhotonSignal()
  {
  }

  //---------------------------------------------------------------------
  std::ostream& operator<< (std::ostream& o, const PhotonSignal& ps)
  {
    o << std::setiosflags(std::ios::fixed) << std::setprecision(2);
    o << "TrackId = "   << std::setw(5) << std::right << ps.TrackId()
      << " TimeMean = " << std::setw(5) << std::right << ps.TimeMean()
      << " Cell# = "    << std::setw(5) << std::right << ps.Cell()
      << " Plane# = "   << std::setw(5) << std::right << ps.Plane()
      << " NPhoton = "  << std::setw(5) << std::right << ps.NPhoton();

    return o;
  }

  //----------------------------------------------------------------------

  bool CompareByPlaneAndCell(const sim::PhotonSignal& a,
			     const sim::PhotonSignal& b)
  {
    if(a.fPlane < b.fPlane) return true;
    if(a.fPlane > b.fPlane) return false;
    return a.fCell < b.fCell;
  }

  //----------------------------------------------------------------------

  void SortByPlaneAndCell(std::vector<sim::PhotonSignal>& c)
  {
    std::sort(c.begin(), c.end(), CompareByPlaneAndCell);
  }

  //----------------------------------------------------------------------

  bool CompareByNPhoton(const sim::PhotonSignal& a,
			const sim::PhotonSignal& b)
  {
    return a.NPhoton() < b.NPhoton();
  }

  //----------------------------------------------------------------------

  void SortByNPhoton(std::vector<sim::PhotonSignal>& c)
  {
    std::sort(c.begin(), c.end(), CompareByNPhoton);
  }

} // end namespace sim
////////////////////////////////////////////////////////////////////////

