void slicer_production_validation() {

  // TODO:
  //   1. Get the stats boxes to automatically draw in the correct places.
  //   2. Have the axes auto zoom according to which histogram has the bigger max.
  //   3. Auto print the canvases.

  // Define initial parameters:
  std::string det  = "FarDet";
  std::string rel1 = "S14-03-06";
  std::string rel2 = "FA14-10-03";
  std::string cat  = "cosmic";
  char name[256];
  double max = 0.0;
  bool print = true;
  
  // Old MC
  TFile *f1 = new TFile("/nova/prod/mc/S14-03-06/reco/validation/hadd/fardet_cosmics_S14-03-06.sim.daq.reco.hist.root");
  // TFile *f1 = new TFile("/nova/prod/mc/S14-03-06/reco/validation/hadd/fardet_genie_rhc_swap_S14-03-06.sim.daq.reco.hist.root");
  gDirectory->cd("slicerana");
  TTree *nt1 = (TTree*)gDirectory->Get("SlicerAna");
  nt1->SetMarkerStyle(2);
  

  // New MC
  TFile *f2 = new TFile("/nova/ana/users/raddatz/FA14-10-03_Reco_Val/FA14-10-03_files/fd_cry.root");
  // TFile *f2 = new TFile("/nova/ana/users/raddatz/FA14-10-03_Reco_Val/FA14-10-03_files/fd_genie_rhc_nonswap.root");
  gDirectory->cd("slicerana");
  TTree *nt2 = (TTree*)gDirectory->Get("SlicerAna");
  nt2->SetMarkerStyle(2);



  double h0min, h0max;
  double h1min, h1max;
  double h2minX, h2maxX;
  double h2minY, h2maxY;
  double h3min, h3max;
  double h9min, h9max;

  double X1, Y1, Y2;

  double pmin, pmax;
  double cmin, cmax;
  double dpmin, dpmax;
  double dcmin, dcmax;

  if(det == "NDOS") {
    h0min = -0.5;
    h0max = 4.5;

    h1min = -0.5;
    h1max = 4.5;

    h2minX = -0.5;
    h2maxX = 6.5; 
    h2minY = 1.0;
    h2maxY = 50000.0;
   
    h3min = -0.5;
    h3max = 4.5;

    h9min = -0.5;
    h9max = 4.5;

    X1 = 200.0;
    Y1 = 64.0;
    Y2 = 96.0;

    pmin = -20.0;
    pmax = 300.0;
    cmin = -10.0;
    cmax = 100.0;
    dpmin = 0.0;
    dpmax = 200.0;
    dcmin = 0.0;
    dcmax = 100.0;
  }
  if(det == "NearDet") {
    h0min = 9.5;
    h0max = 60.5;

    h1min = -0.5;
    h1max = 30.5;

    h2minX = -0.5;
    h2maxX = 30.5; 
    h2minY = 1.0;
    h2maxY = 50000.0;
   
    h3min = -0.5;
    h3max = 4.5;

    h9min = -0.5;
    h9max = 30.5;

    X1 = 214.0;
    Y1 = 96.0;
    Y2 = 96.0;

    pmin = -20.0;
    pmax = 300.0;
    cmin = -10.0;
    cmax = 100.0;
    dpmin = 0.0;
    dpmax = 200.0;
    dcmin = 0.0;
    dcmax = 100.0;
  }
  if(det == "FarDet") {
    h0min = -0.5;
    h0max = 120.5;

    h1min = -0.5;
    h1max = 4.5;

    h2minX = -0.5;
    h2maxX = 100.5; 
    h2minY = 1.0;
    h2maxY = 50000.0;
   
    h3min = -0.5;
    h3max = 100.5;

    h9min = -0.5;
    h9max = 100.5;

    X1 = 896.0;
    Y1 = 384.0;
    Y2 = 384.0;

    pmin = -20.0;
    pmax = 1250.0;
    cmin = -10.0;
    cmax = 400.0;
    dpmin = 0.0;
    dpmax = 600.0;
    dcmin = 0.0;
    dcmax = 400.0;
  }




  
  // book histos...
  TH1F *h0_1 = new TH1F("h0_1","# of MCT (counted by FLS hits);# of nu;count",201,-0.5,200.5);
  TH1F *h0_2 = new TH1F("h0_2","# of MCT (counted by FLS hits);# of nu;count",201,-0.5,200.5);

  TH1F *h1 = new TH1F("h1","# of Nu;# of nu;count",101,-0.5,100.5);

  TH1F *h2_1 = new TH1F("h2_1","# of non-noise slices;# of non-noise slices;count",201,-0.5,200.5);
  TH1F *h2_2 = new TH1F("h2_2","# of non-noise slices;# of non-noise slices;count",201,-0.5,200.5);

  TH1F *h3_1 = new TH1F("h3_1","# of noise slices;# of noise slices;count",101,-0.5,100.5);
  TH1F *h3_2 = new TH1F("h3_2","# of noise slices;# of noise slices;count",101,-0.5,100.5);

  TH2F *h9_1 = new TH2F("h9_1","# of MCT vs. # of non-noise slices;# of MCT;# of slices",201,-0.5,200.5,201,-0.5,200.5);
  TH2F *h9_2 = new TH2F("h9_2","# of MCT vs. # of non-noise slices;# of MCT;# of slices",201,-0.5,200.5,201,-0.5,200.5);

  TCut Nu     = "NumNuFLSHitALL > 0";
  TCut nhit   = "NumHitsX >= 3 && NumHitsY >= 3";
  TCut noise  = "IsNoise == 1";
  TCut cosmic = "nupdg == 0";


  
  // count N slices w/ NHit cut

  // define leaf types and set branch addresses
  int NumNuFLSHitALL;
  int NumNuCellHit;
  int NumMCTCellHitALL;
  std::vector<unsigned int> *IsNoise;
  std::vector<int>          *NumHitsX;
  std::vector<int>          *NumHitsY;
  
  nt1->SetBranchAddress("NumNuFLSHitALL", &NumNuFLSHitALL);
  nt1->SetBranchAddress("NumNuCellHit", &NumNuCellHit);
  nt1->SetBranchAddress("NumNuCellHitALL", &NumMCTCellHitALL);
  nt1->SetBranchAddress("IsNoise",  &IsNoise);
  nt1->SetBranchAddress("NumHitsX", &NumHitsX);
  nt1->SetBranchAddress("NumHitsY", &NumHitsY);
  
  // loop over NTuple entries...
  for(unsigned int i = 0; i < nt1->GetEntries(); ++i) {

    nt1->GetEntry(i);

    unsigned int N = 0;

    for(unsigned int j = 0; j < (*IsNoise).size(); ++j) {     
      if((*IsNoise)[j] == 1) continue;
      if((*NumHitsX)[j] >= 3 && (*NumHitsY)[j] >= 3) N++;
    }

    h2_1->Fill(N);
    h9_1->Fill(NumMCTCellHitALL,N);

  }
  
  nt2->SetBranchAddress("NumMCTFLSHitALL", &NumNuFLSHitALL);
  nt2->SetBranchAddress("NumMCTCellHit", &NumNuCellHit);
  nt2->SetBranchAddress("NumMCTCellHitALL", &NumMCTCellHitALL);
  nt2->SetBranchAddress("IsNoise",  &IsNoise);
  nt2->SetBranchAddress("NumHitsX", &NumHitsX);
  nt2->SetBranchAddress("NumHitsY", &NumHitsY);

  // loop over NTuple entries...
  for(unsigned int i = 0; i < nt2->GetEntries(); ++i) {

    nt2->GetEntry(i);

    unsigned int N = 0;

    for(unsigned int j = 0; j < (*IsNoise).size(); ++j) {     
      if((*IsNoise)[j] == 1) continue;
      if((*NumHitsX)[j] >= 3 && (*NumHitsY)[j] >= 3) N++;
    }

    h2_2->Fill(N);
    h9_2->Fill(NumMCTCellHitALL,N);

  }
  

  
  TCanvas *c0 = new TCanvas("c0", "# of nu (by FLShits)",1000,700);
  c0->cd();
  nt1->Draw("NumNuFLSHitALL >> h0_1");
  nt2->Draw("NumMCTFLSHitALL >> h0_2");
  gStyle->SetOptStat(111111);
  h0_1->SetLineColor(kBlue);
  h0_1->SetLineWidth(2);
  h0_1->Scale(1.0/h0_1->Integral());
  h0_1->SetAxisRange(h0min,h0max,"X");
  h0_1->Draw();
  c0->Update();
  TPaveStats *s01a = h0_1->FindObject("stats");
  s01a->SetX1NDC(0.7);
  s01a->SetX2NDC(0.9);
  s01a->SetY1NDC(0.7);
  s01a->SetY2NDC(0.9);
  s01a->Draw();
  h0_2->SetLineColor(kRed);
  h0_2->SetLineWidth(2);
  h0_2->Scale(1.0/h0_2->Integral());
  h0_2->SetAxisRange(h0min,h0max,"X");
  h0_2->Draw("sames");
  c0->Update();
  TPaveStats *s01b = h0_2->FindObject("stats");
  s01b->SetX1NDC(0.7);
  s01b->SetX2NDC(0.9);
  s01b->SetY1NDC(0.5);
  s01b->SetY2NDC(0.7);
  s01b->Draw();
  sprintf(name,"NMCT_%s_%s_%s_%s.png",det.c_str(),cat.c_str(),rel1.c_str(),rel2.c_str());
  if(print) c0->Print(name);

  TCanvas *c2 = new TCanvas("c2", "# of slices",1000,700);
  c2->cd();
  gStyle->SetOptStat(111111);
  h2_1->SetLineColor(kBlue);
  h2_1->SetLineWidth(2);
  h2_1->Scale(1.0/h2_1->Integral());
  h2_1->SetAxisRange(h2minX,h2maxX,"X");
  h2_1->Draw();
  c2->Update();
  TPaveStats *s02a = h2_1->FindObject("stats");
  s02a->SetX1NDC(0.7);
  s02a->SetX2NDC(0.9);
  s02a->SetY1NDC(0.7);
  s02a->SetY2NDC(0.9);
  s02a->Draw();
  h2_2->SetLineColor(kRed);
  h2_2->SetLineWidth(2);
  h2_2->Scale(1.0/h2_2->Integral());
  h2_2->SetAxisRange(h2minX,h2maxX,"X");
  h2_2->Draw("sames");
  c2->Update();
  TPaveStats *s02b = h2_2->FindObject("stats");
  s02b->SetX1NDC(0.7);
  s02b->SetX2NDC(0.9);
  s02b->SetY1NDC(0.5);
  s02b->SetY2NDC(0.7);
  s02b->Draw();
  sprintf(name,"NSlices_%s_%s_%s_%s.png",det.c_str(),cat.c_str(),rel1.c_str(),rel2.c_str());
  if(print) c2->Print(name);
  
  TCanvas *c3 = new TCanvas("c3", "# of noise slices",1000,700);
  c3->cd();
  nt1->Draw("NumNoise >> h3_1");
  nt2->Draw("NumNoise >> h3_2");
  gStyle->SetOptStat(111111);
  h3_1->SetLineColor(kBlue);
  h3_1->SetLineWidth(2);
  h3_1->Scale(1.0/h3_1->Integral());
  h3_1->SetAxisRange(h3min,h3max,"X");
  h3_1->Draw();
  c3->Update();
  TPaveStats *s03a = h3_1->FindObject("stats");
  s03a->SetX1NDC(0.7);
  s03a->SetX2NDC(0.9);
  s03a->SetY1NDC(0.7);
  s03a->SetY2NDC(0.9);
  s03a->Draw();
  h3_2->SetLineColor(kRed);
  h3_2->SetLineWidth(2);
  h3_2->Scale(1.0/h3_2->Integral());
  h3_2->SetAxisRange(h3min,h3max,"X");
  h3_2->Draw("sames");
  c3->Update();
  TPaveStats *s03b = h3_2->FindObject("stats");
  s03b->SetX1NDC(0.7);
  s03b->SetX2NDC(0.9);
  s03b->SetY1NDC(0.5);
  s03b->SetY2NDC(0.7);
  s03b->Draw();
  sprintf(name,"NNoiseSlices_%s_%s_%s_%s.png",det.c_str(),cat.c_str(),rel1.c_str(),rel2.c_str());
  if(print) c3->Print(name);
  


  TH1F *hC_1  = new TH1F("hC_1","completeness for most pure MCTruth in slice; completeness; count",201,-0.005,2.005);
  TH1F *hP_1  = new TH1F("hP_1","purity for most pure MCTruth in slice; purity; count",201,-0.005,2.005);
  TH1F *hCN_1 = (TH1F*)f1->FindObjectAny("fCompNoise");
  TH1F *hPN_1 = (TH1F*)f1->FindObjectAny("fPurNoise");
  TH1F *hC_2  = new TH1F("hC_2","completeness for most pure MCTruth in slice; completeness; count",201,-0.005,2.005);
  TH1F *hP_2  = new TH1F("hP_2","purity for most pure MCTruth in slice; purity; count",201,-0.005,2.005);
  TH1F *hCN_2 = (TH1F*)f2->FindObjectAny("fCompNoise");
  TH1F *hPN_2 = (TH1F*)f2->FindObjectAny("fPurNoise");
  
  TCanvas *cC = new TCanvas("cC", "comp",1000,700);
  cC->cd();
  nt1->Draw("nu1Eff >> hC_1", nhit && !noise);
  nt2->Draw("nu1Eff >> hC_2", nhit && !noise);
  gStyle->SetOptStat(111111);
  cC->SetLogy();
  hC_1->SetLineColor(kBlue);
  hC_1->SetLineWidth(2);
  hC_1->Scale(1.0/hC_1->Integral());
  hC_1->SetAxisRange(0.0,1.2,"X");
  hC_1->Draw();
  cC->Update();
  TPaveStats *sCa = hC_1->FindObject("stats");
  sCa->SetX1NDC(0.4);
  sCa->SetX2NDC(0.6);
  sCa->SetY1NDC(0.7);
  sCa->SetY2NDC(0.9);
  sCa->Draw();
  hC_2->SetLineColor(kRed);
  hC_2->SetLineWidth(2);
  hC_2->Scale(1.0/hC_2->Integral());
  hC_2->SetAxisRange(0.0,1.2,"X");
  hC_2->Draw("sames");
  cC->Update();
  TPaveStats *sCb = hC_2->FindObject("stats");
  sCb->SetX1NDC(0.4);
  sCb->SetX2NDC(0.6);
  sCb->SetY1NDC(0.5);
  sCb->SetY2NDC(0.7);
  sCb->Draw();
  sprintf(name,"Comp_%s_%s_%s_%s.png",det.c_str(),cat.c_str(),rel1.c_str(),rel2.c_str());
  if(print) cC->Print(name);

  
  TCanvas *cP = new TCanvas("cP", "pur",1000,700);
  cP->cd();
  nt1->Draw("nu1Pur >> hP_1", nhit && !noise);
  nt2->Draw("nu1Pur >> hP_2", nhit && !noise);
  cP->SetLogy();
  gStyle->SetOptStat(111111);
  hP_1->SetLineColor(kBlue);
  hP_1->SetLineWidth(2);
  hP_1->Scale(1.0/hP_1->Integral());
  hP_1->SetAxisRange(0.0,1.2,"X");
  hP_1->Draw();
  cP->Update();
  TPaveStats *sPa = hP_1->FindObject("stats");
  sPa->SetX1NDC(0.4);
  sPa->SetX2NDC(0.6);
  sPa->SetY1NDC(0.7);
  sPa->SetY2NDC(0.9);
  sPa->Draw();
  hP_2->SetLineColor(kRed);
  hP_2->SetLineWidth(2);
  hP_2->Scale(1.0/hP_2->Integral());
  hP_2->SetAxisRange(0.0,1.2,"X");
  hP_2->Draw("sames");
  cC->Update();
  TPaveStats *sPb = hP_2->FindObject("stats");
  sPb->SetX1NDC(0.4);
  sPb->SetX2NDC(0.6);
  sPb->SetY1NDC(0.5);
  sPb->SetY2NDC(0.7);
  sPb->Draw();
  sprintf(name,"Pur_%s_%s_%s_%s.png",det.c_str(),cat.c_str(),rel1.c_str(),rel2.c_str());
  if(print) cP->Print(name);


  TCanvas *cCN = new TCanvas("cCN", "comp NOISE",1000,700);
  cCN->cd();
  cCN->SetLogy();
  gStyle->SetOptStat(111111);
  hCN_1->SetLineColor(kBlue);
  hCN_1->SetLineWidth(2);
  hCN_1->Scale(1.0/hCN_1->Integral());
  hCN_1->Draw();
  cCN->Update();
  TPaveStats *sCNa = hCN_1->FindObject("stats");
  sCNa->SetX1NDC(0.4);
  sCNa->SetX2NDC(0.6);
  sCNa->SetY1NDC(0.7);
  sCNa->SetY2NDC(0.9);
  sCNa->SetName("hCN_1");
  sCNa->Draw();
  hCN_2->SetLineColor(kRed);
  hCN_2->SetLineWidth(2);
  hCN_2->Scale(1.0/hCN_2->Integral());
  hCN_2->Draw("sames");
  cCN->Update();
  TPaveStats *sCNb = hCN_2->FindObject("stats");
  sCNb->SetX1NDC(0.4);
  sCNb->SetX2NDC(0.6);
  sCNb->SetY1NDC(0.5);
  sCNb->SetY2NDC(0.7);
  sCNb->SetName("hCN_2");
  sCNb->Draw();
  sprintf(name,"CompNoise_%s_%s_%s_%s.png",det.c_str(),cat.c_str(),rel1.c_str(),rel2.c_str());
  if(print) cCN->Print(name);

  TCanvas *cPN = new TCanvas("cPN", "pur NOISE",1000,700);
  cPN->cd();
  cPN->SetLogy();
  gStyle->SetOptStat(111111);
  hPN_1->SetLineColor(kBlue);
  hPN_1->SetLineWidth(2);
  hPN_1->Scale(1.0/hPN_1->Integral());
  hPN_1->Draw();
  cPN->Update();
  TPaveStats *sPNa = hPN_1->FindObject("stats");
  sPNa->SetX1NDC(0.4);
  sPNa->SetX2NDC(0.6);
  sPNa->SetY1NDC(0.7);
  sPNa->SetY2NDC(0.9);
  sPNa->SetName("hPN_1");
  sPNa->Draw();
  hPN_2->SetLineColor(kRed);
  hPN_2->SetLineWidth(2);
  hPN_2->Scale(1.0/hPN_2->Integral());
  hPN_2->Draw("sames");
  cPN->Update();
  TPaveStats *sPNb = hPN_2->FindObject("stats");
  sPNb->SetX1NDC(0.4);
  sPNb->SetX2NDC(0.6);
  sPNb->SetY1NDC(0.5);
  sPNb->SetY2NDC(0.7);
  sPNb->SetName("hPN_2");
  sPNb->Draw();
  sprintf(name,"PurNoise_%s_%s_%s_%s.png",det.c_str(),cat.c_str(),rel1.c_str(),rel2.c_str());
  if(print) cPN->Print(name);
  


  
  TH2F *hpcx_1 = new TH2F("hpcx_1","ave hit plane vs. ave hit X cell;plane;X cell",660,-20.0,1300.0,255,-10.0,500.0);
  TH2F *hpcy_1 = new TH2F("hpcy_1","ave hit plane vs. ave hit Y cell;plane;Y cell",660,-20.0,1300.0,255,-10.0,500.0);
  TH2F *hdpcx_1 = new TH2F("hdpcx_1","diff btwn hi and lo for plane vs x cell per slice;delta plane;delta x cell",500,0.0,1000.0,250,0.0,500.0);
  TH2F *hdpcy_1 = new TH2F("hdpcy_1","diff btwn hi and lo for plane vs y cell per slice;delta plane;delta y cell",500,0.0,1000.0,250,0.0,500.0);

  TH2F *hpcx_2 = new TH2F("hpcx_2","ave hit plane vs. ave hit X cell;plane;X cell",660,-20.0,1300.0,255,-10.0,500.0);
  TH2F *hpcy_2 = new TH2F("hpcy_2","ave hit plane vs. ave hit Y cell;plane;Y cell",660,-20.0,1300.0,255,-10.0,500.0);
  TH2F *hdpcx_2 = new TH2F("hdpcx_2","diff btwn hi and lo for plane vs x cell per slice;delta plane;delta x cell",500,0.0,1000.0,250,0.0,500.0);
  TH2F *hdpcy_2 = new TH2F("hdpcy_2","diff btwn hi and lo for plane vs y cell per slice;delta plane;delta y cell",500,0.0,1000.0,250,0.0,500.0);

  TLine *L1x = new TLine(0,0,0,Y1);
  TLine *L2x = new TLine(0,Y1,X1,Y1);
  TLine *L3x = new TLine(0,0,X1,0);
  TLine *L4x = new TLine(X1,0,X1,Y1);
  TLine *L1y = new TLine(0,0,0,Y2);
  TLine *L2y = new TLine(0,Y2,X1,Y2);
  TLine *L3y = new TLine(0,0,X1,0);
  TLine *L4y = new TLine(X1,0,X1,Y2);
  L1x->SetLineWidth(2);
  L2x->SetLineWidth(2);
  L3x->SetLineWidth(2);
  L4x->SetLineWidth(2);
  L1y->SetLineWidth(2);
  L2y->SetLineWidth(2);
  L3y->SetLineWidth(2);
  L4y->SetLineWidth(2);
  
  TCanvas *c5_1 = new TCanvas("c5_1", "ave plane vs. ave X cell - nt1",1000,700);
  c5_1->cd();
  nt1->Draw("CXave:Pave >> hpcx_1", nhit && !noise);
  gStyle->SetOptStat(111111);
  hpcx_1->SetAxisRange(pmin,pmax,"X");
  hpcx_1->SetAxisRange(cmin,cmax,"Y");
  hpcx_1->Draw("colz");
  c5_1->Update();
  TPaveStats *s5_1 = hpcx_1->FindObject("stats");
  s5_1->SetX1NDC(0.7);
  s5_1->SetX2NDC(0.9);
  s5_1->SetY1NDC(0.25);
  s5_1->SetY2NDC(0.75);
  s5_1->Draw();
  L1x->Draw();
  L2x->Draw();
  L3x->Draw();
  L4x->Draw();
  sprintf(name,"PCXave_%s_%s_%s.png",det.c_str(),cat.c_str(),rel1.c_str());
  if(print) c5_1->Print(name);
  
  TCanvas *c5_2 = new TCanvas("c5_2", "ave plane vs. ave X cell - nt2",1000,700);
  c5_2->cd();
  nt2->Draw("CXave:Pave >> hpcx_2", nhit && !noise);
  gStyle->SetOptStat(111111);
  hpcx_2->SetAxisRange(pmin,pmax,"X");
  hpcx_2->SetAxisRange(cmin,cmax,"Y");
  hpcx_2->Draw("colz");
  c5_2->Update();
  TPaveStats *s5_2 = hpcx_2->FindObject("stats");
  s5_2->SetX1NDC(0.7);
  s5_2->SetX2NDC(0.9);
  s5_2->SetY1NDC(0.25);
  s5_2->SetY2NDC(0.75);
  s5_2->Draw();
  L1x->Draw();
  L2x->Draw();
  L3x->Draw();
  L4x->Draw();
  sprintf(name,"PCXave_%s_%s_%s.png",det.c_str(),cat.c_str(),rel2.c_str());
  if(print) c5_2->Print(name);

  TCanvas *c6_1 = new TCanvas("c6_1", "ave plane vs. ave Y cell - nt1",1000,700);
  c6_1->cd();
  nt1->Draw("CYave:Pave >> hpcy_1", nhit && !noise);
  gStyle->SetOptStat(111111);
  hpcy_1->SetAxisRange(pmin,pmax,"X");
  hpcy_1->SetAxisRange(cmin,cmax,"Y"); 
  hpcy_1->Draw("colz");
  c6_1->Update();
  TPaveStats *s6_1 = hpcy_1->FindObject("stats");
  s6_1->SetX1NDC(0.7);
  s6_1->SetX2NDC(0.9);
  s6_1->SetY1NDC(0.25);
  s6_1->SetY2NDC(0.75);
  s6_1->Draw();
  L1y->Draw();
  L2y->Draw();
  L3y->Draw();
  L4y->Draw();
  sprintf(name,"PCYave_%s_%s_%s.png",det.c_str(),cat.c_str(),rel1.c_str());
  if(print) c6_1->Print(name);

  TCanvas *c6_2 = new TCanvas("c6_2", "ave plane vs. ave Y cell - nt2",1000,700);
  c6_2->cd();
  nt2->Draw("CYave:Pave >> hpcy_2", nhit && !noise);
  gStyle->SetOptStat(111111);
  hpcy_2->SetAxisRange(pmin,pmax,"X");
  hpcy_2->SetAxisRange(cmin,cmax,"Y"); 
  hpcy_2->Draw("colz");
  c6_2->Update();
  TPaveStats *s6_2 = hpcy_2->FindObject("stats");
  s6_2->SetX1NDC(0.7);
  s6_2->SetX2NDC(0.9);
  s6_2->SetY1NDC(0.25);
  s6_2->SetY2NDC(0.75);
  s6_2->Draw();
  L1y->Draw();
  L2y->Draw();
  L3y->Draw();
  L4y->Draw();
  sprintf(name,"PCYave_%s_%s_%s.png",det.c_str(),cat.c_str(),rel2.c_str());
  if(print) c6_2->Print(name);
  
  TH2F *hpcxN_1 = new TH2F("hpcxN_1","ave hit plane vs. ave hit X cell NOISE slice;plane;X cell",660,-20.0,1300.0,255,-10.0,500.0);
  TH2F *hpcyN_1 = new TH2F("hpcyN_1","ave hit plane vs. ave hit Y cell NOISE slice;plane;Y cell",660,-20.0,1300.0,255,-10.0,500.0);

  TH2F *hpcxN_2 = new TH2F("hpcxN_2","ave hit plane vs. ave hit X cell NOISE slice;plane;X cell",660,-20.0,1300.0,255,-10.0,500.0);
  TH2F *hpcyN_2 = new TH2F("hpcyN_2","ave hit plane vs. ave hit Y cell NOISE slice;plane;Y cell",660,-20.0,1300.0,255,-10.0,500.0);

  TCanvas *c5N_1 = new TCanvas("c5N_1", "ave plane vs. ave X cell NOISE - nt1",1000,700);
  c5N_1->cd();
  nt1->Draw("CXave:Pave >> hpcxN_1", noise);
  gStyle->SetOptStat(111111);
  hpcxN_1->SetAxisRange(pmin,pmax,"X");
  hpcxN_1->SetAxisRange(cmin,cmax,"Y");
  hpcxN_1->Draw("colz");
  c5N_1->Update();
  TPaveStats *s5N_1 = hpcxN_1->FindObject("stats");
  s5N_1->SetX1NDC(0.7);
  s5N_1->SetX2NDC(0.9);
  s5N_1->SetY1NDC(0.25);
  s5N_1->SetY2NDC(0.75);
  s5N_1->Draw();
  L1x->Draw();
  L2x->Draw();
  L3x->Draw();
  L4x->Draw();
  sprintf(name,"PCXaveNoise_%s_%s_%s.png",det.c_str(),cat.c_str(),rel1.c_str());
  if(print) c5N_1->Print(name);

  TCanvas *c5N_2 = new TCanvas("c5N_2", "ave plane vs. ave X cell NOISE - nt2",1000,700);
  c5N_2->cd();
  nt2->Draw("CXave:Pave >> hpcxN_2", noise);
  gStyle->SetOptStat(111111);
  hpcxN_2->SetAxisRange(pmin,pmax,"X");
  hpcxN_2->SetAxisRange(cmin,cmax,"Y");
  hpcxN_2->Draw("colz");
  c5N_2->Update();
  TPaveStats *s5N_2 = hpcxN_2->FindObject("stats");
  s5N_2->SetX1NDC(0.7);
  s5N_2->SetX2NDC(0.9);
  s5N_2->SetY1NDC(0.25);
  s5N_2->SetY2NDC(0.75);
  s5N_2->Draw();
  L1x->Draw();
  L2x->Draw();
  L3x->Draw();
  L4x->Draw();
  sprintf(name,"PCXaveNoise_%s_%s_%s.png",det.c_str(),cat.c_str(),rel2.c_str());
  if(print) c5N_2->Print(name);

  TCanvas *c6N_1 = new TCanvas("c6N_1", "ave plane vs. ave Y cell NOISE - nt1",1000,700);
  c6N_1->cd();
  nt1->Draw("CYave:Pave >> hpcyN_1", noise);
  gStyle->SetOptStat(111111);
  hpcyN_1->SetAxisRange(pmin,pmax,"X");
  hpcyN_1->SetAxisRange(cmin,cmax,"Y"); 
  hpcyN_1->Draw("colz");
  c6N_1->Update();
  TPaveStats *s6N_1 = hpcyN_1->FindObject("stats");
  s6N_1->SetX1NDC(0.7);
  s6N_1->SetX2NDC(0.9);
  s6N_1->SetY1NDC(0.25);
  s6N_1->SetY2NDC(0.75);
  s6N_1->Draw();
  L1y->Draw();
  L2y->Draw();
  L3y->Draw();
  L4y->Draw();
  sprintf(name,"PCYaveNoise_%s_%s_%s.png",det.c_str(),cat.c_str(),rel1.c_str());
  if(print) c6N_1->Print(name);

  TCanvas *c6N_2 = new TCanvas("c6N_2", "ave plane vs. ave Y cell NOISE - nt2",1000,700);
  c6N_2->cd();
  nt2->Draw("CYave:Pave >> hpcyN_2", noise);
  gStyle->SetOptStat(111111);
  hpcyN_2->SetAxisRange(pmin,pmax,"X");
  hpcyN_2->SetAxisRange(cmin,cmax,"Y"); 
  hpcyN_2->Draw("colz");
  c6N_2->Update();
  TPaveStats *s6N_2 = hpcyN_2->FindObject("stats");
  s6N_2->SetX1NDC(0.7);
  s6N_2->SetX2NDC(0.9);
  s6N_2->SetY1NDC(0.25);
  s6N_2->SetY2NDC(0.75);
  s6N_2->Draw();
  L1y->Draw();
  L2y->Draw();
  L3y->Draw();
  L4y->Draw();
  sprintf(name,"PCYaveNoise_%s_%s_%s.png",det.c_str(),cat.c_str(),rel2.c_str());
  if(print) c6N_2->Print(name);
  


  
  TH1F *ht_1   = new TH1F("ht_1","ave hit time per slice; time [ns]; count",120,-50000.0,550000.0);
  TH1F *htsd_1 = new TH1F("htsd_1","st.dev. of hit time per slice; time [ns]; count",500000,0.0,500000.0);
  TH1F *ht_2   = new TH1F("ht_2","ave hit time per slice; time [ns]; count",120,-50000.0,550000.0);
  TH1F *htsd_2 = new TH1F("htsd_2","st.dev. of hit time per slice; time [ns]; count",500000,0.0,500000.0);

  TCanvas *cT = new TCanvas("cT", "ave time",1000,700);
  cT->cd();
  nt1->Draw("Tave >> ht_1", nhit && !noise);
  nt2->Draw("Tave >> ht_2", nhit && !noise);
  gStyle->SetOptStat(111111);
  ht_1->SetLineColor(kBlue);
  ht_1->SetLineWidth(2);
  ht_1->Scale(1.0/ht_1->Integral());
  ht_1->SetAxisRange(-500.0,600000.0,"X");
  ht_1->Draw();
  cT->Update();
  TPaveStats *sTa = ht_1->FindObject("stats");
  sTa->SetX1NDC(0.7);
  sTa->SetX2NDC(0.9);
  sTa->SetY1NDC(0.7);
  sTa->SetY2NDC(0.9);
  sTa->Draw();
  ht_2->SetLineColor(kRed);
  ht_2->SetLineWidth(2);
  ht_2->Scale(1.0/ht_2->Integral());
  ht_2->SetAxisRange(-500.0,600000.0,"X");
  ht_2->Draw("sames");
  cT->Update();
  TPaveStats *sTb = ht_2->FindObject("stats");
  sTb->SetX1NDC(0.7);
  sTb->SetX2NDC(0.9);
  sTb->SetY1NDC(0.5);
  sTb->SetY2NDC(0.7);
  sTb->Draw();
  sprintf(name,"Tave_%s_%s_%s_%s.png",det.c_str(),cat.c_str(),rel1.c_str(),rel2.c_str());
  if(print) cT->Print(name);

  TCanvas *cTsd = new TCanvas("cTsd", "sd time",1000,700);
  cTsd->cd();
  nt1->Draw("Tsd >> htsd_1", nhit && !noise);
  nt2->Draw("Tsd >> htsd_2", nhit && !noise);
  gStyle->SetOptStat(111111);
  htsd_1->SetLineColor(kBlue);
  htsd_1->SetLineWidth(2);
  htsd_1->Scale(1.0/htsd_1->Integral());
  htsd_1->SetAxisRange(0.0,500.0,"X");
  htsd_1->Draw();
  cTsd->Update();
  TPaveStats *sTsda = htsd_1->FindObject("stats");
  sTsda->SetX1NDC(0.7);
  sTsda->SetX2NDC(0.9);
  sTsda->SetY1NDC(0.7);
  sTsda->SetY2NDC(0.9);
  sTsda->Draw();
  htsd_2->SetLineColor(kRed);
  htsd_2->SetLineWidth(2);
  htsd_2->Scale(1.0/htsd_2->Integral());
  htsd_2->SetAxisRange(0.0,500.0,"X");
  htsd_2->Draw("sames");
  cTsd->Update();
  TPaveStats *sTsdb = htsd_2->FindObject("stats");
  sTsdb->SetX1NDC(0.7);
  sTsdb->SetX2NDC(0.9);
  sTsdb->SetY1NDC(0.5);
  sTsdb->SetY2NDC(0.7);
  sTsdb->Draw();
  sprintf(name,"Tsd_%s_%s_%s_%s.png",det.c_str(),cat.c_str(),rel1.c_str(),rel2.c_str());
  if(print) cTsd->Print(name);

  TH1F *htN_1 = new TH1F("htN_1","ave hit time per NOISE slice; time [ns]; count",1000,0.0,500000.0);
  TH1F *htN_2 = new TH1F("htN_2","ave hit time per NOISE slice; time [ns]; count",1000,0.0,500000.0);

  TCanvas *cTN = new TCanvas("cTN", "ave time NOISE",1000,700);
  cTN->cd();
  nt1->Draw("Tave >> htN_1", noise);
  nt2->Draw("Tave >> htN_2", noise);
  gStyle->SetOptStat(111111);
  htN_1->SetLineColor(kBlue);
  htN_1->SetLineWidth(2);
  htN_1->Scale(1.0/htN_1->Integral());
  htN_1->SetAxisRange(250000.0,300000.0,"X");
  htN_1->Draw();
  cTN->Update();
  TPaveStats *sTNa = htN_1->FindObject("stats");
  sTNa->SetX1NDC(0.7);
  sTNa->SetX2NDC(0.9);
  sTNa->SetY1NDC(0.7);
  sTNa->SetY2NDC(0.9);
  sTNa->Draw();
  htN_2->SetLineColor(kRed);
  htN_2->SetLineWidth(2);
  htN_2->Scale(1.0/htN_2->Integral());
  htN_2->SetAxisRange(250000.0,300000.0,"X");
  htN_2->Draw("sames");
  cTN->Update();
  TPaveStats *sTNb = htN_2->FindObject("stats");
  sTNb->SetX1NDC(0.7);
  sTNb->SetX2NDC(0.9);
  sTNb->SetY1NDC(0.5);
  sTNb->SetY2NDC(0.7);
  sTNb->Draw();
  sprintf(name,"TaveNoise_%s_%s_%s_%s.png",det.c_str(),cat.c_str(),rel1.c_str(),rel2.c_str());
  if(print) cTN->Print(name);
  





  // Below are plots no longer made as part of the standard production validation suite

  /*
  TCanvas *c11 = new TCanvas("c11", "delta plane vs delta X cell",1000,700);
  c11->cd();
  nt1->Draw("(CXhi-CXlo):(Phi-Plo) >> hdpcx", nhit && !noise);
  gStyle->SetOptStat(111111);
  hdpcx->SetAxisRange(dpmin,dpmax,"X");
  hdpcx->SetAxisRange(dcmin,dcmax,"Y");
  hdpcx->Draw("colz");

  TCanvas *c12 = new TCanvas("c12", "delta plane vs delta Y cell",1000,700);
  c12->cd();
  nt1->Draw("(CYhi-CYlo):(Phi-Plo) >> hdpcy", nhit && !noise);
  gStyle->SetOptStat(111111);
  hdpcy->SetAxisRange(dpmin,dpmax,"X");
  hdpcy->SetAxisRange(dcmin,dcmax,"Y");
  hdpcy->Draw("colz");

  TLine *L0 = new TLine(-0.5,-0.5,100.5,100.5);
  L0->SetLineWidth(2);

  TCanvas *c9_1 = new TCanvas("c9_1", "MCT vs. slices - nt1",1000,700);
  c9_1->cd();
  gStyle->SetOptStat(111111);
  h9_1->SetAxisRange(h9min,h9max,"X");
  h9_1->SetAxisRange(h9min,h9max,"Y");
  h9_1->Draw("colz");
  L0->Draw();

  TCanvas *c9_2 = new TCanvas("c9_2", "MCT vs. slices - nt2",1000,700);
  c9_2->cd();
  gStyle->SetOptStat(111111);
  h9_2->SetAxisRange(h9min,h9max,"X");
  h9_2->SetAxisRange(h9min,h9max,"Y");
  h9_2->Draw("colz");
  L0->SetLineWidth(2);
  L0->Draw();
  */

}
