////////////////////////////////////////////////////////////////////////
// \file    SRBeam.h
// \brief   This class contains information about the neutrino production
////////////////////////////////////////////////////////////////////////
#ifndef SRBEAM_H
#define SRBEAM_H

#include "TVector3.h"

namespace caf
{
/** \brief This class contains information about the neutrino production.
* Docs from http://www.hep.utexas.edu/~zarko/wwwgnumi/v19/v19/output_gnumi.html
*/
  class SRBeam
  {
  public:
    SRBeam();
    ~SRBeam();

    void setDefault();

    

    ///  "target vertex". Exit point of parent particle at the target
    TVector3  tv;         
                          
    ///  "target momentum". Parent momentum exiting the target 
    TVector3  tp;         
                          
    ///  Flux run number, indicates which FLUGG job created this flux
    float     runjob;     
                          
    /// Proton event number, indicates which simulated proton was taken
    int       potnum;     
                          
    /// "target particle type". Parent particle ID exiting the target (PDG code)
    int       tptype;     
                          
    ///  "neutrino importance weight". Weight of neutrino parent
    float     nimpwt;     
                          
    ///  "neutrino decay". Decay mode that produced neutrino
    int       ndecay;     
                          
    ///  "vertex". Position of hadron/muon decay
    TVector3  v;          


    ///  "parent decay momentum". Parent momentum at decay point
    TVector3  pdp;        
                          
    ///  "parent particle momentum dx/dx". Parent dx/dz direction at production
    float     ppdxdz;     
                          
    ///  "parent particle momentum dy/dz". Parent dy/dz direction at production
    float     ppdydz;     
                          
    ///  "parent particle p_z". Parent Z momentum at production
    float     pppz;       
                          
    /// "parent particle energy". Parent energy at production
    float     ppenergy;   
                          
    /// "parent particle medium". Tracking medium number where parent was produced
    int       ppmedium;   
                          
    ///  "parent type". Parent PDG code
    int       ptype;      

    ///  "parent particle vertex". Parent production vertex
    TVector3  ppv;        
                          
    ///  Muon neutrino parent momentum, x component 
    TVector3  muparp;   

    ///  Muon neutrino parent energy
    float     mupare;     

    /// Neutrino energy in CM frame 
    float     necm;

    ///  nu parent generation 1=primary proton, 2=secondary, 3=teriary, etc
    int       tgen; 

    /// PDG of parent of the particle exiting the target
    int       tgptype;
    
   /// distance from decay to ray origin (ray origin is the intermediate point on the flux window)
   float     dk2gen;
   
   /// distance from ray origin to event vtx
   float     gen2vtx;
   
   //the best estimate of the true travel distance of the neutrino (fdk2gen+fgen2vtx)
   float     dk2vtx;  
   

  };

} // end namespace

#endif // SRBEAM_H
//////////////////////////////////////////////////////////////////////////
