////////////////////////////////////////////////////////////////////////
// \file    SRGenerator.h
// \brief   Object collecting Generator (GENIE) MC truth variables for caf
// \version
// \author  Nathan Mayer - nathan.mayer@tufts.edu
////////////////////////////////////////////////////////////////////////
#ifndef SRGENERATOR_H
#define SRGENERATOR_H

#include "TVector3.h"
#include <TLorentzVector.h>
#include "StandardRecord/SRGenParticle.h"

#include <vector>

/// \f$ \nu_e \f$ sandbox
namespace caf
{
  /// \f$ \nu_e \f$ sandbox variables
  class SRGenerator
  {
  public:
    SRGenerator();
    ~SRGenerator();
    
    //interactions info
    int fGint; ///<interaction code
    int fGscatter; ///< neutrino scattering code

    double fweight;
    double fprobability;
    double fXsec;
    double fDiffXsec;
    
    //genie::XclsTag info
    int fNumPiPlus;  ///< number of pi pluses in the final state
    int fNumPiMinus; ///< number of pi minuses in the final state
    int fNumPi0;     ///< number of pi0 in the final state
    int fNumProton;  ///< number of protons in the final state
    int fNumNeutron; ///< number of neutrons in the final state
    bool fIsCharm;   ///< did the interaction produce a charmed hadron
    int fResNum;     ///< resonance number

    //kinematics info 
    ///<these are for the interal (on shell) genie kinematics
    //The genie event kinematics are subtle different from the event 
    //kinematics that a experimentalist would calculate Instead of 
    //retriving the genie values for these kinematic variables calcuate 
    //them from the the final state particle while ingnoring the fermi 
    //momentum and the off-shellness of the bound nucleon.
    double fgQ2;
    double fgq2;
    double fgW;
    double fgT;
    double fgX;
    double fgY;
    TLorentzVector fFShadSystP4;
       
    //Target info
    bool fIsSeaQuark;
    int ftgtZ;
    int ftgtA;
    int ftgtPDG; //Target Nucleous(?) PDG
    TLorentzVector fHitNucP4;
    
    //Initial State info
    int fProbePDG;
    TLorentzVector fProbeP4;
    TLorentzVector fVertex;

    //Particles associated with the neutrino interaction
    std::vector <SRGenParticle> fParticleList;
    //SRGenParticle *fFSLepton;
    //SRGenParticle *fProbe;
    int fFSLeptonIndx;
    int fProbeIndx;

    
    //SRGenParticle fFSLepton;
    //SRGenParticle fProbe;
    //SRGenParticle Neutrino() {return fProbe;}
    //SRGenParticle Lepton() {return fFSLepton;}
    
    //GENIE interaction info
    int fHitNucleon;
    int fHitQuark;

    double fVisEfraction;
    double visEinslc;
    
    //private:
    
  };

}

#endif // SRGENERATOR_H
//////////////////////////////////////////////////////////////////////////////

