////////////////////////////////////////////////////////////////////////
// \file    SRLem.h
// \brief   This class contains the LEM PID output.
// \version $Id: SRLem.h,v 1.4 2012-12-07 06:37:29 gsdavies Exp $
// \author  $Author: gsdavies $
// \date    $Date: 2012-12-07 06:37:29 $
////////////////////////////////////////////////////////////////////////
#ifndef SRLEM_H
#define SRLEM_H

namespace caf
{
  /// \brief This class contains the LEM PID output.
  ///
  /// Many variables are some kind of average over the matches. There are
  /// simple means (no suffix), means exponentially weighted by the match
  /// potential ("exp"), and a procedure by which one fits a linear trend and
  /// evaluates it at the potential of the best match ("fit").
  class SRLem
  {
    public:
      ///  Default constructor.
      SRLem();

      /// Default destuctor.
      ~SRLem();

      float pid; ///< The default PID value - normally use this one

      float pidfit;    ///< Fraction of matches that are signal, "fit"
      float pidexp;    ///< Fraction of matches that are signal, "exp"
      float pidexpfit; ///< Fraction of matches that are signal, "expfit"

      float meany;       ///< Mean hadronic y of matches
      float meanyfit;    ///< Hadronic y of matches, "fit"
      float meanyexp;    ///< Hadronic y of matches, "exp"
      float meanysig;    ///< Mean hadronic y of matches that are signal
      float meanysigexp; ///< Hadronic y of matches that are signal, "exp"

      float meanqfrac;       ///< Mean fraction of charge matched
      float meanqfracfit;    ///< Fraction of charge matched, "fit"
      float meanqfracexp;    ///< Fraction of charge matched, "exp"
      float meanqfracsig;    ///< Fraction matched in signal matches
      float meanqfracsigexp; ///< Fraction matched in signal matches "exp"
      float meanqfracbkg;    ///< Mean fraction matched in background matches
      float meanqfracbkgexp; ///< Fraction matched in background matches, "exp"

      float energydiff;    ///< Potential diff between sig and bkg matches
      float energydiffexp; ///< Pot. diff between sig and bkg matches, "exp"

      float enrichfrac;    ///< Fraction of matches that are "enriched"
      float enrichfracexp; ///< Fraction of matches that are "enriched", "exp"

      float fannid;       ///< ANN output, "fann" package
      float fannidenrich; ///< ANN output, including \a enrichfracexp

      float chisig;    ///< chi-square of a fit assuming best match is signal
      float chibkg;    ///< chisq of a fit assuming best match is background
      float chisigexp; ///< chisq of a fit assuming best match is signal, "exp"
      float chibkgexp; ///< chisq of a fit assuming best match is bkg, "exp"

      float avginvE; ///< Fraction of signal matches, 1/E weighting
      float avgexpE; ///< Fraction of signal matches, "exp" weighting
      float avgsigE; ///< Fraction of signal matches, sigmoid weighting

      float tmvabdtg;  ///< TMVA boosted decision tree, gradient boost
      float tmvabdt;   ///< TMVA boosted decision tree, adaptive boost
      float tmvabdtd;  ///< TMVA boosted decision tree, decorrelation
      float tmvabdtmf; ///< TMVA boosted decision tree, Fisher discriminant
      float tmvamlp;   ///< TMVA ANN ("multi layer perceptron")

      float dectree; ///< Decision tree PID

      void setDefault();
  };

} // end namespace

#endif // SRLEM_H
//////////////////////////////////////////////////////////////////////////
