////////////////////////////////////////////////////////////////////////
// \file    SRNumuEnergy.h
// \brief   This is a class for the Numu energy estimator output in the standard record.  
// \version $Id: SRNumuEnergy.h,v 1.1 2012-11-19 19:45:16 rocco Exp $  
// \author  $Author: rocco $
// \date    $Date: 2012-11-19 19:45:16 $
////////////////////////////////////////////////////////////////////////
#ifndef SRNUMUENERGY_H
#define SRNUMUENERGY_H

#include "StandardRecord/SRSlice.h"

namespace caf
{
  /// This is a container for the numu energy estimator output in the standard record.  
  class SRNumuEnergy
    {
    public:
      ///  Default constructor. 
      SRNumuEnergy();
      
      /// Default destuctor.
      virtual ~SRNumuEnergy();

      float      E;              ///< Energy      [GeV]

      float      calccE;         ///< Calorometric charged current neutrino energy [GeV]
      float      trkqeE;         ///< Track length quasielastic neutrino energy [GeV]
      float      trknonqeE;      ///< Track length non-quasielastic neutrino energy [GeV]
      float      trkccE;         ///< Track length cc neutrino energy [GeV]
      float      shiftedtrkccE;  ///< Track length cc neutrino energy [GeV] (with hadE = hadE*0.77)
      float      angleE;         ///< Quasielastic angle formula neutrino energy [GeV]
      float      angleerror;     ///< 1-sigma error from using quasielastic angle formula for neutrino energy
      float      hadcalE;        ///< Hadronic calorometric energy NOT on the muon track[GeV]
      float      hadtrkE;        ///< Hadronic calorometric energy on the muon track[GeV]
      float      ucccE;          ///< Uncontained Energy of numu events (TMVA BDT)
      float      fsE;            ///< Fernanda's scattering energy
      float      ndtrklenact;    ///< Near detector -- muon track length in active region [cm]
      float      ndtrklencat;    ///< Near detector -- muon track length in muon catcher  [cm]
      float      ndtrkcalactE;   ///< Near detector -- muon calorometric energy in active region [GeV]
      float      ndtrkcaltranE;  ///< Near detector -- muon calorometric energy in transition plane [GeV]
      float      ndtrkcalcatE;   ///< Near detector -- muon calorometric energy in muon catcher [GeV]
      float      ndhadcalactE;   ///< Near detector -- hadronic calorometric energy NOT on the muon track in active region [GeV]
      float      ndhadcaltranE;  ///< Near detector -- hadronic calorometric energy NOT on the muon track in transition plane [GeV]
      float      ndhadcalcatE;   ///< Near detector -- hadronic calorometric energy NOT on the muon track in muon catcher [GeV]
      float      ndhadtrkactE;   ///< Near detector -- hadronic calorometric energy on the muon track in active region [GeV]
      float      ndhadtrktranE;  ///< Near detector -- hadronic calorometric energy on the muon track in transition plane [GeV]
      float      ndhadtrkcatE;   ///< Near detector -- hadronic calorometric energy on the muon track in muon catcher [GeV]
      float      ndtrktranx;     ///< Near detector -- if muon track crosses transition plane, the x location [cm]
      float      ndtrktrany;     ///< Near detector -- if muon track crosses transition plane, the y location [cm]
      SRSlice    hadclust;     ///< Cluster information for hits in slice but not on muon track (track with higest ReMId value)

      virtual void setDefault();
      
    };
  
} // end namespace

#endif // SRNUMUENERGY_H
//////////////////////////////////////////////////////////////////////////////
