////////////////////////////////////////////////////////////////////////
// \file    SRTrack.h
// \brief   An SRTrack is a high level track object.  It knows its 
//          direction and length, but does not own its cell hits.
// \version $Id: SRTrack.h,v 1.6 2012-11-13 22:06:10 rocco Exp $
// \author  $Author: rocco $
// \date    $Date: 2012-11-13 22:06:10 $
////////////////////////////////////////////////////////////////////////
#ifndef SRTRACK_H
#define SRTRACK_H
#include "StandardRecord/SRProng.h"

#include "TVector3.h"

namespace caf
{
  
  
  /// A class that is more boiled down than the rb::Track.  This class does not contain cell hits, but 
  /// does know it's energy and direction.  This class is meant to be used in Common Analysis Files as 
  /// part of the StandardRecord.
  class SRTrack
    {
    public:
      ///  Default constructor for SRTrack. 
      SRTrack();
      
      /// Default destuctor. 
      ~SRTrack(){  };

      unsigned short nhit;         ///< number of hits
      unsigned short nhitx;        ///< number of hits in x-view
      unsigned short nhity;        ///< number of hits in y-view
      unsigned short nplane;       ///< number of planes spanned
      unsigned short maxplanecont; ///< maximum number of contiguous planes in prong
      unsigned short maxplanegap;  ///< maximum number of gapped planes in prong
      unsigned short nplanegap;    ///< total number of missing planes on track
      float          calE;         ///< energy based on summed calibrated deposited charge [GeV]
      TVector3       start;        ///< Shower start point in detector coordinates. [cm]
      TVector3       dir;          ///< Shower direction at start point [unit vector recommended]
      float          len;          ///< track length [cm]
      View_t         view;         ///< Prong view caf::kX = 0, caf::kY = 1 or caf::kXorY = 2

      float          lenE;     ///< energy based on track lentgh and MIP assumption [GeV]
      TVector3       stop;     ///< Track end point in detector coordinates.  [cm]
    };
  
} // end namespace

#endif // SRTRACK_H
//////////////////////////////////////////////////////////////////////////////
