////////////////////////////////////////////////////////////////////////
// \file    SRTrackBranch.h
// \brief   An SRTrackBranch contains vectors of SRTracks.  
//          It is intended for use in the Common Analysis ROOT trees.
//
// \version $Id: SRTrackBranch.h,v 1.3 2012-11-13 22:06:10 rocco Exp $
// \author  $Author: rocco $
// \date    $Date: 2012-11-13 22:06:10 $
////////////////////////////////////////////////////////////////////////
#ifndef SRTRACKBRANCH_H
#define SRTRACKBRANCH_H
#include "StandardRecord/SRTrack.h"

#include <vector>

namespace caf
{
  /// A class that is more boiled down than the rb::Track.  This class does not contain cell hits, but 
  /// does know it's energy and direction.  This class is meant to be used in Common Analysis Files as 
  /// part of the StandardRecord.
  class SRTrackBranch
    {
    public:
      ///  Default constructor for SRTrackBranch. 
      SRTrackBranch();
      /// Default destuctor.  
      ~SRTrackBranch();
      
      std::vector<SRTrack> kalman;        ///< 3D Tracks produced by KalmanTrack
      std::vector<SRTrack> kalman2d;      ///< 2D Tracks produced by KalmanTrack
      std::vector<SRTrack> discrete;      ///< 3D tracks produced by DiscreteTrack
      std::vector<SRTrack> discrete2d;    ///< 2D tracks produced by DiscreteTrack
      std::vector<SRTrack> cosmic;        ///< Tracks produced by CosmicTrack

      size_t nkalman;      ///< Number of 3D tracks produced by KalmanTrack (size of vector)
      size_t nkalman2d;    ///< Number of 2D tracks produced by KalmanTrack (size of vector)
      size_t ndiscrete;    ///< Number of 3D tracks produced by DiscreteTrack (size of vector)
      size_t ndiscrete2d;  ///< Number of 2D tracks produced by DiscreteTrack  (size of vector)
      size_t ncosmic;      ///< Number of tracks produced by CosmicTrack  (size of vector)

      void fillSizes();

      
    };
  
} // end namespace

#endif // SRTRACKBRANCH_H
//////////////////////////////////////////////////////////////////////////////
