////////////////////////////////////////////////////////////////////////
// \file    SRTrkME.cxx
// \brief   SRTrkME gives variables describing Michel E's found around
//          at the end of a track
// \author  dpershey
// \date    2014-08-20
////////////////////////////////////////////////////////////////////////
#ifndef SRTRKME_H
#define SRTRKME_H

#include "TVector3.h"
#include "StandardRecord/SRProng.h"// for plane_proj enumeration with kX, kY, kXorY

namespace caf
{
  
  /// The SRTrkME represents output from MichelEFilter.
  class SRTrkME
    {
    public:
      ///  Default constructor for SRProng. 
      SRTrkME();

      ~SRTrkME(){  };
      
      unsigned short  nhit;         ///< number of hits
      
      float           calE;         ///< energy based on summed calibrated deposited charge [GeV]
      float           deltat;       ///< time difference between mean times of track hits and Michel hits [ns]
      float           disttotrack;  ///< Distance between end of track and Michel hits [cm]
      
      bool            atend;        ///< True if Michel hits are at end of track as determined by track reconstruction 
      
      View_t          view;         ///< MichelE view caf::kX = 0, caf::kY = 1 or caf::kXorY = 2

      TVector3        meanpos;      ///< Mean position of Michel hits [cm] 
      
      short           trkidx;       ///< Index of track in trk vector, -1 something went wrong.  (Hopefully not.)

    };
  
} // end namespace

#endif // SRTRKME_H
//////////////////////////////////////////////////////////////////////////////
