////////////////////////////////////////////////////////////////////////
// \file    SRVeto.h
// \brief   Veto class detailing cuts made on slices in the standard record.    
// \author  $Author: Fite $
// \date    $Date: 2014-08-06 09:11:00 $
////////////////////////////////////////////////////////////////////////
#ifndef SRVETO_H
#define SRVETO_H

namespace caf
{
  /// This is a container cuts made during veto process in the standard record.  
  class SRVeto
    {
    public:
      ///  Default constructor. 
      SRVeto();
      
      /// Default destuctor.
      virtual ~SRVeto();

      int          ncell;      ///<Number of hits in the slice
      float        fwddist;    ///<Projected live distance to the edge projected forwards from track end
      float        bakdist;    ///<Projected live distance to the edge projected backwards from track start
      float        ratio;      ///<Number of hits in the track relative to the slice
      float        diry;       ///<Cosine of CosmicTrack with respect to the y-direction
      float        angle;      ///<Cosine of CosmicTrack with respect to the beam direction
      float        anglevar;   ///<Defined as: abs(angle)*(diry+1) - new angle variable for new angle cut
      float        anglevarold;///<Difference of squares between beam angle and Y-direction (Also labeled *CosmicVar* *AnglePID* and *AngleVar*) - the old angle variable for old angle cut
      bool         passthru;   ///<Would this slice pass the (fixed) through-going check?
      bool         passthruold;///<Would this slice pass the (not fixed, no ratio check) through-going check?
      bool         passangle;  ///<Would this slice pass the most recent angle cut?
      bool         passanglefirstanal;///<Would this slice pass the old angle cut used for first analysis
      bool         keep;       ///<Boolean variable that determines whether the slice was actually kept or thrown away (Also labeled *sel* for *selected*); false if ANY cut is false (if fails through-going check, or angle cut, or has number hits < 20)

      virtual void setDefault();
      
    };
  
} // end namespace

#endif // SRVETO_H
//////////////////////////////////////////////////////////////////////////////
