////////////////////////////////////////////////////////////////////////
/// \brief   TODO
/// \author  ricken@fnal.gov
/// \date    Sept. 2012
////////////////////////////////////////////////////////////////////////

#include <cmath>
#include <vector>
#include <string>

// NOvA includes
#include "Calibrator/Calibrator.h"
#include "RecoBase/Vertex.h"
#include "MCCheater/BackTracker.h"
#include "Simulation/Particle.h"
#include "Simulation/ParticleNavigator.h"

// Framework includes
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Services/Registry/ServiceHandle.h"
#include "art/Framework/Services/Optional/TFileService.h"
#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"

#include "TNtuple.h"


namespace ve {
  class VertexEva : public art::EDAnalyzer {
  public:
    explicit VertexEva(fhicl::ParameterSet const& pset);
    virtual ~VertexEva();
    
    void beginJob();
    void analyze(const art::Event& evt);
    void reconfigure(const fhicl::ParameterSet& p);
    
  private:
    std::string fVertexInput;    ///< Input folder from cluster reco

    TNtuple*   displacement;

  };
}


namespace ve{
  
  //......................................................................
  VertexEva::VertexEva(fhicl::ParameterSet const& pset)
    : EDAnalyzer(pset)
  {
    reconfigure(pset);
  }
  
  //......................................................................
  VertexEva::~VertexEva()
  {
  }
  
  //......................................................................
  void VertexEva::reconfigure(const fhicl::ParameterSet& pset)
  {
    fVertexInput    = pset.get< std::string >("VertexInput") ;
  }

  //......................................................................
  void VertexEva::beginJob()
  {
    art::ServiceHandle<art::TFileService> tfs;


    displacement = tfs->make<TNtuple>("displacement","delta",
                                "deltaX:deltaY:deltaZ:deltaT");
  }


  //......................................................................
  ///
  /// In this analysis, the input file should only be single muon events
  /// for simplicity
  ///
  /// 
  void VertexEva::analyze(const art::Event& evt) 
  {

    double mcT, mcX, mcY, mcZ;
    mcT=0;
    mcX=0;
    mcY=0;
    mcZ=0;

    art::ServiceHandle<cheat::BackTracker> bt;
    const sim::ParticleNavigator& pnav = bt->ParticleNavigator();

    for (sim::ParticleNavigator::const_iterator k = pnav.begin(); k != pnav.end(); ++k){
      const sim::Particle* p = (*k).second;
      if (p->PdgCode() == 2212) {
          mcT = p->T();
          mcX = p->Vx();
          mcY = p->Vy();
          mcZ = p->Vz();
          break;
      }
    }

    art::Handle< std::vector<rb::Vertex> > vertexhandle;
    evt.getByLabel(fVertexInput, vertexhandle);
   
    art::Ptr<rb::Vertex> vertex(vertexhandle,0);          //It is supposed to have only 1 vertex /evt

    double rbT, rbX, rbY, rbZ;

    rbT = vertex->GetT();
    rbX = vertex->GetX();
    rbY = vertex->GetY();
    rbZ = vertex->GetZ();
  
    
    displacement->Fill(rbX-mcX,
                       rbY-mcY,
                       rbZ-mcZ,
                       rbT-mcT);
     
    return;
    
  }   //end analyze
  
  DEFINE_ART_MODULE(VertexEva);
  
}//end namespace

