////////////////////////////////////////////////////////////////////////
// Class:       FmmTrackerValidation
// Module Type: analyzer
// File:        FmmTrackerValidation_module.cc
//
// Generated at Wed Dec 18 15:41:32 2013 by Zukai Wang using artmod
// from cetpkgsupport v1_04_02.
////////////////////////////////////////////////////////////////////////
#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"

#include "MCCheater/BackTracker.h"
#include "Calibrator/Calibrator.h"
#include "Simulation/Particle.h"
#include "Simulation/ParticleNavigator.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "Utilities/func/MathUtil.h"
#include "fhiclcpp/ParameterSet.h"
#include "art/Framework/Services/Optional/TFileService.h"

#include "HoughPair.h"
#include "CMap/CMap.h"
#include "Geometry/Geometry.h"
#include "GeometryObjects/CellGeo.h"
#include "GeometryObjects/Geo.h"
#include "GeometryObjects/PlaneGeo.h"

#include "NovaDAQConventions/DAQConventions.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/RecoHit.h"
#include "RecoBase/Cluster.h"
#include "RecoBase/Track.h"

#include "TVector3.h"
#include "TTree.h"
#include "TProfile.h"
#include "TRandom3.h"

#include <algorithm> 
#include <utility>
#include <cmath>
#include <iostream>

namespace zcl {
  class FmmTrackerValidation;
}

class zcl::FmmTrackerValidation : public art::EDAnalyzer {
public:
  explicit FmmTrackerValidation(fhicl::ParameterSet const & pset);
  virtual ~FmmTrackerValidation();

  void analyze(const art::Event & evt);
  void beginJob();
  void endJob();
  
private:
  std::string fTrackerInput;

  const int    ntrials = 100;   //Number of MC trilas in determining the fitting error
  const double X_err   = 1.8;
  const double Y_err   = 1.8;
  const double Z_err   = 2.82;

  //Each entry is corresponding to 1 reconstructed track
  TTree   * RCV_Tree;
  TRandom * genR = new TRandom3();
  //Purely MC info
  double beta_MC;
  double Edep_MC;
  double xi_MC, xf_MC;
  double yi_MC, yf_MC;
  double zi_MC, zf_MC;
  double ti_MC, tf_MC;
  int    nxhits_MC, nyhits_MC; 
  double trlMC;

  //RC info
  int    nxhits, nyhits;
  int    nhits_sat, nhits_true;
  double PE_tot;
  double MIP_tot;
  double ADC_tot;
  double GeV_tot;

  double tns_mean, tns_rms, tns_min, tns_max;
  int max_gap, miss_plane, p_cross, p_overlap;

  //Notice the start and end point in the fmmtrack might be flipped:
  double Xi, Yi, Zi, Ti;
  double Xf, Yf, Zf, Tf;

  double t_min, t_max;
  double x_tmin, x_tmax, y_tmin, y_tmax, z_tmin, z_tmax; 

  double lxsqr, lysqr;

  double vrc_sat;      // cm/ns
  double tSat_mean;
  double tSat_rms;
  double vSat_err;
  double vrc_nonsat;   // cm/ns
  double tnonSat_mean;
  double tnonSat_rms;
  double vnonSat_err;
  double Chisqr_TS;
  double Chisqr_TN;

  double adc_W;

  double dist2(double dx, double dy);
  double doca2(double x, double y, double x0, double y0,  double dydx); 

  //Linear Regression function
  double vfit(std::vector<TVector3> & Positions, 
	      std::vector<double> & Ts,
	      std::vector<double> & Terr,
	      double & v_err,
	      double & Tmean,
	      double & Trms,
	      double & dTsqr);  
};

zcl::FmmTrackerValidation::FmmTrackerValidation(fhicl::ParameterSet const & p) :
  EDAnalyzer(p),
  fTrackerInput      (p.get< std::string > ("TrackerInput")      )
{
}

zcl::FmmTrackerValidation::~FmmTrackerValidation()
{
  // Clean up dynamic memory and other resources here.
}

void zcl::FmmTrackerValidation::beginJob()
{
  art::ServiceHandle<art::TFileService> tfs;

  //Combining cheater info and the RC info to make the data structure better
  //One RC track info per entry, if no tracks reconstructed in this event, fill in 1 entry anyway tagging the 
  //reconstruction failed
  RCV_Tree = tfs->make<TTree>("RCV_Tree","Information from RC track info");

  RCV_Tree -> Branch("betaMC",  &beta_MC,  "betaMC/D");
  RCV_Tree -> Branch("edepMC",  &Edep_MC,  "edepMC/D");
  RCV_Tree -> Branch("tiMC",    &ti_MC,    "tiMC/D");
  RCV_Tree -> Branch("xiMC",    &xi_MC,    "xiMC/D");
  RCV_Tree -> Branch("yiMC",    &yi_MC,    "yiMC/D");
  RCV_Tree -> Branch("ziMC",    &zi_MC,    "ziMC/D");
  RCV_Tree -> Branch("tfMC",    &tf_MC,    "tfMC/D");
  RCV_Tree -> Branch("xfMC",    &xf_MC,    "xfMC/D");
  RCV_Tree -> Branch("yfMC",    &yf_MC,    "yfMC/D");
  RCV_Tree -> Branch("zfMC",    &zf_MC,    "zfMC/D");
  RCV_Tree -> Branch("xhitsMC", &nxhits_MC,  "xhitsMC/I");
  RCV_Tree -> Branch("yhitsMC", &nyhits_MC,  "yhitsMC/I");
  RCV_Tree -> Branch("trlMC",   &trlMC,      "trlMC/D");

  RCV_Tree -> Branch("nxhits",  &nxhits,    "nxhits/I");
  RCV_Tree -> Branch("nyhits",  &nyhits,    "nyhits/I");
  RCV_Tree -> Branch("nhits_true",  &nhits_true,   "nhits_true/I");
  RCV_Tree -> Branch("nhits_sat",   &nhits_sat,    "nhits_sat/I");
  RCV_Tree -> Branch("PE_tot",  &PE_tot,   "PE_tot/D");
  RCV_Tree -> Branch("MIP_tot", &MIP_tot,  "MIP_tot/D");
  RCV_Tree -> Branch("ADC_tot", &ADC_tot,  "ADC_tot/D");
  RCV_Tree -> Branch("GeV_tot", &GeV_tot,  "GeV_tot/D");  

  //Missing Planes and the max gap:
  RCV_Tree -> Branch("max_gap",     &max_gap,       "max_gap/I");
  RCV_Tree -> Branch("miss_plane",  &miss_plane,    "miss_plane/I");
  RCV_Tree -> Branch("p_cross",     &p_cross,       "p_cross/I");
  RCV_Tree -> Branch("p_overlap",   &p_overlap,     "p_overlap/I");  

  RCV_Tree -> Branch("Xi",   &Xi,    "Xi/D");
  RCV_Tree -> Branch("Xf",   &Xf,    "Xf/D");
  RCV_Tree -> Branch("Yi",   &Yi,    "Yi/D");
  RCV_Tree -> Branch("Yf",   &Yf,    "Yf/D");
  RCV_Tree -> Branch("Zi",   &Zi,    "Zi/D");
  RCV_Tree -> Branch("Zf",   &Zf,    "Zf/D");
  RCV_Tree -> Branch("Ti",   &Ti,    "Ti/D");
  RCV_Tree -> Branch("Tf",   &Tf,    "Tf/D");

  RCV_Tree -> Branch("tns_mean",  &tns_mean,  "tns_mean/D");
  RCV_Tree -> Branch("tns_rms",   &tns_rms,   "tns_rms/D");
  RCV_Tree -> Branch("tns_min",   &tns_min,   "tns_min/D");
  RCV_Tree -> Branch("tns_max",   &tns_max,   "tns_max/D");  

  //This is double checking the velocity check:
  RCV_Tree -> Branch("t_min",       &t_min,       "t_min/D");
  RCV_Tree -> Branch("t_max",       &t_max,       "t_max/D");
  RCV_Tree -> Branch("x_tmin",      &x_tmin,      "x_tmin/D");
  RCV_Tree -> Branch("x_tmax",      &x_tmax,      "x_tmax/D");
  RCV_Tree -> Branch("y_tmin",      &y_tmin,      "y_tmin/D");
  RCV_Tree -> Branch("y_tmax",      &y_tmax,      "y_tmax/D");
  RCV_Tree -> Branch("z_tmin",      &z_tmin,      "z_tmin/D");
  RCV_Tree -> Branch("z_tmax",      &z_tmax,      "z_tmax/D");

  //fitting parameters:
  RCV_Tree -> Branch("lxsqr",   &lxsqr,  "lxsqr/D");
  RCV_Tree -> Branch("lysqr",   &lysqr,  "lysqr/D");

  RCV_Tree -> Branch("vrc_sat",   &vrc_sat,  "vrc_sat/D");
  RCV_Tree -> Branch("tSat_mean", &tSat_mean,"tSat_mean/D");
  RCV_Tree -> Branch("tSat_rms",  &tSat_rms, "tSat_rms/D");
  RCV_Tree -> Branch("vSat_err",  &vSat_err, "vsat_err/D");
  RCV_Tree -> Branch("vrc_nonsat",   &vrc_nonsat,  "vrc_nonsat/D");
  RCV_Tree -> Branch("tnonSat_mean", &tnonSat_mean,"tnonSat_mean/D");
  RCV_Tree -> Branch("tnonSat_rms",  &tnonSat_rms, "tnonSat_rms/D");
  RCV_Tree -> Branch("vnonSat_err",  &vnonSat_err, "vnonSat_err/D");
  RCV_Tree -> Branch("Chisqr_TS",    &Chisqr_TS,   "Chisqr_TS/D");
  RCV_Tree -> Branch("Chisqr_TN",    &Chisqr_TN,   "Chisqr_TN/D");

  RCV_Tree -> Branch("adc_W", &adc_W, "adc_W/D");
}

void zcl::FmmTrackerValidation::analyze(const art::Event & evt)
{
  
  art::ServiceHandle<geo::Geometry> geom;
  art::ServiceHandle<cheat::BackTracker> bt;
  art::ServiceHandle<calib::Calibrator> calib;

  std::vector<int   > plane_fls;
  std::vector<int   > cell_fls;
  std::vector<double> t_fls;

  //Get MC info first:
  const sim::ParticleNavigator& pnav = bt->ParticleNavigator();
  for (sim::ParticleNavigator::const_iterator i = pnav.begin(); i != pnav.end(); ++i) {
    nxhits_MC = 0;
    nyhits_MC = 0;
    trlMC     = 0;

    const sim::Particle *p = (*i).second;
    if (p->PdgCode()!=42 ) continue;
    const std::vector<sim::FLSHit>& flshits = bt->ParticleToFLSHit(p->TrackId());
    unsigned ntruehits_Tot = flshits.size();
 
    if ( ntruehits_Tot==0 ) continue;
    double momentum2 = (p->P())*(p->P());
    double mass2     = (p->Mass())*(p->Mass());
    beta_MC = sqrt(momentum2 / (momentum2+mass2) );

    ti_MC = flshits[0].GetEntryT();
    xi_MC = flshits[0].GetEntryX();
    yi_MC = flshits[0].GetEntryY();
    zi_MC = flshits[0].GetEntryZ();
    tf_MC = flshits[0].GetExitT();
    xf_MC = flshits[0].GetExitX();
    yf_MC = flshits[0].GetExitY();
    zf_MC = flshits[0].GetExitZ();
    Edep_MC = flshits[0].GetEdep();
    trlMC   = flshits[0].GetTotalPathLength();

    unsigned int planeID_i = flshits[0].GetPlaneID();
    unsigned int cellID_i  = flshits[0].GetCellID();
    unsigned int planeID_f = planeID_i;
    unsigned int cellID_f  = cellID_i;

    if (planeID_i%2) ++nxhits_MC;
    else ++nyhits_MC;

    plane_fls.emplace_back(planeID_i);
    cell_fls.emplace_back(cellID_i);
    t_fls.emplace_back(ti_MC);

    for (unsigned h =1; h!= flshits.size(); ++h) {
      double ti = flshits[h].GetEntryT();
      double tf = flshits[h].GetExitT(); 
      plane_fls.emplace_back(flshits[h].GetPlaneID());
      cell_fls.emplace_back(flshits[h].GetCellID());
      t_fls.emplace_back(ti);
      if (ti < ti_MC) {
	ti_MC = ti;
	xi_MC = flshits[h].GetEntryX();
	yi_MC = flshits[h].GetEntryY();
	zi_MC = flshits[h].GetEntryZ();
	planeID_i = flshits[h].GetPlaneID();
	cellID_i  = flshits[h].GetCellID();
      }
      if (tf > tf_MC) {
	tf_MC = tf;
	xf_MC = flshits[h].GetExitX();
	yf_MC = flshits[h].GetExitY();
	zf_MC = flshits[h].GetExitZ();
	planeID_f = flshits[h].GetPlaneID();
	cellID_f  = flshits[h].GetCellID();
      }

      if (flshits[h].GetPlaneID() % 2) ++nxhits_MC;
      else ++nyhits_MC;

      Edep_MC += flshits[h].GetEdep();
      trlMC   += flshits[h].GetTotalPathLength();
    }

    //By now, you have got the local coordinate, you have to translate it to world:
    const geo::CellGeo* startCell = geom->Plane(planeID_i)->Cell(cellID_i);
    double local[3], world[3];
    local[0] = xi_MC;
    local[1] = yi_MC;
    local[2] = zi_MC;
    startCell->LocalToWorld(local,world);
    xi_MC = world[0];
    yi_MC = world[1];
    zi_MC = world[2];

    const geo::CellGeo* stopCell = geom->Plane(planeID_f)->Cell(cellID_f);
    local[0] = xf_MC;
    local[1] = yf_MC;
    local[2] = zf_MC;
    stopCell->LocalToWorld(local,world);
    xf_MC = world[0];
    yf_MC = world[1];
    zf_MC = world[2];
    //1 monopole per event, so you can break out from the loop:
    break;
  }

  //=====================================================================================
  art::Handle< std::vector<rb::Track> > trks;
  evt.getByLabel(fTrackerInput,trks);
  nxhits = 0;
  nyhits = 0;
  nhits_sat  = 0;
  nhits_true = 0;
  ADC_tot = 0;
  if (trks->size() < 1) {
    std::cout<<"There is no track reconstructed in this event!"<<std::endl;
    RCV_Tree->Fill();
    return;
  }

  bool filled = false;

  //take the entire 3D track:
  for (unsigned trkIdx = 0; trkIdx != trks->size(); ++trkIdx ) {
    const rb::Track& track = (*trks)[trkIdx];

    if (track.NCell()<6) continue;

    tns_mean  = 0; //ADC Weighed average time
    double ADC_sum = 0;

    //With the first round of loop, can determine the following info: t_mean 

    for (unsigned i = 0; i != track.NCell(); ++i) {
      const art::Ptr<rb::CellHit> chit = track.Cell(i);
      double tns = chit->TNS();
      double adc = chit->ADC();
      tns_mean += tns*adc;
      ADC_sum  += adc; 
    }

    tns_mean /= ADC_sum;
    //=============================================================

    //Just determine the RMS of the time:
    tns_rms = 0;
    for (unsigned i = 0; i != track.NCell(); ++i) {
      const art::Ptr<rb::CellHit> chit = track.Cell(i);
      double tns = chit->TNS();
      double adc = chit->ADC();
      tns_rms = (tns-tns_mean)*adc*(tns-tns_mean)*adc;
    }
    tns_rms = sqrt(tns_rms/ADC_sum+250000/12.);

    //==============================================================
    //Now, loop through the hits again to throw away the hits outside the 4 sigma range,
    //Make sure they get updated!
    tns_min  = 99999999;    
    tns_max  = -99999999;
    nxhits = 0;
    nyhits = 0;
    nhits_sat  = 0;
    nhits_true = 0;
    ADC_tot = 0;
    std::vector<int> planes;

    int P_x0=9999;
    int P_x1=0;
    int P_y0=9999;
    int P_y1=0;
 
    for (unsigned i = 0; i != track.NCell(); ++i) {
      const art::Ptr<rb::CellHit> chit = track.Cell(i);
      double tns = chit->TNS();
      if (tns > tns_mean + 4*tns_rms || tns < tns_mean - 4*tns_rms) continue;
      if (tns > tns_max) tns_max = tns;
      if (tns < tns_min) tns_min = tns;
      planes.emplace_back(chit->Plane());
      ADC_tot += chit->ADC();

      if (chit->View()==geo::kX) {
	++nxhits;
	if (chit->Plane() < P_x0) P_x0 = chit->Plane();
	if (chit->Plane() > P_x1) P_x1 = chit->Plane();
      }
      else {
	++nyhits;
	if (chit->Plane() < P_y0) P_y0 = chit->Plane();
	if (chit->Plane() > P_y1) P_y1 = chit->Plane();
      }
    }
    
    if (planes.size()<3) continue;

    if (P_x1-P_x0<3) continue;
    if (P_y1-P_y0<3) continue;

    //Now calculate the missing planes and the max gap:
    std::sort (planes.begin(), planes.end());
    max_gap    = 0;
    miss_plane = 0;
    p_cross    = planes.back()-planes.front()+1;
    p_overlap  = 1;
    bool start_flag = false;
    for (unsigned pidx =0; pidx+1<planes.size(); ++pidx) {
      int dP = planes[pidx+1] - planes[pidx] - 1;
      //Calculate the number of planes in the overlapped region:
      if (!start_flag) {
	if ( (planes[pidx+1] - planes[pidx]) % 2 ) start_flag = true;
      }
      else {
	if ( (planes[pidx+1] - planes[pidx]) % 2 ) ++p_overlap;
      }
      if (dP > 0 ) {
	miss_plane += dP;
	if (dP > max_gap) max_gap = dP;
      }
    }
    //========================================================
    //Loop through the hits again: this time, determine dxdz and dydz

    std::vector<double> x0s_x, x1s_x, y0s_y, y1s_y;
    std::vector<double> x0s_z, x1s_z, y0s_z, y1s_z;
    for (unsigned i = 0; i != track.NCell(); ++i) {
      const art::Ptr<rb::CellHit> chit = track.Cell(i);
      double tns = chit->TNS();
      if (tns > tns_mean + 4*tns_rms || tns < tns_mean - 4*tns_rms) continue;

      double xyz[3];
      geom->Plane(chit->Plane())->Cell(chit->Cell())->GetCenter(xyz);
      if (chit->Plane() == P_x0 ) {
	x0s_x.emplace_back(xyz[0]); 
	x0s_z.emplace_back(xyz[2]); 
      }
      if (chit->Plane() == P_x1 ) {
	x1s_x.emplace_back(xyz[0]); 
	x1s_z.emplace_back(xyz[2]); 
      }

      if (chit->Plane() == P_y0 ) {
	y0s_y.emplace_back(xyz[1]); 
	y0s_z.emplace_back(xyz[2]); 
      }
      if (chit->Plane() == P_y1 ) {
	y1s_y.emplace_back(xyz[1]); 
	y1s_z.emplace_back(xyz[2]); 
      }
    }

    double z_x0=0;
    double x_x0=0;
    double z_y0=0;
    double y_y0=0;
    double z_x1=0;
    double z_y1=0;

    double dx = 0;
    double dy = 0;
    double dxdz = 0;
    double dydz = 0;


    for (unsigned h0 = 0; h0 != x0s_x.size(); ++h0) {
      for (unsigned h1 = 0; h1 != x1s_x.size(); ++h1) {
	if (fabs(dx) < fabs(x1s_x[h1]-x0s_x[h0])) {
	  dx   = x1s_x[h1]-x0s_x[h0];
	  dxdz = dx/(x1s_z[h1]-x0s_z[h0]); 
	  z_x0 = x0s_z[h0];
	  x_x0 = x0s_x[h0];
	  z_x1 = x1s_z[h1];
	}
      }
    }

    for (unsigned h0 = 0; h0 != y0s_y.size(); ++h0) {
      for (unsigned h1 = 0; h1 != y1s_y.size(); ++h1) {
	if (fabs(dy) < fabs(y1s_y[h1]-y0s_y[h0])) {
	  dy   = y1s_y[h1]-y0s_y[h0];
	  dydz = dy/(y1s_z[h1]-y0s_z[h0]); 
	  z_y0 = y0s_z[h0];
	  y_y0 = y0s_y[h0];
	  z_y1 = y1s_z[h1];
	}
      }
    }

    //Now you can determine the two ends:
    double zi = std::min(z_x0,z_y0);
    double zf = std::max(z_x1,z_y1);

    double xi = dxdz*(zi-z_x0)+x_x0;
    double xf = dxdz*(zf-z_x0)+x_x0;
    double yi = dydz*(zi-z_y0)+y_y0;
    double yf = dydz*(zf-z_y0)+y_y0;

    TVector3 Pi(xi,yi,zi);  // Low Z end
    TVector3 Pf(xf,yf,zf);  // High Z end

    //Now, we know the direction of the track, can provide the W position,
    //Loop through the hits attached to the track again!
    PE_tot  = 0;
    MIP_tot = 0;
    GeV_tot = 0;
    double t_min =  9999999;
    double t_max = -9999999;
    
    std::vector<double>   ts_sat,terrs_sat;
    std::vector<double>   ts_unsat,terrs_unsat;
    std::vector<TVector3> pos_sat, pos_unsat;

    lxsqr = 0;
    lysqr = 0;

    //In the following loop, determine the ADC center from the geometry center:
    TVector3 adc_Center(0,0,0);
    TVector3 geo_Center(0,0,0);
    ADC_tot = 0;

    for (unsigned i = 0; i != track.NCell(); ++i) {
      const art::Ptr<rb::CellHit> chit = track.Cell(i);
      double tns = chit->TNS();
      if (tns > tns_mean + 4*tns_rms || tns < tns_mean - 4*tns_rms) continue;
    
      //Interpolate the W position:
      double W;
      double xyz[3];
      geom->Plane(chit->Plane())->Cell(chit->Cell())->GetCenter(xyz);   

      if (chit->View()==geo::kX) {
	W = dydz*(xyz[2]-z_y0)+y_y0;
	lxsqr += doca2(xyz[2],xyz[0],z_x0,x_x0, dxdz);
      }
      else {
	W = dxdz*(xyz[2]-z_x0)+x_x0;
	lysqr += doca2(xyz[2],xyz[1],z_y0,y_y0, dydz);
      }

      rb::RecoHit rhit(calib->MakeRecoHit(*chit,W));

      if (!rhit.IsCalibrated()) continue;

      PE_tot  += rhit.PECorr();
      MIP_tot += rhit.MIP();
      GeV_tot += rhit.GeV(); 

      TVector3 pos(rhit.X(), rhit.Y(), rhit.Z());
      double st = calib->GetTimeRes(*chit); 

      adc_Center += (chit->ADC())*pos;
      geo_Center += pos;
      ADC_tot    += chit->ADC();

      if (!bt->IsNoise(chit) || (chit->ADC()>3100 && chit->TNS() > ti_MC-1000 && chit->TNS() < tf_MC+1000) )
	++nhits_true;

      if (chit->ADC(3) == 4095) {
	pos_sat.emplace_back(pos-Pi);
	ts_sat.emplace_back(rhit.T()-tns_min);
	terrs_sat.emplace_back(st);
      }

      else {
	pos_unsat.emplace_back(pos-Pi);
	ts_unsat.emplace_back(rhit.T()-tns_min);
	terrs_unsat.emplace_back(st);
      }

      if (rhit.T() < t_min ) {
	x_tmin = rhit.X();
	y_tmin = rhit.Y();
	z_tmin = rhit.Z();
	t_min  = rhit.T();
      } 

      if (rhit.T() > t_max ) {
	x_tmax = rhit.X();
	y_tmax = rhit.Y();
	z_tmax = rhit.Z();
	t_max  = rhit.T();
      } 
    }

    adc_Center = adc_Center*(1/double(ADC_tot));
    geo_Center = geo_Center*(1/double(ts_sat.size()+ts_unsat.size()));

    adc_W = (adc_Center-geo_Center).Mag();

    lxsqr /= double(nxhits);
    lysqr /= double(nyhits);

    nhits_sat = pos_sat.size();

    vrc_sat = vfit(pos_sat, ts_sat, terrs_sat, vSat_err, tSat_mean, tSat_rms, Chisqr_TS);
    vrc_nonsat = vfit(pos_unsat, ts_unsat, terrs_unsat, vnonSat_err, tnonSat_mean, tnonSat_rms, Chisqr_TN);

    if (nhits_sat<2) {
      //Make them apparently invalid
      vrc_sat   = 60;
      vSat_err  = -1;
      tSat_rms  = -1;
      tSat_mean = -1e6;
    }

    if (pos_unsat.size()<2) {
      //Make them apparently invalid
      vrc_nonsat  = 60;
      vnonSat_err = -1;
      tnonSat_rms = -1;
      tnonSat_mean = -1e6;
    }

    //Just set the lowZ end to be the starting point:
    Xi = xi;
    Yi = yi;
    Zi = zi;
    Ti = t_min;
    Xf = xf;
    Yf = yf;
    Zf = zf;    
    Tf = t_max;

    RCV_Tree->Fill();
    filled = true;
  }

  if (!filled) RCV_Tree->Fill();
  
} //end analyzing 

double zcl::FmmTrackerValidation::dist2(double dx, double dy) {
  return dx*dx+dy*dy;
} 

double zcl::FmmTrackerValidation::doca2(double x, double y, double x0, double y0,  double dydx) 
{
  if (fabs(x-x0) < 0.1 && fabs(y-y0)<0.1) return 0;
  double cos2 = (x-x0 + dydx*(y-y0))*(x-x0 + dydx*(y-y0))/dist2(x-x0,y-y0)/dist2(1,dydx);
  double docasqr = dist2(x-x0,y-y0)*(1-cos2);
  //  std::cout<<"docasqr: "<<docasqr<<"  cos2:"<<cos2<<std::endl;
  return docasqr;
}


double zcl::FmmTrackerValidation::vfit(std::vector<TVector3> & Positions, 
				       std::vector<double> & Ts, 
				       std::vector<double> & Terr,
				       double & v_err,
				       double & Tmean,
				       double & Trms,
				       double & dTsqr)
{
  int m = Ts.size();
  double sumL  = 0;
  double sumT  = 0;
  double sumLT = 0;
  double sumLL = 0;

  std::vector<double> Vs;

  for (int i = 0; i!=m; ++i ) {
    sumL  += Positions[i].Mag();
    sumT  += Ts[i];
    sumLT += Positions[i].Mag() * Ts[i];
    sumLL += Positions[i].Mag() * Positions[i].Mag();
  }

  Tmean = sumT/double(m);
  double v = (sumL*sumL-m*sumLL)/(sumT*sumL-m*sumLT);
  if (v > 30) v = 30;
  if (v <-30) v = -30;

  Vs.emplace_back(v);

  for (int t = 0; t!=ntrials; ++t) {
    sumL  = 0;
    sumT  = 0;
    sumLT = 0;
    sumLL = 0;
    for (int i = 0; i!=m; ++i ) {
      double xi = genR->Uniform(Positions[i].X()-X_err,Positions[i].X()+X_err);
      double yi = genR->Uniform(Positions[i].Y()-Y_err,Positions[i].Y()+Y_err);
      double zi = genR->Uniform(Positions[i].Z()-Z_err,Positions[i].Z()+Z_err);
      double ti = genR->Gaus(Ts[i], Terr[i]);
      double li = sqrt(xi*xi+yi*yi+zi*zi);

      sumL  += li;
      sumT  += ti;
      sumLT += li*ti;
      sumLL += li*li;
    }
    v = (sumL*sumL-m*sumLL)/(sumT*sumL-m*sumLT);
    if (v > 30) v = 30;
    if (v <-30) v = -30;
    Vs.emplace_back(v);
  }

  //Calculate the mean of v, and standard error:
  double v_mean = 0;
  v_err = 0;
  for (unsigned j = 0; j!=Vs.size(); ++j) 
    v_mean += Vs[j];
  v_mean /= double(Vs.size());

  for (unsigned j = 0; j!=Vs.size(); ++j) 
    v_err += (Vs[j]-v_mean)*(Vs[j]-v_mean);

  v_err = sqrt(v_err)/double(Vs.size());

  if (v_mean < 0.003 && v_mean > 0) 
    v_mean = 0.003;
  
  else if (v_mean > -0.003 && v_mean < 0) 
    v_mean = -0.003;
  
  double t0 = Tmean-sumL/(m+0.)/v_mean;

  Trms  = 0;
  dTsqr = 0;
  for (int i = 0; i!=m; ++i ) {
    Trms += (Ts[i]-Tmean)*(Ts[i]-Tmean);
    dTsqr+= (Ts[i]-t0-Positions[i].Mag()/v_mean)* (Ts[i]-t0-Positions[i].Mag()/v_mean);
  }
  Trms  = sqrt(Trms/(m+0.));
  dTsqr = sqrt(dTsqr/(m+0.));

  return v_mean;
} 


void zcl::FmmTrackerValidation::endJob(){

}

DEFINE_ART_MODULE(zcl::FmmTrackerValidation)
