////////////////////////////////////////////////////////////////////////
// Class:       SMMCluster
// Module Type: producer
// File:        SMMCluster_module.cc
//
// Generated at Wed Dec 18 15:41:32 2013 by Zukai Wang using artmod
// from cetpkgsupport v1_04_02.
////////////////////////////////////////////////////////////////////////
#include "art/Framework/Core/EDProducer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"

#include "MCCheater/BackTracker.h"
#include "Calibrator/Calibrator.h"
#include "Simulation/Particle.h"
#include "Simulation/ParticleNavigator.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "art/Framework/Services/Optional/TFileService.h"

#include "HoughPair.h"
#include "CMap/CMap.h"
#include "Geometry/Geometry.h"
#include "GeometryObjects/CellGeo.h"
#include "GeometryObjects/PlaneGeo.h"

#include "NovaDAQConventions/DAQConventions.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/RecoHit.h"
#include "RecoBase/Cluster.h"

#include "TH3.h"
#include "TVector3.h"
#include "TTree.h"

#include <algorithm> 
#include <utility>
#include <cmath>
#include <iostream>

namespace zcl {
  class SMMCluster;
}

class zcl::SMMCluster : public art::EDProducer {
public:
  explicit SMMCluster(fhicl::ParameterSet const & pset);
  virtual ~SMMCluster();

  void produce(art::Event & evt);
  void beginJob();
  void endJob();

  int MissC(art::ServiceHandle<geo::Geometry> const & geom,
	    art::PtrVector<rb::CellHit> & hitlist, 
	    TVector3 & startC,  //C, Z, T, sigmaT... I am just using this as a container 
	    TVector3 & stopC);

  void LinFit(std::vector<TVector3> const & hits,
	      TVector3 const h0,
	      TVector3 & vinfo); 

  double calcW(double z, double w1, double w2, double z1, double z2); 
 
  private:
  std::string fClusterInput;    ///< Input folder from cluster reco
  double _sigmaT;
  double _deltaZ;

  TTree* RC_Tree;
  int    evtID, subRunID, RunID;
  double beta_MC;
  double Edep_MC;
  double xi_MC, xf_MC;
  double yi_MC, yf_MC;
  double zi_MC, zf_MC;
  double ti_MC, tf_MC;
  int    nxhits_MC, nyhits_MC; 
  double ti_RC;
  double tf_RC;
  double xi_RC, sigmaXi, xf_RC, sigmaXf;
  double yi_RC, sigmaYi, yf_RC, sigmaYf;
  double zi_RC, sigmaZi, zf_RC, sigmaZf;
  int    missH_X, missH_Y, nxhits_RC, nyhits_RC;
  double xPE_RC, yPE_RC;
  int    noise_Xhits, noise_Yhits, noise_XPE, noise_YPE;  //This is actually cheater's info
  double Vx_RC, Vy_RC, Vz_xRC, Vz_yRC;
  double sigmaT2_X, sigmaT2_Y;
  bool triggered, reconstructed;

};

zcl::SMMCluster::SMMCluster(fhicl::ParameterSet const & p) :
  fClusterInput      (p.get< std::string > ("ClusterInput")      ),
  _sigmaT            (4687.5),
  _deltaZ            (50)
{
  // Call appropriate Produces<>() functions here.
  produces< std::vector<rb::Cluster> >();
}

zcl::SMMCluster::~SMMCluster()
{
  // Clean up dynamic memory and other resources here.
}

void zcl::SMMCluster::beginJob()
{
  art::ServiceHandle<art::TFileService> tfs;

  RC_Tree = tfs->make<TTree>("SMMRC_Tree","Information from cluster RC info");
  RC_Tree -> Branch("triggered",  &triggered,   "triggered/O");
  RC_Tree -> Branch("reconstructed",  &reconstructed,   "reconstructed/O");
  RC_Tree -> Branch("event",  &evtID,      "event/I");
  RC_Tree -> Branch("subRun", &subRunID,   "subRun/I");     
  RC_Tree -> Branch("Run",    &RunID,      "Run/I");
  RC_Tree -> Branch("ti",     &ti_RC,      "ti/D");
  RC_Tree -> Branch("tf",     &tf_RC,      "tf/D");
  RC_Tree -> Branch("xi",     &xi_RC,      "xi/D");
  RC_Tree -> Branch("yi",     &yi_RC,      "yi/D");
  RC_Tree -> Branch("zi",     &zi_RC,      "zi/D");
  RC_Tree -> Branch("xf",     &xf_RC,      "xf/D");
  RC_Tree -> Branch("yf",     &yf_RC,      "yf/D");
  RC_Tree -> Branch("zf",     &zf_RC,      "zf/D");
  RC_Tree -> Branch("sigmaxi",     &sigmaXi,      "sigmaxi/D");
  RC_Tree -> Branch("sigmayi",     &sigmaYi,      "sigmayi/D");
  RC_Tree -> Branch("sigmazi",     &sigmaZi,      "sigmazi/D");
  RC_Tree -> Branch("sigmaxf",     &sigmaXf,      "sigmaxf/D");
  RC_Tree -> Branch("sigmayf",     &sigmaYf,      "sigmayf/D");
  RC_Tree -> Branch("sigmazf",     &sigmaZf,      "sigmazf/D");
  RC_Tree -> Branch("missH_X",     &missH_X,      "missHX/I");  //Missing hits: overall discountinuity in plane
  RC_Tree -> Branch("missH_Y",     &missH_Y,      "missHY/I");
  RC_Tree -> Branch("nHits_X",     &nxhits_RC,    "nXHits/I");
  RC_Tree -> Branch("nHits_Y",     &nyhits_RC,    "nYHits/I");
  RC_Tree -> Branch("PE_X",        &xPE_RC,       "PE_X/D");
  RC_Tree -> Branch("PE_Y",        &yPE_RC,       "PE_Y/D");
  RC_Tree -> Branch("Vx",          &Vx_RC,        "Vx/D");
  RC_Tree -> Branch("Vy",          &Vy_RC,        "Vy/D");
  RC_Tree -> Branch("Vz_x",        &Vz_xRC,       "Vz_x/D");
  RC_Tree -> Branch("Vz_y",        &Vz_yRC,       "Vz_y/D");
  RC_Tree -> Branch("sigmaT2_X",   &sigmaT2_X,    "sigmaT2_X/D");
  RC_Tree -> Branch("sigmaT2_Y",   &sigmaT2_Y,    "sigmaT2_Y/D");

  //Cheater info part
  RC_Tree -> Branch("betaMC",  &beta_MC,  "beta/D");
  RC_Tree -> Branch("edepMC",  &Edep_MC,  "edep/D");
  RC_Tree -> Branch("tiMC",    &ti_MC,    "ti/D");
  RC_Tree -> Branch("xiMC",    &xi_MC,    "xi/D");
  RC_Tree -> Branch("yiMC",    &yi_MC,    "yi/D");
  RC_Tree -> Branch("ziMC",    &zi_MC,    "zi/D");
  RC_Tree -> Branch("tfMC",    &tf_MC,    "tf/D");
  RC_Tree -> Branch("xfMC",    &xf_MC,    "xf/D");
  RC_Tree -> Branch("yfMC",    &yf_MC,    "yf/D");
  RC_Tree -> Branch("zfMC",    &zf_MC,    "zf/D");
  RC_Tree -> Branch("xhits", &nxhits_MC,  "xhits/I");
  RC_Tree -> Branch("yhits", &nyhits_MC,  "yhits/I");
  RC_Tree -> Branch("noise_Xhits", &noise_Xhits,  "noise_Xhits/I");
  RC_Tree -> Branch("noise_Yhits", &noise_Yhits,  "noise_Yhits/I");
  RC_Tree -> Branch("noise_XPE",  &noise_XPE,   "noise_XPE/D");
  RC_Tree -> Branch("noise_YPE",  &noise_YPE,   "noise_YPE/D");

}

void zcl::SMMCluster::produce(art::Event & evt)
{
  evtID    = evt.id().event();
  subRunID = evt.subRun();
  RunID    = evt.run();

  art::ServiceHandle<geo::Geometry> geom;
  art::ServiceHandle<cheat::BackTracker> bt;
  art::ServiceHandle<calib::Calibrator> calib;

  //Get MC info first:
  const sim::ParticleNavigator& pnav = bt->ParticleNavigator();
  for (sim::ParticleNavigator::const_iterator i = pnav.begin(); i != pnav.end(); ++i) {
    nxhits_MC = 0;
    nyhits_MC = 0;

    const sim::Particle *p = (*i).second;
    if (p->PdgCode()!=42 ) continue;
    const std::vector<sim::FLSHit>& flshits = bt->ParticleToFLSHit(p->TrackId());
    unsigned ntruehits_Tot = flshits.size();
    if ( ntruehits_Tot==0 ) continue;
    double momentum2 = (p->P())*(p->P());
    double mass2     = (p->Mass())*(p->Mass());
    beta_MC = sqrt(momentum2 / (momentum2+mass2) );

    ti_MC = flshits[0].GetEntryT();
    xi_MC = flshits[0].GetEntryX();
    yi_MC = flshits[0].GetEntryY();
    zi_MC = flshits[0].GetEntryZ();
    tf_MC = flshits[0].GetExitT();
    xf_MC = flshits[0].GetExitX();
    yf_MC = flshits[0].GetExitY();
    zf_MC = flshits[0].GetExitZ();
    Edep_MC = flshits[0].GetEdep();

    unsigned int planeID_i = flshits[0].GetPlaneID();
    unsigned int cellID_i  = flshits[0].GetCellID();
    unsigned int planeID_f = planeID_i;
    unsigned int cellID_f  = cellID_i;

    if (planeID_i%2) ++nxhits_MC;
    else ++nyhits_MC;
   
    for (size_t h =1; h!= flshits.size(); ++h) {
      double ti = flshits[h].GetEntryT();
      double tf = flshits[h].GetExitT(); 
      if (ti < ti_MC) {
	ti_MC = ti;
	xi_MC = flshits[h].GetEntryX();
	yi_MC = flshits[h].GetEntryY();
	zi_MC = flshits[h].GetEntryZ();
	planeID_i = flshits[h].GetPlaneID();
	cellID_i  = flshits[h].GetCellID();
      }
      if (tf > tf_MC) {
	tf_MC = tf;
	xf_MC = flshits[h].GetExitX();
	yf_MC = flshits[h].GetExitY();
	zf_MC = flshits[h].GetExitZ();
	planeID_f = flshits[h].GetPlaneID();
	cellID_f  = flshits[h].GetCellID();
      }

      if (flshits[h].GetPlaneID() % 2) ++nxhits_MC;
      else ++nyhits_MC;
     
      Edep_MC += flshits[h].GetEdep();
    }

    //By now, you have got the local coordinate, you have to translate it to world:
    const geo::CellGeo* startCell = geom->Plane(planeID_i)->Cell(cellID_i);
    double local[3], world[3];
    local[0] = xi_MC;
    local[1] = yi_MC;
    local[2] = zi_MC;
    startCell->LocalToWorld(local,world);
    xi_MC = world[0];
    yi_MC = world[1];
    zi_MC = world[2];

    const geo::CellGeo* stopCell = geom->Plane(planeID_f)->Cell(cellID_f);
    local[0] = xf_MC;
    local[1] = yf_MC;
    local[2] = zf_MC;
    stopCell->LocalToWorld(local,world);
    xf_MC = world[0];
    yf_MC = world[1];
    zf_MC = world[2];

    //1 monopole per event, so you can break out from the loop:
    break;
  }

  //MC info ready, fill the tree:


  //=============================MC/RC Separation Line============================================

  //initialization of triggering and reconstruction status:
  triggered = false;
  reconstructed = false; 

  art::Handle< std::vector<rb::Cluster> > slices;
  evt.getByLabel(fClusterInput, slices);

  //Devide the clusters from different views:
  std::vector<rb::Cluster> Xclusters;
  std::vector<rb::Cluster> Yclusters;

  for (unsigned sliceIdx = 0; sliceIdx != slices->size(); ++sliceIdx) {
    const rb::Cluster& slice = (*slices)[sliceIdx];
    if ((slice.Cell(0)) -> View()==geo::kX) Xclusters.emplace_back(slice);
    else Yclusters.emplace_back(slice);
  }

  if (Xclusters.empty()) 
    RC_Tree -> Fill();

  else  triggered = true;

  /*
  std::cout<<"X sp clusters: "<<Xclusters.size()
	   <<"   Y sp clusters: "<<Yclusters.size()<<std::endl;
  */
  std::unique_ptr<std::vector<rb::Cluster> > clustercol(new std::vector<rb::Cluster>);

  //Your mission: merge 2D tracks and meanwhile record the information from both cheater and RC
  for (unsigned xsid = 0 ; xsid!= Xclusters.size(); ++xsid) {
    TVector3 X_start, X_stop;
    art::PtrVector<rb::CellHit> xhitlist = Xclusters[xsid].AllCells();
    missH_X = MissC(geom, xhitlist, X_start, X_stop);
    nxhits_RC = xhitlist.size();
    
    for (unsigned ysid = 0; ysid!= Yclusters.size(); ++ysid) {
      TVector3 Y_start, Y_stop;
      art::PtrVector<rb::CellHit> yhitlist = Yclusters[ysid].AllCells();
      missH_Y = MissC(geom, yhitlist, Y_start, Y_stop);
      nyhits_RC = yhitlist.size();
 
      //Check whether to merge these 2:
      if (X_start.Z() > Y_start.Z()+_sigmaT || X_start.Z() < Y_start.Z()-_sigmaT) continue; 
      if (X_stop.Z()  > Y_stop.Z() +_sigmaT || X_stop.Z()  < Y_stop.Z() -_sigmaT) continue;
      if (X_start.Y() > Y_start.Y()+_deltaZ || X_start.Y() < Y_start.Y()-_deltaZ) continue;
      if (X_stop.Y()  > Y_stop.Y() +_deltaZ || X_stop.Y()  < Y_stop.Y() -_deltaZ) continue;

      //if all these requirements have been reached, merge!
      reconstructed = true;
      rb::Cluster SMMcluster3D;  
      std::vector<TVector3> xhits;
 
      xPE_RC      = 0;
      noise_Xhits = 0;
      noise_XPE   = 0;
      for (int i= 0; i!=nxhits_RC; ++i) {
	const art::Ptr<rb::CellHit> chit = xhitlist[i];

	double xyz[3];
	int p = chit->Plane();
	int c = chit->Cell();
	geom->Plane(p)->Cell(c)->GetCenter(xyz);

	rb::RecoHit rhit(calib->MakeRecoHit( *chit,
					     calcW(xyz[2],
						   Y_start.X(),
						   Y_stop.X(),
						   Y_start.Y(),
						   Y_stop.Y()) )  );

	if (!rhit.IsCalibrated()) continue;

	TVector3 hit_st(xyz[0],xyz[2],rhit.T());
	xhits.emplace_back(hit_st);
	SMMcluster3D.Add(chit);
	xPE_RC += rhit.PECorr();
	if (bt->IsNoise(chit)) {
	  ++noise_Xhits;
	  noise_XPE += rhit.PECorr();
	}
      }
    
      std::vector<TVector3> yhits;
      yPE_RC = 0;
      noise_Yhits = 0;
      noise_YPE  = 0;
      for (int i= 0; i!=nyhits_RC; ++i) {
	const art::Ptr<rb::CellHit> chit = yhitlist[i];
	double xyz[3];
	int p = chit->Plane();
	int c = chit->Cell();
	geom->Plane(p)->Cell(c)->GetCenter(xyz);

	rb::RecoHit rhit(calib->MakeRecoHit(*chit,
					    calcW(xyz[2],
						  X_start.X(),
						  X_stop.X(),
						  X_start.Y(),
						  X_stop.Y()) )  );

	if (!rhit.IsCalibrated()) continue;

	TVector3 hit_st(xyz[1],xyz[2],rhit.T());
	yhits.emplace_back(hit_st);
	SMMcluster3D.Add(chit);
	yPE_RC += rhit.PECorr();
	if (bt->IsNoise(chit)) {
	  ++noise_Yhits;
	  noise_YPE += rhit.PECorr();
	}
      }

      clustercol->emplace_back(SMMcluster3D); 

      //Now, do the vx ,vy,vz fit:
      TVector3 X_vinfo, Y_vinfo;
      LinFit(xhits,X_start,X_vinfo);
      LinFit(yhits,Y_start,Y_vinfo);

      Vx_RC      = X_vinfo.X();
      Vz_xRC     = X_vinfo.Y();
      sigmaT2_X  = X_vinfo.Z();
      Vy_RC      = Y_vinfo.X();
      Vz_yRC     = Y_vinfo.Y();
      sigmaT2_Y  = Y_vinfo.Z();

      //By now, the 3D cluster has been reconstructed, calculate the info and fill the RC tree:
      //The remaining infomation to be calculated:
      //sigmaX,Y,Z,T, (XYZ)_if
      sigmaXi = 1.8;   //1.8cm is the half size of a cell width
      sigmaYi = 1.8;

      double sigmaT2 = (sigmaT2_X + sigmaT2_Y)/(nxhits_RC+nyhits_RC+0.);

      //If the track is forward going:
      if (Vz_xRC+Vz_yRC > 0) {
	//First, start point:
	if (X_start.Y() < Y_start.Y()) {
	  //This means, the first hit is in X view:
	  xi_RC   = X_start.X();
	  zi_RC   = X_start.Y(); 
	  sigmaZi = Y_start.Y()-zi_RC-6.6;
	  yi_RC   = Y_start.X() - (Y_start.Y() - zi_RC)*Vy_RC/Vz_yRC;	
	  ti_RC   = X_start.Z();
	  sigmaYi = sqrt(sigmaYi*sigmaYi + Vy_RC*Vy_RC*sigmaT2);
	}
	else {
	  //This means, the first hit is in Y view:
	  yi_RC   = Y_start.X();
	  zi_RC   = Y_start.Y(); 
	  sigmaZi = X_start.Y() - zi_RC - 6.6;
	  xi_RC   = X_start.X() - (X_start.Y() - zi_RC)*Vx_RC/Vz_xRC;	
	  ti_RC   = Y_start.Z();
	  sigmaXi = sqrt(sigmaXi*sigmaXi + Vx_RC*Vx_RC*sigmaT2);
	}

	//Now, stop point:
	if (X_stop.Y() > Y_stop.Y()) {
	  //This means, the last hit is in X view:
	  xf_RC   = X_stop.X();
	  zf_RC   = X_stop.Y(); 
	  sigmaZf = zf_RC - Y_stop.Y() - 6.6;
	  yf_RC   = Y_stop.X() + (zf_RC-Y_stop.Y() )*Vy_RC/Vz_yRC;
	  ti_RC   = X_stop.Z();
	  sigmaYi = sqrt(sigmaYi*sigmaYi+ Vy_RC*Vy_RC*sigmaT2);
	}
	else {
	  //This means, the last hit is in Y view:
	  yf_RC   = Y_stop.X();
	  zf_RC   = Y_stop.Y(); 
	  sigmaZf = zf_RC-Y_stop.Y()-6.6;
	  xf_RC   = X_stop.X() + (X_stop.Y() - zi_RC)*Vx_RC/Vz_xRC;
	  tf_RC   = Y_stop.Z();
	  sigmaXi = sqrt(sigmaXi*sigmaXi+ Vx_RC*Vx_RC*sigmaT2);
	}
      }

      //Or, this track is going backwards: 
      else {
	if (X_start.Y() > Y_start.Y()) {
	  //This means, the first hit is in X view: Xz > Yz
	  xi_RC   = X_start.X();
	  zi_RC   = X_start.Y(); 
	  sigmaZi = zi_RC-Y_start.Y()-6.6;
	  yi_RC   = Y_start.X() + (sigmaZi+6.6)*Vy_RC/Vz_yRC;	
	  ti_RC   = X_start.Z();
	  sigmaYi = sqrt(sigmaYi*sigmaYi + Vy_RC*Vy_RC*sigmaT2);
	}
	else {
	  //This means, the first hit is in Y view: Yz > Xz
	  yi_RC   = Y_start.X();
	  zi_RC   = Y_start.Y(); 
	  sigmaZi = zi_RC - X_start.Y() - 6.6;
	  xi_RC   = X_start.X() + (sigmaZi+6.6)*Vx_RC/Vz_xRC;	
	  ti_RC   = Y_start.Z();
	  sigmaXi = sqrt(sigmaXi*sigmaXi + Vx_RC*Vx_RC*sigmaT2);
	}

	//Now, stop point:
	if (X_stop.Y() < Y_stop.Y()) {
	  //This means, the last hit is in X view: Xz < Yz
	  xf_RC   = X_stop.X();
	  zf_RC   = X_stop.Y(); 
	  sigmaZf = Y_stop.Y() - zf_RC - 6.6;
	  yf_RC   = Y_stop.X() - (sigmaZf+6.6)*Vy_RC/Vz_yRC;
	  ti_RC   = X_stop.Z();
	  sigmaYi = sqrt(sigmaYi*sigmaYi+ Vy_RC*Vy_RC*sigmaT2);
	}
	else {
	  //This means, the last hit is in Y view: Xz > Yz
	  yf_RC   = Y_stop.X();
	  zf_RC   = Y_stop.Y(); 
	  sigmaZf = Y_stop.Y()-zf_RC-6.6;
	  xf_RC   = X_stop.X() - (sigmaZf+6.6)*Vx_RC/Vz_xRC;
	  tf_RC   = Y_stop.Z();
	  sigmaXi = sqrt(sigmaXi*sigmaXi+ Vx_RC*Vx_RC*sigmaT2);
	}
      }

      sigmaZi = sqrt(sigmaZi*sigmaZi+7.9524);
      sigmaZf = sqrt(sigmaZf*sigmaZf+7.9524);

      RC_Tree -> Fill();

    }
  }

  //If triggered but not reconstructed, fill the mc info anyway:
  if (triggered && !reconstructed)    RC_Tree -> Fill();

  //std::cout<<"beta: "<< beta_MC<<std::endl;
  evt.put(std::move(clustercol));
} //end production function

void zcl::SMMCluster::endJob(){

}

//Predetermine the first and last hit in this 2D cluster
int zcl::SMMCluster::MissC(art::ServiceHandle<geo::Geometry> const & geom,
			   art::PtrVector<rb::CellHit> & hitlist,
			   TVector3 & startC,      //x(y)/cm, z/cm, tns
			   TVector3 & stopC      
			   )
{
  int missHits = 0;
  int nhits = hitlist.size();
  rb::SortByPlane(hitlist);

  const art::Ptr<rb::CellHit> chit0 = hitlist[0];
  const art::Ptr<rb::CellHit> chitf = hitlist[nhits-1];

  float t0 = chit0->TNS();
  int   c0 = chit0->Cell();
  int   p0 = chit0->Plane();
  float tf = chitf->TNS();
  int   cf = chitf->Cell();
  int   pf = chitf->Plane();

  int   p_pre = p0;

  if (c0>cf)     //largest c corresponds to c0 in that plane
    {
      for (int i=0; i!= nhits; ++i) {
	const art::Ptr<rb::CellHit> chiti = hitlist[i];
	int pi = chiti->Plane();
	int ci = chiti->Cell();
	if (pi==p0 && ci>c0) c0 = ci;
	if (pi==pf && ci<cf) c0 = cf;
	if (pi>p_pre+2) {
	  missHits+=(pi-p_pre)/2;
	} 
	p_pre=pi;
      }
    }

  else 
    {
      for (int i=0; i!= nhits; ++i) {
	const art::Ptr<rb::CellHit> chiti = hitlist[i];
	int pi = chiti->Plane();
	int ci = chiti->Cell();
	if (pi==p0 && ci<c0) c0 = ci;
	if (pi==pf && ci>cf) c0 = cf;
	if (pi>p_pre+2) missHits+=(pi-p_pre)/2; 
	p_pre=pi;
      }
    }

  double xyz0[3],xyzf[3];
  geom->Plane(p0)->Cell(c0)->GetCenter(xyz0);
  geom->Plane(pf)->Cell(cf)->GetCenter(xyzf);

  double x0, xf;
  if (chit0->View()==geo::kX){
    x0 = xyz0[0];
    xf = xyzf[0];
  }

  else {
    x0 = xyz0[1];
    xf = xyzf[1];
  }

  if (t0<tf) {
    startC.SetXYZ(x0,xyz0[2],t0);
    stopC.SetXYZ(xf,xyzf[2],tf);
  }
  else {
    startC.SetXYZ(xf,xyzf[2],tf);
    stopC.SetXYZ(x0,xyz0[2],t0);
  }

  return missHits;
}

void zcl::SMMCluster::LinFit(std::vector<TVector3> const & hits,
			     TVector3 const h0,  //First hit
			     TVector3 & vinfo)   //vx, vy, sigmaT2 
{
  float xt = 0;
  float x0 = h0.X();
  float yt = 0;
  float y0 = h0.Y();
  float tt = 0;
  float t0 = h0.Z();
 
  for (unsigned i = 0; i!=hits.size(); ++i) {
    xt += (hits[i].X()-x0)*(hits[i].Z()-t0);
    yt += (hits[i].Y()-y0)*(hits[i].Z()-t0); 
    tt += (hits[i].Z()-t0)*(hits[i].Z()-t0);
  }

  double vx = xt/tt;
  double vy = yt/tt;
  double sigmaT2=0;

  for (unsigned i = 0; i!=hits.size(); ++i) {
    float A = hits[i].Z()-t0-(hits[i].X()-x0)/vx;
    float B = hits[i].Z()-t0-(hits[i].Y()-y0)/vy;
    sigmaT2 += A*A+B*B;
  }

  vinfo.SetXYZ(vx,vy,sigmaT2);
}

double zcl::SMMCluster::calcW(double z, double w1, double w2, double z1, double z2) {
  double w =  (w2*(z-z1)+w1*(z2-z))/(z2-z1);
  if (w > 761) w = 761;
  else if (w < -761) w = -761;

  return w;
}


DEFINE_ART_MODULE(zcl::SMMCluster)
