////////////////////////////////////////////////////////////////////////
// Class:       SPCluster
// Module Type: producer
// File:        SPCluster_module.cc
//
// Generated at Wed Dec 18 15:41:32 2013 by Zukai Wang using artmod
// from cetpkgsupport v1_04_02.
////////////////////////////////////////////////////////////////////////
#include "art/Framework/Core/EDProducer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "art/Framework/Services/Optional/TFileService.h"

#include "HoughPair.h"
#include "CMap/CMap.h"
#include "Geometry/Geometry.h"
#include "GeometryObjects/CellGeo.h"
#include "GeometryObjects/PlaneGeo.h"

#include "NovaDAQConventions/DAQConventions.h"
#include "RecoBase/CellHit.h"
#include "RecoBase/Cluster.h"

#include "TH3.h"
#include "TVector3.h"
#include "TTree.h"

#include <algorithm> 
#include <utility>
#include <cmath>

namespace zcl {
  class SPCluster;
}

class zcl::SPCluster : public art::EDProducer {
public:
  explicit SPCluster(fhicl::ParameterSet const & pset);
  virtual ~SPCluster();

  void produce(art::Event & evt);
  void beginJob();
  void endJob();

  bool SurfAssign(art::Ptr<rb::CellHit> const & hptr);
  bool ContaineAssign(art::Ptr<rb::CellHit> const & hptr);

  //Return the indices of the earliest and latest matching elements in the other view:
  bool ViewCheck(art::Ptr<rb::CellHit> const &,
		 art::Ptr<rb::CellHit> const &);  

  float Dist(art::Ptr<rb::CellHit> const &,
	     art::Ptr<rb::CellHit> const &);  

  //This function return the index of the hit which earlier than and closest to the time you assigned
  unsigned IndexLocator(std::vector<art::Ptr<rb::CellHit> > & hit_ptrs,
			float tns);

  bool dupCheck(std::vector<art::Ptr<rb::CellHit> > & hit_ptrs,
		art::Ptr<rb::CellHit>& chit);

private:
  // FHICL Parameters:
  std::string fCellHitInput;
  float _hf_min;     //The threshold of hits/distance
  //One should consider the speed of MM going to the next plane in the other view, 
  //so the timing cut should be looser in accomodating slower monopoles
  unsigned _sigmaT;    //Time resolution (ns) for single hit: same view parameter
  unsigned _deltaT;    //for checking mate hits on the other view
  unsigned _minHits;   //precut on hits number limit
  float _deltaP_S;    //This is used for surface determination and also for view matching determination
  float _deltaC_S;    //For surface determination
  float _deltaP_C;    //This is used for surface determination and also for view matching determination
  float _deltaC_C;    //For surface determination
  float _sigmaL;    // Triangle cut for the max additional ratio allowed
  float _Xmax;
  float _Ymax;
  float _DistMin;   //Min distance requirement    
  float _vproMin;   //Max beta: 0.01
  float _vproMax;   //Min beta: 4e-5
  float _df;        //Maximum fraction of (plane gap)/(plane range) allowed

};

zcl::SPCluster::SPCluster(fhicl::ParameterSet const & p) :
  fCellHitInput      (p.get< std::string  >("CellHitInput")      ),
  _hf_min     (p.get< float    >("hf_min")),     //Min hit number density in unit distance(in cell unit)
  _sigmaT     (1500),      //100TDC = 1562.5ns
  _deltaT     (4687.5),    //300TDC 
  _minHits    (30),
  _deltaP_S   (4),
  _deltaC_S   (10),
  _deltaP_C   (2),
  _deltaC_C   (2),
  _sigmaL     (0.02),
  _Xmax       (p.get<float>("Xmax")), 
  _Ymax       (p.get<float>("Ymax")),            
  _DistMin    (p.get<float>("DistMin")),
  _vproMin    (p.get<float>("vproMin")),
  _vproMax    (p.get<float>("vproMax")),
  _df         (0.1)
{
  // Call appropriate Produces<>() functions here.
  produces< std::vector<rb::Cluster> >();
}

zcl::SPCluster::~SPCluster()
{
  // Clean up dynamic memory and other resources here.
}

void zcl::SPCluster::beginJob()
{
}

void zcl::SPCluster::produce(art::Event & evt)
{
  art::ServiceHandle<geo::Geometry> geom;
  art::Handle< std::vector<rb::CellHit> > hitcol;
  evt.getByLabel(fCellHitInput, hitcol);
  // Load the cell hits into a vector for easy use
  std::vector<art::Ptr<rb::CellHit > > hitlist[2];

  //This process imitated the ADC filtering process
  for(unsigned int i = 0; i < hitcol->size(); ++i){
    art::Ptr<rb::CellHit> hit_ptr(hitcol, i);
    //Make a selection of hits here: first 2 diblocks
    
    if (hit_ptr->ADC() > 50 && hit_ptr->Plane() < _Xmax+1) {
      unsigned v = 0;
      if (hit_ptr->View()==geo::kY ) v = 1;
      hitlist[v].emplace_back(hit_ptr);
    } 
  }
  

  //Sort the hits by time:
  rb::SortByTime(hitlist[0]);
  rb::SortByTime(hitlist[1]);

  //X, Y, T  (plane, cell, tns)
  std::vector<art::Ptr<rb::CellHit> > c_PS[2];     //They will contain all Valid non surface hits on each view
  std::vector<art::Ptr<rb::CellHit> > s_PS[2];     //They will contain all Valid surface and mate hits on each view

  std::unique_ptr<std::vector<rb::Cluster> > clustercol(new std::vector<rb::Cluster>);

  //View Check and doing surface hits collecion
  for (unsigned v = 0; v!=2; ++v) {
    for (unsigned i = 0; i!= hitlist[v].size(); ++i ) {
      art::Ptr<rb::CellHit> const& hit = hitlist[v][i];
      float t = hit->TNS();
      unsigned start_id = IndexLocator(hitlist[1-v], t-_deltaT);
      unsigned stop_id  = IndexLocator(hitlist[1-v], t+_deltaT); 
      bool met = false;
      for (unsigned j = start_id; j<stop_id; ++j ) {
	art::Ptr<rb::CellHit> const& buddy = hitlist[1-v][j];
	if (ViewCheck(hit, buddy)) {
	  if (!met && ContaineAssign(hit) ) {
	    c_PS[v].emplace_back(hit);
	    met=true;
	  }
	  else if (SurfAssign(hit) || SurfAssign(buddy)) {
	    s_PS[v].emplace_back(hit);
	    break;
	  }
	}
      }
    }
  }

  for (unsigned v=0; v!=2; ++v) {
    if (c_PS[v].size()<_minHits) {
      evt.put(std::move(clustercol));
      return;
    }
    if (s_PS[v].size()<2) {
      evt.put(std::move(clustercol));
      return;
    }
  } 

  for (int v=0; v!=2; ++v) {
    //  std::cout<<"View: "<<v<<std::endl;
    //If all X view hits have all been reviewed and no cluster has been reconstructed, you can stop here:
    if (v==1 && clustercol->empty() ) break;

    //Try to find entry hit on valid surface hits containers
    for (unsigned i=0; i<s_PS[v].size()-1; ++i) {
      //     bool found = false;
      art::Ptr<rb::CellHit> hit1 = s_PS[v][i];
      float ti = hit1->TNS();
      int   pi = hit1->Plane();
      int   ci = hit1->Cell();

      //      std::cout<<"potential Pi: "<<pi<<"  Ci: "<<ci<<std::endl;
     

      unsigned ini = IndexLocator(c_PS[v], ti-_deltaT);
      //      std::cout<<"ini:"<<ini<<std::endl;

      if (ini+_minHits+1 > c_PS[v].size() ) break;
      
      for (unsigned j=s_PS[v].size()-1; j>i && ini+_minHits+1 < c_PS[v].size(); --j) {
	art::Ptr<rb::CellHit> hit2 = s_PS[v][j];
	float tf = hit2->TNS();
	int   pf = hit2->Plane();
	int   cf = hit2->Cell();

	if (tf +_deltaT  < c_PS[v][ini+_minHits]->TNS()) {
	  //	  std::cout<<"break due to tf too small"<<std::endl;
	  break;
	}
	if (tf > c_PS[v][c_PS[v].size()-1]->TNS() +_deltaT ){
	  //	  std::cout<<"continue due to tf too large"<<std::endl;
	  continue;
	}

	unsigned   fin = IndexLocator(c_PS[v], tf+_deltaT);
	float Distance = Dist(hit1, hit2);

	if (Distance < _DistMin) {
	  continue;
	}
	if (fin<ini+Distance*_hf_min) {
	  //	  std::cout<<"fin:"<<fin<<std::endl;
	  continue;
	}
	
	float vpro = (tf-ti)/Distance;
	//	std::cout<<"vpro: "<<vpro<<std::endl;
	if (tf > ti+Distance*_vproMax+_sigmaT) {
	  //	  std::cout<<"tf too large"<<std::endl;
	  continue;
	}
	else if (tf+_sigmaT < ti+Distance*_vproMin) {
	  //	  std::cout<<"tf too small"<<std::endl;
	  continue;
	}

	//Here we implemente the CCB(continuity-check and break) mechanisim
	//First, calculate the time window for checking the previous hit on track:
	int _dP  = abs(pf-pi)*_df;
	float dt = (tf-ti)*_df;

	//Fish is coming!
	unsigned h_counts = 0;
	float disCut = std::min(Distance*_sigmaL,float(3));

	unsigned h_cut = Distance*_hf_min;
	rb::Cluster Cluster;

	int last_P = pi;
	float last_T = ti;
	for (unsigned k=ini; k!=fin; ++k) {
	  art::Ptr<rb::CellHit> Mid = c_PS[v][k];
	  int pk = Mid->Plane();
	  int ck = Mid->Cell();
	  float tk = Mid->TNS();

	  //	  std::cout<<"k-ini: "<<k-ini<<std::endl;
	  if (tk>last_T+dt+_sigmaT) {
	    //    std::cout<<"hits attached: "<<h_counts<<"  current Plane: "<<pk<<" curent time: "<<tk-last_T<<" view:"<<v<<std::endl;
	    if (pk> last_P+_dP || pk+_dP<last_P) break;
	  }

	  if (pk > pi && pk > pf) continue;
	  else if (pk < pi && pk < pf) continue;
	  if (ck > ci && ck > cf) continue;
	  else if (ck < ci && ck < cf) continue;

	  float t1 = tk-ti;
	  float t2 = tf-tk;
	  float d1 = Dist(hit1, Mid);
	  float d2 = Dist(hit2, Mid);

	  if (d1+d2 > Distance+disCut) continue;

	  if (d1>d2) {
	    if (t1 < d1*vpro + _sigmaT && t1 > d1*vpro - _sigmaT) {
	      ++h_counts;
	      Cluster.Add(Mid);
	      //refresh the previous hit:
	      last_P= pk;
	      last_T= tk;
	    }
	  }
	  else {
	    if (t2 > d2*vpro - _sigmaT && t2 < d2*vpro + _sigmaT) {
	      ++h_counts;
	      Cluster.Add(Mid);
	      //refresh the previous hit:
	      last_P= pk;
	      last_T= tk;
	    }
	  }
	}

	if (h_counts > h_cut ) {
	  //Now delete the hits from the container:c_PS[v]
	  for (unsigned idx =0; idx != Cluster.NCell(); ++idx) {
	    auto it = find(c_PS[v].begin(), c_PS[v].end(), Cluster.Cell(idx));
	    c_PS[v].erase(it);
	  }
	  
	  Cluster.Add(hit1);
	  Cluster.Add(hit2);
	  clustercol->emplace_back(Cluster);
	
	  
	}	  
      }
    } // 1 view finished
  }

  evt.put(std::move(clustercol));
} //end production function

void zcl::SPCluster::endJob(){

}

bool zcl::SPCluster::SurfAssign(art::Ptr<rb::CellHit> const & hptr) 
{
  if      ( hptr->Cell()  > _Ymax - _deltaC_S ) return true;
  else if ( hptr->Cell()  < _deltaC_S         ) return true;
  else if ( hptr->Plane() < _deltaP_S         ) return true;
  else if ( hptr->Plane() > _Xmax - _deltaP_S ) return true;

  return false;

}

bool zcl::SPCluster::ContaineAssign(art::Ptr<rb::CellHit> const & hptr)
{
  if      ( hptr->Cell()  > _Ymax - _deltaC_C ) return false;
  else if ( hptr->Cell()  < _deltaC_C         ) return false;
  else if ( hptr->Plane() < _deltaP_C         ) return false;
  else if ( hptr->Plane() > _Xmax - _deltaP_C ) return false;

  return true;
}

//Note: Ts < Tm (Looking forwards). Time check is already done in the main loop.
bool zcl::SPCluster::ViewCheck(art::Ptr<rb::CellHit> const & hptr1,
			       art::Ptr<rb::CellHit> const & hptr2)
{
  if (hptr1->TNS() > hptr2->TNS()+_deltaT) return false;
  if (hptr2->TNS() > hptr1->TNS()+_deltaT) return false;
  if (hptr1->Plane() > hptr2->Plane() ) {
    if (hptr1->Plane() - hptr2->Plane() < _deltaP_S ) return true;
  }
  else if (hptr2->Plane() - hptr1->Plane() < _deltaP_S ) return true;
  return false; 
}

float zcl::SPCluster::Dist(art::Ptr<rb::CellHit> const & hptr1,
			   art::Ptr<rb::CellHit> const & hptr2)
{
  float dx = ((hptr1->Plane() )-  (hptr2->Plane()) )*1.6795;
  float dy = (hptr1->Cell() ) -  (hptr2->Cell());
  return sqrt(dx*dx+dy*dy);
}  

//It should return the index before T
unsigned zcl::SPCluster::IndexLocator(std::vector<art::Ptr<rb::CellHit> > & hit_ptrs,
				      float tns)
{
  unsigned i = 0;
  unsigned f = hit_ptrs.size()-1;

  if (tns < (hit_ptrs[0])->TNS()) return i;
  if (tns > (hit_ptrs[f])->TNS()) return f;

  while (f>i+1) {
    unsigned m = (f+i)/2;
    if (tns >= (hit_ptrs[m])->TNS() ) i = m;
    else f = m; 
  }
  return i;
}

bool zcl::SPCluster::dupCheck(std::vector<art::Ptr<rb::CellHit> > & hit_ptrs,
			      art::Ptr<rb::CellHit> & chit)
{
  if (hit_ptrs.empty()) return false;
  float t = chit->TNS();

  unsigned f = hit_ptrs.size()-1;
  if (t > hit_ptrs[f]->TNS()) return false;
  unsigned i = 0;

  while (f>i+5) {
    unsigned m = (f+i)/2;
    if (t >= hit_ptrs[m]->TNS() ) i = m;
    else f = m+1; 
  }

  for (unsigned idx = i; idx!=f; ++idx) {
    if (chit==hit_ptrs[idx]) 
      return true;
  }

  return false;

}

DEFINE_ART_MODULE(zcl::SPCluster)
