////////////////////////////////////////////////////////////////////////
/// \file G4Alg.h
//
/// \version $Id: G4Alg.h,v 1.3 2012-09-13 15:23:13 brebel Exp $
/// \author  brebel@fnal.gov
////////////////////////////////////////////////////////////////////////
#ifndef G4NOVA_G4ALG_H
#define G4NOVA_G4ALG_H

// ART includes
#include "art/Framework/Principal/Handle.h"
#include "art/Persistency/Common/Ptr.h"

namespace art { 
  class Event; 
  class ParameterSet;
}
namespace g4b{
    class G4Helper;
}
namespace simb{
    class MCTruth;
}
namespace sim{
    class FLSHitList;
    class Particle;
}

namespace g4n {

  /// An algorithm to pass interaction information to Geant4 and create hits and particle lists
  class G4Alg {
  public:
    explicit G4Alg(fhicl::ParameterSet const &pset);
    virtual ~G4Alg();                        

    void RunGeant(std::vector< art::Handle< std::vector<simb::MCTruth> > >& mclists,
		  std::vector< sim::FLSHitList >&                           flshitlist,
		  std::vector< sim::Particle >&                             particlelist,
		  std::vector< std::vector<std::pair<size_t, size_t> > >&   pListLimits);

    void RunGeant(std::vector< const simb::MCTruth* >&                      mctruths,
		  std::vector< sim::FLSHitList >&                           flshitlist,
		  std::vector< sim::Particle >&                             particlelist,
		  std::map<int, size_t >&                                   trackIDToMCTruthIndex);

    void RunGeant(art::Ptr<simb::MCTruth>                                   mctruth,
		  std::vector< sim::FLSHitList >&                           flshitlist,
		  std::vector< sim::Particle >&                             particlelist,
		  int                                                       trackIDOffset=-1);

    void RunGeant(const simb::MCTruth*                                      mctruth,
		  std::vector< sim::FLSHitList >&                           flshitlist,
		  std::vector< sim::Particle >&                             particlelist,
		  int                                                       trackIDOffset=-1);

  private:

    void CollectFLSHitsAndParticles(std::vector< sim::FLSHitList >&         flshitlist,
				    std::vector< sim::Particle >&           particlelist);
		  
    void ConfigUserActionManager(std::vector<std::string> const& actionList,
                                 fhicl::ParameterSet const& pset);
    void OverridePDGLifetimes();

    g4b::G4Helper*  fG4Help;              ///< G4Helper object
    std::string     fG4MacroPath;         ///< path to G4 macro
    std::string     fG4PhysListName;      ///< name of G4 physics list to use
    double          fEnergyThresh;        ///< threshold for stopping the tracking of a particle in GeV
    bool            fManyParticles;       ///< if true, keep individual track ids from processes like compt and brem
    bool            fSparseTrajectories;  ///< if true, only save necessary points in particle trajectories
    bool            fIsRockVetoUsed;      ///< Is RockVeto check used?
    bool            fSlimSimParticles;    ///< if true, only save particles which lead to an FLS hit
    bool            fIsBigBoxUsed;        ///< Do we need to use the BigBox cut?
                                          ///< If a particle leaves the DetectorBigBox, stop tracking it
    double          fDetectorBigBoxRange; ///< Distance to the edge of the DetectorEnclosure box in cm
                                          ///< to define the DetectorBigBox.
    std::string     fGenModuleLabel;      ///< label of module that made the particles to track

    int             fPlaIndex;            ///< index of the ParticleListAction in the UserActionManager
    int             fFlsIndex;            ///< index of the FLSHitListAction in the UserActionManager

    std::vector<std::string> fUserActions;///< UserAction classes 

    std::vector<int>    fOverrideLifetimePDG; ///< list of pdg code to change
    std::vector<double> fOverrideLifetimeNS;  ///< new lifetime of PDG in ns
  };
}

#endif // G4NOVA_G4ALG_H
////////////////////////////////////////////////////////////////////////
