//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
// $Id: NovaG4MonopoleEquation.hh,v 1.2 2012-09-20 21:45:42 greenc Exp $
// GEANT4 tag $Name: not supported by cvs2svn $
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//
//
// class NovaG4MonopoleEquation
//
// Class description:
//
// This is the right-hand side of equation of motion in a combined
// electric and magnetic field for magnetic monopoles.

// History:
// - Created. V.Grichine, 10.11.98
// - Modified. S.Burdin, 30.04.10
// 	       B.Bozsogi, 15.06.10
//  2012-02-04:  R. Hatcher - incorporate into NOvA framework from base G4 v4.9.4.p02
// -------------------------------------------------------------------

#ifndef NOVAG4MONOPOLEEQUATION_hh
#define NOVAG4MONOPOLEEQUATION_hh

#include "Geant4/G4EquationOfMotion.hh"
#include "Geant4/G4ElectroMagneticField.hh"

namespace g4n {

class NovaG4MonopoleEquation : public G4EquationOfMotion
{
  public:  // with description

  NovaG4MonopoleEquation(G4ElectroMagneticField *emField );
      // : G4EquationOfMotion( emField ) {;}

    ~NovaG4MonopoleEquation() {;} 

    void  SetChargeMomentumMass(G4double particleMagneticCharge,
                                G4double particleElectricCharge,
                                G4double mass);
                                 
    void EvaluateRhsGivenB(const G4double y[],
                           const G4double Field[],
                                 G4double dydx[] ) const;
      // Given the value of the electromagnetic field, this function 
      // calculates the value of the derivative dydx.

  private:

    G4double        fMagCharge ;
    G4double        fElCharge;
    G4double        fMassCof;
};

} // end-of-namespace g4n

#endif
